@@ -7,7 +7,7 @@
  * computers, as long as there is a network connection to the
  * computer where the application is running.
  *
- * Only some of the windowing systems where GTK+ runs have the
+ * Only some of the windowing systems where GTK runs have the
  * concept of multiple displays. (The X Window System is the
  * main example.) Other windowing systems can only handle one
  * keyboard and mouse, and combine all monitors into
@@ -68,32 +68,25 @@ enum
 static GtkWidget *
 find_toplevel_at_pointer (GdkDisplay *display)
 {
-  GdkWindow *pointer_window;
+  GdkSurface *pointer_window;
   GtkWidget *widget = NULL;
 
-  pointer_window = gdk_device_get_window_at_position (gtk_get_current_event_device (),
-                                                      NULL, NULL);
+  pointer_window = gdk_device_get_surface_at_position (gtk_get_current_event_device (), NULL, NULL);
 
-  /* The user data field of a GdkWindow is used to store a pointer
-   * to the widget that created it.
-   */
   if (pointer_window)
-    {
-      gpointer widget_ptr;
-      gdk_window_get_user_data (pointer_window, &widget_ptr);
-      widget = widget_ptr;
-    }
+    widget = GTK_WIDGET (gtk_root_get_for_surface (pointer_window));
 
-  return widget ? gtk_widget_get_toplevel (widget) : NULL;
+  return widget;
 }
 
-static gboolean
-button_release_event_cb (GtkWidget       *widget,
-                         GdkEventButton  *event,
-                         gboolean        *clicked)
+static void
+released_cb (GtkGestureMultiPress *gesture,
+             guint                 n_press,
+             gdouble               x,
+             gdouble               y,
+             gboolean             *clicked)
 {
   *clicked = TRUE;
-  return TRUE;
 }
 
 /* Asks the user to click on a window, then waits for them click
@@ -101,17 +94,16 @@ button_release_event_cb (GtkWidget       *widget,
  * window under the pointer, or NULL, if there is none.
  */
 static GtkWidget *
-query_for_toplevel (GdkScreen  *screen,
+query_for_toplevel (GdkDisplay *display,
                     const char *prompt)
 {
-  GdkDisplay *display = gdk_screen_get_display (screen);
   GtkWidget *popup, *label, *frame;
   GdkCursor *cursor;
   GtkWidget *toplevel = NULL;
   GdkDevice *device;
 
   popup = gtk_window_new (GTK_WINDOW_POPUP);
-  gtk_window_set_screen (GTK_WINDOW (popup), screen);
+  gtk_window_set_display (GTK_WINDOW (popup), display);
   gtk_window_set_modal (GTK_WINDOW (popup), TRUE);
   gtk_window_set_position (GTK_WINDOW (popup), GTK_WIN_POS_CENTER);
 
@@ -123,35 +115,38 @@ query_for_toplevel (GdkScreen  *screen,
   g_object_set (label, "margin", 10, NULL);
   gtk_container_add (GTK_CONTAINER (frame), label);
 
-  gtk_widget_show_all (popup);
-  cursor = gdk_cursor_new_from_name (display, "crosshair");
+  gtk_widget_show (popup);
+  cursor = gdk_cursor_new_from_name ("crosshair", NULL);
   device = gtk_get_current_event_device ();
 
   if (gdk_seat_grab (gdk_device_get_seat (device),
-                     gtk_widget_get_window (popup),
+                     gtk_widget_get_surface (popup),
                      GDK_SEAT_CAPABILITY_ALL_POINTING,
                      FALSE, cursor, NULL, NULL, NULL) == GDK_GRAB_SUCCESS)
     {
+      GtkGesture *gesture = gtk_gesture_multi_press_new ();
       gboolean clicked = FALSE;
 
-      g_signal_connect (popup, "button-release-event",
-                        G_CALLBACK (button_release_event_cb), &clicked);
+      g_signal_connect (gesture, "released",
+                        G_CALLBACK (released_cb), &clicked);
+      gtk_widget_add_controller (popup, GTK_EVENT_CONTROLLER (gesture));
 
-      /* Process events until clicked is set by button_release_event_cb.
+      /* Process events until clicked is set by our button release event handler.
        * We pass in may_block=TRUE since we want to wait if there
        * are no events currently.
        */
       while (!clicked)
         g_main_context_iteration (NULL, TRUE);
 
-      toplevel = find_toplevel_at_pointer (gdk_screen_get_display (screen));
+      gdk_seat_ungrab (gdk_device_get_seat (device));
+
+      toplevel = find_toplevel_at_pointer (display);
       if (toplevel == popup)
         toplevel = NULL;
     }
 
   g_object_unref (cursor);
   gtk_widget_destroy (popup);
-  gdk_display_flush (display);                 /* Really release the grab */
 
   return toplevel;
 }
@@ -162,17 +157,17 @@ query_for_toplevel (GdkScreen  *screen,
 static void
 query_change_display (ChangeDisplayInfo *info)
 {
-  GdkScreen *screen = gtk_widget_get_screen (info->window);
+  GdkDisplay *display = gtk_widget_get_display (info->window);
   GtkWidget *toplevel;
 
-  toplevel = query_for_toplevel (screen,
+  toplevel = query_for_toplevel (display,
                                  "Please select the toplevel\n"
-                                 "to move to the new screen");
+                                 "to move to the new display");
 
   if (toplevel)
-    gtk_window_set_screen (GTK_WINDOW (toplevel), gdk_display_get_default_screen (info->current_display));
+    gtk_window_set_display (GTK_WINDOW (toplevel), info->current_display);
   else
-    gdk_display_beep (gdk_screen_get_display (screen));
+    gdk_display_beep (display);
 }
 
 /* Called when the user clicks on a button in our dialog or
@@ -224,7 +219,6 @@ open_display_cb (GtkWidget         *button,
   gtk_container_add (GTK_CONTAINER (content_area), display_entry);
 
   gtk_widget_grab_focus (display_entry);
-  gtk_widget_show_all (gtk_bin_get_child (GTK_BIN (dialog)));
 
   while (!result)
     {
@@ -305,7 +299,7 @@ create_frame (ChangeDisplayInfo *info,
   *frame = gtk_frame_new (title);
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
-  gtk_container_set_border_width (GTK_CONTAINER (hbox), 8);
+  g_object_set (hbox, "margin", 8, NULL);
   gtk_container_add (GTK_CONTAINER (*frame), hbox);
 
   scrollwin = gtk_scrolled_window_new (NULL, NULL);
@@ -313,7 +307,7 @@ create_frame (ChangeDisplayInfo *info,
                                   GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrollwin),
                                        GTK_SHADOW_IN);
-  gtk_box_pack_start (GTK_BOX (hbox), scrollwin, TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (hbox), scrollwin);
 
   *tree_view = gtk_tree_view_new ();
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (*tree_view), FALSE);
@@ -323,7 +317,7 @@ create_frame (ChangeDisplayInfo *info,
   gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
 
   *button_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
-  gtk_box_pack_start (GTK_BOX (hbox), *button_vbox, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (hbox), *button_vbox);
 
   if (!info->size_group)
     info->size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
@@ -363,11 +357,11 @@ create_display_frame (ChangeDisplayInfo *info)
 
   button = left_align_button_new ("_Open...");
   g_signal_connect (button, "clicked",  G_CALLBACK (open_display_cb), info);
-  gtk_box_pack_start (GTK_BOX (button_vbox), button, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (button_vbox), button);
 
   button = left_align_button_new ("_Close");
   g_signal_connect (button, "clicked",  G_CALLBACK (close_display_cb), info);
-  gtk_box_pack_start (GTK_BOX (button_vbox), button, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (button_vbox), button);
 
   info->display_model = (GtkTreeModel *)gtk_list_store_new (DISPLAY_NUM_COLUMNS,
                                                             G_TYPE_STRING,
@@ -539,15 +533,15 @@ do_changedisplay (GtkWidget *do_widget)
       content_area = gtk_dialog_get_content_area (GTK_DIALOG (info->window));
 
       vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
-      gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
-      gtk_box_pack_start (GTK_BOX (content_area), vbox, TRUE, TRUE, 0);
+      g_object_set (vbox, "margin", 8, NULL);
+      gtk_container_add (GTK_CONTAINER (content_area), vbox);
 
       frame = create_display_frame (info);
-      gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
+      gtk_container_add (GTK_CONTAINER (vbox), frame);
 
       initialize_displays (info);
 
-      gtk_widget_show_all (info->window);
+      gtk_widget_show (info->window);
       return info->window;
     }
   else
