@@ -13,19 +13,22 @@ show_parsing_error (GtkCssProvider *provider,
                     const GError   *error,
                     GtkTextBuffer  *buffer)
 {
+  const GtkCssLocation *start_location, *end_location;
   GtkTextIter start, end;
   const char *tag_name;
 
+  start_location = gtk_css_section_get_start_location (section);
   gtk_text_buffer_get_iter_at_line_index (buffer,
                                           &start,
-                                          gtk_css_section_get_start_line (section),
-                                          gtk_css_section_get_start_position (section));
+                                          start_location->lines,
+                                          start_location->line_bytes);
+  end_location = gtk_css_section_get_end_location (section);
   gtk_text_buffer_get_iter_at_line_index (buffer,
                                           &end,
-                                          gtk_css_section_get_end_line (section),
-                                          gtk_css_section_get_end_position (section));
+                                          end_location->lines,
+                                          end_location->line_bytes);
 
-  if (g_error_matches (error, GTK_CSS_PROVIDER_ERROR, GTK_CSS_PROVIDER_ERROR_DEPRECATED))
+  if (error->domain == GTK_CSS_PARSER_WARNING)
     tag_name = "warning";
   else
     tag_name = "error";
@@ -45,10 +48,8 @@ css_text_changed (GtkTextBuffer  *buffer,
   gtk_text_buffer_remove_all_tags (buffer, &start, &end);
 
   text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-  gtk_css_provider_load_from_data (provider, text, -1, NULL);
+  gtk_css_provider_load_from_data (provider, text, -1);
   g_free (text);
-
-  gtk_style_context_reset_widgets (gdk_screen_get_default ());
 }
 
 static void
@@ -110,7 +111,7 @@ do_css_basics (GtkWidget *do_widget)
     }
 
   if (!gtk_widget_get_visible (window))
-    gtk_widget_show_all (window);
+    gtk_widget_show (window);
   else
     gtk_widget_destroy (window);
 
