@@ -19,7 +19,7 @@ hex_spin_input (GtkSpinButton *spin_button,
   gchar *err;
   gdouble res;
 
-  buf = gtk_entry_get_text (GTK_ENTRY (spin_button));
+  buf = gtk_editable_get_text (GTK_EDITABLE (spin_button));
   res = strtol (buf, &err, 16);
   *new_val = res;
   if (*err)
@@ -33,16 +33,16 @@ hex_spin_output (GtkSpinButton *spin_button)
 {
   GtkAdjustment *adjustment;
   gchar *buf;
-  gint val;
+  gdouble val;
 
   adjustment = gtk_spin_button_get_adjustment (spin_button);
-  val = (gint) gtk_adjustment_get_value (adjustment);
+  val = gtk_adjustment_get_value (adjustment);
   if (fabs (val) < 1e-5)
     buf = g_strdup ("0x00");
   else
-    buf = g_strdup_printf ("0x%.2X", val);
-  if (strcmp (buf, gtk_entry_get_text (GTK_ENTRY (spin_button))))
-    gtk_entry_set_text (GTK_ENTRY (spin_button), buf);
+    buf = g_strdup_printf ("0x%.2X", (gint) val);
+  if (strcmp (buf, gtk_editable_get_text (GTK_EDITABLE (spin_button))))
+    gtk_editable_set_text (GTK_EDITABLE (spin_button), buf);
   g_free (buf);
 
   return TRUE;
@@ -60,7 +60,7 @@ time_spin_input (GtkSpinButton *spin_button,
   gchar *endh;
   gchar *endm;
 
-  text = gtk_entry_get_text (GTK_ENTRY (spin_button));
+  text = gtk_editable_get_text (GTK_EDITABLE (spin_button));
   str = g_strsplit (text, ":", 2);
 
   if (g_strv_length (str) == 2)
@@ -99,8 +99,8 @@ time_spin_output (GtkSpinButton *spin_button)
   hours = gtk_adjustment_get_value (adjustment) / 60.0;
   minutes = (hours - floor (hours)) * 60.0;
   buf = g_strdup_printf ("%02.0f:%02.0f", floor (hours), floor (minutes + 0.5));
-  if (strcmp (buf, gtk_entry_get_text (GTK_ENTRY (spin_button))))
-    gtk_entry_set_text (GTK_ENTRY (spin_button), buf);
+  if (strcmp (buf, gtk_editable_get_text (GTK_EDITABLE (spin_button))))
+    gtk_editable_set_text (GTK_EDITABLE (spin_button), buf);
   g_free (buf);
 
   return TRUE;
@@ -132,7 +132,7 @@ month_spin_input (GtkSpinButton *spin_button,
   for (i = 1; i <= 12; i++)
     {
       tmp1 = g_ascii_strup (month[i - 1], -1);
-      tmp2 = g_ascii_strup (gtk_entry_get_text (GTK_ENTRY (spin_button)), -1);
+      tmp2 = g_ascii_strup (gtk_editable_get_text (GTK_EDITABLE (spin_button)), -1);
       if (strstr (tmp1, tmp2) == tmp1)
         found = TRUE;
       g_free (tmp1);
@@ -162,8 +162,8 @@ month_spin_output (GtkSpinButton *spin_button)
   for (i = 1; i <= 12; i++)
     if (fabs (value - (double)i) < 1e-5)
       {
-        if (strcmp (month[i-1], gtk_entry_get_text (GTK_ENTRY (spin_button))))
-          gtk_entry_set_text (GTK_ENTRY (spin_button), month[i-1]);
+        if (strcmp (month[i-1], gtk_editable_get_text (GTK_EDITABLE (spin_button))))
+          gtk_editable_set_text (GTK_EDITABLE (spin_button), month[i-1]);
       }
 
   return TRUE;
@@ -201,8 +201,8 @@ do_spinbutton (GtkWidget *do_widget)
                                       NULL);
     gtk_builder_connect_signals (builder, NULL);
     window = GTK_WIDGET (gtk_builder_get_object (builder, "window"));
-    gtk_window_set_screen (GTK_WINDOW (window),
-                           gtk_widget_get_screen (do_widget));
+    gtk_window_set_display (GTK_WINDOW (window),
+                            gtk_widget_get_display (do_widget));
     gtk_window_set_title (GTK_WINDOW (window), "Spin Buttons");
     gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
     g_signal_connect (window, "destroy",
@@ -245,7 +245,7 @@ do_spinbutton (GtkWidget *do_widget)
   }
 
   if (!gtk_widget_get_visible (window))
-    gtk_widget_show_all (window);
+    gtk_widget_show (window);
   else
     gtk_widget_destroy (window);
 
