@@ -21,17 +21,17 @@
 /**
  * SECTION:gdkglcontext
  * @Title: GdkGLContext
- * @Short_description: OpenGL context
+ * @Short_description: OpenGL draw context
  *
  * #GdkGLContext is an object representing the platform-specific
- * OpenGL drawing context.
+ * OpenGL draw context.
  *
- * #GdkGLContexts are created for a #GdkWindow using
- * gdk_window_create_gl_context(), and the context will match
- * the #GdkVisual of the window.
+ * #GdkGLContexts are created for a #GdkSurface using
+ * gdk_surface_create_gl_context(), and the context will match the
+ * the characteristics of the surface.
  *
  * A #GdkGLContext is not tied to any particular normal framebuffer.
- * For instance, it cannot draw to the #GdkWindow back buffer. The GDK
+ * For instance, it cannot draw to the #GdkSurface back buffer. The GDK
  * repaint system is in full control of the painting to that. Instead,
  * you can create render buffers or textures and use gdk_cairo_draw_from_gl()
  * in the draw function of your widget to draw them. Then GDK will handle
@@ -46,14 +46,14 @@
  * ## Creating a new OpenGL context ##
  *
  * In order to create a new #GdkGLContext instance you need a
- * #GdkWindow, which you typically get during the realize call
+ * #GdkSurface, which you typically get during the realize call
  * of a widget.
  *
  * A #GdkGLContext is not realized until either gdk_gl_context_make_current(),
  * or until it is realized using gdk_gl_context_realize(). It is possible to
  * specify details of the GL context like the OpenGL version to be used, or
  * whether the GL context should have extra state validation enabled after
- * calling gdk_window_create_gl_context() by calling gdk_gl_context_realize().
+ * calling gdk_surface_create_gl_context() by calling gdk_gl_context_realize().
  * If the realization fails you have the option to change the settings of the
  * #GdkGLContext and try again.
  *
@@ -76,6 +76,13 @@
  * that is currently set by calling gdk_gl_context_clear_current().
  */
 
+/**
+ * GdkGLContext:
+ *
+ * The GdkGLContext struct contains only private fields and
+ * should not be accessed directly.
+ */
+
 #include "config.h"
 
 #include "gdkglcontextprivate.h"
@@ -88,8 +95,6 @@
 #include <epoxy/gl.h>
 
 typedef struct {
-  GdkDisplay *display;
-  GdkWindow *window;
   GdkGLContext *shared_context;
 
   int major;
@@ -100,25 +105,25 @@ typedef struct {
   guint use_texture_rectangle : 1;
   guint has_gl_framebuffer_blit : 1;
   guint has_frame_terminator : 1;
+  guint has_khr_debug : 1;
+  guint use_khr_debug : 1;
   guint has_unpack_subimage : 1;
-  guint has_sync : 1;
-  guint has_texture_format_bgra : 1;
+  guint has_debug_output : 1;
   guint extensions_checked : 1;
   guint debug_enabled : 1;
   guint forward_compatible : 1;
   guint is_legacy : 1;
-  guint has_tex_param : 1;
 
   int use_es;
 
+  int max_debug_label_length;
+
   GdkGLContextPaintData *paint_data;
 } GdkGLContextPrivate;
 
 enum {
   PROP_0,
 
-  PROP_DISPLAY,
-  PROP_WINDOW,
   PROP_SHARED_CONTEXT,
 
   LAST_PROP
@@ -128,10 +133,21 @@ static GParamSpec *obj_pspecs[LAST_PROP] = { NULL, };
 
 G_DEFINE_QUARK (gdk-gl-error-quark, gdk_gl_error)
 
-G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GdkGLContext, gdk_gl_context, G_TYPE_OBJECT)
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GdkGLContext, gdk_gl_context, GDK_TYPE_DRAW_CONTEXT)
 
 static GPrivate thread_current_context = G_PRIVATE_INIT (g_object_unref);
 
+static void
+gdk_gl_context_clear_old_updated_area (GdkGLContext *context)
+{
+  int i;
+
+  for (i = 0; i < 2; i++)
+    {
+      g_clear_pointer (&context->old_updated_area[i], cairo_region_destroy);
+    }
+}
+
 static void
 gdk_gl_context_dispose (GObject *gobject)
 {
@@ -139,12 +155,12 @@ gdk_gl_context_dispose (GObject *gobject)
   GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
   GdkGLContext *current;
 
+  gdk_gl_context_clear_old_updated_area (context);
+
   current = g_private_get (&thread_current_context);
   if (current == context)
     g_private_replace (&thread_current_context, NULL);
 
-  g_clear_object (&priv->display);
-  g_clear_object (&priv->window);
   g_clear_object (&priv->shared_context);
 
   G_OBJECT_CLASS (gdk_gl_context_parent_class)->dispose (gobject);
@@ -170,34 +186,6 @@ gdk_gl_context_set_property (GObject      *gobject,
 
   switch (prop_id)
     {
-    case PROP_DISPLAY:
-      {
-        GdkDisplay *display = g_value_get_object (value);
-
-        if (display)
-          g_object_ref (display);
-
-        if (priv->display)
-          g_object_unref (priv->display);
-
-        priv->display = display;
-      }
-      break;
-
-    case PROP_WINDOW:
-      {
-        GdkWindow *window = g_value_get_object (value);
-
-        if (window)
-          g_object_ref (window);
-
-        if (priv->window)
-          g_object_unref (priv->window);
-
-        priv->window = window;
-      }
-      break;
-
     case PROP_SHARED_CONTEXT:
       {
         GdkGLContext *context = g_value_get_object (value);
@@ -222,14 +210,6 @@ gdk_gl_context_get_property (GObject    *gobject,
 
   switch (prop_id)
     {
-    case PROP_DISPLAY:
-      g_value_set_object (value, priv->display);
-      break;
-
-    case PROP_WINDOW:
-      g_value_set_object (value, priv->window);
-      break;
-
     case PROP_SHARED_CONTEXT:
       g_value_set_object (value, priv->shared_context);
       break;
@@ -241,9 +221,10 @@ gdk_gl_context_get_property (GObject    *gobject,
 
 void
 gdk_gl_context_upload_texture (GdkGLContext    *context,
-                               cairo_surface_t *image_surface,
+                               const guchar    *data,
                                int              width,
                                int              height,
+                               int              stride,
                                guint            texture_target)
 {
   GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
@@ -257,21 +238,19 @@ gdk_gl_context_upload_texture (GdkGLContext    *context,
       (priv->use_es && (priv->gl_version >= 30 || priv->has_unpack_subimage)))
     {
       glPixelStorei (GL_UNPACK_ALIGNMENT, 4);
-      glPixelStorei (GL_UNPACK_ROW_LENGTH, cairo_image_surface_get_stride (image_surface) / 4);
+      glPixelStorei (GL_UNPACK_ROW_LENGTH, stride / 4);
 
       if (priv->use_es)
         glTexImage2D (texture_target, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE,
-                      cairo_image_surface_get_data (image_surface));
+                      data);
       else
         glTexImage2D (texture_target, 0, GL_RGBA, width, height, 0, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV,
-                      cairo_image_surface_get_data (image_surface));
+                      data);
 
       glPixelStorei (GL_UNPACK_ROW_LENGTH, 0);
     }
   else
     {
-      GLvoid *data = cairo_image_surface_get_data (image_surface);
-      int stride = cairo_image_surface_get_stride (image_surface);
       int i;
 
       if (priv->use_es)
@@ -279,14 +258,14 @@ gdk_gl_context_upload_texture (GdkGLContext    *context,
           glTexImage2D (texture_target, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
 
           for (i = 0; i < height; i++)
-            glTexSubImage2D (texture_target, 0, 0, i, width, 1, GL_RGBA, GL_UNSIGNED_BYTE, (unsigned char*) data + (i * stride));
+            glTexSubImage2D (texture_target, 0, 0, i, width, 1, GL_RGBA, GL_UNSIGNED_BYTE, data + (i * stride));
         }
       else
         {
           glTexImage2D (texture_target, 0, GL_RGBA, width, height, 0, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, NULL);
 
           for (i = 0; i < height; i++)
-            glTexSubImage2D (texture_target, 0, 0, i, width, 1, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, (unsigned char*) data + (i * stride));
+            glTexSubImage2D (texture_target, 0, 0, i, width, 1, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, data + (i * stride));
         }
     }
 }
@@ -301,51 +280,99 @@ gdk_gl_context_real_realize (GdkGLContext  *self,
   return FALSE;
 }
 
+static cairo_region_t *
+gdk_gl_context_real_get_damage (GdkGLContext *context)
+{
+  GdkSurface *surface = gdk_draw_context_get_surface (GDK_DRAW_CONTEXT (context));
+
+  return cairo_region_create_rectangle (&(GdkRectangle) {
+                                            0, 0,
+                                            gdk_surface_get_width (surface),
+                                            gdk_surface_get_height (surface)
+                                        });
+}
+
+static void
+gdk_gl_context_real_begin_frame (GdkDrawContext *draw_context,
+                                 cairo_region_t *region)
+{
+  GdkGLContext *context = GDK_GL_CONTEXT (draw_context);
+  GdkSurface *surface;
+  GdkGLContext *shared;
+  cairo_region_t *damage;
+  int ww, wh;
+
+  shared = gdk_gl_context_get_shared_context (context);
+  if (shared)
+    {
+      GDK_DRAW_CONTEXT_GET_CLASS (GDK_DRAW_CONTEXT (shared))->begin_frame (GDK_DRAW_CONTEXT (shared), region);
+      return;
+    }
+
+  damage = GDK_GL_CONTEXT_GET_CLASS (context)->get_damage (context);
+  cairo_region_union (region, damage);
+  cairo_region_destroy (damage);
+
+  surface = gdk_draw_context_get_surface (draw_context);
+  ww = gdk_surface_get_width (surface) * gdk_surface_get_scale_factor (surface);
+  wh = gdk_surface_get_height (surface) * gdk_surface_get_scale_factor (surface);
+
+  gdk_gl_context_make_current (context);
+
+  /* Initial setup */
+  glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
+  glDisable (GL_DEPTH_TEST);
+  glDisable (GL_BLEND);
+  glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
+
+  glViewport (0, 0, ww, wh);
+}
+
+static void
+gdk_gl_context_real_end_frame (GdkDrawContext *draw_context,
+                               cairo_region_t *painted)
+{
+  GdkGLContext *context = GDK_GL_CONTEXT (draw_context);
+  GdkGLContext *shared;
+
+  shared = gdk_gl_context_get_shared_context (context);
+  if (shared)
+    {
+      GDK_DRAW_CONTEXT_GET_CLASS (GDK_DRAW_CONTEXT (shared))->end_frame (GDK_DRAW_CONTEXT (shared), painted);
+      return;
+    }
+
+  if (context->old_updated_area[1])
+    cairo_region_destroy (context->old_updated_area[1]);
+  context->old_updated_area[1] = context->old_updated_area[0];
+  context->old_updated_area[0] = cairo_region_reference (painted);
+}
+
+static void
+gdk_gl_context_surface_resized (GdkDrawContext *draw_context)
+{
+  GdkGLContext *context = GDK_GL_CONTEXT (draw_context);
+
+  gdk_gl_context_clear_old_updated_area (context);
+}
+
 static void
 gdk_gl_context_class_init (GdkGLContextClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GdkDrawContextClass *draw_context_class = GDK_DRAW_CONTEXT_CLASS (klass);
 
   klass->realize = gdk_gl_context_real_realize;
+  klass->get_damage = gdk_gl_context_real_get_damage;
 
-  /**
-   * GdkGLContext:display:
-   *
-   * The #GdkDisplay used to create the #GdkGLContext.
-   *
-   * Since: 3.16
-   */
-  obj_pspecs[PROP_DISPLAY] =
-    g_param_spec_object ("display",
-                         P_("Display"),
-                         P_("The GDK display used to create the GL context"),
-                         GDK_TYPE_DISPLAY,
-                         G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY |
-                         G_PARAM_STATIC_STRINGS);
-
-  /**
-   * GdkGLContext:window:
-   *
-   * The #GdkWindow the gl context is bound to.
-   *
-   * Since: 3.16
-   */
-  obj_pspecs[PROP_WINDOW] =
-    g_param_spec_object ("window",
-                         P_("Window"),
-                         P_("The GDK window bound to the GL context"),
-                         GDK_TYPE_WINDOW,
-                         G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY |
-                         G_PARAM_STATIC_STRINGS);
+  draw_context_class->begin_frame = gdk_gl_context_real_begin_frame;
+  draw_context_class->end_frame = gdk_gl_context_real_end_frame;
+  draw_context_class->surface_resized = gdk_gl_context_surface_resized;
 
   /**
    * GdkGLContext:shared-context:
    *
    * The #GdkGLContext that this context is sharing data with, or %NULL
-   *
-   * Since: 3.16
    */
   obj_pspecs[PROP_SHARED_CONTEXT] =
     g_param_spec_object ("shared-context",
@@ -372,34 +399,9 @@ gdk_gl_context_init (GdkGLContext *self)
   priv->use_es = -1;
 }
 
-/*< private >
- * gdk_gl_context_end_frame:
- * @context: a #GdkGLContext
- * @painted: The area that has been redrawn this frame
- * @damage: The area that we know is actually different from the last frame
- *
- * Copies the back buffer to the front buffer.
- *
- * This function may call `glFlush()` implicitly before returning; it
- * is not recommended to call `glFlush()` explicitly before calling
- * this function.
- *
- * Since: 3.16
- */
-void
-gdk_gl_context_end_frame (GdkGLContext   *context,
-                          cairo_region_t *painted,
-                          cairo_region_t *damage)
-{
-  g_return_if_fail (GDK_IS_GL_CONTEXT (context));
-
-  GDK_GL_CONTEXT_GET_CLASS (context)->end_frame (context, painted, damage);
-}
-
 GdkGLContextPaintData *
 gdk_gl_context_get_paint_data (GdkGLContext *context)
 {
-
   GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
 
   if (priv->paint_data == NULL)
@@ -436,14 +438,87 @@ gdk_gl_context_has_frame_terminator (GdkGLContext *context)
   return priv->has_frame_terminator;
 }
 
-gboolean
-gdk_gl_context_has_tex_param (GdkGLContext *context)
+void
+gdk_gl_context_push_debug_group (GdkGLContext *context,
+                                 const char   *message)
 {
   GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
 
-  return priv->has_tex_param;
+  if (priv->use_khr_debug)
+    glPushDebugGroupKHR (GL_DEBUG_SOURCE_APPLICATION, 0, -1, message);
 }
 
+void
+gdk_gl_context_push_debug_group_printf (GdkGLContext *context,
+                                        const char   *format,
+                                        ...)
+{
+  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
+  gchar *message;
+  va_list args;
+
+  if (priv->use_khr_debug)
+    {
+      int msg_len;
+
+      va_start (args, format);
+      message = g_strdup_vprintf (format, args);
+      va_end (args);
+
+      msg_len = MIN (priv->max_debug_label_length, strlen (message) - 1);
+      glPushDebugGroupKHR (GL_DEBUG_SOURCE_APPLICATION, 0, msg_len, message);
+      g_free (message);
+    }
+}
+
+void
+gdk_gl_context_pop_debug_group (GdkGLContext *context)
+{
+  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
+
+  if (priv->use_khr_debug)
+    glPopDebugGroupKHR ();
+}
+
+void
+gdk_gl_context_label_object (GdkGLContext *context,
+                             guint         identifier,
+                             guint         name,
+                             const char   *label)
+{
+  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
+
+  if (priv->use_khr_debug)
+    glObjectLabel (identifier, name, -1, label);
+}
+
+void
+gdk_gl_context_label_object_printf  (GdkGLContext *context,
+                                     guint         identifier,
+                                     guint         name,
+                                     const char   *format,
+                                     ...)
+{
+  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
+  gchar *message;
+  va_list args;
+
+  if (priv->use_khr_debug)
+    {
+      int msg_len;
+
+      va_start (args, format);
+      message = g_strdup_vprintf (format, args);
+      va_end (args);
+
+      msg_len = MIN (priv->max_debug_label_length, strlen (message) - 1);
+
+      glObjectLabel (identifier, name, msg_len, message);
+      g_free (message);
+    }
+}
+
+
 gboolean
 gdk_gl_context_has_unpack_subimage (GdkGLContext *context)
 {
@@ -452,22 +527,6 @@ gdk_gl_context_has_unpack_subimage (GdkGLContext *context)
   return priv->has_unpack_subimage;
 }
 
-gboolean
-gdk_gl_context_has_sync (GdkGLContext *context)
-{
-  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
-
-  return priv->has_sync;
-}
-
-gboolean
-gdk_gl_context_has_texture_format_bgra (GdkGLContext *context)
-{
-  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
-
-  return priv->has_texture_format_bgra;
-}
-
 /**
  * gdk_gl_context_set_debug_enabled:
  * @context: a #GdkGLContext
@@ -479,8 +538,6 @@ gdk_gl_context_has_texture_format_bgra (GdkGLContext *context)
  *
  * The #GdkGLContext must not be realized or made current prior to
  * calling this function.
- *
- * Since: 3.16
  */
 void
 gdk_gl_context_set_debug_enabled (GdkGLContext *context,
@@ -503,8 +560,6 @@ gdk_gl_context_set_debug_enabled (GdkGLContext *context,
  * Retrieves the value set using gdk_gl_context_set_debug_enabled().
  *
  * Returns: %TRUE if debugging is enabled
- *
- * Since: 3.16
  */
 gboolean
 gdk_gl_context_get_debug_enabled (GdkGLContext *context)
@@ -530,8 +585,6 @@ gdk_gl_context_get_debug_enabled (GdkGLContext *context)
  *
  * The #GdkGLContext must not be realized or made current prior to calling
  * this function.
- *
- * Since: 3.16
  */
 void
 gdk_gl_context_set_forward_compatible (GdkGLContext *context,
@@ -554,8 +607,6 @@ gdk_gl_context_set_forward_compatible (GdkGLContext *context,
  * Retrieves the value set using gdk_gl_context_set_forward_compatible().
  *
  * Returns: %TRUE if the context should be forward compatible
- *
- * Since: 3.16
  */
 gboolean
 gdk_gl_context_get_forward_compatible (GdkGLContext *context)
@@ -579,8 +630,6 @@ gdk_gl_context_get_forward_compatible (GdkGLContext *context)
  *
  * The #GdkGLContext must not be realized or made current prior to calling
  * this function.
- *
- * Since: 3.16
  */
 void
 gdk_gl_context_set_required_version (GdkGLContext *context,
@@ -588,6 +637,7 @@ gdk_gl_context_set_required_version (GdkGLContext *context,
                                      int           minor)
 {
   GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
+  GdkDisplay *display;
   int version, min_ver;
 
   g_return_if_fail (GDK_IS_GL_CONTEXT (context));
@@ -604,7 +654,9 @@ gdk_gl_context_set_required_version (GdkGLContext *context,
   /* Enforce a minimum context version number of 3.2 */
   version = (major * 100) + minor;
 
-  if (priv->use_es > 0 || (_gdk_gl_flags & GDK_GL_GLES) != 0)
+  display = gdk_draw_context_get_display (GDK_DRAW_CONTEXT (context));
+
+  if (priv->use_es > 0 || GDK_DISPLAY_DEBUG_CHECK (display, GL_GLES))
     min_ver = 200;
   else
     min_ver = 302;
@@ -626,8 +678,6 @@ gdk_gl_context_set_required_version (GdkGLContext *context,
  *
  * Retrieves the major and minor version requested by calling
  * gdk_gl_context_set_required_version().
- *
- * Since: 3.16
  */
 void
 gdk_gl_context_get_required_version (GdkGLContext *context,
@@ -635,12 +685,15 @@ gdk_gl_context_get_required_version (GdkGLContext *context,
                                      int          *minor)
 {
   GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
+  GdkDisplay *display;
   int default_major, default_minor;
   int maj, min;
 
   g_return_if_fail (GDK_IS_GL_CONTEXT (context));
 
-  if (priv->use_es > 0 || (_gdk_gl_flags & GDK_GL_GLES) != 0)
+  display = gdk_draw_context_get_display (GDK_DRAW_CONTEXT (context));
+
+  if (priv->use_es > 0 || GDK_DISPLAY_DEBUG_CHECK (display, GL_GLES))
     {
       default_major = 2;
       default_minor = 0;
@@ -689,8 +742,6 @@ gdk_gl_context_get_required_version (GdkGLContext *context,
  * kind of shader programs to load.
  *
  * Returns: %TRUE if the GL context is in legacy mode
- *
- * Since: 3.20
  */
 gboolean
 gdk_gl_context_is_legacy (GdkGLContext *context)
@@ -730,8 +781,6 @@ gdk_gl_context_set_is_legacy (GdkGLContext *context,
  * You should check the return value of gdk_gl_context_get_use_es() after
  * calling gdk_gl_context_realize() to decide whether to use the OpenGL or
  * OpenGL ES API, extensions, or shaders.
- *
- * Since: 3.22
  */
 void
 gdk_gl_context_set_use_es (GdkGLContext *context,
@@ -753,8 +802,6 @@ gdk_gl_context_set_use_es (GdkGLContext *context,
  * Checks whether the @context is using an OpenGL or OpenGL ES profile.
  *
  * Returns: %TRUE if the #GdkGLContext is using an OpenGL ES profile
- *
- * Since: 3.22
  */
 gboolean
 gdk_gl_context_get_use_es (GdkGLContext *context)
@@ -769,6 +816,99 @@ gdk_gl_context_get_use_es (GdkGLContext *context)
   return priv->use_es > 0;
 }
 
+#ifdef G_ENABLE_CONSISTENCY_CHECKS
+static void
+gl_debug_message_callback (GLenum        source,
+                           GLenum        type,
+                           GLuint        id,
+                           GLenum        severity,
+                           GLsizei       length,
+                           const GLchar *message,
+                           const void   *user_data)
+{
+  const char *message_source;
+  const char *message_type;
+  const char *message_severity;
+
+  if (severity == GL_DEBUG_SEVERITY_NOTIFICATION)
+    return;
+
+  switch (source)
+    {
+    case GL_DEBUG_SOURCE_API:
+      message_source = "API";
+      break;
+    case GL_DEBUG_SOURCE_WINDOW_SYSTEM:
+      message_source = "Window System";
+      break;
+    case GL_DEBUG_SOURCE_SHADER_COMPILER:
+      message_source = "Shader Compiler";
+      break;
+    case GL_DEBUG_SOURCE_THIRD_PARTY:
+      message_source = "Third Party";
+      break;
+    case GL_DEBUG_SOURCE_APPLICATION:
+      message_source = "Application";
+      break;
+    case GL_DEBUG_SOURCE_OTHER:
+    default:
+      message_source = "Other";
+    }
+
+  switch (type)
+    {
+    case GL_DEBUG_TYPE_ERROR:
+      message_type = "Error";
+      break;
+    case GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR:
+      message_type = "Deprecated Behavior";
+      break;
+    case GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR:
+      message_type = "Undefined Behavior";
+      break;
+    case GL_DEBUG_TYPE_PORTABILITY:
+      message_type = "Portability";
+      break;
+    case GL_DEBUG_TYPE_PERFORMANCE:
+      message_type = "Performance";
+      break;
+    case GL_DEBUG_TYPE_MARKER:
+      message_type = "Marker";
+      break;
+    case GL_DEBUG_TYPE_PUSH_GROUP:
+      message_type = "Push Group";
+      break;
+    case GL_DEBUG_TYPE_POP_GROUP:
+      message_type = "Pop Group";
+      break;
+    case GL_DEBUG_TYPE_OTHER:
+    default:
+      message_type = "Other";
+    }
+
+  switch (severity)
+    {
+    case GL_DEBUG_SEVERITY_HIGH:
+      message_severity = "High";
+      break;
+    case GL_DEBUG_SEVERITY_MEDIUM:
+      message_severity = "Medium";
+      break;
+    case GL_DEBUG_SEVERITY_LOW:
+      message_severity = "Low";
+      break;
+    case GL_DEBUG_SEVERITY_NOTIFICATION:
+      message_severity = "Notification";
+      break;
+    default:
+      message_severity = "Unknown";
+    }
+
+  g_warning ("OPENGL:\n    Source: %s\n    Type: %s\n    Severity: %s\n    Message: %s",
+             message_source, message_type, message_severity, message);
+}
+#endif
+
 /**
  * gdk_gl_context_realize:
  * @context: a #GdkGLContext
@@ -779,8 +919,6 @@ gdk_gl_context_get_use_es (GdkGLContext *context)
  * It is safe to call this function on a realized #GdkGLContext.
  *
  * Returns: %TRUE if the context is realized
- *
- * Since: 3.16
  */
 gboolean
 gdk_gl_context_realize (GdkGLContext  *context,
@@ -802,6 +940,7 @@ static void
 gdk_gl_context_check_extensions (GdkGLContext *context)
 {
   GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
+  GdkDisplay *display;
   gboolean has_npot, has_texture_rectangle;
 
   if (!priv->realized)
@@ -815,51 +954,58 @@ gdk_gl_context_check_extensions (GdkGLContext *context)
   if (priv->use_es < 0)
     priv->use_es = !epoxy_is_desktop_gl ();
 
+  priv->has_debug_output = epoxy_has_gl_extension ("GL_ARB_debug_output");
+
+#ifdef G_ENABLE_CONSISTENCY_CHECKS
+  if (priv->has_debug_output)
+    {
+      gdk_gl_context_make_current (context);
+      glEnable (GL_DEBUG_OUTPUT);
+      glEnable (GL_DEBUG_OUTPUT_SYNCHRONOUS);
+      glDebugMessageCallback (gl_debug_message_callback, NULL);
+    }
+#endif
+
   if (priv->use_es)
     {
       has_npot = priv->gl_version >= 20;
       has_texture_rectangle = FALSE;
 
-      /* This should check for GL_NV_framebuffer_blit as well - see extension at:
+      /* This should check for GL_NV_framebuffer_blit - see extension at:
        *
        * https://www.khronos.org/registry/gles/extensions/NV/NV_framebuffer_blit.txt
-       *
-       * for ANGLE, we can enable bit blitting if we have the
-       * GL_ANGLE_framebuffer_blit extension
        */
-      if (epoxy_has_gl_extension ("GL_ANGLE_framebuffer_blit"))
-        priv->has_gl_framebuffer_blit = TRUE;
-      else
-        priv->has_gl_framebuffer_blit = FALSE;
+      priv->has_gl_framebuffer_blit = FALSE;
 
       /* No OES version */
       priv->has_frame_terminator = FALSE;
 
       priv->has_unpack_subimage = epoxy_has_gl_extension ("GL_EXT_unpack_subimage");
-      priv->has_sync = priv->gl_version >= 30;
-      priv->has_texture_format_bgra = epoxy_has_gl_extension ("GL_EXT_texture_format_BGRA8888");
-      priv->has_tex_param = priv->gl_version >= 31;
+      priv->has_khr_debug = epoxy_has_gl_extension ("GL_KHR_debug");
     }
   else
     {
-      has_npot = priv->gl_version >= 20 || epoxy_has_gl_extension ("GL_ARB_texture_non_power_of_two");
-      has_texture_rectangle = priv->gl_version >= 31 || epoxy_has_gl_extension ("GL_ARB_texture_rectangle");
+      has_npot = epoxy_has_gl_extension ("GL_ARB_texture_non_power_of_two");
+      has_texture_rectangle = epoxy_has_gl_extension ("GL_ARB_texture_rectangle");
 
-      priv->has_gl_framebuffer_blit = priv->gl_version >= 30 || epoxy_has_gl_extension ("GL_EXT_framebuffer_blit");
+      priv->has_gl_framebuffer_blit = epoxy_has_gl_extension ("GL_EXT_framebuffer_blit");
       priv->has_frame_terminator = epoxy_has_gl_extension ("GL_GREMEDY_frame_terminator");
       priv->has_unpack_subimage = TRUE;
-      priv->has_tex_param = priv->gl_version >= 10 &&
-                            epoxy_is_desktop_gl ();
-      priv->has_sync = priv->gl_version >= 32 ||
-                       epoxy_has_gl_extension ("GL_ARB_sync") ||
-                       epoxy_has_gl_extension ("GL_APPLE_sync");
+      priv->has_khr_debug = epoxy_has_gl_extension ("GL_KHR_debug");
 
       /* We asked for a core profile, but we didn't get one, so we're in legacy mode */
       if (priv->gl_version < 32)
         priv->is_legacy = TRUE;
     }
 
-  if (!priv->use_es && G_UNLIKELY (_gdk_gl_flags & GDK_GL_TEXTURE_RECTANGLE))
+  display = gdk_draw_context_get_display (GDK_DRAW_CONTEXT (context));
+
+  if (priv->has_khr_debug && GDK_DISPLAY_DEBUG_CHECK (display, GL_DEBUG))
+    {
+      priv->use_khr_debug = TRUE;
+      glGetIntegerv (GL_MAX_LABEL_LENGTH, &priv->max_debug_label_length);
+    }
+  if (!priv->use_es && GDK_DISPLAY_DEBUG_CHECK (display, GL_TEXTURE_RECT))
     priv->use_texture_rectangle = TRUE;
   else if (has_npot)
     priv->use_texture_rectangle = FALSE;
@@ -868,14 +1014,15 @@ gdk_gl_context_check_extensions (GdkGLContext *context)
   else
     g_warning ("GL implementation doesn't support any form of non-power-of-two textures");
 
-  GDK_NOTE (OPENGL,
-            g_message ("%s version: %d.%d (%s)\n"
+  GDK_DISPLAY_NOTE (display, OPENGL,
+    g_message ("%s version: %d.%d (%s)\n"
                        "* GLSL version: %s\n"
                        "* Extensions checked:\n"
                        " - GL_ARB_texture_non_power_of_two: %s\n"
                        " - GL_ARB_texture_rectangle: %s\n"
                        " - GL_EXT_framebuffer_blit: %s\n"
                        " - GL_GREMEDY_frame_terminator: %s\n"
+                       " - GL_KHR_debug: %s\n"
                        "* Using texture rectangle: %s",
                        priv->use_es ? "OpenGL ES" : "OpenGL",
                        priv->gl_version / 10, priv->gl_version % 10,
@@ -885,6 +1032,7 @@ gdk_gl_context_check_extensions (GdkGLContext *context)
                        has_texture_rectangle ? "yes" : "no",
                        priv->has_gl_framebuffer_blit ? "yes" : "no",
                        priv->has_frame_terminator ? "yes" : "no",
+                       priv->has_khr_debug ? "yes" : "no",
                        priv->use_texture_rectangle ? "yes" : "no"));
 
   priv->extensions_checked = TRUE;
@@ -895,8 +1043,6 @@ gdk_gl_context_check_extensions (GdkGLContext *context)
  * @context: a #GdkGLContext
  *
  * Makes the @context the current one.
- *
- * Since: 3.16
  */
 void
 gdk_gl_context_make_current (GdkGLContext *context)
@@ -907,8 +1053,7 @@ gdk_gl_context_make_current (GdkGLContext *context)
   g_return_if_fail (GDK_IS_GL_CONTEXT (context));
 
   current = g_private_get (&thread_current_context);
-  if (current == context &&
-      gdk_display_is_gl_context_current (priv->display, context))
+  if (current == context)
     return;
 
   /* we need to realize the GdkGLContext if it wasn't explicitly realized */
@@ -925,7 +1070,7 @@ gdk_gl_context_make_current (GdkGLContext *context)
         }
     }
 
-  if (gdk_display_make_gl_context_current (priv->display, context))
+  if (gdk_display_make_gl_context_current (gdk_draw_context_get_display (GDK_DRAW_CONTEXT (context)), context))
     {
       g_private_replace (&thread_current_context, g_object_ref (context));
       gdk_gl_context_check_extensions (context);
@@ -939,37 +1084,29 @@ gdk_gl_context_make_current (GdkGLContext *context)
  * Retrieves the #GdkDisplay the @context is created for
  *
  * Returns: (nullable) (transfer none): a #GdkDisplay or %NULL
- *
- * Since: 3.16
  */
 GdkDisplay *
 gdk_gl_context_get_display (GdkGLContext *context)
 {
-  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
-
   g_return_val_if_fail (GDK_IS_GL_CONTEXT (context), NULL);
 
-  return priv->display;
+  return gdk_draw_context_get_display (GDK_DRAW_CONTEXT (context));
 }
 
 /**
- * gdk_gl_context_get_window:
+ * gdk_gl_context_get_surface:
  * @context: a #GdkGLContext
  *
- * Retrieves the #GdkWindow used by the @context.
+ * Retrieves the #GdkSurface used by the @context.
  *
- * Returns: (nullable) (transfer none): a #GdkWindow or %NULL
- *
- * Since: 3.16
+ * Returns: (nullable) (transfer none): a #GdkSurface or %NULL
  */
-GdkWindow *
-gdk_gl_context_get_window (GdkGLContext *context)
+GdkSurface *
+gdk_gl_context_get_surface (GdkGLContext *context)
 {
-  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
-
   g_return_val_if_fail (GDK_IS_GL_CONTEXT (context), NULL);
 
-  return priv->window;
+  return gdk_draw_context_get_surface (GDK_DRAW_CONTEXT (context));
 }
 
 /**
@@ -979,8 +1116,6 @@ gdk_gl_context_get_window (GdkGLContext *context)
  * Retrieves the #GdkGLContext that this @context share data with.
  *
  * Returns: (nullable) (transfer none): a #GdkGLContext or %NULL
- *
- * Since: 3.16
  */
 GdkGLContext *
 gdk_gl_context_get_shared_context (GdkGLContext *context)
@@ -1001,8 +1136,6 @@ gdk_gl_context_get_shared_context (GdkGLContext *context)
  * Retrieves the OpenGL version of the @context.
  *
  * The @context must be realized prior to calling this function.
- *
- * Since: 3.16
  */
 void
 gdk_gl_context_get_version (GdkGLContext *context,
@@ -1027,8 +1160,6 @@ gdk_gl_context_get_version (GdkGLContext *context,
  *
  * Any OpenGL call after this function returns will be ignored
  * until gdk_gl_context_make_current() is called.
- *
- * Since: 3.16
  */
 void
 gdk_gl_context_clear_current (void)
@@ -1038,9 +1169,7 @@ gdk_gl_context_clear_current (void)
   current = g_private_get (&thread_current_context);
   if (current != NULL)
     {
-      GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (current);
-
-      if (gdk_display_make_gl_context_current (priv->display, NULL))
+      if (gdk_display_make_gl_context_current (gdk_draw_context_get_display (GDK_DRAW_CONTEXT (current)), NULL))
         g_private_replace (&thread_current_context, NULL);
     }
 }
@@ -1051,8 +1180,6 @@ gdk_gl_context_clear_current (void)
  * Retrieves the current #GdkGLContext.
  *
  * Returns: (nullable) (transfer none): the current #GdkGLContext, or %NULL
- *
- * Since: 3.16
  */
 GdkGLContext *
 gdk_gl_context_get_current (void)
@@ -1061,44 +1188,5 @@ gdk_gl_context_get_current (void)
 
   current = g_private_get (&thread_current_context);
 
-  if (current)
-    {
-      GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (current);
-      if (!gdk_display_is_gl_context_current (priv->display, current))
-        {
-          g_private_replace (&thread_current_context, NULL);
-          current = NULL;
-        }
-    }
-
   return current;
 }
-
-/**
- * gdk_gl_get_flags:
- *
- * Returns the currently active GL flags.
- *
- * Returns: the GL flags
- *
- * Since: 3.16
- */
-GdkGLFlags
-gdk_gl_get_flags (void)
-{
-  return _gdk_gl_flags;
-}
-
-/**
- * gdk_gl_set_flags:
- * @flags: #GdkGLFlags to set
- *
- * Sets GL flags.
- *
- * Since: 3.16
- */
-void
-gdk_gl_set_flags (GdkGLFlags flags)
-{
-  _gdk_gl_flags = flags;
-}
