@@ -22,7 +22,8 @@
 #ifndef __GDK_WAYLAND_DISPLAY__
 #define __GDK_WAYLAND_DISPLAY__
 
-#include <config.h>
+#include "config.h"
+
 #include <stdint.h>
 #include <wayland-client.h>
 #include <wayland-cursor.h>
@@ -32,44 +33,54 @@
 #include <gdk/wayland/xdg-shell-client-protocol.h>
 #include <gdk/wayland/xdg-shell-unstable-v6-client-protocol.h>
 #include <gdk/wayland/xdg-foreign-unstable-v1-client-protocol.h>
-#include <gdk/wayland/xdg-foreign-unstable-v2-client-protocol.h>
 #include <gdk/wayland/keyboard-shortcuts-inhibit-unstable-v1-client-protocol.h>
 #include <gdk/wayland/server-decoration-client-protocol.h>
 #include <gdk/wayland/xdg-output-unstable-v1-client-protocol.h>
-#include <gdk/wayland/primary-selection-unstable-v1-client-protocol.h>
-#ifdef HAVE_XDG_ACTIVATION
-#include <gdk/wayland/xdg-activation-v1-client-protocol.h>
-#endif
 
 #include <glib.h>
 #include <gdk/gdkkeys.h>
-#include <gdk/gdkwindow.h>
+#include <gdk/gdksurface.h>
 #include <gdk/gdkinternals.h>
 #include <gdk/gdk.h>		/* For gdk_get_program_class() */
 
 #include "gdkdisplayprivate.h"
+#include "gdkwaylanddevice.h"
 
 #include <epoxy/egl.h>
 
 G_BEGIN_DECLS
 
-#define GDK_WAYLAND_MAX_THEME_SCALE 4
+#define GDK_WAYLAND_MAX_THEME_SCALE 3
 #define GDK_WAYLAND_THEME_SCALES_COUNT GDK_WAYLAND_MAX_THEME_SCALE
 
 #define GDK_ZWP_POINTER_GESTURES_V1_VERSION 1
 
 typedef struct _GdkWaylandSelection GdkWaylandSelection;
 
+typedef struct {
+        gboolean     antialias;
+        gboolean     hinting;
+        gint         dpi;
+        const gchar *rgba;
+        const gchar *hintstyle;
+} GsdXftSettings;
+
 typedef enum _GdkWaylandShellVariant
 {
   GDK_WAYLAND_SHELL_VARIANT_XDG_SHELL,
-  GDK_WAYLAND_SHELL_VARIANT_ZXDG_SHELL_V6,
+  GDK_WAYLAND_SHELL_VARIANT_ZXDG_SHELL_V6
 } GdkWaylandShellVariant;
 
 struct _GdkWaylandDisplay
 {
   GdkDisplay parent_instance;
-  GdkScreen *screen;
+  GList *toplevels;
+
+  GHashTable *settings;
+  GsdXftSettings xft_settings;
+  GDBusProxy *settings_portal;
+
+  guint32    shell_capabilities;
 
   /* Startup notification */
   gchar *startup_notification_id;
@@ -78,7 +89,6 @@ struct _GdkWaylandDisplay
   guint32 serial;
 
   uint32_t xdg_wm_base_id;
-  int xdg_wm_base_version;
   uint32_t zxdg_shell_v6_id;
   GdkWaylandShellVariant shell_variant;
 
@@ -94,20 +104,13 @@ struct _GdkWaylandDisplay
   struct wl_data_device_manager *data_device_manager;
   struct wl_subcompositor *subcompositor;
   struct zwp_pointer_gestures_v1 *pointer_gestures;
-  struct gtk_primary_selection_device_manager *gtk_primary_selection_manager;
-  struct zwp_primary_selection_device_manager_v1 *zwp_primary_selection_manager_v1;
+  struct gtk_primary_selection_device_manager *primary_selection_manager;
   struct zwp_tablet_manager_v2 *tablet_manager;
-  struct zxdg_exporter_v1 *xdg_exporter_v1;
-  struct zxdg_importer_v1 *xdg_importer_v1;
-  struct zxdg_exporter_v2 *xdg_exporter_v2;
-  struct zxdg_importer_v2 *xdg_importer_v2;
+  struct zxdg_exporter_v1 *xdg_exporter;
+  struct zxdg_importer_v1 *xdg_importer;
   struct zwp_keyboard_shortcuts_inhibit_manager_v1 *keyboard_shortcuts_inhibit;
   struct org_kde_kwin_server_decoration_manager *server_decoration_manager;
   struct zxdg_output_manager_v1 *xdg_output_manager;
-  uint32_t xdg_output_version;
-#ifdef HAVE_XDG_ACTIVATION
-  struct xdg_activation_v1 *xdg_activation;
-#endif
 
   GList *async_roundtrips;
 
@@ -126,7 +129,7 @@ struct _GdkWaylandDisplay
   struct wl_cursor_theme *scaled_cursor_themes[GDK_WAYLAND_THEME_SCALES_COUNT];
   gchar *cursor_theme_name;
   int cursor_theme_size;
-  GHashTable *cursor_cache;
+  GHashTable *cursor_surface_cache;
 
   GSource *event_source;
 
@@ -134,17 +137,11 @@ struct _GdkWaylandDisplay
   int seat_version;
   int data_device_manager_version;
   int gtk_shell_version;
-  int xdg_output_manager_version;
-#ifdef HAVE_XDG_ACTIVATION
-  int xdg_activation_version;
-#endif
 
   uint32_t server_decoration_mode;
 
   struct xkb_context *xkb_context;
 
-  GdkWaylandSelection *selection;
-
   GPtrArray *monitors;
 
   gint64 last_bell_time_ms;
@@ -159,7 +156,6 @@ struct _GdkWaylandDisplay
   guint have_egl_buffer_age : 1;
   guint have_egl_swap_buffers_with_damage : 1;
   guint have_egl_surfaceless_context : 1;
-  EGLint egl_min_swap_interval;
 };
 
 struct _GdkWaylandDisplayClass
