@@ -26,7 +26,7 @@
 
 #include "gdkwaylanddisplay.h"
 #include "gdkwaylandglcontext.h"
-#include "gdkwaylandwindow.h"
+#include "gdkwaylandsurface.h"
 #include "gdkprivate-wayland.h"
 
 #include "gdkinternals.h"
@@ -35,71 +35,7 @@
 
 G_DEFINE_TYPE (GdkWaylandGLContext, gdk_wayland_gl_context, GDK_TYPE_GL_CONTEXT)
 
-static void gdk_x11_gl_context_dispose (GObject *gobject);
-
-void
-gdk_wayland_window_invalidate_for_new_frame (GdkWindow      *window,
-                                             cairo_region_t *update_area)
-{
-  cairo_rectangle_int_t window_rect;
-  GdkDisplay *display = gdk_window_get_display (window);
-  GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (display);
-  GdkWaylandGLContext *context_wayland;
-  int buffer_age;
-  gboolean invalidate_all;
-  EGLSurface egl_surface;
-
-  /* Minimal update is ok if we're not drawing with gl */
-  if (window->gl_paint_context == NULL)
-    return;
-
-  context_wayland = GDK_WAYLAND_GL_CONTEXT (window->gl_paint_context);
-  buffer_age = 0;
-
-  egl_surface = gdk_wayland_window_get_egl_surface (window->impl_window,
-                                                    context_wayland->egl_config);
-
-  if (display_wayland->have_egl_buffer_age)
-    {
-      gdk_gl_context_make_current (window->gl_paint_context);
-      eglQuerySurface (display_wayland->egl_display, egl_surface,
-		       EGL_BUFFER_AGE_EXT, &buffer_age);
-    }
-
-  invalidate_all = FALSE;
-  if (buffer_age == 0 || buffer_age >= 4)
-    invalidate_all = TRUE;
-  else
-    {
-      if (buffer_age >= 2)
-        {
-          if (window->old_updated_area[0])
-            cairo_region_union (update_area, window->old_updated_area[0]);
-          else
-            invalidate_all = TRUE;
-        }
-      if (buffer_age >= 3)
-        {
-          if (window->old_updated_area[1])
-            cairo_region_union (update_area, window->old_updated_area[1]);
-          else
-            invalidate_all = TRUE;
-        }
-    }
-
-  if (invalidate_all)
-    {
-      window_rect.x = 0;
-      window_rect.y = 0;
-      window_rect.width = gdk_window_get_width (window);
-      window_rect.height = gdk_window_get_height (window);
-
-      /* If nothing else is known, repaint everything so that the back
-       * buffer is fully up-to-date for the swapbuffer
-       */
-      cairo_region_union_rectangle (update_area, &window_rect);
-    }
-}
+static void gdk_wayland_gl_context_dispose (GObject *gobject);
 
 #define N_EGL_ATTRS     16
 
@@ -120,9 +56,9 @@ gdk_wayland_gl_context_realize (GdkGLContext *context,
   gdk_gl_context_get_required_version (context, &major, &minor);
   debug_bit = gdk_gl_context_get_debug_enabled (context);
   forward_bit = gdk_gl_context_get_forward_compatible (context);
-  legacy_bit = (_gdk_gl_flags & GDK_GL_LEGACY) != 0 ||
+  legacy_bit = GDK_DISPLAY_DEBUG_CHECK (display, GL_LEGACY) ||
                (share != NULL && gdk_gl_context_is_legacy (share));
-  use_es = (_gdk_gl_flags & GDK_GL_GLES) != 0 ||
+  use_es = GDK_DISPLAY_DEBUG_CHECK (display, GL_GLES) ||
            (share != NULL && gdk_gl_context_get_use_es (share));
 
   flags = 0;
@@ -166,7 +102,8 @@ gdk_wayland_gl_context_realize (GdkGLContext *context,
   context_attribs[i++] = EGL_NONE;
   g_assert (i < N_EGL_ATTRS);
 
-  GDK_NOTE (OPENGL, g_message ("Creating EGL context version %d.%d (debug:%s, forward:%s, legacy:%s, es:%s)",
+  GDK_DISPLAY_NOTE (display, OPENGL,
+            g_message ("Creating EGL context version %d.%d (debug:%s, forward:%s, legacy:%s, es:%s)",
                                major, minor,
                                debug_bit ? "yes" : "no",
                                forward_bit ? "yes" : "no",
@@ -193,7 +130,8 @@ gdk_wayland_gl_context_realize (GdkGLContext *context,
       legacy_bit = TRUE;
       use_es = FALSE;
 
-      GDK_NOTE (OPENGL, g_message ("eglCreateContext failed, switching to legacy"));
+      GDK_DISPLAY_NOTE (display, OPENGL,
+                g_message ("eglCreateContext failed, switching to legacy"));
       ctx = eglCreateContext (display_wayland->egl_display,
                               context_wayland->egl_config,
                               share != NULL ? GDK_WAYLAND_GL_CONTEXT (share)->egl_context
@@ -209,7 +147,7 @@ gdk_wayland_gl_context_realize (GdkGLContext *context,
       return FALSE;
     }
 
-  GDK_NOTE (OPENGL, g_message ("Created EGL context[%p]", ctx));
+  GDK_DISPLAY_NOTE (display, OPENGL, g_message ("Created EGL context[%p]", ctx));
 
   context_wayland->egl_context = ctx;
 
@@ -219,34 +157,95 @@ gdk_wayland_gl_context_realize (GdkGLContext *context,
   return TRUE;
 }
 
-static void
-gdk_wayland_gl_context_end_frame (GdkGLContext   *context,
-                                  cairo_region_t *painted,
-                                  cairo_region_t *damage)
+static cairo_region_t *
+gdk_wayland_gl_context_get_damage (GdkGLContext *context)
 {
-  GdkWindow *window = gdk_gl_context_get_window (context);
-  GdkDisplay *display = gdk_window_get_display (window);
+  GdkDisplay *display = gdk_draw_context_get_display (GDK_DRAW_CONTEXT (context));
+  GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (display);
+  EGLSurface egl_surface;
+  GdkSurface *surface = gdk_draw_context_get_surface (GDK_DRAW_CONTEXT (context));
+  int buffer_age = 0;
+
+  if (display_wayland->have_egl_buffer_age)
+    {
+      GdkGLContext *shared;
+      GdkWaylandGLContext *shared_wayland;
+
+      shared = gdk_gl_context_get_shared_context (context);
+      if (shared == NULL)
+        shared = context;
+      shared_wayland = GDK_WAYLAND_GL_CONTEXT (shared);
+
+      egl_surface = gdk_wayland_surface_get_egl_surface (surface->impl_surface,
+                                                        shared_wayland->egl_config);
+      gdk_gl_context_make_current (shared);
+      eglQuerySurface (display_wayland->egl_display, egl_surface,
+                       EGL_BUFFER_AGE_EXT, &buffer_age);
+
+      switch (buffer_age)
+        {
+          case 1:
+            return cairo_region_create ();
+            break;
+
+          case 2:
+            if (context->old_updated_area[0])
+              return cairo_region_copy (context->old_updated_area[0]);
+            break;
+
+          case 3:
+            if (context->old_updated_area[0] &&
+                context->old_updated_area[1])
+              {
+                cairo_region_t *damage = cairo_region_copy (context->old_updated_area[0]);
+                cairo_region_union (damage, context->old_updated_area[1]);
+                return damage;
+              }
+            break;
+
+          default:
+            ;
+        }
+    }
+
+  return GDK_GL_CONTEXT_CLASS (gdk_wayland_gl_context_parent_class)->get_damage (context);
+}
+
+static void
+gdk_wayland_gl_context_end_frame (GdkDrawContext *draw_context,
+                                  cairo_region_t *painted)
+{
+  GdkGLContext *context = GDK_GL_CONTEXT (draw_context);
+  GdkSurface *surface = gdk_gl_context_get_surface (context);
+  GdkDisplay *display = gdk_surface_get_display (surface);
   GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (display);
   GdkWaylandGLContext *context_wayland = GDK_WAYLAND_GL_CONTEXT (context);
   EGLSurface egl_surface;
 
+  GDK_DRAW_CONTEXT_CLASS (gdk_wayland_gl_context_parent_class)->end_frame (draw_context, painted);
+  if (gdk_gl_context_get_shared_context (context))
+    return;
+
   gdk_gl_context_make_current (context);
 
-  egl_surface = gdk_wayland_window_get_egl_surface (window->impl_window,
+  egl_surface = gdk_wayland_surface_get_egl_surface (surface->impl_surface,
                                                     context_wayland->egl_config);
 
+  gdk_wayland_surface_sync (surface);
+  gdk_wayland_surface_request_frame (surface);
+
   if (display_wayland->have_egl_swap_buffers_with_damage)
     {
-      int i, j, n_rects = cairo_region_num_rectangles (damage);
+      int i, j, n_rects = cairo_region_num_rectangles (painted);
       EGLint *rects = g_new (EGLint, n_rects * 4);
       cairo_rectangle_int_t rect;
-      int window_height = gdk_window_get_height (window);
+      int surface_height = gdk_surface_get_height (surface);
 
       for (i = 0, j = 0; i < n_rects; i++)
         {
-          cairo_region_get_rectangle (damage, i, &rect);
+          cairo_region_get_rectangle (painted, i, &rect);
           rects[j++] = rect.x;
-          rects[j++] = window_height - rect.height - rect.y;
+          rects[j++] = surface_height - rect.height - rect.y;
           rects[j++] = rect.width;
           rects[j++] = rect.height;
         }
@@ -261,12 +260,15 @@ static void
 gdk_wayland_gl_context_class_init (GdkWaylandGLContextClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GdkDrawContextClass *draw_context_class = GDK_DRAW_CONTEXT_CLASS (klass);
   GdkGLContextClass *context_class = GDK_GL_CONTEXT_CLASS (klass);
 
-  gobject_class->dispose = gdk_x11_gl_context_dispose;
+  gobject_class->dispose = gdk_wayland_gl_context_dispose;
+
+  draw_context_class->end_frame = gdk_wayland_gl_context_end_frame;
 
   context_class->realize = gdk_wayland_gl_context_realize;
-  context_class->end_frame = gdk_wayland_gl_context_end_frame;
+  context_class->get_damage = gdk_wayland_gl_context_get_damage;
 }
 
 static void
@@ -309,13 +311,11 @@ gdk_wayland_get_display (GdkWaylandDisplay *display_wayland)
 }
 
 gboolean
-gdk_wayland_display_init_gl (GdkDisplay *display,
-                             GdkGLContext *share)
+gdk_wayland_display_init_gl (GdkDisplay *display)
 {
   GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (display);
   EGLint major, minor;
   EGLDisplay dpy;
-  gboolean use_es;
 
   if (display_wayland->have_egl)
     return TRUE;
@@ -328,10 +328,8 @@ gdk_wayland_display_init_gl (GdkDisplay *display,
   if (!eglInitialize (dpy, &major, &minor))
     return FALSE;
 
-  use_es = (_gdk_gl_flags & GDK_GL_GLES) != 0 ||
-           (share != NULL && gdk_gl_context_get_use_es (share));
-  if (!eglBindAPI (use_es ? EGL_OPENGL_ES_API : EGL_OPENGL_API))
-      return FALSE;
+  if (!eglBindAPI (EGL_OPENGL_API))
+    return FALSE;
 
   display_wayland->egl_display = dpy;
   display_wayland->egl_major_version = major;
@@ -351,7 +349,7 @@ gdk_wayland_display_init_gl (GdkDisplay *display,
   display_wayland->have_egl_surfaceless_context =
     epoxy_has_egl_extension (dpy, "EGL_KHR_surfaceless_context");
 
-  GDK_NOTE (OPENGL,
+  GDK_DISPLAY_NOTE (display, OPENGL,
             g_message ("EGL API version %d.%d found\n"
                        " - Vendor: %s\n"
                        " - Version: %s\n"
@@ -371,19 +369,15 @@ gdk_wayland_display_init_gl (GdkDisplay *display,
 #define MAX_EGL_ATTRS   30
 
 static gboolean
-find_eglconfig_for_window (GdkWindow  *window,
+find_eglconfig_for_surface (GdkSurface  *surface,
                            EGLConfig  *egl_config_out,
-                           EGLint     *min_swap_interval_out,
                            GError    **error)
 {
-  GdkDisplay *display = gdk_window_get_display (window);
+  GdkDisplay *display = gdk_surface_get_display (surface);
   GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (display);
-  GdkVisual *visual = gdk_window_get_visual (window);
   EGLint attrs[MAX_EGL_ATTRS];
   EGLint count;
-  EGLConfig *configs, chosen_config;
-  gboolean use_rgba;
-
+  EGLConfig *configs;
   int i = 0;
 
   attrs[i++] = EGL_SURFACE_TYPE;
@@ -393,24 +387,13 @@ find_eglconfig_for_window (GdkWindow  *window,
   attrs[i++] = EGL_RGB_BUFFER;
 
   attrs[i++] = EGL_RED_SIZE;
-  attrs[i++] = 8;
+  attrs[i++] = 1;
   attrs[i++] = EGL_GREEN_SIZE;
-  attrs[i++] = 8;
+  attrs[i++] = 1;
   attrs[i++] = EGL_BLUE_SIZE;
-  attrs[i++] = 8;
-
-  use_rgba = (visual == gdk_screen_get_rgba_visual (gdk_display_get_default_screen (display)));
-
-  if (use_rgba)
-    {
-      attrs[i++] = EGL_ALPHA_SIZE;
-      attrs[i++] = 8;
-    }
-  else
-    {
-      attrs[i++] = EGL_ALPHA_SIZE;
-      attrs[i++] = 0;
-    }
+  attrs[i++] = 1;
+  attrs[i++] = EGL_ALPHA_SIZE;
+  attrs[i++] = 1;
 
   attrs[i++] = EGL_NONE;
   g_assert (i < MAX_EGL_ATTRS);
@@ -434,20 +417,9 @@ find_eglconfig_for_window (GdkWindow  *window,
     }
 
   /* Pick first valid configuration i guess? */
-  chosen_config = configs[0];
-
-  if (!eglGetConfigAttrib (display_wayland->egl_display, chosen_config,
-                           EGL_MIN_SWAP_INTERVAL, min_swap_interval_out))
-    {
-      g_set_error_literal (error, GDK_GL_ERROR,
-                           GDK_GL_ERROR_NOT_AVAILABLE,
-                           "Could not retrieve the minimum swap interval");
-      g_free (configs);
-      return FALSE;
-    }
 
   if (egl_config_out != NULL)
-    *egl_config_out = chosen_config;
+    *egl_config_out = configs[0];
 
   g_free (configs);
 
@@ -455,17 +427,17 @@ find_eglconfig_for_window (GdkWindow  *window,
 }
 
 GdkGLContext *
-gdk_wayland_window_create_gl_context (GdkWindow     *window,
+gdk_wayland_surface_create_gl_context (GdkSurface     *surface,
 				      gboolean       attached,
                                       GdkGLContext  *share,
                                       GError       **error)
 {
-  GdkDisplay *display = gdk_window_get_display (window);
+  GdkDisplay *display = gdk_surface_get_display (surface);
   GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (display);
   GdkWaylandGLContext *context;
   EGLConfig config;
 
-  if (!gdk_wayland_display_init_gl (display, share))
+  if (!gdk_wayland_display_init_gl (display))
     {
       g_set_error_literal (error, GDK_GL_ERROR,
                            GDK_GL_ERROR_NOT_AVAILABLE,
@@ -481,14 +453,11 @@ gdk_wayland_window_create_gl_context (GdkWindow     *window,
       return NULL;
     }
 
-  if (!find_eglconfig_for_window (window, &config,
-                                  &display_wayland->egl_min_swap_interval,
-                                  error))
+  if (!find_eglconfig_for_surface (surface, &config, error))
     return NULL;
 
   context = g_object_new (GDK_TYPE_WAYLAND_GL_CONTEXT,
-                          "display", display,
-                          "window", window,
+                          "surface", surface,
                           "shared-context", share,
                           NULL);
 
@@ -499,22 +468,22 @@ gdk_wayland_window_create_gl_context (GdkWindow     *window,
 }
 
 static void
-gdk_x11_gl_context_dispose (GObject *gobject)
+gdk_wayland_gl_context_dispose (GObject *gobject)
 {
   GdkWaylandGLContext *context_wayland = GDK_WAYLAND_GL_CONTEXT (gobject);
 
   if (context_wayland->egl_context != NULL)
     {
       GdkGLContext *context = GDK_GL_CONTEXT (gobject);
-      GdkWindow *window = gdk_gl_context_get_window (context);
-      GdkDisplay *display = gdk_window_get_display (window);
+      GdkSurface *surface = gdk_gl_context_get_surface (context);
+      GdkDisplay *display = gdk_surface_get_display (surface);
       GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (display);
 
       if (eglGetCurrentContext () == context_wayland->egl_context)
         eglMakeCurrent(display_wayland->egl_display, EGL_NO_SURFACE, EGL_NO_SURFACE,
                        EGL_NO_CONTEXT);
 
-      GDK_NOTE (OPENGL, g_message ("Destroying EGL context"));
+      GDK_DISPLAY_NOTE (display, OPENGL, g_message ("Destroying EGL context"));
 
       eglDestroyContext (display_wayland->egl_display,
                          context_wayland->egl_context);
@@ -524,22 +493,13 @@ gdk_x11_gl_context_dispose (GObject *gobject)
   G_OBJECT_CLASS (gdk_wayland_gl_context_parent_class)->dispose (gobject);
 }
 
-gboolean
-gdk_wayland_display_is_gl_context_current (GdkDisplay   *display,
-                                           GdkGLContext *context)
-{
-  GdkWaylandGLContext *context_wayland = GDK_WAYLAND_GL_CONTEXT (context);
-
-  return context_wayland->egl_context == eglGetCurrentContext ();
-}
-
 gboolean
 gdk_wayland_display_make_gl_context_current (GdkDisplay   *display,
                                              GdkGLContext *context)
 {
   GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (display);
   GdkWaylandGLContext *context_wayland;
-  GdkWindow *window;
+  GdkSurface *surface;
   EGLSurface egl_surface;
 
   if (context == NULL)
@@ -550,16 +510,16 @@ gdk_wayland_display_make_gl_context_current (GdkDisplay   *display,
     }
 
   context_wayland = GDK_WAYLAND_GL_CONTEXT (context);
-  window = gdk_gl_context_get_window (context);
+  surface = gdk_gl_context_get_surface (context);
 
-  if (context_wayland->is_attached)
-    egl_surface = gdk_wayland_window_get_egl_surface (window->impl_window, context_wayland->egl_config);
+  if (context_wayland->is_attached || gdk_draw_context_is_in_frame (GDK_DRAW_CONTEXT (context)))
+    egl_surface = gdk_wayland_surface_get_egl_surface (surface->impl_surface, context_wayland->egl_config);
   else
     {
       if (display_wayland->have_egl_surfaceless_context)
 	egl_surface = EGL_NO_SURFACE;
       else
-	egl_surface = gdk_wayland_window_get_dummy_egl_surface (window->impl_window,
+	egl_surface = gdk_wayland_surface_get_dummy_egl_surface (surface->impl_surface,
 								context_wayland->egl_config);
     }
 
@@ -570,10 +530,5 @@ gdk_wayland_display_make_gl_context_current (GdkDisplay   *display,
       return FALSE;
     }
 
-  if (display_wayland->egl_min_swap_interval == 0)
-    eglSwapInterval (display_wayland->egl_display, 0);
-  else
-    g_debug ("Can't disable GL swap interval");
-
   return TRUE;
 }
