@@ -29,13 +29,14 @@ struct _GdkWaylandMonitor {
   guint32 id;
   guint32 version;
   struct wl_output *output;
+  gboolean added;
 
   struct zxdg_output_v1 *xdg_output;
-  /* Raw wl_output data */
-  GdkRectangle output_geometry;
-  /* Raw xdg_output data */
-  GdkRectangle xdg_output_geometry;
-  char *name;
+  /* Size and position, can be either from wl_output or xdg_output */
+  int32_t x;
+  int32_t y;
+  int32_t width;
+  int32_t height;
   gboolean wl_output_done;
   gboolean xdg_output_done;
 };
