@@ -17,19 +17,20 @@
 
 #include "config.h"
 
-#include <gdk/gdkwindow.h>
+#include <gdk/gdksurface.h>
 
 #include <windowsx.h>
 #include <objbase.h>
 
 #include "gdkwin32.h"
 #include "gdkdevice-wintab.h"
+#include "gdkdisplay-win32.h"
 
 G_DEFINE_TYPE (GdkDeviceWintab, gdk_device_wintab, GDK_TYPE_DEVICE)
 
 static gboolean
 gdk_device_wintab_get_history (GdkDevice      *device,
-                               GdkWindow      *window,
+                               GdkSurface      *window,
                                guint32         start,
                                guint32         stop,
                                GdkTimeCoord ***events,
@@ -66,7 +67,7 @@ get_current_mask (void)
 
 static void
 gdk_device_wintab_get_state (GdkDevice       *device,
-                             GdkWindow       *window,
+                             GdkSurface       *window,
                              gdouble         *axes,
                              GdkModifierType *mask)
 {
@@ -92,25 +93,16 @@ gdk_device_wintab_get_state (GdkDevice       *device,
 }
 
 static void
-gdk_device_wintab_set_window_cursor (GdkDevice *device,
-                                     GdkWindow *window,
+gdk_device_wintab_set_surface_cursor (GdkDevice *device,
+                                     GdkSurface *window,
                                      GdkCursor *cursor)
 {
 }
 
-static void
-gdk_device_wintab_warp (GdkDevice *device,
-                        GdkScreen *screen,
-                        gdouble   x,
-                        gdouble   y)
-{
-}
-
 static void
 gdk_device_wintab_query_state (GdkDevice        *device,
-                               GdkWindow        *window,
-                               GdkWindow       **root_window,
-                               GdkWindow       **child_window,
+                               GdkSurface        *window,
+                               GdkSurface       **child_window,
                                gdouble          *root_x,
                                gdouble          *root_y,
                                gdouble          *win_x,
@@ -118,44 +110,51 @@ gdk_device_wintab_query_state (GdkDevice        *device,
                                GdkModifierType  *mask)
 {
   GdkDeviceWintab *device_wintab;
-  GdkScreen *screen;
   POINT point;
   HWND hwnd, hwndc;
-  GdkWindowImplWin32 *impl;
+  int scale;
 
   device_wintab = GDK_DEVICE_WINTAB (device);
-  screen = gdk_window_get_screen (window);
-  impl = GDK_WINDOW_IMPL_WIN32 (window->impl);
-
-  hwnd = GDK_WINDOW_HWND (window);
-  _gdk_win32_get_cursor_pos (&point);
-
-  if (root_x)
-    *root_x = (point.x + _gdk_offset_x) / impl->window_scale;
-
-  if (root_y)
-    *root_y = (point.y + _gdk_offset_y) / impl->window_scale;
-
-  if (window == gdk_screen_get_root_window (screen))
+  if (window)
     {
-      if (win_x)
-        *win_x = (point.x + _gdk_offset_x) / impl->window_scale;
-
-      if (win_y)
-        *win_y = (point.y + _gdk_offset_y) / impl->window_scale;
+      scale = GDK_SURFACE_IMPL_WIN32 (window->impl)->surface_scale;
+      hwnd = GDK_SURFACE_HWND (window);
     }
   else
     {
-      ScreenToClient (hwnd, &point);
+      GdkDisplay *display = gdk_device_get_display (device);
 
-      if (win_x)
-        *win_x = point.x / impl->window_scale;
-
-      if (win_y)
-        *win_y = point.y / impl->window_scale;
+      scale = GDK_WIN32_DISPLAY (display)->surface_scale;
+      hwnd = NULL;
     }
 
-  if (child_window)
+  GetCursorPos (&point);
+
+  if (root_x)
+    *root_x = point.x / scale;
+
+  if (root_y)
+    *root_y = point.y / scale;
+
+  if (hwnd)
+    ScreenToClient (hwnd, &point);
+
+  if (win_x)
+    *win_x = point.x / scale;
+
+  if (win_y)
+    *win_y = point.y / scale;
+
+  if (!window)
+    {
+      if (win_x)
+        *win_x += _gdk_offset_x;
+
+      if (win_y)
+        *win_y += _gdk_offset_y;
+    }
+
+  if (hwnd && child_window)
     {
       hwndc = ChildWindowFromPoint (hwnd, point);
 
@@ -165,9 +164,6 @@ gdk_device_wintab_query_state (GdkDevice        *device,
         *child_window = NULL; /* Direct child unknown to gdk */
     }
 
-  if (root_window)
-    *root_window = gdk_screen_get_root_window (screen);
-
   if (mask)
     {
       *mask = get_current_mask ();
@@ -182,10 +178,10 @@ gdk_device_wintab_query_state (GdkDevice        *device,
 
 static GdkGrabStatus
 gdk_device_wintab_grab (GdkDevice    *device,
-                        GdkWindow    *window,
+                        GdkSurface    *window,
                         gboolean      owner_events,
                         GdkEventMask  event_mask,
-                        GdkWindow    *confine_to,
+                        GdkSurface    *confine_to,
                         GdkCursor    *cursor,
                         guint32       time_)
 {
@@ -198,8 +194,8 @@ gdk_device_wintab_ungrab (GdkDevice *device,
 {
 }
 
-static GdkWindow *
-gdk_device_wintab_window_at_position (GdkDevice       *device,
+static GdkSurface *
+gdk_device_wintab_surface_at_position (GdkDevice       *device,
                                       gdouble         *win_x,
                                       gdouble         *win_y,
                                       GdkModifierType *mask,
@@ -208,31 +204,24 @@ gdk_device_wintab_window_at_position (GdkDevice       *device,
   return NULL;
 }
 
-static void
-gdk_device_wintab_select_window_events (GdkDevice    *device,
-                                        GdkWindow    *window,
-                                        GdkEventMask  event_mask)
-{
-}
-
 void
 _gdk_device_wintab_translate_axes (GdkDeviceWintab *device_wintab,
-                                   GdkWindow       *window,
+                                   GdkSurface       *window,
                                    gdouble         *axes,
                                    gdouble         *x,
                                    gdouble         *y)
 {
   GdkDevice *device;
-  GdkWindow *impl_window;
+  GdkSurface *impl_surface;
   gint root_x, root_y;
   gdouble temp_x, temp_y;
   gint i;
 
   device = GDK_DEVICE (device_wintab);
-  impl_window = _gdk_window_get_impl_window (window);
+  impl_surface = _gdk_surface_get_impl_surface (window);
   temp_x = temp_y = 0;
 
-  gdk_window_get_origin (impl_window, &root_x, &root_y);
+  gdk_surface_get_origin (impl_surface, &root_x, &root_y);
 
   for (i = 0; i < gdk_device_get_n_axes (device); i++)
     {
@@ -244,15 +233,28 @@ _gdk_device_wintab_translate_axes (GdkDeviceWintab *device_wintab,
         {
         case GDK_AXIS_X:
         case GDK_AXIS_Y:
-          if (gdk_device_get_mode (device) == GDK_MODE_WINDOW)
-            _gdk_device_translate_window_coord (device, window, i,
+          if (gdk_device_get_mode (device) == GDK_MODE_SURFACE)
+            _gdk_device_translate_surface_coord (device, window, i,
                                                 device_wintab->last_axis_data[i],
                                                 &axes[i]);
           else
-            _gdk_device_translate_screen_coord (device, window,
-                                                root_x, root_y, i,
-                                                device_wintab->last_axis_data[i],
-                                                &axes[i]);
+            {
+              HMONITOR hmonitor;
+              MONITORINFO minfo = {sizeof (MONITORINFO),};
+
+              hmonitor = MonitorFromWindow (GDK_SURFACE_HWND (window),
+                                            MONITOR_DEFAULTTONEAREST);
+              GetMonitorInfo (hmonitor, &minfo);
+
+              /* XXX: the dimensions from minfo may need to be scaled for HiDPI usage */
+              _gdk_device_translate_screen_coord (device, window,
+                                                  root_x, root_y,
+                                                  minfo.rcWork.right - minfo.rcWork.left,
+                                                  minfo.rcWork.bottom - minfo.rcWork.top,
+                                                  i,
+                                                  device_wintab->last_axis_data[i],
+                                                  &axes[i]);
+            }
           if (use == GDK_AXIS_X)
             temp_x = axes[i];
           else if (use == GDK_AXIS_Y)
@@ -281,13 +283,11 @@ gdk_device_wintab_class_init (GdkDeviceWintabClass *klass)
 
   device_class->get_history = gdk_device_wintab_get_history;
   device_class->get_state = gdk_device_wintab_get_state;
-  device_class->set_window_cursor = gdk_device_wintab_set_window_cursor;
-  device_class->warp = gdk_device_wintab_warp;
+  device_class->set_surface_cursor = gdk_device_wintab_set_surface_cursor;
   device_class->query_state = gdk_device_wintab_query_state;
   device_class->grab = gdk_device_wintab_grab;
   device_class->ungrab = gdk_device_wintab_ungrab;
-  device_class->window_at_position = gdk_device_wintab_window_at_position;
-  device_class->select_window_events = gdk_device_wintab_select_window_events;
+  device_class->surface_at_position = gdk_device_wintab_surface_at_position;
 }
 
 static void
