@@ -22,14 +22,14 @@
 
 #include "gdkglcontext-x11.h"
 #include "gdkdisplay-x11.h"
+#include "gdkprivate-x11.h"
 #include "gdkscreen-x11.h"
 
 #include "gdkx11display.h"
 #include "gdkx11glcontext.h"
 #include "gdkx11screen.h"
-#include "gdkx11window.h"
-#include "gdkx11visual.h"
-#include "gdkvisualprivate.h"
+#include "gdkx11surface.h"
+#include "gdkvisual-x11.h"
 #include "gdkx11property.h"
 #include <X11/Xatom.h>
 
@@ -79,16 +79,16 @@ drawable_info_free (gpointer data_)
 }
 
 static DrawableInfo *
-get_glx_drawable_info (GdkWindow *window)
+get_glx_drawable_info (GdkSurface *surface)
 {
-  return g_object_get_data (G_OBJECT (window), "-gdk-x11-window-glx-info");
+  return g_object_get_data (G_OBJECT (surface), "-gdk-x11-surface-glx-info");
 }
 
 static void
-set_glx_drawable_info (GdkWindow    *window,
+set_glx_drawable_info (GdkSurface    *surface,
                        DrawableInfo *info)
 {
-  g_object_set_data_full (G_OBJECT (window), "-gdk-x11-window-glx-info",
+  g_object_set_data_full (G_OBJECT (surface), "-gdk-x11-surface-glx-info",
                           info,
                           drawable_info_free);
 }
@@ -118,127 +118,34 @@ maybe_wait_for_vblank (GdkDisplay  *display,
     }
 }
 
-void
-gdk_x11_window_invalidate_for_new_frame (GdkWindow      *window,
-                                         cairo_region_t *update_area)
-{
-  cairo_rectangle_int_t window_rect;
-  GdkDisplay *display = gdk_window_get_display (window);
-  GdkX11Display *display_x11 = GDK_X11_DISPLAY (display);
-  Display *dpy = gdk_x11_display_get_xdisplay (display);
-  GdkX11GLContext *context_x11;
-  unsigned int buffer_age;
-  gboolean invalidate_all;
-
-  /* Minimal update is ok if we're not drawing with gl */
-  if (window->gl_paint_context == NULL)
-    return;
-
-  context_x11 = GDK_X11_GL_CONTEXT (window->gl_paint_context);
-
-  buffer_age = 0;
-
-  context_x11->do_blit_swap = FALSE;
-
-  if (display_x11->has_glx_buffer_age)
-    {
-      gdk_gl_context_make_current (window->gl_paint_context);
-      glXQueryDrawable(dpy, context_x11->drawable,
-		       GLX_BACK_BUFFER_AGE_EXT, &buffer_age);
-    }
-
-
-  invalidate_all = FALSE;
-  if (buffer_age >= 4)
-    {
-      cairo_rectangle_int_t whole_window = { 0, 0, gdk_window_get_width (window), gdk_window_get_height (window) };
-
-      if (gdk_gl_context_has_framebuffer_blit (window->gl_paint_context) &&
-          cairo_region_contains_rectangle (update_area, &whole_window) != CAIRO_REGION_OVERLAP_IN)
-        {
-          context_x11->do_blit_swap = TRUE;
-        }
-      else
-        invalidate_all = TRUE;
-    }
-  else
-    {
-      if (buffer_age == 0)
-        {
-          invalidate_all = TRUE;
-        }
-      if (buffer_age >= 2)
-        {
-          if (window->old_updated_area[0])
-            cairo_region_union (update_area, window->old_updated_area[0]);
-          else
-            invalidate_all = TRUE;
-        }
-      if (buffer_age >= 3)
-        {
-          if (window->old_updated_area[1])
-            cairo_region_union (update_area, window->old_updated_area[1]);
-          else
-            invalidate_all = TRUE;
-        }
-    }
-
-  if (invalidate_all)
-    {
-      window_rect.x = 0;
-      window_rect.y = 0;
-      window_rect.width = gdk_window_get_width (window);
-      window_rect.height = gdk_window_get_height (window);
-
-      /* If nothing else is known, repaint everything so that the back
-         buffer is fully up-to-date for the swapbuffer */
-      cairo_region_union_rectangle (update_area, &window_rect);
-    }
-
-}
-
 static void
-gdk_gl_blit_region (GdkWindow *window, cairo_region_t *region)
-{
-  int n_rects, i;
-  int scale = gdk_window_get_scale_factor (window);
-  int wh = gdk_window_get_height (window);
-  cairo_rectangle_int_t rect;
-
-  n_rects = cairo_region_num_rectangles (region);
-  for (i = 0; i < n_rects; i++)
-    {
-      cairo_region_get_rectangle (region, i, &rect);
-      glScissor (rect.x * scale, (wh - rect.y - rect.height) * scale, rect.width * scale, rect.height * scale);
-      glBlitFramebuffer (rect.x * scale, (wh - rect.y - rect.height) * scale, (rect.x + rect.width) * scale, (wh - rect.y) * scale,
-                         rect.x * scale, (wh - rect.y - rect.height) * scale, (rect.x + rect.width) * scale, (wh - rect.y) * scale,
-                         GL_COLOR_BUFFER_BIT, GL_NEAREST);
-    }
-}
-
-static void
-gdk_x11_gl_context_end_frame (GdkGLContext *context,
-                              cairo_region_t *painted,
-                              cairo_region_t *damage)
+gdk_x11_gl_context_end_frame (GdkDrawContext *draw_context,
+                              cairo_region_t *painted)
 {
+  GdkGLContext *context = GDK_GL_CONTEXT (draw_context);
   GdkX11GLContext *context_x11 = GDK_X11_GL_CONTEXT (context);
-  GdkWindow *window = gdk_gl_context_get_window (context);
+  GdkSurface *surface = gdk_gl_context_get_surface (context);
   GdkDisplay *display = gdk_gl_context_get_display (context);
   Display *dpy = gdk_x11_display_get_xdisplay (display);
   GdkX11Display *display_x11 = GDK_X11_DISPLAY (display);
+  //GdkRectangle whole_window;
   DrawableInfo *info;
   GLXDrawable drawable;
 
+  GDK_DRAW_CONTEXT_CLASS (gdk_x11_gl_context_parent_class)->end_frame (draw_context, painted);
+  if (gdk_gl_context_get_shared_context (context))
+    return;
+
   gdk_gl_context_make_current (context);
 
-  info = get_glx_drawable_info (window);
+  info = get_glx_drawable_info (surface);
 
-  drawable = context_x11->drawable;
+  drawable = context_x11->attached_drawable;
 
-  GDK_NOTE (OPENGL,
+  GDK_DISPLAY_NOTE (display, OPENGL,
             g_message ("Flushing GLX buffers for drawable %lu (window: %lu), frame sync: %s",
                        (unsigned long) drawable,
-                       (unsigned long) gdk_x11_window_get_xid (window),
+                       (unsigned long) gdk_x11_surface_get_xid (surface),
                        context_x11->do_frame_sync ? "yes" : "no"));
 
   /* if we are going to wait for the vertical refresh manually
@@ -274,24 +181,66 @@ gdk_x11_gl_context_end_frame (GdkGLContext *context,
         }
     }
 
-  if (context_x11->do_blit_swap)
-    {
-      glDrawBuffer(GL_FRONT);
-      glReadBuffer(GL_BACK);
-      gdk_gl_blit_region (window, painted);
-      glDrawBuffer(GL_BACK);
-      glFlush();
+  gdk_x11_surface_pre_damage (surface);
 
-      if (gdk_gl_context_has_frame_terminator (context))
-        glFrameTerminatorGREMEDY ();
-    }
-  else
-    glXSwapBuffers (dpy, drawable);
+  glXSwapBuffers (dpy, drawable);
 
   if (context_x11->do_frame_sync && info != NULL && display_x11->has_glx_video_sync)
     glXGetVideoSyncSGI (&info->last_frame_counter);
 }
 
+static cairo_region_t *
+gdk_x11_gl_context_get_damage (GdkGLContext *context)
+{
+  GdkDisplay *display = gdk_draw_context_get_display (GDK_DRAW_CONTEXT (context));
+  GdkX11Display *display_x11 = GDK_X11_DISPLAY (display);
+  Display *dpy = gdk_x11_display_get_xdisplay (display);
+  unsigned int buffer_age = 0;
+
+  if (display_x11->has_glx_buffer_age)
+    {
+      GdkGLContext *shared;
+      GdkX11GLContext *shared_x11;
+
+      shared = gdk_gl_context_get_shared_context (context);
+      if (shared == NULL)
+        shared = context;
+      shared_x11 = GDK_X11_GL_CONTEXT (shared);
+
+      gdk_gl_context_make_current (shared);
+      glXQueryDrawable (dpy, shared_x11->attached_drawable,
+                        GLX_BACK_BUFFER_AGE_EXT, &buffer_age);
+
+      switch (buffer_age)
+        {
+          case 1:
+            return cairo_region_create ();
+            break;
+
+          case 2:
+            if (context->old_updated_area[0])
+              return cairo_region_copy (context->old_updated_area[0]);
+            break;
+
+          case 3:
+            if (context->old_updated_area[0] &&
+                context->old_updated_area[1])
+              {
+                cairo_region_t *damage = cairo_region_copy (context->old_updated_area[0]);
+                cairo_region_union (damage, context->old_updated_area[1]);
+                return damage;
+              }
+            break;
+
+          default:
+            ;
+        }
+
+    }
+
+  return GDK_GL_CONTEXT_CLASS (gdk_x11_gl_context_parent_class)->get_damage (context);
+}
+
 typedef struct {
   Display *display;
   GLXDrawable drawable;
@@ -424,16 +373,16 @@ glx_pixmap_get (cairo_surface_t *surface, guint texture_target)
 
 static gboolean
 gdk_x11_gl_context_texture_from_surface (GdkGLContext *paint_context,
-					 cairo_surface_t *surface,
+					 cairo_surface_t *cairo_surface,
 					 cairo_region_t *region)
 {
   GdkGLXPixmap *glx_pixmap;
   double device_x_offset, device_y_offset;
   cairo_rectangle_int_t rect;
   int n_rects, i;
-  GdkWindow *window;
-  int unscaled_window_height;
-  int window_scale;
+  GdkSurface *surface;
+  int unscaled_surface_height;
+  int surface_scale;
   unsigned int texture_id;
   gboolean use_texture_rectangle;
   guint target;
@@ -446,7 +395,7 @@ gdk_x11_gl_context_texture_from_surface (GdkGLContext *paint_context,
   if (!display_x11->has_glx_texture_from_pixmap)
     return FALSE;
 
-  if (cairo_surface_get_type (surface) != CAIRO_SURFACE_TYPE_XLIB)
+  if (cairo_surface_get_type (cairo_surface) != CAIRO_SURFACE_TYPE_XLIB)
     return FALSE;
 
   use_texture_rectangle = gdk_gl_context_use_texture_rectangle (paint_context);
@@ -455,19 +404,19 @@ gdk_x11_gl_context_texture_from_surface (GdkGLContext *paint_context,
   else
     target = GL_TEXTURE_2D;
 
-  glx_pixmap = glx_pixmap_get (surface, target);
+  glx_pixmap = glx_pixmap_get (cairo_surface, target);
   if (glx_pixmap == NULL)
     return FALSE;
 
-  GDK_NOTE (OPENGL, g_message ("Using GLX_EXT_texture_from_pixmap to draw surface"));
+  GDK_DISPLAY_NOTE (GDK_DISPLAY (display_x11), OPENGL, g_message ("Using GLX_EXT_texture_from_pixmap to draw surface"));
 
-  window = gdk_gl_context_get_window (paint_context)->impl_window;
-  window_scale = gdk_window_get_scale_factor (window);
-  gdk_window_get_unscaled_size (window, NULL, &unscaled_window_height);
+  surface = gdk_gl_context_get_surface (paint_context)->impl_surface;
+  surface_scale = gdk_surface_get_scale_factor (surface);
+  gdk_surface_get_unscaled_size (surface, NULL, &unscaled_surface_height);
 
   sx = sy = 1;
-  cairo_surface_get_device_scale (window->current_paint.surface, &sx, &sy);
-  cairo_surface_get_device_offset (surface, &device_x_offset, &device_y_offset);
+  cairo_surface_get_device_scale (cairo_surface, &sx, &sy);
+  cairo_surface_get_device_offset (cairo_surface, &device_x_offset, &device_y_offset);
 
   /* Ensure all the X stuff are synced before we read it back via texture-from-pixmap */
   glXWaitX();
@@ -488,11 +437,11 @@ gdk_x11_gl_context_texture_from_surface (GdkGLContext *paint_context,
   n_rects = cairo_region_num_rectangles (region);
   quads = g_new (GdkTexturedQuad, n_rects);
 
-#define FLIP_Y(_y) (unscaled_window_height - (_y))
+#define FLIP_Y(_y) (unscaled_surface_height - (_y))
 
   cairo_region_get_extents (region, &rect);
-  glScissor (rect.x * window_scale, FLIP_Y((rect.y + rect.height) * window_scale),
-             rect.width * window_scale, rect.height * window_scale);
+  glScissor (rect.x * surface_scale, FLIP_Y((rect.y + rect.height) * surface_scale),
+             rect.width * surface_scale, rect.height * surface_scale);
 
   for (i = 0; i < n_rects; i++)
     {
@@ -512,14 +461,14 @@ gdk_x11_gl_context_texture_from_surface (GdkGLContext *paint_context,
         }
       else
         {
-          uscale = 1.0 / cairo_xlib_surface_get_width (surface);
-          vscale = 1.0 / cairo_xlib_surface_get_height (surface);
+          uscale = 1.0 / cairo_xlib_surface_get_width (cairo_surface);
+          vscale = 1.0 / cairo_xlib_surface_get_height (cairo_surface);
         }
 
       {
         GdkTexturedQuad quad = {
-          rect.x * window_scale, FLIP_Y(rect.y * window_scale),
-          (rect.x + rect.width) * window_scale, FLIP_Y((rect.y + rect.height) * window_scale),
+          rect.x * surface_scale, FLIP_Y(rect.y * surface_scale),
+          (rect.x + rect.width) * surface_scale, FLIP_Y((rect.y + rect.height) * surface_scale),
           uscale * src_x, vscale * src_y,
           uscale * (src_x + src_width), vscale * (src_y + src_height),
         };
@@ -623,17 +572,16 @@ gdk_x11_gl_context_realize (GdkGLContext  *context,
   GdkX11Display *display_x11;
   GdkDisplay *display;
   GdkX11GLContext *context_x11;
-  GLXWindow drawable;
   XVisualInfo *xvisinfo;
   Display *dpy;
   DrawableInfo *info;
   GdkGLContext *share;
-  GdkWindow *window;
+  GdkSurface *surface;
   gboolean debug_bit, compat_bit, legacy_bit, es_bit;
   int major, minor, flags;
 
-  window = gdk_gl_context_get_window (context);
-  display = gdk_window_get_display (window);
+  surface = gdk_gl_context_get_surface (context);
+  display = gdk_surface_get_display (surface);
   dpy = gdk_x11_display_get_xdisplay (display);
   context_x11 = GDK_X11_GL_CONTEXT (context);
   display_x11 = GDK_X11_DISPLAY (display);
@@ -644,11 +592,9 @@ gdk_x11_gl_context_realize (GdkGLContext  *context,
   compat_bit = gdk_gl_context_get_forward_compatible (context);
 
   /* If there is no glXCreateContextAttribsARB() then we default to legacy */
-  legacy_bit = !display_x11->has_glx_create_context ||
-               (_gdk_gl_flags & GDK_GL_LEGACY) != 0;
+  legacy_bit = !display_x11->has_glx_create_context || GDK_DISPLAY_DEBUG_CHECK (display, GL_LEGACY);
 
-  es_bit = ((_gdk_gl_flags & GDK_GL_GLES) != 0 ||
-            (share != NULL && gdk_gl_context_get_use_es (share))) &&
+  es_bit = (GDK_DISPLAY_DEBUG_CHECK (display, GL_GLES) || (share != NULL && gdk_gl_context_get_use_es (share))) &&
            (display_x11->has_glx_create_context && display_x11->has_glx_create_es2_context);
 
   /* We cannot share legacy contexts with core profile ones, so the
@@ -664,7 +610,7 @@ gdk_x11_gl_context_realize (GdkGLContext  *context,
   if (compat_bit)
     flags |= GLX_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB;
 
-  GDK_NOTE (OPENGL,
+  GDK_DISPLAY_NOTE (display, OPENGL,
             g_message ("Creating GLX context (version:%d.%d, debug:%s, forward:%s, legacy:%s, es:%s)",
                        major, minor,
                        debug_bit ? "yes" : "no",
@@ -678,7 +624,7 @@ gdk_x11_gl_context_realize (GdkGLContext  *context,
    */
   if (legacy_bit && !GDK_X11_DISPLAY (display)->has_glx_create_context)
     {
-      GDK_NOTE (OPENGL, g_message ("Creating legacy GL context on request"));
+      GDK_DISPLAY_NOTE (display, OPENGL, g_message ("Creating legacy GL context on request"));
       context_x11->glx_context = create_legacy_context (display, context_x11->glx_config, share);
     }
   else
@@ -701,7 +647,7 @@ gdk_x11_gl_context_realize (GdkGLContext  *context,
           minor = 0;
         }
 
-      GDK_NOTE (OPENGL, g_message ("Creating GL3 context"));
+      GDK_DISPLAY_NOTE (display, OPENGL, g_message ("Creating GL3 context"));
       context_x11->glx_context = create_gl3_context (display,
                                                      context_x11->glx_config,
                                                      share,
@@ -710,7 +656,7 @@ gdk_x11_gl_context_realize (GdkGLContext  *context,
       /* Fall back to legacy in case the GL3 context creation failed */
       if (context_x11->glx_context == NULL)
         {
-          GDK_NOTE (OPENGL, g_message ("Creating fallback legacy context"));
+          GDK_DISPLAY_NOTE (display, OPENGL, g_message ("Creating fallback legacy context"));
           context_x11->glx_context = create_legacy_context (display, context_x11->glx_config, share);
           legacy_bit = TRUE;
           es_bit = FALSE;
@@ -733,7 +679,7 @@ gdk_x11_gl_context_realize (GdkGLContext  *context,
 
   xvisinfo = find_xvisinfo_for_fbconfig (display, context_x11->glx_config);
 
-  info = get_glx_drawable_info (window->impl_window);
+  info = get_glx_drawable_info (surface->impl_surface);
   if (info == NULL)
     {
       XSetWindowAttributes attrs;
@@ -762,7 +708,7 @@ gdk_x11_gl_context_realize (GdkGLContext  *context,
       if (GDK_X11_DISPLAY (display)->glx_version >= 13)
         {
           info->glx_drawable = glXCreateWindow (dpy, context_x11->glx_config,
-                                                gdk_x11_window_get_xid (window->impl_window),
+                                                gdk_x11_surface_get_xid (surface->impl_surface),
                                                 NULL);
           info->dummy_glx = glXCreateWindow (dpy, context_x11->glx_config, info->dummy_xwin, NULL);
         }
@@ -781,20 +727,17 @@ gdk_x11_gl_context_realize (GdkGLContext  *context,
           return FALSE;
         }
 
-      set_glx_drawable_info (window->impl_window, info);
+      set_glx_drawable_info (surface->impl_surface, info);
     }
 
   XFree (xvisinfo);
 
-  if (context_x11->is_attached)
-    drawable = info->glx_drawable ? info->glx_drawable : gdk_x11_window_get_xid (window->impl_window);
-  else
-    drawable = info->dummy_glx ? info->dummy_glx : info->dummy_xwin;
+  context_x11->attached_drawable = info->glx_drawable ? info->glx_drawable : gdk_x11_surface_get_xid (surface->impl_surface);
+  context_x11->unattached_drawable = info->dummy_glx ? info->dummy_glx : info->dummy_xwin;
 
   context_x11->is_direct = glXIsDirect (dpy, context_x11->glx_context);
-  context_x11->drawable = drawable;
 
-  GDK_NOTE (OPENGL,
+  GDK_DISPLAY_NOTE (display, OPENGL,
             g_message ("Realized GLX context[%p], %s",
                        context_x11->glx_context,
                        context_x11->is_direct ? "direct" : "indirect"));
@@ -816,7 +759,7 @@ gdk_x11_gl_context_dispose (GObject *gobject)
       if (glXGetCurrentContext () == context_x11->glx_context)
         glXMakeContextCurrent (dpy, None, None, NULL);
 
-      GDK_NOTE (OPENGL, g_message ("Destroying GLX context"));
+      GDK_DISPLAY_NOTE (display, OPENGL, g_message ("Destroying GLX context"));
       glXDestroyContext (dpy, context_x11->glx_context);
       context_x11->glx_context = NULL;
     }
@@ -828,12 +771,15 @@ static void
 gdk_x11_gl_context_class_init (GdkX11GLContextClass *klass)
 {
   GdkGLContextClass *context_class = GDK_GL_CONTEXT_CLASS (klass);
+  GdkDrawContextClass *draw_context_class = GDK_DRAW_CONTEXT_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   context_class->realize = gdk_x11_gl_context_realize;
-  context_class->end_frame = gdk_x11_gl_context_end_frame;
+  context_class->get_damage = gdk_x11_gl_context_get_damage;
   context_class->texture_from_surface = gdk_x11_gl_context_texture_from_surface;
 
+  draw_context_class->end_frame = gdk_x11_gl_context_end_frame;
+
   gobject_class->dispose = gdk_x11_gl_context_dispose;
 }
 
@@ -844,9 +790,9 @@ gdk_x11_gl_context_init (GdkX11GLContext *self)
 }
 
 gboolean
-gdk_x11_screen_init_gl (GdkScreen *screen)
+gdk_x11_screen_init_gl (GdkX11Screen *screen)
 {
-  GdkDisplay *display = gdk_screen_get_display (screen);
+  GdkDisplay *display = GDK_SCREEN_DISPLAY (screen);
   GdkX11Display *display_x11 = GDK_X11_DISPLAY (display);
   Display *dpy;
   int error_base, event_base;
@@ -855,7 +801,7 @@ gdk_x11_screen_init_gl (GdkScreen *screen)
   if (display_x11->have_glx)
     return TRUE;
 
-  if (_gdk_gl_flags & GDK_GL_DISABLE)
+  if (GDK_DISPLAY_DEBUG_CHECK (display, GL_DISABLE))
     return FALSE;
 
   dpy = gdk_x11_display_get_xdisplay (display);
@@ -866,7 +812,7 @@ gdk_x11_screen_init_gl (GdkScreen *screen)
   if (!glXQueryExtension (dpy, &error_base, &event_base))
     return FALSE;
 
-  screen_num = GDK_X11_SCREEN (screen)->screen_num;
+  screen_num = screen->screen_num;
 
   display_x11->have_glx = TRUE;
 
@@ -893,7 +839,7 @@ gdk_x11_screen_init_gl (GdkScreen *screen)
   display_x11->has_glx_visual_rating =
     epoxy_has_glx_extension (dpy, screen_num, "GLX_EXT_visual_rating");
 
-  GDK_NOTE (OPENGL,
+  GDK_DISPLAY_NOTE (display, OPENGL,
             g_message ("GLX version %d.%d found\n"
                        " - Vendor: %s\n"
                        " - Checked extensions:\n"
@@ -921,18 +867,16 @@ gdk_x11_screen_init_gl (GdkScreen *screen)
 #define MAX_GLX_ATTRS   30
 
 static gboolean
-find_fbconfig_for_visual (GdkDisplay   *display,
-			  GdkVisual    *visual,
-                          GLXFBConfig  *fb_config_out,
-                          GError      **error)
+find_fbconfig (GdkDisplay   *display,
+               GLXFBConfig  *fb_config_out,
+               GError      **error)
 {
   static int attrs[MAX_GLX_ATTRS];
   Display *dpy = gdk_x11_display_get_xdisplay (display);
   GLXFBConfig *configs;
   int n_configs, i;
-  gboolean use_rgba;
   gboolean retval = FALSE;
-  VisualID xvisual_id = XVisualIDFromVisual(gdk_x11_visual_get_xvisual (visual));
+  VisualID xvisual_id = XVisualIDFromVisual (gdk_x11_display_get_window_visual (GDK_X11_DISPLAY (display)));
 
   i = 0;
   attrs[i++] = GLX_DRAWABLE_TYPE;
@@ -951,8 +895,7 @@ find_fbconfig_for_visual (GdkDisplay   *display,
   attrs[i++] = GLX_BLUE_SIZE;
   attrs[i++] = 1;
 
-  use_rgba = (visual == gdk_screen_get_rgba_visual (gdk_display_get_default_screen (display)));
-  if (use_rgba)
+  if (gdk_display_is_rgba (display))
     {
       attrs[i++] = GLX_ALPHA_SIZE;
       attrs[i++] = 1;
@@ -1020,7 +963,7 @@ struct glvisualinfo {
 };
 
 static gboolean
-visual_compatible (const GdkVisual *a, const GdkVisual *b)
+visual_compatible (const GdkX11Visual *a, const GdkX11Visual *b)
 {
   return a->type == b->type &&
     a->depth == b->depth &&
@@ -1032,7 +975,7 @@ visual_compatible (const GdkVisual *a, const GdkVisual *b)
 }
 
 static gboolean
-visual_is_rgba (const GdkVisual *visual)
+visual_is_rgba (const GdkX11Visual *visual)
 {
   return
     visual->depth == 32 &&
@@ -1043,12 +986,12 @@ visual_is_rgba (const GdkVisual *visual)
 
 /* This picks a compatible (as in has the same X visual details) visual
    that has "better" characteristics on the GL side */
-static GdkVisual *
+static GdkX11Visual *
 pick_better_visual_for_gl (GdkX11Screen *x11_screen,
                            struct glvisualinfo *gl_info,
-                           GdkVisual *compatible)
+                           GdkX11Visual *compatible)
 {
-  GdkVisual *visual;
+  GdkX11Visual *visual;
   int i;
   gboolean want_alpha = visual_is_rgba (compatible);
 
@@ -1181,9 +1124,8 @@ save_cached_gl_visuals (GdkDisplay *display, int system, int rgba)
 }
 
 void
-_gdk_x11_screen_update_visuals_for_gl (GdkScreen *screen)
+_gdk_x11_screen_update_visuals_for_gl (GdkX11Screen *x11_screen)
 {
-  GdkX11Screen *x11_screen;
   GdkDisplay *display;
   GdkX11Display *display_x11;
   Display *dpy;
@@ -1191,7 +1133,6 @@ _gdk_x11_screen_update_visuals_for_gl (GdkScreen *screen)
   int i;
   int system_visual_id, rgba_visual_id;
 
-  x11_screen = GDK_X11_SCREEN (screen);
   display = x11_screen->display;
   display_x11 = GDK_X11_DISPLAY (display);
   dpy = gdk_x11_display_get_xdisplay (display);
@@ -1202,7 +1143,7 @@ _gdk_x11_screen_update_visuals_for_gl (GdkScreen *screen)
     {
       for (i = 0; i < x11_screen->nvisuals; i++)
         {
-          GdkVisual *visual = x11_screen->visuals[i];
+          GdkX11Visual *visual = x11_screen->visuals[i];
           int visual_id = gdk_x11_visual_get_xvisual (visual)->visualid;
 
           if (visual_id == system_visual_id)
@@ -1214,7 +1155,7 @@ _gdk_x11_screen_update_visuals_for_gl (GdkScreen *screen)
       return;
     }
 
-  if (!gdk_x11_screen_init_gl (screen))
+  if (!gdk_x11_screen_init_gl (x11_screen))
     return;
 
   gl_info = g_new0 (struct glvisualinfo, x11_screen->nvisuals);
@@ -1262,19 +1203,18 @@ _gdk_x11_screen_update_visuals_for_gl (GdkScreen *screen)
 }
 
 GdkGLContext *
-gdk_x11_window_create_gl_context (GdkWindow    *window,
+gdk_x11_surface_create_gl_context (GdkSurface    *surface,
                                   gboolean      attached,
                                   GdkGLContext *share,
                                   GError      **error)
 {
   GdkDisplay *display;
   GdkX11GLContext *context;
-  GdkVisual *visual;
   GLXFBConfig config;
 
-  display = gdk_window_get_display (window);
+  display = gdk_surface_get_display (surface);
 
-  if (!gdk_x11_screen_init_gl (gdk_window_get_screen (window)))
+  if (!gdk_x11_screen_init_gl (GDK_SURFACE_SCREEN (surface)))
     {
       g_set_error_literal (error, GDK_GL_ERROR,
                            GDK_GL_ERROR_NOT_AVAILABLE,
@@ -1282,13 +1222,11 @@ gdk_x11_window_create_gl_context (GdkWindow    *window,
       return NULL;
     }
 
-  visual = gdk_window_get_visual (window);
-  if (!find_fbconfig_for_visual (display, visual, &config, error))
+  if (!find_fbconfig (display, &config, error))
     return NULL;
 
   context = g_object_new (GDK_TYPE_X11_GL_CONTEXT,
-                          "display", display,
-                          "window", window,
+                          "surface", surface,
                           "shared-context", share,
                           NULL);
 
@@ -1304,9 +1242,8 @@ gdk_x11_display_make_gl_context_current (GdkDisplay   *display,
 {
   GdkX11GLContext *context_x11;
   Display *dpy = gdk_x11_display_get_xdisplay (display);
-  GdkWindow *window;
-  GdkScreen *screen;
   gboolean do_frame_sync = FALSE;
+  GLXWindow drawable;
 
   if (context == NULL)
     {
@@ -1322,28 +1259,30 @@ gdk_x11_display_make_gl_context_current (GdkDisplay   *display,
       return FALSE;
     }
 
-  GDK_NOTE (OPENGL,
-            g_message ("Making GLX context current to drawable %lu",
-                       (unsigned long) context_x11->drawable));
+  if (context_x11->is_attached || gdk_draw_context_is_in_frame (GDK_DRAW_CONTEXT (context)))
+    drawable = context_x11->attached_drawable;
+  else
+    drawable = context_x11->unattached_drawable;
 
-  if (!glXMakeContextCurrent (dpy, context_x11->drawable, context_x11->drawable,
+  GDK_DISPLAY_NOTE (display, OPENGL,
+            g_message ("Making GLX context %p current to drawable %lu",
+                       context, (unsigned long) drawable));
+
+  if (!glXMakeContextCurrent (dpy, drawable, drawable,
                               context_x11->glx_context))
     {
-      GDK_NOTE (OPENGL,
+      GDK_DISPLAY_NOTE (display, OPENGL,
                 g_message ("Making GLX context current failed"));
       return FALSE;
     }
 
   if (context_x11->is_attached && GDK_X11_DISPLAY (display)->has_glx_swap_interval)
     {
-      window = gdk_gl_context_get_window (context);
-
       /* If the WM is compositing there is no particular need to delay
        * the swap when drawing on the offscreen, rendering to the screen
        * happens later anyway, and its up to the compositor to sync that
        * to the vblank. */
-      screen = gdk_window_get_screen (window);
-      do_frame_sync = ! gdk_screen_is_composited (screen);
+      do_frame_sync = ! gdk_display_is_composited (display);
 
       if (do_frame_sync != context_x11->do_frame_sync)
         {
@@ -1368,8 +1307,6 @@ gdk_x11_display_make_gl_context_current (GdkDisplay   *display,
  * Retrieves the version of the GLX implementation.
  *
  * Returns: %TRUE if GLX is available
- *
- * Since: 3.16
  */
 gboolean
 gdk_x11_display_get_glx_version (GdkDisplay *display,
@@ -1381,7 +1318,7 @@ gdk_x11_display_get_glx_version (GdkDisplay *display,
   if (!GDK_IS_X11_DISPLAY (display))
     return FALSE;
 
-  if (!gdk_x11_screen_init_gl (gdk_display_get_default_screen (display)))
+  if (!gdk_x11_screen_init_gl (GDK_X11_DISPLAY (display)->screen))
     return FALSE;
 
   if (major != NULL)
