@@ -30,6 +30,7 @@
 #include <gtk/gtk.h>
 #include "gtktextviewaccessibleprivate.h"
 #include "gtktextbufferprivate.h"
+#include "gtktextviewprivate.h"
 #include "gtk/gtkwidgetprivate.h"
 
 struct _GtkTextViewAccessiblePrivate
@@ -59,13 +60,11 @@ static void       mark_set_cb           (GtkTextBuffer    *buffer,
 
 static void atk_editable_text_interface_init      (AtkEditableTextIface      *iface);
 static void atk_text_interface_init               (AtkTextIface              *iface);
-static void atk_streamable_content_interface_init (AtkStreamableContentIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GtkTextViewAccessible, gtk_text_view_accessible, GTK_TYPE_CONTAINER_ACCESSIBLE,
                          G_ADD_PRIVATE (GtkTextViewAccessible)
                          G_IMPLEMENT_INTERFACE (ATK_TYPE_EDITABLE_TEXT, atk_editable_text_interface_init)
-                         G_IMPLEMENT_INTERFACE (ATK_TYPE_TEXT, atk_text_interface_init)
-                         G_IMPLEMENT_INTERFACE (ATK_TYPE_STREAMABLE_CONTENT, atk_streamable_content_interface_init))
+                         G_IMPLEMENT_INTERFACE (ATK_TYPE_TEXT, atk_text_interface_init))
 
 
 static void
@@ -413,7 +412,7 @@ gtk_text_view_accessible_get_caret_offset (AtkText *text)
 
   widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
-    return 0;
+    return -1;
 
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
   return get_insert_offset (buffer);
@@ -450,9 +449,9 @@ gtk_text_view_accessible_get_offset_at_point (AtkText      *text,
 {
   GtkTextView *view;
   GtkTextIter iter;
-  gint x_widget, y_widget, x_window, y_window, buff_x, buff_y;
+  gint x_widget, y_widget, x_surface, y_surface, buff_x, buff_y;
   GtkWidget *widget;
-  GdkWindow *window;
+  GdkSurface *surface;
   GdkRectangle rect;
 
   widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
@@ -460,8 +459,8 @@ gtk_text_view_accessible_get_offset_at_point (AtkText      *text,
     return -1;
 
   view = GTK_TEXT_VIEW (widget);
-  window = gtk_text_view_get_window (view, GTK_TEXT_WINDOW_WIDGET);
-  gdk_window_get_origin (window, &x_widget, &y_widget);
+  surface = gtk_widget_get_surface (widget);
+  gdk_surface_get_origin (surface, &x_widget, &y_widget);
 
   if (coords == ATK_XY_SCREEN)
     {
@@ -470,11 +469,11 @@ gtk_text_view_accessible_get_offset_at_point (AtkText      *text,
     }
   else if (coords == ATK_XY_WINDOW)
     {
-      window = gdk_window_get_toplevel (window);
-      gdk_window_get_origin (window, &x_window, &y_window);
+      surface = gdk_surface_get_toplevel (surface);
+      gdk_surface_get_origin (surface, &x_surface, &y_surface);
 
-      x = x - x_widget + x_window;
-      y = y - y_widget + y_window;
+      x = x - x_widget + x_surface;
+      y = y - y_widget + y_surface;
     }
   else
     return -1;
@@ -512,8 +511,8 @@ gtk_text_view_accessible_get_character_extents (AtkText      *text,
   GtkTextIter iter;
   GtkWidget *widget;
   GdkRectangle rectangle;
-  GdkWindow *window;
-  gint x_widget, y_widget, x_window, y_window;
+  GdkSurface *surface;
+  gint x_widget, y_widget, x_surface, y_surface;
 
   *x = 0;
   *y = 0;
@@ -529,23 +528,23 @@ gtk_text_view_accessible_get_character_extents (AtkText      *text,
   gtk_text_buffer_get_iter_at_offset (buffer, &iter, offset);
   gtk_text_view_get_iter_location (view, &iter, &rectangle);
 
-  window = gtk_text_view_get_window (view, GTK_TEXT_WINDOW_WIDGET);
-  if (window == NULL)
+  surface = gtk_widget_get_surface (widget);
+  if (surface == NULL)
     return;
 
-  gdk_window_get_origin (window, &x_widget, &y_widget);
+  gdk_surface_get_origin (surface, &x_widget, &y_widget);
 
   *height = rectangle.height;
   *width = rectangle.width;
 
-  gtk_text_view_buffer_to_window_coords (view, GTK_TEXT_WINDOW_WIDGET,
+  gtk_text_view_buffer_to_surface_coords (view, GTK_TEXT_WINDOW_WIDGET,
     rectangle.x, rectangle.y, x, y);
   if (coords == ATK_XY_WINDOW)
     {
-      window = gdk_window_get_toplevel (window);
-      gdk_window_get_origin (window, &x_window, &y_window);
-      *x += x_widget - x_window;
-      *y += y_widget - y_window;
+      surface = gdk_surface_get_toplevel (surface);
+      gdk_surface_get_origin (surface, &x_surface, &y_surface);
+      *x += x_widget - x_surface;
+      *y += y_widget - y_surface;
     }
   else if (coords == ATK_XY_SCREEN)
     {
@@ -1129,15 +1128,15 @@ gtk_text_view_accessible_get_default_attributes (AtkText *text)
                                        text_attrs->appearance.underline);
 
   value = g_strdup_printf ("%u,%u,%u",
-                           text_attrs->appearance.bg_color.red,
-                           text_attrs->appearance.bg_color.green,
-                           text_attrs->appearance.bg_color.blue);
+                           (guint)(text_attrs->appearance.bg_rgba->red * 65535),
+                           (guint)(text_attrs->appearance.bg_rgba->green * 65535),
+                           (guint)(text_attrs->appearance.bg_rgba->blue * 65535));
   attributes = add_text_attribute (attributes, ATK_TEXT_ATTR_BG_COLOR, value);
 
   value = g_strdup_printf ("%u,%u,%u",
-                           text_attrs->appearance.fg_color.red,
-                           text_attrs->appearance.fg_color.green,
-                           text_attrs->appearance.fg_color.blue);
+                           (guint)(text_attrs->appearance.fg_rgba->red),
+                           (guint)(text_attrs->appearance.fg_rgba->green),
+                           (guint)(text_attrs->appearance.fg_rgba->blue));
   attributes = add_text_attribute (attributes, ATK_TEXT_ATTR_FG_COLOR, value);
 
   value = g_strdup_printf ("%g", text_attrs->font_scale);
@@ -1309,99 +1308,6 @@ gtk_text_view_accessible_set_selection (AtkText *text,
     return FALSE;
 }
 
-static gboolean
-gtk_text_view_accessible_scroll_substring_to(AtkText *text,
-                                             gint start_offset,
-                                             gint end_offset,
-                                             AtkScrollType type)
-{
-  GtkTextView *view;
-  GtkWidget *widget;
-  GtkTextBuffer *buffer;
-  GtkTextIter iter;
-  gdouble xalign = -1.0, yalign = -1.0;
-  gboolean use_align = TRUE;
-  gint offset, rtl = 0;
-
-  if (end_offset < start_offset)
-    return FALSE;
-
-  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
-  if (widget == NULL)
-    return FALSE;
-
-  view = GTK_TEXT_VIEW (widget);
-  buffer = gtk_text_view_get_buffer (view);
-
-  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
-    rtl = 1;
-
-  /*
-   * Opportunistically pick which offset should be used to calculate
-   * the scrolling factor.
-   *
-   * Considering only an extremity of the substring is good enough when
-   * the selected string doesn't include line break and isn't larger than
-   * the visible rectangle.
-   */
-  switch (type)
-    {
-    case ATK_SCROLL_TOP_LEFT:
-      offset = (rtl) ? end_offset : start_offset;
-      xalign = 0.0;
-      yalign = 0.0;
-      break;
-    case ATK_SCROLL_BOTTOM_RIGHT:
-      offset = (rtl) ? start_offset : end_offset;
-      xalign = 1.0;
-      yalign = 1.0;
-      break;
-    case ATK_SCROLL_TOP_EDGE:
-      offset = start_offset;
-      yalign = 0.0;
-      break;
-    case ATK_SCROLL_BOTTOM_EDGE:
-      offset = end_offset;
-      yalign = 1.0;
-      break;
-    case ATK_SCROLL_LEFT_EDGE:
-      offset = (rtl) ? end_offset : start_offset;
-      xalign = 0.0;
-      break;
-    case ATK_SCROLL_RIGHT_EDGE:
-      offset = (rtl) ? start_offset : end_offset;
-      xalign = 1.0;
-      break;
-    case ATK_SCROLL_ANYWHERE:
-      offset = start_offset;
-      use_align = FALSE;
-      xalign = yalign = 0.0;
-      break;
-    default:
-      return FALSE;
-    }
-
-  gtk_text_buffer_get_iter_at_offset (buffer, &iter, offset);
-
-  /* Get current iter location to be able to scroll in a single direction. */
-  if (use_align && (xalign == -1.0 || yalign == -1.0))
-    {
-      GdkRectangle rect, irect;
-
-      gtk_text_view_get_visible_rect (view, &rect);
-      gtk_text_view_get_iter_location (view, &iter, &irect);
-
-      if (xalign == -1.0)
-        xalign = ((gdouble) (irect.x - rect.x)) / (rect.width - 1);
-      if (yalign == -1.0)
-        yalign = ((gdouble) (irect.y - rect.y)) / (rect.height - 1);
-    }
-
-  gtk_text_view_scroll_to_iter (view, &iter, 0, use_align, xalign, yalign);
-
-  return TRUE;
-}
-
 static void
 atk_text_interface_init (AtkTextIface *iface)
 {
@@ -1422,7 +1328,6 @@ atk_text_interface_init (AtkTextIface *iface)
   iface->set_selection = gtk_text_view_accessible_set_selection;
   iface->get_run_attributes = gtk_text_view_accessible_get_run_attributes;
   iface->get_default_attributes = gtk_text_view_accessible_get_default_attributes;
-  iface->scroll_substring_to = gtk_text_view_accessible_scroll_substring_to;
 }
 
 /* atkeditabletext.h */
@@ -1440,7 +1345,7 @@ gtk_text_view_accessible_set_run_attributes (AtkEditableText *text,
   GtkTextIter start;
   GtkTextIter end;
   gint j;
-  GdkColor *color;
+  GdkRGBA *color;
   gchar** RGB_vals;
   GSList *l;
 
@@ -1549,21 +1454,21 @@ gtk_text_view_accessible_set_run_attributes (AtkEditableText *text,
       else if (!strcmp (name, atk_text_attribute_get_name (ATK_TEXT_ATTR_BG_COLOR)))
         {
           RGB_vals = g_strsplit (value, ",", 3);
-          color = g_malloc (sizeof (GdkColor));
-          color->red = atoi (RGB_vals[0]);
-          color->green = atoi (RGB_vals[1]);
-          color->blue = atoi (RGB_vals[2]);
-          g_object_set (G_OBJECT (tag), "background-gdk", color, NULL);
+          color = g_malloc (sizeof (GdkRGBA));
+          color->red = atoi (RGB_vals[0]) / 65535.0;
+          color->green = atoi (RGB_vals[1]) / 65535.0;
+          color->blue = atoi (RGB_vals[2]) / 65535.0;
+          g_object_set (G_OBJECT (tag), "background-rgba", color, NULL);
         }
  
       else if (!strcmp (name, atk_text_attribute_get_name (ATK_TEXT_ATTR_FG_COLOR)))
         {
           RGB_vals = g_strsplit (value, ",", 3);
-          color = g_malloc (sizeof (GdkColor));
-          color->red = atoi (RGB_vals[0]);
-          color->green = atoi (RGB_vals[1]);
-          color->blue = atoi (RGB_vals[2]);
-          g_object_set (G_OBJECT (tag), "foreground-gdk", color, NULL);
+          color = g_malloc (sizeof (GdkRGBA));
+          color->red = atoi (RGB_vals[0]) / 65535.0;
+          color->green = atoi (RGB_vals[1]) / 65535.0;
+          color->blue = atoi (RGB_vals[2]) / 65535.0;
+          g_object_set (G_OBJECT (tag), "foreground-rgba", color, NULL);
         }
 
       else if (!strcmp (name, atk_text_attribute_get_name (ATK_TEXT_ATTR_STRETCH)))
@@ -1700,7 +1605,7 @@ gtk_text_view_accessible_copy_text (AtkEditableText *text,
   GtkTextBuffer *buffer;
   GtkTextIter start, end;
   gchar *str;
-  GtkClipboard *clipboard;
+  GdkClipboard *clipboard;
 
   widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
@@ -1712,8 +1617,8 @@ gtk_text_view_accessible_copy_text (AtkEditableText *text,
   gtk_text_buffer_get_iter_at_offset (buffer, &end, end_pos);
   str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
 
-  clipboard = gtk_widget_get_clipboard (widget, GDK_SELECTION_CLIPBOARD);
-  gtk_clipboard_set_text (clipboard, str, -1);
+  clipboard = gtk_widget_get_clipboard (widget);
+  gdk_clipboard_set_text (clipboard, str);
 }
 
 static void
@@ -1726,7 +1631,7 @@ gtk_text_view_accessible_cut_text (AtkEditableText *text,
   GtkTextBuffer *buffer;
   GtkTextIter start, end;
   gchar *str;
-  GtkClipboard *clipboard;
+  GdkClipboard *clipboard;
 
   widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
@@ -1740,8 +1645,8 @@ gtk_text_view_accessible_cut_text (AtkEditableText *text,
   gtk_text_buffer_get_iter_at_offset (buffer, &start, start_pos);
   gtk_text_buffer_get_iter_at_offset (buffer, &end, end_pos);
   str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-  clipboard = gtk_widget_get_clipboard (widget, GDK_SELECTION_CLIPBOARD);
-  gtk_clipboard_set_text (clipboard, str, -1);
+  clipboard = gtk_widget_get_clipboard (widget);
+  gdk_clipboard_set_text (clipboard, str);
   gtk_text_buffer_delete (buffer, &start, &end);
 }
 
@@ -1777,17 +1682,20 @@ typedef struct
 } PasteData;
 
 static void
-paste_received (GtkClipboard *clipboard,
-                const gchar  *text,
+paste_received (GObject      *clipboard,
+                GAsyncResult *result,
                 gpointer      data)
 {
   PasteData* paste = data;
   GtkTextIter pos_itr;
+  char *text;
 
+  text = gdk_clipboard_read_text_finish (GDK_CLIPBOARD (clipboard), result, NULL);
   if (text)
     {
       gtk_text_buffer_get_iter_at_offset (paste->buffer, &pos_itr, paste->position);
       gtk_text_buffer_insert (paste->buffer, &pos_itr, text, -1);
+      g_free (text);
     }
 
   g_object_unref (paste->buffer);
@@ -1801,7 +1709,7 @@ gtk_text_view_accessible_paste_text (AtkEditableText *text,
   GtkWidget *widget;
   GtkTextBuffer *buffer;
   PasteData paste;
-  GtkClipboard *clipboard;
+  GdkClipboard *clipboard;
 
   widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (text));
   if (widget == NULL)
@@ -1816,8 +1724,8 @@ gtk_text_view_accessible_paste_text (AtkEditableText *text,
   paste.position = position;
 
   g_object_ref (paste.buffer);
-  clipboard = gtk_widget_get_clipboard (widget, GDK_SELECTION_CLIPBOARD);
-  gtk_clipboard_request_text (clipboard, paste_received, &paste);
+  clipboard = gtk_widget_get_clipboard (widget);
+  gdk_clipboard_read_text_async (clipboard, NULL, paste_received, &paste);
 }
 
 static void
@@ -1933,146 +1841,6 @@ mark_set_cb (GtkTextBuffer *buffer,
     }
 }
 
-static gint
-gail_streamable_content_get_n_mime_types (AtkStreamableContent *streamable)
-{
-  GtkWidget *widget;
-  GtkTextBuffer *buffer;
-  gint n_mime_types = 0;
-
-  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (streamable));
-  if (widget == NULL)
-    return 0;
-
-  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
-  if (buffer)
-    {
-      gint i;
-      gboolean advertises_plaintext = FALSE;
-      GdkAtom *atoms;
-
-      atoms = gtk_text_buffer_get_serialize_formats (buffer, &n_mime_types);
-      for (i = 0; i < n_mime_types-1; ++i)
-        if (!strcmp ("text/plain", gdk_atom_name (atoms[i])))
-            advertises_plaintext = TRUE;
-      if (!advertises_plaintext)
-        n_mime_types++;
-    }
-
-  return n_mime_types;
-}
-
-static const gchar *
-gail_streamable_content_get_mime_type (AtkStreamableContent *streamable,
-                                       gint                  i)
-{
-  GtkWidget *widget;
-  GtkTextBuffer *buffer;
-
-  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (streamable));
-  if (widget == NULL)
-    return 0;
-
-  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
-  if (buffer)
-    {
-      gint n_mime_types = 0;
-      GdkAtom *atoms;
-
-      atoms = gtk_text_buffer_get_serialize_formats (buffer, &n_mime_types);
-      if (i < n_mime_types)
-        return gdk_atom_name (atoms [i]);
-      else if (i == n_mime_types)
-        return "text/plain";
-    }
-
-  return NULL;
-}
-
-static GIOChannel *
-gail_streamable_content_get_stream (AtkStreamableContent *streamable,
-                                    const gchar          *mime_type)
-{
-  GtkWidget *widget;
-  GtkTextBuffer *buffer;
-  gint i, n_mime_types = 0;
-  GdkAtom *atoms;
-
-  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (streamable));
-  if (widget == NULL)
-    return 0;
-
-  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
-  if (!buffer)
-    return NULL;
-
-  atoms = gtk_text_buffer_get_serialize_formats (buffer, &n_mime_types);
-
-  for (i = 0; i < n_mime_types; ++i)
-    {
-      if (!strcmp ("text/plain", mime_type) ||
-          !strcmp (gdk_atom_name (atoms[i]), mime_type))
-        {
-          guint8 *cbuf;
-          GError *err = NULL;
-          gsize len, written;
-          gchar tname[80];
-          GtkTextIter start, end;
-          GIOChannel *gio = NULL;
-          int fd;
-
-          gtk_text_buffer_get_iter_at_offset (buffer, &start, 0);
-          gtk_text_buffer_get_iter_at_offset (buffer, &end, -1);
-          if (!strcmp ("text/plain", mime_type))
-            {
-              cbuf = (guint8*) gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-              len = strlen ((const char *) cbuf);
-            }
-          else
-            {
-              cbuf = gtk_text_buffer_serialize (buffer, buffer, atoms[i], &start, &end, &len);
-            }
-          g_snprintf (tname, 20, "streamXXXXXX");
-          fd = g_mkstemp (tname);
-          gio = g_io_channel_unix_new (fd);
-          g_io_channel_set_encoding (gio, NULL, &err);
-          if (!err)
-            g_io_channel_write_chars (gio, (const char *) cbuf, (gssize) len, &written, &err);
-          else
-            g_message ("%s", err->message);
-          if (!err)
-            g_io_channel_seek_position (gio, 0, G_SEEK_SET, &err);
-          else
-            g_message ("%s", err->message);
-          if (!err)
-            g_io_channel_flush (gio, &err);
-          else
-            g_message ("%s", err->message);
-          if (err)
-            {
-              g_message ("<error writing to stream [%s]>", tname);
-              g_error_free (err);
-            }
-          /* make sure the file is removed on unref of the giochannel */
-          else
-            {
-              g_unlink (tname);
-              return gio;
-            }
-        }
-    }
-
-  return NULL;
-}
-
-static void
-atk_streamable_content_interface_init (AtkStreamableContentIface *iface)
-{
-  iface->get_n_mime_types = gail_streamable_content_get_n_mime_types;
-  iface->get_mime_type = gail_streamable_content_get_mime_type;
-  iface->get_stream = gail_streamable_content_get_stream;
-}
-
 void
 _gtk_text_view_accessible_set_buffer (GtkTextView   *textview,
                                       GtkTextBuffer *old_buffer)
