@@ -19,17 +19,19 @@
 
 #include "gtkcellrenderertext.h"
 
-#include <stdlib.h>
-
 #include "gtkeditable.h"
 #include "gtkentry.h"
-#include "gtksizerequest.h"
-#include "gtkmarshalers.h"
 #include "gtkintl.h"
+#include "gtkmarshalers.h"
 #include "gtkprivate.h"
+#include "gtksizerequest.h"
+#include "gtksnapshot.h"
+#include "gtkstylecontext.h"
 #include "gtktreeprivate.h"
+
 #include "a11y/gtktextcellaccessible.h"
 
+#include <stdlib.h>
 
 /**
  * SECTION:gtkcellrenderertext
@@ -55,8 +57,8 @@ static void gtk_cell_renderer_text_set_property  (GObject                  *obje
 						  guint                     param_id,
 						  const GValue             *value,
 						  GParamSpec               *pspec);
-static void gtk_cell_renderer_text_render     (GtkCellRenderer          *cell,
-					       cairo_t                  *cr,
+static void gtk_cell_renderer_text_snapshot   (GtkCellRenderer          *cell,
+					       GtkSnapshot              *snapshot,
 					       GtkWidget                *widget,
 					       const GdkRectangle       *background_area,
 					       const GdkRectangle       *cell_area,
@@ -112,8 +114,6 @@ enum {
   /* Style args */
   PROP_BACKGROUND,
   PROP_FOREGROUND,
-  PROP_BACKGROUND_GDK,
-  PROP_FOREGROUND_GDK,
   PROP_BACKGROUND_RGBA,
   PROP_FOREGROUND_RGBA,
   PROP_FONT,
@@ -243,7 +243,7 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
   object_class->get_property = gtk_cell_renderer_text_get_property;
   object_class->set_property = gtk_cell_renderer_text_set_property;
 
-  cell_class->render = gtk_cell_renderer_text_render;
+  cell_class->snapshot = gtk_cell_renderer_text_snapshot;
   cell_class->start_editing = gtk_cell_renderer_text_start_editing;
   cell_class->get_preferred_width = gtk_cell_renderer_text_get_preferred_width;
   cell_class->get_preferred_height = gtk_cell_renderer_text_get_preferred_height;
@@ -285,28 +285,10 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                            NULL,
                            GTK_PARAM_WRITABLE);
 
-  /**
-   * GtkCellRendererText:background-gdk:
-   *
-   * Background color as a #GdkColor
-   *
-   * Deprecated: 3.4: Use #GtkCellRendererText:background-rgba instead.
-   */
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  text_cell_renderer_props[PROP_BACKGROUND_GDK] =
-      g_param_spec_boxed ("background-gdk",
-                          P_("Background color"),
-                          P_("Background color as a GdkColor"),
-                          GDK_TYPE_COLOR,
-                          GTK_PARAM_READWRITE | G_PARAM_DEPRECATED);
-G_GNUC_END_IGNORE_DEPRECATIONS
-
   /**
    * GtkCellRendererText:background-rgba:
    *
    * Background color as a #GdkRGBA
-   *
-   * Since: 3.0
    */
   text_cell_renderer_props[PROP_BACKGROUND_RGBA] =
       g_param_spec_boxed ("background-rgba",
@@ -321,28 +303,10 @@ G_GNUC_END_IGNORE_DEPRECATIONS
                            NULL,
                            GTK_PARAM_WRITABLE);
 
-  /**
-   * GtkCellRendererText:foreground-gdk:
-   *
-   * Foreground color as a #GdkColor
-   *
-   * Deprecated: 3.4: Use #GtkCellRendererText:foreground-rgba instead.
-   */
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  text_cell_renderer_props[PROP_FOREGROUND_GDK] =
-      g_param_spec_boxed ("foreground-gdk",
-                          P_("Foreground color"),
-                          P_("Foreground color as a GdkColor"),
-                          GDK_TYPE_COLOR,
-                          GTK_PARAM_READWRITE | G_PARAM_DEPRECATED);
-G_GNUC_END_IGNORE_DEPRECATIONS
-
   /**
    * GtkCellRendererText:foreground-rgba:
    *
    * Foreground color as a #GdkRGBA
-   *
-   * Since: 3.0
    */
   text_cell_renderer_props[PROP_FOREGROUND_RGBA] =
       g_param_spec_boxed ("foreground-rgba",
@@ -362,7 +326,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
   text_cell_renderer_props[PROP_FONT] =
       g_param_spec_string ("font",
                            P_("Font"),
-                           P_("Font description as a string, e.g. \"Sans Italic 12\""),
+                           P_("Font description as a string, e.g. “Sans Italic 12”"),
                            NULL,
                            GTK_PARAM_READWRITE);
 
@@ -465,7 +429,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
                            P_("Language"),
                            P_("The language this text is in, as an ISO code. "
                               "Pango can use this as a hint when rendering the text. "
-                              "If you don't understand this parameter, you probably don't need it"),
+                              "If you don’t understand this parameter, you probably don’t need it"),
                            NULL,
                            GTK_PARAM_READWRITE);
 
@@ -476,8 +440,6 @@ G_GNUC_END_IGNORE_DEPRECATIONS
    * does not have enough room to display the entire string. Setting it to
    * %PANGO_ELLIPSIZE_NONE turns off ellipsizing. See the wrap-width property
    * for another way of making the text fit into a given width.
-   *
-   * Since: 2.6
    */
   text_cell_renderer_props[PROP_ELLIPSIZE] =
       g_param_spec_enum ("ellipsize",
@@ -495,8 +457,6 @@ G_GNUC_END_IGNORE_DEPRECATIONS
    * The desired width of the cell, in characters. If this property is set to
    * -1, the width will be calculated automatically, otherwise the cell will
    * request either 3 characters or the property value, whichever is greater.
-   *
-   * Since: 2.6
    **/
   text_cell_renderer_props[PROP_WIDTH_CHARS] =
       g_param_spec_int ("width-chars",
@@ -517,8 +477,6 @@ G_GNUC_END_IGNORE_DEPRECATIONS
    * cell should not receive any greater allocation unless it is
    * set to expand in its #GtkCellLayout and all of the cell's siblings
    * have received their natural width.
-   *
-   * Since: 3.0
    **/
   text_cell_renderer_props[PROP_MAX_WIDTH_CHARS] =
       g_param_spec_int ("max-width-chars",
@@ -534,8 +492,6 @@ G_GNUC_END_IGNORE_DEPRECATIONS
    * Specifies how to break the string into multiple lines, if the cell
    * renderer does not have enough room to display the entire string.
    * This property has no effect unless the wrap-width property is set.
-   *
-   * Since: 2.8
    */
   text_cell_renderer_props[PROP_WRAP_MODE] =
       g_param_spec_enum ("wrap-mode",
@@ -553,8 +509,6 @@ G_GNUC_END_IGNORE_DEPRECATIONS
    * Specifies the minimum width at which the text is wrapped. The wrap-mode property can
    * be used to influence at what character positions the line breaks can be placed.
    * Setting wrap-width to -1 turns wrapping off.
-   *
-   * Since: 2.8
    */
   text_cell_renderer_props[PROP_WRAP_WIDTH] =
       g_param_spec_int ("wrap-width",
@@ -572,8 +526,6 @@ G_GNUC_END_IGNORE_DEPRECATIONS
    * Note that this property describes how to align the lines of text in
    * case there are several of them. The "xalign" property of #GtkCellRenderer,
    * on the other hand, sets the horizontal alignment of the whole text.
-   *
-   * Since: 2.10
    */
   text_cell_renderer_props[PROP_ALIGN] =
       g_param_spec_enum ("alignment",
@@ -588,8 +540,6 @@ G_GNUC_END_IGNORE_DEPRECATIONS
    *
    * The text that will be displayed in the #GtkCellRenderer if
    * #GtkCellRendererText:editable is %TRUE and the cell is empty.
-   *
-   * Since 3.6
    */
   text_cell_renderer_props[PROP_PLACEHOLDER_TEXT] =
       g_param_spec_string ("placeholder-text",
@@ -689,9 +639,6 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 		  G_TYPE_NONE, 2,
 		  G_TYPE_STRING,
 		  G_TYPE_STRING);
-  g_signal_set_va_marshaller (text_cell_renderer_signals [EDITED],
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              _gtk_marshal_VOID__STRING_STRINGv);
 
   gtk_cell_renderer_class_set_accessible_type (cell_class, GTK_TYPE_TEXT_CELL_ACCESSIBLE);
 }
@@ -735,9 +682,9 @@ get_property_font_set_mask (guint prop_id)
       return PANGO_FONT_MASK_STRETCH;
     case PROP_SIZE_SET:
       return PANGO_FONT_MASK_SIZE;
+    default:
+      return 0;
     }
-
-  return 0;
 }
 
 static void
@@ -763,30 +710,6 @@ gtk_cell_renderer_text_get_property (GObject        *object,
       g_value_set_boolean (value, priv->single_paragraph);
       break;
 
-    case PROP_BACKGROUND_GDK:
-      {
-        GdkColor color;
-
-        color.red = (guint16) (priv->background.red * 65535);
-        color.green = (guint16) (priv->background.green * 65535);
-        color.blue = (guint16) (priv->background.blue * 65535);
-
-        g_value_set_boxed (value, &color);
-      }
-      break;
-
-    case PROP_FOREGROUND_GDK:
-      {
-        GdkColor color;
-
-        color.red = (guint16) (priv->foreground.red * 65535);
-        color.green = (guint16) (priv->foreground.green * 65535);
-        color.blue = (guint16) (priv->foreground.blue * 65535);
-
-        g_value_set_boxed (value, &color);
-      }
-      break;
-
     case PROP_BACKGROUND_RGBA:
       g_value_set_boxed (value, &priv->background);
       break;
@@ -1191,8 +1114,6 @@ gtk_cell_renderer_text_set_property (GObject      *object,
           set_bg_color (celltext, &rgba);
         else
           g_warning ("Don't know color '%s'", g_value_get_string (value));
-
-        g_object_notify_by_pspec (object, text_cell_renderer_props[PROP_BACKGROUND_GDK]);
       }
       break;
 
@@ -1206,54 +1127,6 @@ gtk_cell_renderer_text_set_property (GObject      *object,
           set_fg_color (celltext, &rgba);
         else
           g_warning ("Don't know color '%s'", g_value_get_string (value));
-
-        g_object_notify_by_pspec (object, text_cell_renderer_props[PROP_FOREGROUND_GDK]);
-      }
-      break;
-
-    case PROP_BACKGROUND_GDK:
-      {
-        GdkColor *color;
-
-        color = g_value_get_boxed (value);
-        if (color)
-          {
-            GdkRGBA rgba;
-
-            rgba.red = color->red / 65535.;
-            rgba.green = color->green / 65535.;
-            rgba.blue = color->blue / 65535.;
-            rgba.alpha = 1;
-
-            set_bg_color (celltext, &rgba);
-          }
-        else
-          {
-            set_bg_color (celltext, NULL);
-          }
-      }
-      break;
-
-    case PROP_FOREGROUND_GDK:
-      {
-        GdkColor *color;
-
-        color = g_value_get_boxed (value);
-        if (color)
-          {
-            GdkRGBA rgba;
-
-            rgba.red = color->red / 65535.;
-            rgba.green = color->green / 65535.;
-            rgba.blue = color->blue / 65535.;
-            rgba.alpha = 1;
-
-            set_fg_color (celltext, &rgba);
-          }
-        else
-          {
-            set_fg_color (celltext, NULL);
-          }
       }
       break;
 
@@ -1333,6 +1206,8 @@ gtk_cell_renderer_text_set_property (GObject      *object,
 					     g_value_get_double (value) * PANGO_SCALE);
 	    g_object_notify_by_pspec (object, pspec);
 	    break;
+          default:
+            break;
 	  }
 
 	if (priv->fixed_height_rows != -1)
@@ -1650,6 +1525,9 @@ get_layout (GtkCellRendererText *celltext,
           uline = PANGO_UNDERLINE_DOUBLE;
           break;
 
+        case PANGO_UNDERLINE_DOUBLE:
+        case PANGO_UNDERLINE_LOW:
+        case PANGO_UNDERLINE_ERROR:
         default:
           break;
         }
@@ -1734,16 +1612,14 @@ get_size (GtkCellRenderer    *cell,
   if (priv->calc_fixed_height)
     {
       GtkStyleContext *style_context;
-      GtkStateFlags state;
       PangoContext *context;
       PangoFontMetrics *metrics;
       PangoFontDescription *font_desc;
       gint row_height;
 
       style_context = gtk_widget_get_style_context (widget);
-      state = gtk_widget_get_state_flags (widget);
 
-      gtk_style_context_get (style_context, state, "font", &font_desc, NULL);
+      gtk_style_context_get (style_context, "font", &font_desc, NULL);
       pango_font_description_merge_static (font_desc, priv->font, TRUE);
 
       if (priv->scale_set)
@@ -1825,12 +1701,12 @@ get_size (GtkCellRenderer    *cell,
 }
 
 static void
-gtk_cell_renderer_text_render (GtkCellRenderer      *cell,
-			       cairo_t              *cr,
-			       GtkWidget            *widget,
-			       const GdkRectangle   *background_area,
-			       const GdkRectangle   *cell_area,
-			       GtkCellRendererState  flags)
+gtk_cell_renderer_text_snapshot (GtkCellRenderer      *cell,
+			         GtkSnapshot          *snapshot,
+                                 GtkWidget            *widget,
+                                 const GdkRectangle   *background_area,
+                                 const GdkRectangle   *cell_area,
+                                 GtkCellRendererState  flags)
 
 {
   GtkCellRendererText *celltext = GTK_CELL_RENDERER_TEXT (cell);
@@ -1848,9 +1724,12 @@ gtk_cell_renderer_text_render (GtkCellRenderer      *cell,
 
   if (priv->background_set && (flags & GTK_CELL_RENDERER_SELECTED) == 0)
     {
-      gdk_cairo_rectangle (cr, background_area);
-      gdk_cairo_set_source_rgba (cr, &priv->background);
-      cairo_fill (cr);
+      gtk_snapshot_append_color (snapshot,
+                                 &priv->background,
+                                 &GRAPHENE_RECT_INIT(
+                                     background_area->x, background_area->y,
+                                     background_area->width, background_area->height
+                                 ));
     }
 
   gtk_cell_renderer_get_padding (cell, &xpad, &ypad);
@@ -1864,17 +1743,18 @@ gtk_cell_renderer_text_render (GtkCellRenderer      *cell,
   pango_layout_get_pixel_extents (layout, NULL, &rect);
   x_offset = x_offset - rect.x;
 
-  cairo_save (cr);
+  gtk_snapshot_push_clip (snapshot,
+                          &GRAPHENE_RECT_INIT(
+                              cell_area->x, cell_area->y,
+                              cell_area->width, cell_area->height
+                          ));
 
-  gdk_cairo_rectangle (cr, cell_area);
-  cairo_clip (cr);
+  gtk_snapshot_render_layout (snapshot, context,
+                              cell_area->x + x_offset + xpad,
+                              cell_area->y + y_offset + ypad,
+                              layout);
 
-  gtk_render_layout (context, cr,
-                     cell_area->x + x_offset + xpad,
-                     cell_area->y + y_offset + ypad,
-                     layout);
-
-  cairo_restore (cr);
+  gtk_snapshot_pop (snapshot);
 
   g_object_unref (layout);
 }
@@ -1919,7 +1799,7 @@ gtk_cell_renderer_text_editing_done (GtkCellEditable *entry,
     return;
 
   path = g_object_get_data (G_OBJECT (entry), GTK_CELL_RENDERER_TEXT_PATH);
-  new_text = gtk_entry_get_text (GTK_ENTRY (entry));
+  new_text = gtk_editable_get_text (GTK_EDITABLE (entry));
 
   g_signal_emit (data, text_cell_renderer_signals[EDITED], 0, path, new_text);
 }
@@ -1952,9 +1832,8 @@ gtk_cell_renderer_text_popup_unmap (GtkMenu *menu,
   if (priv->entry_menu_popdown_timeout)
     return;
 
-  priv->entry_menu_popdown_timeout = gdk_threads_add_timeout (500, popdown_timeout,
-                                                    data);
-  g_source_set_name_by_id (priv->entry_menu_popdown_timeout, "[gtk+] popdown_timeout");
+  priv->entry_menu_popdown_timeout = g_timeout_add (500, popdown_timeout, data);
+  g_source_set_name_by_id (priv->entry_menu_popdown_timeout, "[gtk] popdown_timeout");
 }
 
 static void
@@ -1978,26 +1857,26 @@ gtk_cell_renderer_text_populate_popup (GtkEntry *entry,
                     G_CALLBACK (gtk_cell_renderer_text_popup_unmap), data);
 }
 
-static gboolean
-gtk_cell_renderer_text_focus_out_event (GtkWidget *entry,
-		                        GdkEvent  *event,
-					gpointer   data)
+static void
+gtk_cell_renderer_text_focus_changed (GtkWidget  *entry,
+                                      GParamSpec *pspec,
+                                      gpointer    data)
 {
   GtkCellRendererTextPrivate *priv;
 
   priv = GTK_CELL_RENDERER_TEXT (data)->priv;
 
+  if (gtk_widget_has_focus (entry))
+    return;
+
   if (priv->in_entry_menu)
-    return FALSE;
+    return;
 
   g_object_set (entry,
                 "editing-canceled", TRUE,
                 NULL);
   gtk_cell_editable_editing_done (GTK_CELL_EDITABLE (entry));
   gtk_cell_editable_remove_widget (GTK_CELL_EDITABLE (entry));
-
-  /* entry needs focus-out-event */
-  return FALSE;
 }
 
 static GtkCellEditable *
@@ -2027,10 +1906,10 @@ gtk_cell_renderer_text_start_editing (GtkCellRenderer      *cell,
 
   gtk_entry_set_has_frame (GTK_ENTRY (priv->entry), FALSE);
   gtk_entry_set_alignment (GTK_ENTRY (priv->entry), xalign);
-  gtk_entry_set_width_chars (GTK_ENTRY (priv->entry), 5);
+  gtk_editable_set_width_chars (GTK_EDITABLE (priv->entry), 5);
 
   if (priv->text)
-    gtk_entry_set_text (GTK_ENTRY (priv->entry), priv->text);
+    gtk_editable_set_text (GTK_EDITABLE (priv->entry), priv->text);
   g_object_set_data_full (G_OBJECT (priv->entry), I_(GTK_CELL_RENDERER_TEXT_PATH), g_strdup (path), g_free);
   
   gtk_editable_select_region (GTK_EDITABLE (priv->entry), 0, -1);
@@ -2046,8 +1925,8 @@ gtk_cell_renderer_text_start_editing (GtkCellRenderer      *cell,
 		    "editing-done",
 		    G_CALLBACK (gtk_cell_renderer_text_editing_done),
 		    celltext);
-  priv->focus_out_id = g_signal_connect_after (priv->entry, "focus-out-event",
-					       G_CALLBACK (gtk_cell_renderer_text_focus_out_event),
+  priv->focus_out_id = g_signal_connect_after (priv->entry, "notify::has-focus",
+					       G_CALLBACK (gtk_cell_renderer_text_focus_changed),
 					       celltext);
   priv->populate_popup_id =
     g_signal_connect (priv->entry, "populate-popup",
