@@ -29,6 +29,7 @@
 #include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtksizegroup.h"
+#include "gtkstylecontext.h"
 
 #include <math.h>
 
@@ -58,8 +59,6 @@
  * # CSS names
  *
  * GtkColorChooserWidget has a single CSS node with name colorchooser.
- *
- * Since: 3.4
  */
 
 struct _GtkColorChooserWidgetPrivate
@@ -414,8 +413,6 @@ add_palette (GtkColorChooserWidget  *cc,
           gtk_grid_attach (GTK_GRID (grid), p, line, pos, 1, 1);
        }
     }
-
-  gtk_widget_show_all (grid);
 }
 
 static void
@@ -431,72 +428,81 @@ remove_default_palette (GtkColorChooserWidget *cc)
 static void
 add_default_palette (GtkColorChooserWidget *cc)
 {
-  const gchar *default_colors[9][5] = {
-    { "#99c1f1", "#62a0ea", "#3584e4", "#1c71d8", "#1a5fb4" }, /* Blue */
-    { "#8ff0a4", "#57e389", "#33d17a", "#2ec27e", "#26a269" }, /* Green */
-    { "#f9f06b", "#f8e45c", "#f6d32d", "#f5c211", "#e5a50a" }, /* Yellow */
-    { "#ffbe6f", "#ffa348", "#ff7800", "#e66100", "#c64600" }, /* Orange */
-    { "#f66151", "#ed333b", "#e01b24", "#c01c28", "#a51d2d" }, /* Red */
-    { "#dc8add", "#c061cb", "#9141ac", "#813d9c", "#613583" }, /* Purple */
-    { "#cdab8f", "#b5835a", "#986a44", "#865e3c", "#63452c" }, /* Brown */
-    { "#ffffff", "#f6f5f4", "#deddda", "#c0bfbc", "#9a9996" }, /* Light */
-    { "#77767b", "#5e5c64", "#3d3846", "#241f31", "#000000" } /* Dark */
+  const gchar *default_colors[9][3] = {
+    { "#ef2929", "#cc0000", "#a40000" }, /* Scarlet Red */
+    { "#fcaf3e", "#f57900", "#ce5c00" }, /* Orange */
+    { "#fce94f", "#edd400", "#c4a000" }, /* Butter */
+    { "#8ae234", "#73d216", "#4e9a06" }, /* Chameleon */
+    { "#729fcf", "#3465a4", "#204a87" }, /* Sky Blue */
+    { "#ad7fa8", "#75507b", "#5c3566" }, /* Plum */
+    { "#e9b96e", "#c17d11", "#8f5902" }, /* Chocolate */
+    { "#888a85", "#555753", "#2e3436" }, /* Aluminum 1 */
+    { "#eeeeec", "#d3d7cf", "#babdb6" }  /* Aluminum 2 */
   };
   const gchar *color_names[] = {
-    NC_("Color name", "Very Light Blue"),
-    NC_("Color name", "Light Blue"),
-    NC_("Color name", "Blue"),
-    NC_("Color name", "Dark Blue"),
-    NC_("Color name", "Very Dark Blue"),
-    NC_("Color name", "Very Light Green"),
-    NC_("Color name", "Light Green"),
-    NC_("Color name", "Green"),
-    NC_("Color name", "Dark Green"),
-    NC_("Color name", "Very Dark Green"),
-    NC_("Color name", "Very Light Yellow"),
-    NC_("Color name", "Light Yellow"),
-    NC_("Color name", "Yellow"),
-    NC_("Color name", "Dark Yellow"),
-    NC_("Color name", "Very Dark Yellow"),
-    NC_("Color name", "Very Light Orange"),
+    NC_("Color name", "Light Scarlet Red"),
+    NC_("Color name", "Scarlet Red"),
+    NC_("Color name", "Dark Scarlet Red"),
     NC_("Color name", "Light Orange"),
     NC_("Color name", "Orange"),
     NC_("Color name", "Dark Orange"),
-    NC_("Color name", "Very Dark Orange"),
-    NC_("Color name", "Very Light Red"),
-    NC_("Color name", "Light Red"),
-    NC_("Color name", "Red"),
-    NC_("Color name", "Dark Red"),
-    NC_("Color name", "Very Dark Red"),
-    NC_("Color name", "Very Light Purple"),
-    NC_("Color name", "Light Purple"),
-    NC_("Color name", "Purple"),
-    NC_("Color name", "Dark Purple"),
-    NC_("Color name", "Very Dark Purple"),
-    NC_("Color name", "Very Light Brown"),
-    NC_("Color name", "Light Brown"),
-    NC_("Color name", "Brown"),
-    NC_("Color name", "Dark Brown"),
-    NC_("Color name", "Very Dark Brown"),
-    NC_("Color name", "White"),
-    NC_("Color name", "Light Gray 1"),
-    NC_("Color name", "Light Gray 2"),
-    NC_("Color name", "Light Gray 3"),
-    NC_("Color name", "Light Gray 4"),
-    NC_("Color name", "Dark Gray 1"),
-    NC_("Color name", "Dark Gray 2"),
-    NC_("Color name", "Dark Gray 3"),
-    NC_("Color name", "Dark Gray 4"),
-    NC_("Color name", "Black"),
+    NC_("Color name", "Light Butter"),
+    NC_("Color name", "Butter"),
+    NC_("Color name", "Dark Butter"),
+    NC_("Color name", "Light Chameleon"),
+    NC_("Color name", "Chameleon"),
+    NC_("Color name", "Dark Chameleon"),
+    NC_("Color name", "Light Sky Blue"),
+    NC_("Color name", "Sky Blue"),
+    NC_("Color name", "Dark Sky Blue"),
+    NC_("Color name", "Light Plum"),
+    NC_("Color name", "Plum"),
+    NC_("Color name", "Dark Plum"),
+    NC_("Color name", "Light Chocolate"),
+    NC_("Color name", "Chocolate"),
+    NC_("Color name", "Dark Chocolate"),
+    NC_("Color name", "Light Aluminum 1"),
+    NC_("Color name", "Aluminum 1"),
+    NC_("Color name", "Dark Aluminum 1"),
+    NC_("Color name", "Light Aluminum 2"),
+    NC_("Color name", "Aluminum 2"),
+    NC_("Color name", "Dark Aluminum 2")
   };
-  GdkRGBA colors[9*5];
+  const gchar *default_grays[9] = {
+    "#000000", /* black */
+    "#2e3436", /* very dark gray */
+    "#555753", /* darker gray */
+    "#888a85", /* dark gray */
+    "#babdb6", /* medium gray */
+    "#d3d7cf", /* light gray */
+    "#eeeeec", /* lighter gray */
+    "#f3f3f3", /* very light gray */
+    "#ffffff"  /* white */
+  };
+  const gchar *gray_names[] = {
+    NC_("Color name", "Black"),
+    NC_("Color name", "Very Dark Gray"),
+    NC_("Color name", "Darker Gray"),
+    NC_("Color name", "Dark Gray"),
+    NC_("Color name", "Medium Gray"),
+    NC_("Color name", "Light Gray"),
+    NC_("Color name", "Lighter Gray"),
+    NC_("Color name", "Very Light Gray"),
+    NC_("Color name", "White")
+  };
+  GdkRGBA colors[9*3];
   gint i, j;
 
   for (i = 0; i < 9; i++)
-    for (j = 0; j < 5; j++)
-      gdk_rgba_parse (&colors[i*5 + j], default_colors[i][j]);
+    for (j = 0; j < 3; j++)
+      gdk_rgba_parse (&colors[i*3 + j], default_colors[i][j]);
 
-  add_palette (cc, GTK_ORIENTATION_VERTICAL, 5, 9*5, colors, color_names);
+  add_palette (cc, GTK_ORIENTATION_VERTICAL, 3, 9*3, colors, color_names);
+
+  for (i = 0; i < 9; i++)
+    gdk_rgba_parse (&colors[i], default_grays[i]);
+
+  add_palette (cc, GTK_ORIENTATION_HORIZONTAL, 9, 9, colors, gray_names);
 
   cc->priv->has_default_palette = TRUE;
 }
@@ -526,14 +532,14 @@ gtk_color_chooser_widget_init (GtkColorChooserWidget *cc)
 
   add_default_palette (cc);
 
-  cc->priv->custom = box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
-  g_object_set (box, "margin-top", 12, NULL);
-  gtk_box_pack_end (GTK_BOX (cc->priv->palette), box, FALSE, TRUE, 0);
-
   /* translators: label for the custom section in the color chooser */
   cc->priv->custom_label = label = gtk_label_new (_("Custom"));
   gtk_widget_set_halign (label, GTK_ALIGN_START);
-  gtk_box_pack_end (GTK_BOX (cc->priv->palette), label, FALSE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (cc->priv->palette), label);
+
+  cc->priv->custom = box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
+  g_object_set (box, "margin-top", 12, NULL);
+  gtk_container_add (GTK_CONTAINER (cc->priv->palette), box);
 
   cc->priv->button = button = gtk_color_swatch_new ();
   gtk_widget_set_name (button, "add-color-button");
@@ -545,8 +551,8 @@ gtk_color_chooser_widget_init (GtkColorChooserWidget *cc)
   gtk_color_swatch_set_selectable (GTK_COLOR_SWATCH (button), FALSE);
   gtk_container_add (GTK_CONTAINER (box), button);
 
-  cc->priv->settings = g_settings_new ("org.gtk.Settings.ColorChooser");
-  variant = g_settings_get_value (cc->priv->settings, "custom-colors");
+  cc->priv->settings = g_settings_new ("org.gtk.gtk4.Settings.ColorChooser");
+  variant = g_settings_get_value (cc->priv->settings, I_("custom-colors"));
   g_variant_iter_init (&iter, variant);
   i = 0;
   p = NULL;
@@ -565,7 +571,7 @@ gtk_color_chooser_widget_init (GtkColorChooserWidget *cc)
       connect_custom_signals (p, cc);
       gtk_container_add (GTK_CONTAINER (box), p);
 
-      if (i == 9)
+      if (i == 8)
         break;
     }
   g_variant_unref (variant);
@@ -580,18 +586,13 @@ gtk_color_chooser_widget_init (GtkColorChooserWidget *cc)
   gtk_container_add (GTK_CONTAINER (cc), box);
   gtk_container_add (GTK_CONTAINER (box), cc->priv->editor);
 
-  g_settings_get (cc->priv->settings, "selected-color", "(bdddd)",
+  g_settings_get (cc->priv->settings, I_("selected-color"), "(bdddd)",
                   &selected,
                   &color.red, &color.green, &color.blue, &color.alpha);
   if (selected)
     gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (cc), &color);
 
-  gtk_widget_show_all (GTK_WIDGET (cc));
   gtk_widget_hide (GTK_WIDGET (cc->priv->editor));
-  gtk_widget_hide (GTK_WIDGET (cc));
-
-  gtk_widget_set_no_show_all (cc->priv->palette, TRUE);
-  gtk_widget_set_no_show_all (cc->priv->editor, TRUE);
 
   cc->priv->size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
   gtk_size_group_add_widget (cc->priv->size_group, cc->priv->palette);
@@ -688,14 +689,12 @@ gtk_color_chooser_widget_class_init (GtkColorChooserWidgetClass *class)
    * The ::show-editor property is %TRUE when the color chooser
    * is showing the single-color editor. It can be set to switch
    * the color chooser into single-color editing mode.
-   *
-   * Since: 3.4
    */
   g_object_class_install_property (object_class, PROP_SHOW_EDITOR,
       g_param_spec_boolean ("show-editor", P_("Show editor"), P_("Show editor"),
                             FALSE, GTK_PARAM_READWRITE));
 
-  gtk_widget_class_set_css_name (GTK_WIDGET_CLASS (class), "colorchooser");
+  gtk_widget_class_set_css_name (GTK_WIDGET_CLASS (class), I_("colorchooser"));
 }
 
 /* GtkColorChooser implementation {{{1 */
@@ -747,8 +746,7 @@ add_custom_color (GtkColorChooserWidget *cc,
   gtk_color_swatch_set_can_drop (GTK_COLOR_SWATCH (p), TRUE);
   connect_custom_signals (p, cc);
 
-  gtk_container_add (GTK_CONTAINER (cc->priv->custom), p);
-  gtk_box_reorder_child (GTK_BOX (cc->priv->custom), p, 1);
+  gtk_box_insert_child_after (GTK_BOX (cc->priv->custom), p, gtk_widget_get_first_child (cc->priv->custom));
   gtk_widget_show (p);
 
   select_swatch (cc, GTK_COLOR_SWATCH (p));
@@ -824,8 +822,6 @@ gtk_color_chooser_widget_iface_init (GtkColorChooserInterface *iface)
  * Creates a new #GtkColorChooserWidget.
  *
  * Returns: a new #GtkColorChooserWidget
- *
- * Since: 3.4
  */
 GtkWidget *
 gtk_color_chooser_widget_new (void)
