@@ -33,9 +33,10 @@
 #include "gtkoverlay.h"
 #include "gtkadjustment.h"
 #include "gtklabel.h"
-#include "gtkrender.h"
 #include "gtkspinbutton.h"
 #include "gtkstylecontext.h"
+#include "gtkeventcontrollerkey.h"
+#include "gtkroot.h"
 
 #include <math.h>
 
@@ -105,7 +106,7 @@ entry_set_rgba (GtkColorEditor *editor,
                           scale_round (color->red, 255),
                           scale_round (color->green, 255),
                           scale_round (color->blue, 255));
-  gtk_entry_set_text (GTK_ENTRY (editor->priv->entry), text);
+  gtk_editable_set_text (GTK_EDITABLE (editor->priv->entry), text);
   editor->priv->text_changed = FALSE;
   g_free (text);
 }
@@ -132,13 +133,13 @@ entry_apply (GtkWidget      *entry,
   g_free (text);
 }
 
-static gboolean
-entry_focus_out (GtkWidget      *entry,
-                 GdkEventFocus  *event,
-                 GtkColorEditor *editor)
+static void
+entry_focus_changed (GtkWidget      *entry,
+                     GParamSpec     *pspec,
+                     GtkColorEditor *editor)
 {
-  entry_apply (entry, editor);
-  return FALSE;
+  if (!gtk_widget_has_focus (entry))
+    entry_apply (entry, editor);
 }
 
 static void
@@ -224,7 +225,7 @@ popup_edit (GtkWidget      *widget,
     {
       dismiss_current_popup (editor);
       toplevel = gtk_widget_get_toplevel (GTK_WIDGET (editor));
-      g_set_object (&editor->priv->popdown_focus, gtk_window_get_focus (GTK_WINDOW (toplevel)));
+      g_set_object (&editor->priv->popdown_focus, gtk_root_get_focus (GTK_ROOT (toplevel)));
       editor->priv->current_popup = popup;
       editor->priv->popup_position = position;
       gtk_widget_show (popup);
@@ -233,11 +234,13 @@ popup_edit (GtkWidget      *widget,
 }
 
 static gboolean
-popup_key_press (GtkWidget      *popup,
-                 GdkEventKey    *event,
-                 GtkColorEditor *editor)
+popup_key_pressed (GtkEventController *controller,
+                   guint               keyval,
+                   guint               keycode,
+                   GdkModifierType     state,
+                   GtkColorEditor     *editor)
 {
-  if (event->keyval == GDK_KEY_Escape)
+  if (keyval == GDK_KEY_Escape)
     {
       dismiss_current_popup (editor);
       return TRUE;
@@ -272,7 +275,7 @@ get_child_position (GtkOverlay     *overlay,
       if (gtk_widget_get_direction (GTK_WIDGET (overlay)) == GTK_TEXT_DIR_RTL)
         allocation->x = 0;
       else
-        allocation->x = gtk_widget_get_allocated_width (GTK_WIDGET (overlay)) - req.width;
+        allocation->x = gtk_widget_get_width (GTK_WIDGET (overlay)) - req.width;
     }
   else if (widget == editor->priv->h_popup)
     {
@@ -303,8 +306,8 @@ get_child_position (GtkOverlay     *overlay,
   else
     return FALSE;
 
-  allocation->x = CLAMP (allocation->x, 0, gtk_widget_get_allocated_width (GTK_WIDGET (overlay)) - req.width);
-  allocation->y = CLAMP (allocation->y, 0, gtk_widget_get_allocated_height (GTK_WIDGET (overlay)) - req.height);
+  allocation->x = CLAMP (allocation->x, 0, gtk_widget_get_width (GTK_WIDGET (overlay)) - req.width);
+  allocation->y = CLAMP (allocation->y, 0, gtk_widget_get_height (GTK_WIDGET (overlay)) - req.height);
 
   return TRUE;
 }
@@ -340,24 +343,6 @@ scaled_adjustment (GtkAdjustment *a,
   return as;
 }
 
-static gboolean
-popup_draw (GtkWidget      *popup,
-            cairo_t        *cr,
-            GtkColorEditor *editor)
-{
-  GtkStyleContext *context;
-  gint width, height;
-
-  context = gtk_widget_get_style_context (popup);
-  width = gtk_widget_get_allocated_width (popup);
-  height = gtk_widget_get_allocated_height (popup);
-
-  gtk_render_background (context, cr, 0, 0, width, height);
-  gtk_render_frame (context, cr, 0, 0, width, height);
-
-  return FALSE;
-}
-
 static void
 color_picked (GObject      *source,
               GAsyncResult *res,
@@ -390,6 +375,8 @@ pick_color (GtkButton      *button,
 static void
 gtk_color_editor_init (GtkColorEditor *editor)
 {
+  GtkEventController *controller;
+
   editor->priv = gtk_color_editor_get_instance_private (editor);
   editor->priv->use_alpha = TRUE;
 
@@ -398,14 +385,6 @@ gtk_color_editor_init (GtkColorEditor *editor)
   g_type_ensure (GTK_TYPE_COLOR_SWATCH);
   gtk_widget_init_template (GTK_WIDGET (editor));
 
-  /* Some post processing is needed in code to set this up */
-  gtk_widget_set_events (editor->priv->swatch,
-			 gtk_widget_get_events (editor->priv->swatch)
-                                 & ~(GDK_BUTTON_PRESS_MASK
-                                     | GDK_BUTTON_RELEASE_MASK
-                                     | GDK_KEY_PRESS_MASK
-                                     | GDK_KEY_RELEASE_MASK));
-
   if (gtk_widget_get_direction (editor->priv->h_slider) == GTK_TEXT_DIR_RTL)
     gtk_style_context_add_class (gtk_widget_get_style_context (editor->priv->h_slider),
                                  "marks-before");
@@ -427,6 +406,19 @@ gtk_color_editor_init (GtkColorEditor *editor)
   gtk_overlay_add_overlay (GTK_OVERLAY (editor->priv->overlay), editor->priv->h_popup);
   gtk_overlay_add_overlay (GTK_OVERLAY (editor->priv->overlay), editor->priv->a_popup);
 
+  controller = gtk_event_controller_key_new ();
+  g_signal_connect (controller, "key-pressed", G_CALLBACK (popup_key_pressed), editor);
+  gtk_widget_add_controller (editor->priv->h_entry, controller);
+  controller = gtk_event_controller_key_new ();
+  g_signal_connect (controller, "key-pressed", G_CALLBACK (popup_key_pressed), editor);
+  gtk_widget_add_controller (editor->priv->s_entry, controller);
+  controller = gtk_event_controller_key_new ();
+  g_signal_connect (controller, "key-pressed", G_CALLBACK (popup_key_pressed), editor);
+  gtk_widget_add_controller (editor->priv->v_entry, controller);
+  controller = gtk_event_controller_key_new ();
+  g_signal_connect (controller, "key-pressed", G_CALLBACK (popup_key_pressed), editor);
+  gtk_widget_add_controller (editor->priv->a_entry, controller);
+
   gtk_style_context_remove_class (gtk_widget_get_style_context (editor->priv->swatch), "activatable");
 
   editor->priv->picker = gtk_color_picker_new ();
@@ -546,13 +538,11 @@ gtk_color_editor_class_init (GtkColorEditorClass *class)
   gtk_widget_class_bind_template_child_private (widget_class, GtkColorEditor, picker_button);
 
   gtk_widget_class_bind_template_callback (widget_class, hsv_changed);
-  gtk_widget_class_bind_template_callback (widget_class, popup_draw);
-  gtk_widget_class_bind_template_callback (widget_class, popup_key_press);
   gtk_widget_class_bind_template_callback (widget_class, dismiss_current_popup);
   gtk_widget_class_bind_template_callback (widget_class, get_child_position);
   gtk_widget_class_bind_template_callback (widget_class, entry_text_changed);
   gtk_widget_class_bind_template_callback (widget_class, entry_apply);
-  gtk_widget_class_bind_template_callback (widget_class, entry_focus_out);
+  gtk_widget_class_bind_template_callback (widget_class, entry_focus_changed);
   gtk_widget_class_bind_template_callback (widget_class, popup_edit);
   gtk_widget_class_bind_template_callback (widget_class, pick_color);
 }
