@@ -23,11 +23,11 @@
 
 #include "gtkcssanimationprivate.h"
 #include "gtkcssarrayvalueprivate.h"
+#include "gtkcssdynamicprivate.h"
 #include "gtkcssenumvalueprivate.h"
 #include "gtkcssinheritvalueprivate.h"
 #include "gtkcssinitialvalueprivate.h"
 #include "gtkcssnumbervalueprivate.h"
-#include "gtkcsssectionprivate.h"
 #include "gtkcssshorthandpropertyprivate.h"
 #include "gtkcssstaticstyleprivate.h"
 #include "gtkcssstringvalueprivate.h"
@@ -44,7 +44,8 @@ static GtkCssValue *
 gtk_css_animated_style_get_value (GtkCssStyle *style,
                                   guint        id)
 {
-  GtkCssAnimatedStyle *animated = GTK_CSS_ANIMATED_STYLE (style);
+  /* This is called a lot, so we avoid a dynamic type check here */
+  GtkCssAnimatedStyle *animated = (GtkCssAnimatedStyle *) style;
 
   if (animated->animated_values &&
       id < animated->animated_values->len &&
@@ -147,11 +148,30 @@ GtkCssValue *
 gtk_css_animated_style_get_intrinsic_value (GtkCssAnimatedStyle *style,
                                             guint                id)
 {
-  gtk_internal_return_val_if_fail (GTK_IS_CSS_ANIMATED_STYLE (style), NULL);
-
   return gtk_css_style_get_value (style->style, id);
 }
 
+static GSList *
+gtk_css_animated_style_create_dynamic (GSList      *animations,
+                                       GtkCssStyle *style,
+                                       gint64       timestamp)
+{
+  guint i;
+
+  /* XXX: Check animations if they have dynamic values */
+
+  for (i = 0; i < GTK_CSS_PROPERTY_N_PROPERTIES; i++)
+    {
+      if (gtk_css_value_is_dynamic (gtk_css_style_get_value (style, i)))
+        {
+          animations = g_slist_append (animations, gtk_css_dynamic_new (timestamp));
+          break;
+        }
+    }
+
+  return animations;
+}
+
 /* TRANSITIONS */
 
 typedef struct _TransitionInfo TransitionInfo;
@@ -165,33 +185,23 @@ transition_info_add (TransitionInfo    infos[GTK_CSS_PROPERTY_N_PROPERTIES],
                      GtkStyleProperty *property,
                      guint             index)
 {
-  if (property == NULL)
+  if (GTK_IS_CSS_SHORTHAND_PROPERTY (property))
     {
+      GtkCssShorthandProperty *shorthand = (GtkCssShorthandProperty *) property;
+      guint len = _gtk_css_shorthand_property_get_n_subproperties (shorthand);
       guint i;
 
-      for (i = 0; i < _gtk_css_style_property_get_n_properties (); i++)
-        {
-          GtkCssStyleProperty *prop = _gtk_css_style_property_lookup_by_id (i);
-
-          transition_info_add (infos, GTK_STYLE_PROPERTY (prop), index);
-        }
-    }
-  else if (GTK_IS_CSS_SHORTHAND_PROPERTY (property))
-    {
-      GtkCssShorthandProperty *shorthand = GTK_CSS_SHORTHAND_PROPERTY (property);
-      guint i;
-
-      for (i = 0; i < _gtk_css_shorthand_property_get_n_subproperties (shorthand); i++)
+      for (i = 0; i < len; i++)
         {
           GtkCssStyleProperty *prop = _gtk_css_shorthand_property_get_subproperty (shorthand, i);
 
           transition_info_add (infos, GTK_STYLE_PROPERTY (prop), index);
         }
     }
-  else if (GTK_IS_CSS_STYLE_PROPERTY (property))
+  else
     {
       guint id;
-      
+
       if (!_gtk_css_style_property_is_animated (GTK_CSS_STYLE_PROPERTY (property)))
         return;
 
@@ -200,10 +210,6 @@ transition_info_add (TransitionInfo    infos[GTK_CSS_PROPERTY_N_PROPERTIES],
       infos[id].index = index;
       infos[id].pending = TRUE;
     }
-  else
-    {
-      g_assert_not_reached ();
-    }
 }
 
 static void
@@ -219,15 +225,22 @@ transition_infos_set (TransitionInfo  infos[GTK_CSS_PROPERTY_N_PROPERTIES],
 
       prop_value = _gtk_css_array_value_get_nth (transitions, i);
       if (g_ascii_strcasecmp (_gtk_css_ident_value_get (prop_value), "all") == 0)
-        property = NULL;
+        {
+          const guint len = _gtk_css_style_property_get_n_properties ();
+          guint j;
+
+          for (j = 0; j < len; j++)
+            {
+              property = GTK_STYLE_PROPERTY (_gtk_css_style_property_lookup_by_id (j));
+              transition_info_add (infos, property, i);
+            }
+        }
       else
         {
           property = _gtk_style_property_lookup (_gtk_css_ident_value_get (prop_value));
-          if (property == NULL)
-            continue;
+          if (property)
+            transition_info_add (infos, property, i);
         }
-      
-      transition_info_add (infos, property, i);
     }
 }
 
@@ -332,12 +345,12 @@ gtk_css_animated_style_find_animation (GSList     *animations,
 }
 
 static GSList *
-gtk_css_animated_style_create_css_animations (GSList                  *animations,
-                                              GtkCssStyle             *base_style,
-                                              GtkCssStyle             *parent_style,
-                                              gint64                   timestamp,
-                                              GtkStyleProviderPrivate *provider,
-                                              GtkCssStyle             *source)
+gtk_css_animated_style_create_css_animations (GSList           *animations,
+                                              GtkCssStyle      *base_style,
+                                              GtkCssStyle      *parent_style,
+                                              gint64            timestamp,
+                                              GtkStyleProvider *provider,
+                                              GtkCssStyle      *source)
 {
   GtkCssValue *durations, *delays, *timing_functions, *animation_names;
   GtkCssValue *iteration_counts, *directions, *play_states, *fill_modes;
@@ -377,7 +390,7 @@ gtk_css_animated_style_create_css_animations (GSList                  *animation
         }
       else
         {
-          keyframes = _gtk_style_provider_private_get_keyframes (provider, name);
+          keyframes = gtk_style_provider_get_keyframes (provider, name);
           if (keyframes == NULL)
             continue;
 
@@ -418,11 +431,11 @@ gtk_css_animated_style_apply_animations (GtkCssAnimatedStyle *style)
 }
 
 GtkCssStyle *
-gtk_css_animated_style_new (GtkCssStyle             *base_style,
-                            GtkCssStyle             *parent_style,
-                            gint64                   timestamp,
-                            GtkStyleProviderPrivate *provider,
-                            GtkCssStyle             *previous_style)
+gtk_css_animated_style_new (GtkCssStyle      *base_style,
+                            GtkCssStyle      *parent_style,
+                            gint64            timestamp,
+                            GtkStyleProvider *provider,
+                            GtkCssStyle      *previous_style)
 {
   GtkCssAnimatedStyle *result;
   GSList *animations;
@@ -440,6 +453,7 @@ gtk_css_animated_style_new (GtkCssStyle             *base_style,
   if (previous_style != NULL)
     animations = gtk_css_animated_style_create_css_transitions (animations, base_style, timestamp, previous_style);
   animations = gtk_css_animated_style_create_css_animations (animations, base_style, parent_style, timestamp, provider, previous_style);
+  animations = gtk_css_animated_style_create_dynamic (animations, base_style, timestamp);
 
   if (animations == NULL)
     return g_object_ref (base_style);
@@ -466,12 +480,9 @@ gtk_css_animated_style_new_advance (GtkCssAnimatedStyle *source,
   gtk_internal_return_val_if_fail (GTK_IS_CSS_ANIMATED_STYLE (source), NULL);
   gtk_internal_return_val_if_fail (GTK_IS_CSS_STYLE (base), NULL);
   
-  if (timestamp == 0)
+  if (timestamp == 0 || timestamp == source->current_time)
     return g_object_ref (source->style);
 
-  if (timestamp == source->current_time)
-    return g_object_ref (GTK_CSS_STYLE (source));
-
   gtk_internal_return_val_if_fail (timestamp > source->current_time, NULL);
 
   animations = NULL;
