@@ -22,7 +22,9 @@
 #include <string.h>
 
 #include "gtkcssimageurlprivate.h"
-#include "gtkcssimagesurfaceprivate.h"
+
+#include "gtkcssimageinvalidprivate.h"
+#include "gtkcssimagepaintableprivate.h"
 #include "gtkstyleproviderprivate.h"
 
 G_DEFINE_TYPE (GtkCssImageUrl, _gtk_css_image_url, GTK_TYPE_CSS_IMAGE)
@@ -31,9 +33,8 @@ static GtkCssImage *
 gtk_css_image_url_load_image (GtkCssImageUrl  *url,
                               GError         **error)
 {
-  GdkPixbuf *pixbuf;
+  GdkTexture *texture;
   GError *local_error = NULL;
-  GFileInputStream *input;
 
   if (url->loaded_image)
     return url->loaded_image;
@@ -46,49 +47,39 @@ gtk_css_image_url_load_image (GtkCssImageUrl  *url,
       char *uri = g_file_get_uri (url->file);
       char *resource_path = g_uri_unescape_string (uri + strlen ("resource://"), NULL);
 
-      pixbuf = gdk_pixbuf_new_from_resource (resource_path, &local_error);
+      texture = gdk_texture_new_from_resource (resource_path);
+
       g_free (resource_path);
       g_free (uri);
     }
   else
     {
-      input = g_file_read (url->file, NULL, &local_error);
-      if (input != NULL)
-	{
-          pixbuf = gdk_pixbuf_new_from_stream (G_INPUT_STREAM (input), NULL, &local_error);
-          g_object_unref (input);
-	}
-      else
-        {
-          pixbuf = NULL;
-        }
+      texture = gdk_texture_new_from_file (url->file, &local_error);
     }
 
-  if (pixbuf == NULL)
+  if (texture == NULL)
     {
-      cairo_surface_t *empty;
-
       if (error)
         {
           char *uri;
 
           uri = g_file_get_uri (url->file);
           g_set_error (error,
-                       GTK_CSS_PROVIDER_ERROR,
-                       GTK_CSS_PROVIDER_ERROR_FAILED,
+                       GTK_CSS_PARSER_ERROR,
+                       GTK_CSS_PARSER_ERROR_FAILED,
                        "Error loading image '%s': %s", uri, local_error->message);
-          g_error_free (local_error);
           g_free (uri);
        }
-
-      empty = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, 0, 0);
-      url->loaded_image = _gtk_css_image_surface_new (empty);
-      cairo_surface_destroy (empty);
-      return url->loaded_image;
+      
+      url->loaded_image = gtk_css_image_invalid_new ();
+    }
+  else
+    {
+      url->loaded_image = gtk_css_image_paintable_new (GDK_PAINTABLE (texture), GDK_PAINTABLE (texture));
+      g_object_unref (texture);
     }
 
-  url->loaded_image = _gtk_css_image_surface_new_for_pixbuf (pixbuf);
-  g_object_unref (pixbuf);
+  g_clear_error (&local_error);
 
   return url->loaded_image;
 }
@@ -118,22 +109,22 @@ gtk_css_image_url_get_aspect_ratio (GtkCssImage *image)
 }
 
 static void
-gtk_css_image_url_draw (GtkCssImage        *image,
-                        cairo_t            *cr,
-                        double              width,
-                        double              height)
+gtk_css_image_url_snapshot (GtkCssImage *image,
+                            GtkSnapshot *snapshot,
+                            double       width,
+                            double       height)
 {
   GtkCssImageUrl *url = GTK_CSS_IMAGE_URL (image);
 
-  _gtk_css_image_draw (gtk_css_image_url_load_image (url, NULL), cr, width, height);
+  gtk_css_image_snapshot (gtk_css_image_url_load_image (url, NULL), snapshot, width, height);
 }
 
 static GtkCssImage *
-gtk_css_image_url_compute (GtkCssImage             *image,
-                           guint                    property_id,
-                           GtkStyleProviderPrivate *provider,
-                           GtkCssStyle             *style,
-                           GtkCssStyle             *parent_style)
+gtk_css_image_url_compute (GtkCssImage      *image,
+                           guint             property_id,
+                           GtkStyleProvider *provider,
+                           GtkCssStyle      *style,
+                           GtkCssStyle      *parent_style)
 {
   GtkCssImageUrl *url = GTK_CSS_IMAGE_URL (image);
   GtkCssImage *copy;
@@ -143,20 +134,38 @@ gtk_css_image_url_compute (GtkCssImage             *image,
   if (error)
     {
       GtkCssSection *section = gtk_css_style_get_section (style, property_id);
-      _gtk_style_provider_private_emit_error (provider, section, error);
+      gtk_style_provider_emit_error (provider, section, error);
       g_error_free (error);
     }
 
   return g_object_ref (copy);
 }
 
+static gboolean
+gtk_css_image_url_equal (GtkCssImage *image1,
+                         GtkCssImage *image2)
+{
+  GtkCssImageUrl *url1 = GTK_CSS_IMAGE_URL (image1);
+  GtkCssImageUrl *url2 = GTK_CSS_IMAGE_URL (image2);
+
+  return g_file_equal (url1->file, url2->file);
+}
+
+static gboolean
+gtk_css_image_url_is_invalid (GtkCssImage *image)
+{
+  GtkCssImageUrl *url = GTK_CSS_IMAGE_URL (image);
+
+  return gtk_css_image_is_invalid (gtk_css_image_url_load_image (url, NULL));
+}
+
 static gboolean
 gtk_css_image_url_parse (GtkCssImage  *image,
                          GtkCssParser *parser)
 {
   GtkCssImageUrl *url = GTK_CSS_IMAGE_URL (image);
 
-  url->file = _gtk_css_parser_read_url (parser);
+  url->file = gtk_css_parser_consume_url (parser);
   if (url->file == NULL)
     return FALSE;
 
@@ -193,9 +202,11 @@ _gtk_css_image_url_class_init (GtkCssImageUrlClass *klass)
   image_class->get_height = gtk_css_image_url_get_height;
   image_class->get_aspect_ratio = gtk_css_image_url_get_aspect_ratio;
   image_class->compute = gtk_css_image_url_compute;
-  image_class->draw = gtk_css_image_url_draw;
+  image_class->snapshot = gtk_css_image_url_snapshot;
   image_class->parse = gtk_css_image_url_parse;
   image_class->print = gtk_css_image_url_print;
+  image_class->equal = gtk_css_image_url_equal;
+  image_class->is_invalid = gtk_css_image_url_is_invalid;
 
   object_class->dispose = gtk_css_image_url_dispose;
 }
