@@ -19,7 +19,7 @@
 #define __GTK_CSS_TYPES_PRIVATE_H__
 
 #include <glib-object.h>
-#include <gtk/gtkstylecontext.h>
+#include <gsk/gsk.h>
 
 G_BEGIN_DECLS
 
@@ -27,7 +27,6 @@ typedef union _GtkCssMatcher GtkCssMatcher;
 typedef struct _GtkCssNode GtkCssNode;
 typedef struct _GtkCssNodeDeclaration GtkCssNodeDeclaration;
 typedef struct _GtkCssStyle GtkCssStyle;
-typedef struct _GtkStyleProviderPrivate GtkStyleProviderPrivate; /* dummy typedef */
 
 #define GTK_CSS_CHANGE_CLASS                          (1ULL <<  0)
 #define GTK_CSS_CHANGE_NAME                           (1ULL <<  1)
@@ -95,26 +94,26 @@ typedef guint64 GtkCssChange;
 
 /*
  * GtkCssAffects:
- * @GTK_CSS_AFFECTS_FOREGROUND: The foreground rendering is affected.
+ * @GTK_CSS_AFFECTS_CONTENT: The content rendering is affected.
  *   This does not include things that affect the font. For those,
- *   see @GTK_CSS_AFFECTS_FONT.
+ *   see @GTK_CSS_AFFECTS_TEXT.
  * @GTK_CSS_AFFECTS_BACKGROUND: The background rendering is affected.
  * @GTK_CSS_AFFECTS_BORDER: The border styling is affected.
- * @GTK_CSS_AFFECTS_PANGO_LAYOUT: Font rendering is affected.
- * @GTK_CSS_AFFECTS_FONT: The font is affected and should be reloaded
- *   if it was cached.
- * @GTK_CSS_AFFECTS_TEXT: Text rendering is affected.
+ * @GTK_CSS_AFFECTS_ICON_SIZE: Icon size is affected.
  * @GTK_CSS_AFFECTS_TEXT_ATTRS: Text attributes are affected.
+ * @GTK_CSS_AFFECTS_TEXT_SIZE: Text size is affected.
+ * @GTK_CSS_AFFECTS_TEXT_CONTENT: Text rendering is affected, but size or
+ *   attributes are not.
  * @GTK_CSS_AFFECTS_ICON: Fullcolor icons and their rendering is affected.
  * @GTK_CSS_AFFECTS_SYMBOLIC_ICON: Symbolic icons and their rendering is affected.
- * @GTK_CSS_AFFECTS_OUTLINE: The outline styling is affected. Outlines
- *   only affect elements that can be focused.
- * @GTK_CSS_AFFECTS_CLIP: Changes in this property may have an effect
- *   on the clipping area of the element. Changes in these properties
- *   should cause a reevaluation of the element's clip area.
+ * @GTK_CSS_AFFECTS_OUTLINE: The outline styling is affected.
  * @GTK_CSS_AFFECTS_SIZE: Changes in this property may have an effect
  *   on the allocated size of the element. Changes in these properties
  *   should cause a recomputation of the element's allocated size.
+ * @GTK_CSS_AFFECTS_POSTEFFECT: An effect is applied after drawing that changes
+ * @GTK_CSS_AFFECTS_TEXT: Affects anything related to text rendering.
+ * @GTK_CSS_AFFECTS_REDRAW: Affects anything that requires redraw.
+ * @GTK_CSS_AFFECTS_TRANSFORM: Affects the element transformation.
  *
  * The generic effects that a CSS property can have. If a value is
  * set, then the property will have an influence on that feature.
@@ -122,25 +121,32 @@ typedef guint64 GtkCssChange;
  * Note that multiple values can be set.
  */
 typedef enum {
-  GTK_CSS_AFFECTS_FOREGROUND = (1 << 0),
-  GTK_CSS_AFFECTS_BACKGROUND = (1 << 1),
-  GTK_CSS_AFFECTS_BORDER = (1 << 2),
-  GTK_CSS_AFFECTS_FONT = (1 << 3),
-  GTK_CSS_AFFECTS_TEXT = (1 << 4),
-  GTK_CSS_AFFECTS_TEXT_ATTRS = (1 << 5),
-  GTK_CSS_AFFECTS_ICON = (1 << 6),
-  GTK_CSS_AFFECTS_SYMBOLIC_ICON = (1 << 7),
-  GTK_CSS_AFFECTS_OUTLINE = (1 << 8),
-  GTK_CSS_AFFECTS_CLIP = (1 << 9),
-  GTK_CSS_AFFECTS_SIZE = (1 << 10)
+  GTK_CSS_AFFECTS_CONTENT       = (1 << 0),
+  GTK_CSS_AFFECTS_BACKGROUND    = (1 << 1),
+  GTK_CSS_AFFECTS_BORDER        = (1 << 2),
+  GTK_CSS_AFFECTS_ICON_SIZE     = (1 << 3),
+  GTK_CSS_AFFECTS_TEXT_ATTRS    = (1 << 4),
+  GTK_CSS_AFFECTS_TEXT_SIZE     = (1 << 5),
+  GTK_CSS_AFFECTS_TEXT_CONTENT  = (1 << 6),
+  GTK_CSS_AFFECTS_ICON          = (1 << 7),
+  GTK_CSS_AFFECTS_SYMBOLIC_ICON = (1 << 8),
+  GTK_CSS_AFFECTS_OUTLINE       = (1 << 9),
+  GTK_CSS_AFFECTS_SIZE          = (1 << 10),
+  GTK_CSS_AFFECTS_POSTEFFECT    = (1 << 11),
+  GTK_CSS_AFFECTS_TRANSFORM     = (1 << 12),
 } GtkCssAffects;
 
-#define GTK_CSS_AFFECTS_REDRAW (GTK_CSS_AFFECTS_FOREGROUND |    \
+#define GTK_CSS_AFFECTS_REDRAW (GTK_CSS_AFFECTS_CONTENT |       \
                                 GTK_CSS_AFFECTS_BACKGROUND |    \
                                 GTK_CSS_AFFECTS_BORDER |        \
                                 GTK_CSS_AFFECTS_ICON |          \
                                 GTK_CSS_AFFECTS_SYMBOLIC_ICON | \
-                                GTK_CSS_AFFECTS_OUTLINE)
+                                GTK_CSS_AFFECTS_OUTLINE |       \
+                                GTK_CSS_AFFECTS_POSTEFFECT)
+
+#define GTK_CSS_AFFECTS_TEXT (GTK_CSS_AFFECTS_TEXT_SIZE | \
+                              GTK_CSS_AFFECTS_TEXT_CONTENT)
+
 
 enum { /*< skip >*/
   GTK_CSS_PROPERTY_COLOR,
@@ -151,13 +157,19 @@ enum { /*< skip >*/
   GTK_CSS_PROPERTY_BACKGROUND_COLOR,
   GTK_CSS_PROPERTY_FONT_FAMILY,
   GTK_CSS_PROPERTY_FONT_STYLE,
-  GTK_CSS_PROPERTY_FONT_VARIANT,
   GTK_CSS_PROPERTY_FONT_WEIGHT,
   GTK_CSS_PROPERTY_FONT_STRETCH,
   GTK_CSS_PROPERTY_LETTER_SPACING,
   GTK_CSS_PROPERTY_TEXT_DECORATION_LINE,
   GTK_CSS_PROPERTY_TEXT_DECORATION_COLOR,
   GTK_CSS_PROPERTY_TEXT_DECORATION_STYLE,
+  GTK_CSS_PROPERTY_FONT_KERNING,
+  GTK_CSS_PROPERTY_FONT_VARIANT_LIGATURES,
+  GTK_CSS_PROPERTY_FONT_VARIANT_POSITION,
+  GTK_CSS_PROPERTY_FONT_VARIANT_CAPS,
+  GTK_CSS_PROPERTY_FONT_VARIANT_NUMERIC,
+  GTK_CSS_PROPERTY_FONT_VARIANT_ALTERNATES,
+  GTK_CSS_PROPERTY_FONT_VARIANT_EAST_ASIAN,
   GTK_CSS_PROPERTY_TEXT_SHADOW,
   GTK_CSS_PROPERTY_BOX_SHADOW,
   GTK_CSS_PROPERTY_MARGIN_TOP,
@@ -204,9 +216,13 @@ enum { /*< skip >*/
   GTK_CSS_PROPERTY_BORDER_IMAGE_SLICE,
   GTK_CSS_PROPERTY_BORDER_IMAGE_WIDTH,
   GTK_CSS_PROPERTY_ICON_SOURCE,
+  GTK_CSS_PROPERTY_ICON_SIZE,
   GTK_CSS_PROPERTY_ICON_SHADOW,
   GTK_CSS_PROPERTY_ICON_STYLE,
   GTK_CSS_PROPERTY_ICON_TRANSFORM,
+  GTK_CSS_PROPERTY_ICON_FILTER,
+  GTK_CSS_PROPERTY_BORDER_SPACING,
+  GTK_CSS_PROPERTY_TRANSFORM,
   GTK_CSS_PROPERTY_MIN_WIDTH,
   GTK_CSS_PROPERTY_MIN_HEIGHT,
   GTK_CSS_PROPERTY_TRANSITION_PROPERTY,
@@ -222,35 +238,15 @@ enum { /*< skip >*/
   GTK_CSS_PROPERTY_ANIMATION_DELAY,
   GTK_CSS_PROPERTY_ANIMATION_FILL_MODE,
   GTK_CSS_PROPERTY_OPACITY,
-  GTK_CSS_PROPERTY_ICON_EFFECT,
-  GTK_CSS_PROPERTY_ENGINE,
-  GTK_CSS_PROPERTY_GTK_KEY_BINDINGS,
+  GTK_CSS_PROPERTY_FILTER,
   GTK_CSS_PROPERTY_CARET_COLOR,
   GTK_CSS_PROPERTY_SECONDARY_CARET_COLOR,
   GTK_CSS_PROPERTY_FONT_FEATURE_SETTINGS,
+  GTK_CSS_PROPERTY_FONT_VARIATION_SETTINGS,
   /* add more */
   GTK_CSS_PROPERTY_N_PROPERTIES
 };
 
-typedef enum /*< skip >*/ {
-  GTK_CSS_BLEND_MODE_COLOR,
-  GTK_CSS_BLEND_MODE_COLOR_BURN,
-  GTK_CSS_BLEND_MODE_COLOR_DODGE,
-  GTK_CSS_BLEND_MODE_DARKEN,
-  GTK_CSS_BLEND_MODE_DIFFERENCE,
-  GTK_CSS_BLEND_MODE_EXCLUSION,
-  GTK_CSS_BLEND_MODE_HARD_LIGHT,
-  GTK_CSS_BLEND_MODE_HUE,
-  GTK_CSS_BLEND_MODE_LIGHTEN,
-  GTK_CSS_BLEND_MODE_LUMINOSITY,
-  GTK_CSS_BLEND_MODE_MULTIPLY,
-  GTK_CSS_BLEND_MODE_NORMAL,
-  GTK_CSS_BLEND_MODE_OVERLAY,
-  GTK_CSS_BLEND_MODE_SATURATE,
-  GTK_CSS_BLEND_MODE_SCREEN,
-  GTK_CSS_BLEND_MODE_SOFT_LIGHT
-} GtkCssBlendMode;
-
 typedef enum /*< skip >*/ {
   GTK_CSS_IMAGE_BUILTIN_NONE,
   GTK_CSS_IMAGE_BUILTIN_CHECK,
@@ -269,14 +265,6 @@ typedef enum /*< skip >*/ {
   GTK_CSS_IMAGE_BUILTIN_EXPANDER_VERTICAL_LEFT_EXPANDED,
   GTK_CSS_IMAGE_BUILTIN_EXPANDER_HORIZONTAL_RIGHT_EXPANDED,
   GTK_CSS_IMAGE_BUILTIN_EXPANDER_VERTICAL_RIGHT_EXPANDED,
-  GTK_CSS_IMAGE_BUILTIN_GRIP_TOPLEFT,
-  GTK_CSS_IMAGE_BUILTIN_GRIP_TOP,
-  GTK_CSS_IMAGE_BUILTIN_GRIP_TOPRIGHT,
-  GTK_CSS_IMAGE_BUILTIN_GRIP_RIGHT,
-  GTK_CSS_IMAGE_BUILTIN_GRIP_BOTTOMRIGHT,
-  GTK_CSS_IMAGE_BUILTIN_GRIP_BOTTOM,
-  GTK_CSS_IMAGE_BUILTIN_GRIP_BOTTOMLEFT,
-  GTK_CSS_IMAGE_BUILTIN_GRIP_LEFT,
   GTK_CSS_IMAGE_BUILTIN_PANE_SEPARATOR,
   GTK_CSS_IMAGE_BUILTIN_HANDLE,
   GTK_CSS_IMAGE_BUILTIN_SPINNER
@@ -307,12 +295,6 @@ typedef enum /*< skip >*/ {
   GTK_CSS_FILL_BOTH
 } GtkCssFillMode;
 
-typedef enum /*< skip >*/ {
-  GTK_CSS_ICON_EFFECT_NONE,
-  GTK_CSS_ICON_EFFECT_HIGHLIGHT,
-  GTK_CSS_ICON_EFFECT_DIM
-} GtkCssIconEffect;
-
 typedef enum /*< skip >*/ {
   GTK_CSS_ICON_STYLE_REQUESTED,
   GTK_CSS_ICON_STYLE_REGULAR,
@@ -353,13 +335,6 @@ typedef enum /*< skip >*/ {
   GTK_CSS_LEFT
 } GtkCssSide;
 
-typedef enum /*< skip >*/ {
-  GTK_CSS_TOP_LEFT,
-  GTK_CSS_TOP_RIGHT,
-  GTK_CSS_BOTTOM_RIGHT,
-  GTK_CSS_BOTTOM_LEFT
-} GtkCssCorner;
-
 typedef enum /*< skip >*/ {
   GTK_CSS_DIMENSION_PERCENTAGE,
   GTK_CSS_DIMENSION_NUMBER,
@@ -393,7 +368,72 @@ typedef enum /*< skip >*/ {
   GTK_CSS_MS,
 } GtkCssUnit;
 
-cairo_operator_t        _gtk_css_blend_mode_get_operator         (GtkCssBlendMode    mode);
+typedef enum /*< skip >*/ {
+  GTK_CSS_FONT_KERNING_AUTO,
+  GTK_CSS_FONT_KERNING_NORMAL,
+  GTK_CSS_FONT_KERNING_NONE
+} GtkCssFontKerning;
+
+typedef enum /*< skip >*/ {
+  GTK_CSS_FONT_VARIANT_LIGATURE_NORMAL                     = 1 << 0,
+  GTK_CSS_FONT_VARIANT_LIGATURE_NONE                       = 1 << 1,
+  GTK_CSS_FONT_VARIANT_LIGATURE_COMMON_LIGATURES           = 1 << 2,
+  GTK_CSS_FONT_VARIANT_LIGATURE_NO_COMMON_LIGATURES        = 1 << 3,
+  GTK_CSS_FONT_VARIANT_LIGATURE_DISCRETIONARY_LIGATURES    = 1 << 4,
+  GTK_CSS_FONT_VARIANT_LIGATURE_NO_DISCRETIONARY_LIGATURES = 1 << 5,
+  GTK_CSS_FONT_VARIANT_LIGATURE_HISTORICAL_LIGATURES       = 1 << 6,
+  GTK_CSS_FONT_VARIANT_LIGATURE_NO_HISTORICAL_LIGATURES    = 1 << 7,
+  GTK_CSS_FONT_VARIANT_LIGATURE_CONTEXTUAL                 = 1 << 8,
+  GTK_CSS_FONT_VARIANT_LIGATURE_NO_CONTEXTUAL              = 1 << 9
+} GtkCssFontVariantLigature;
+
+typedef enum /*< skip >*/ {
+  GTK_CSS_FONT_VARIANT_POSITION_NORMAL,
+  GTK_CSS_FONT_VARIANT_POSITION_SUB,
+  GTK_CSS_FONT_VARIANT_POSITION_SUPER
+} GtkCssFontVariantPosition;
+
+typedef enum /*< skip >*/ {
+  GTK_CSS_FONT_VARIANT_NUMERIC_NORMAL             = 1 << 0,
+  GTK_CSS_FONT_VARIANT_NUMERIC_LINING_NUMS        = 1 << 1,
+  GTK_CSS_FONT_VARIANT_NUMERIC_OLDSTYLE_NUMS      = 1 << 2,
+  GTK_CSS_FONT_VARIANT_NUMERIC_PROPORTIONAL_NUMS  = 1 << 3,
+  GTK_CSS_FONT_VARIANT_NUMERIC_TABULAR_NUMS       = 1 << 4,
+  GTK_CSS_FONT_VARIANT_NUMERIC_DIAGONAL_FRACTIONS = 1 << 5,
+  GTK_CSS_FONT_VARIANT_NUMERIC_STACKED_FRACTIONS  = 1 << 6,
+  GTK_CSS_FONT_VARIANT_NUMERIC_ORDINAL            = 1 << 7,
+  GTK_CSS_FONT_VARIANT_NUMERIC_SLASHED_ZERO       = 1 << 8
+} GtkCssFontVariantNumeric;
+
+typedef enum /*< skip >*/ {
+  GTK_CSS_FONT_VARIANT_CAPS_NORMAL,
+  GTK_CSS_FONT_VARIANT_CAPS_SMALL_CAPS,
+  GTK_CSS_FONT_VARIANT_CAPS_ALL_SMALL_CAPS,
+  GTK_CSS_FONT_VARIANT_CAPS_PETITE_CAPS,
+  GTK_CSS_FONT_VARIANT_CAPS_ALL_PETITE_CAPS,
+  GTK_CSS_FONT_VARIANT_CAPS_UNICASE,
+  GTK_CSS_FONT_VARIANT_CAPS_TITLING_CAPS
+} GtkCssFontVariantCaps;
+
+typedef enum /*< skip >*/ {
+  GTK_CSS_FONT_VARIANT_ALTERNATE_NORMAL,
+  GTK_CSS_FONT_VARIANT_ALTERNATE_HISTORICAL_FORMS
+} GtkCssFontVariantAlternate;
+
+typedef enum /*< skip >*/ {
+  GTK_CSS_FONT_VARIANT_EAST_ASIAN_NORMAL       = 1 << 0,
+  GTK_CSS_FONT_VARIANT_EAST_ASIAN_JIS78        = 1 << 1,
+  GTK_CSS_FONT_VARIANT_EAST_ASIAN_JIS83        = 1 << 2,
+  GTK_CSS_FONT_VARIANT_EAST_ASIAN_JIS90        = 1 << 3,
+  GTK_CSS_FONT_VARIANT_EAST_ASIAN_JIS04        = 1 << 4,
+  GTK_CSS_FONT_VARIANT_EAST_ASIAN_SIMPLIFIED   = 1 << 5,
+  GTK_CSS_FONT_VARIANT_EAST_ASIAN_TRADITIONAL  = 1 << 6,
+  GTK_CSS_FONT_VARIANT_EAST_ASIAN_FULL_WIDTH   = 1 << 7,
+  GTK_CSS_FONT_VARIANT_EAST_ASIAN_PROPORTIONAL = 1 << 8,
+  GTK_CSS_FONT_VARIANT_EAST_ASIAN_RUBY         = 1 << 9
+} GtkCssFontVariantEastAsian;
+
+cairo_operator_t        _gtk_css_blend_mode_get_operator         (GskBlendMode       mode);
 
 GtkCssChange            _gtk_css_change_for_sibling              (GtkCssChange       match);
 GtkCssChange            _gtk_css_change_for_child                (GtkCssChange       match);
@@ -404,10 +444,6 @@ char *                  gtk_css_change_to_string                 (GtkCssChange
 void                    gtk_css_change_print                     (GtkCssChange       change,
                                                                   GString           *string);
 
-/* for lack of better place to put it */
-/* mirror what cairo does */
-#define gtk_rgba_is_clear(rgba) ((rgba)->alpha < ((double)0x00ff / (double)0xffff))
-
 G_END_DECLS
 
 #endif /* __GTK_CSS_TYPES_PRIVATE_H__ */
