@@ -13,9 +13,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -31,7 +29,6 @@
 #include "gtktreedatalist.h"
 #include "gtktreednd.h"
 #include "gtktreemodel.h"
-#include "gtkalias.h"
 
 /*** Structure: how GtkFileSystemModel works
  *
@@ -42,7 +39,7 @@
  *
  *   2. The caller populates the model by hand, with files not necessarily in the same folder.  This happens
  *      if you use _gtk_file_system_model_new() and then _gtk_file_system_model_add_and_query_file().  This is
- *      the special kind of usage for "search" and "recent-files", where the file chooser gives the model the
+ *      the special kind of usage for “search” and “recent-files”, where the file chooser gives the model the
  *      files to be displayed.
  *
  * Internal data structure
@@ -73,7 +70,7 @@
  * Each FileModelNode has a node->visible field, which indicates whether the node is visible in the GtkTreeView.
  * A node may be invisible if, for example, it corresponds to a hidden file and the file chooser is not showing
  * hidden files.  Also, a file filter may be explicitly set onto the model, for example, to only show files that
- * match "*.jpg".  In this case, node->filtered_out says whether the node failed the filter.  The ultimate
+ * match “*.jpg”.  In this case, node->filtered_out says whether the node failed the filter.  The ultimate
  * decision on whether a node is visible or not in the treeview is distilled into the node->visible field.
  * The reason for having a separate node->filtered_out field is so that the file chooser can query whether
  * a (filtered-out) folder should be made sensitive in the GUI.
@@ -85,12 +82,12 @@
  * the treeview to array indexes in our array of files.  And thus we introduce a bit of terminology:
  *
  *   index - An index in the model->files array.  All variables/fields that represent indexes are either called
- *   "index" or "i_*", or simply "i" for things like loop counters.
+ *   “index” or “i_*”, or simply “i” for things like loop counters.
  *
  *   row - An index in the GtkTreeView, i.e. the index of a row within the outward-facing API of the
  *   GtkFileSystemModel.  However, note that our rows are 1-based, not 0-based, for the reason explained in the
- *   following paragraph.  Variables/fields that represent visible rows are called "row", or "r_*", or simply
- *   "r".
+ *   following paragraph.  Variables/fields that represent visible rows are called “row”, or “r_*”, or simply
+ *   “r”.
  *
  * Each FileModelNode has a node->row field which is the number of visible rows in the treeview, *before and
  * including* that node.  This means that node->row is 1-based, instead of 0-based --- this makes some code
@@ -98,11 +95,11 @@
  * turn the 0-based treepath into a 1-based row for our purposes.  If a node is not visible, it will have the
  * same row number as its closest preceding visible node.
  *
- * We try to compute the node->row fields lazily.  A node is said to be "valid" if its node->row is accurate.
+ * We try to compute the node->row fields lazily.  A node is said to be “valid” if its node->row is accurate.
  * For this, the model keeps a model->n_nodes_valid field which is the count of valid nodes starting from the
  * beginning of the model->files array.  When a node changes its information, or when a node gets deleted, that
  * node and the following ones get invalidated by simply setting model->n_nodes_valid to the array index of the
- * node.  If the model happens to need a node's row number and that node is in the model->files array after
+ * node.  If the model happens to need a node’s row number and that node is in the model->files array after
  * model->n_nodes_valid, then the nodes get re-validated up to the sought node.  See node_validate_rows() for
  * this logic.
  *
@@ -247,7 +244,7 @@ static void remove_file (GtkFileSystemModel *model,
  *
  * If you want to validate up to an index or up to a row, specify the index or
  * the row you want and specify G_MAXUINT for the other argument.  Pass
- * G_MAXUINT for both arguments for "validate everything".
+ * G_MAXUINT for both arguments for “validate everything”.
  */
 static void
 node_validate_rows (GtkFileSystemModel *model, guint up_to_index, guint up_to_row)
@@ -1095,7 +1092,7 @@ _gtk_file_system_model_class_init (GtkFileSystemModelClass *class)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkFileSystemModelClass, finished_loading),
 		  NULL, NULL,
-		  _gtk_marshal_VOID__POINTER,
+		  NULL,
 		  G_TYPE_NONE, 1, G_TYPE_POINTER);
 }
 
@@ -1140,8 +1137,6 @@ gtk_file_system_model_got_files (GObject *object, GAsyncResult *res, gpointer da
   GList *walk, *files;
   GError *error = NULL;
 
-  gdk_threads_enter ();
-
   files = g_file_enumerator_next_files_finish (enumerator, res, &error);
 
   if (files)
@@ -1149,11 +1144,11 @@ gtk_file_system_model_got_files (GObject *object, GAsyncResult *res, gpointer da
       if (model->dir_thaw_source == 0)
         {
           freeze_updates (model);
-          model->dir_thaw_source = gdk_threads_add_timeout_full (IO_PRIORITY + 1,
-                                                                 50,
-                                                                 thaw_func,
-                                                                 model,
-                                                                 NULL);
+          model->dir_thaw_source = g_timeout_add_full (IO_PRIORITY + 1, 50,
+                                                       thaw_func,
+                                                       model,
+                                                       NULL);
+          g_source_set_name_by_id (model->dir_thaw_source, "[gtk] thaw_func");
         }
 
       for (walk = files; walk; walk = walk->next)
@@ -1206,8 +1201,6 @@ gtk_file_system_model_got_files (GObject *object, GAsyncResult *res, gpointer da
       if (error)
         g_error_free (error);
     }
-
-  gdk_threads_leave ();
 }
 
 static void
@@ -1224,16 +1217,12 @@ gtk_file_system_model_query_done (GObject *     object,
   if (info == NULL)
     return;
 
-  gdk_threads_enter ();
-
   _gtk_file_system_model_update_file (model, file, info);
 
   id = node_get_for_file (model, file);
   gtk_file_system_model_sort_node (model, id);
 
   g_object_unref (info);
-
-  gdk_threads_leave ();
 }
 
 static void
@@ -1258,14 +1247,16 @@ gtk_file_system_model_monitor_change (GFileMonitor *      monitor,
                                  model);
         break;
       case G_FILE_MONITOR_EVENT_DELETED:
-	gdk_threads_enter ();
         remove_file (model, file);
-	gdk_threads_leave ();
         break;
       case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
         /* FIXME: use freeze/thaw with this somehow? */
       case G_FILE_MONITOR_EVENT_PRE_UNMOUNT:
       case G_FILE_MONITOR_EVENT_UNMOUNTED:
+      case G_FILE_MONITOR_EVENT_MOVED:
+      case G_FILE_MONITOR_EVENT_RENAMED:
+      case G_FILE_MONITOR_EVENT_MOVED_IN:
+      case G_FILE_MONITOR_EVENT_MOVED_OUT:
       default:
         /* ignore these */
         break;
@@ -1279,8 +1270,6 @@ gtk_file_system_model_got_enumerator (GObject *dir, GAsyncResult *res, gpointer
   GFileEnumerator *enumerator;
   GError *error = NULL;
 
-  gdk_threads_enter ();
-
   enumerator = g_file_enumerate_children_finish (G_FILE (dir), res, &error);
   if (enumerator == NULL)
     {
@@ -1309,8 +1298,6 @@ gtk_file_system_model_got_enumerator (GObject *dir, GAsyncResult *res, gpointer
                           G_CALLBACK (gtk_file_system_model_monitor_change),
                           model);
     }
-
-  gdk_threads_leave ();
 }
 
 static void
@@ -1396,7 +1383,7 @@ _gtk_file_system_model_new_valist (GtkFileSystemModelGetValue get_func,
  * to the list using _gtk_file_system_model_add_and_query_file()
  * or _gtk_file_system_model_update_file().
  *
- * Return value: the newly created #GtkFileSystemModel
+ * Returns: the newly created #GtkFileSystemModel
  **/
 GtkFileSystemModel *
 _gtk_file_system_model_new (GtkFileSystemModelGetValue get_func,
@@ -1433,7 +1420,7 @@ _gtk_file_system_model_new (GtkFileSystemModelGetValue get_func,
  * it will also monitor the drectory and update the model's
  * contents to reflect changes, if the @directory supports monitoring.
  * 
- * Return value: the newly created #GtkFileSystemModel
+ * Returns: the newly created #GtkFileSystemModel
  **/
 GtkFileSystemModel *
 _gtk_file_system_model_new_for_directory (GFile *                    dir,
@@ -1650,17 +1637,14 @@ _gtk_file_system_model_iter_is_filtered_out (GtkFileSystemModel *model,
  * @model: a #GtkFileSystemModel
  * @iter: a #GtkTreeIter pointing to a row of @model
  * 
- * Gets the #GFileInfo structure for a particular row
+ * Gets the #GFileInfo-struct for a particular row
  * of @model.
  * 
- * Return value: a #GFileInfo structure. This structure
+ * Returns: a #GFileInfo-struct. This value
  *   is owned by @model and must not be modified or freed.
  *   If you want to keep the information for later use,
- *   you must take a reference, since the structure may be
- *   freed on later changes to the file system.  If you have
- *   called _gtk_file_system_model_add_editable() and the @iter
- *   corresponds to the row that this function returned, the
- *   return value will be NULL.
+ *   you must take a reference, since the #GFileInfo-struct may be
+ *   freed on later changes to the file system.
  **/
 GFileInfo *
 _gtk_file_system_model_get_info (GtkFileSystemModel *model,
@@ -1683,7 +1667,7 @@ _gtk_file_system_model_get_info (GtkFileSystemModel *model,
  * 
  * Gets the file for a particular row in @model. 
  *
- * Return value: the file. This object is owned by @model and
+ * Returns: the file. This object is owned by @model and
  *   or freed. If you want to save the path for later use,
  *   you must take a ref, since the object may be freed
  *   on later changes to the file system.
@@ -1967,6 +1951,23 @@ _gtk_file_system_model_update_file (GtkFileSystemModel *model,
     emit_row_changed_for_node (model, id);
 }
 
+void
+_gtk_file_system_model_update_files (GtkFileSystemModel *model,
+                                     GList              *files,
+                                     GList              *infos)
+{
+  GList *l, *i;
+
+  g_return_if_fail (GTK_IS_FILE_SYSTEM_MODEL (model));
+
+  freeze_updates (model);
+
+  for (l = files, i = infos; l; l = l->next, i = i->next)
+    _gtk_file_system_model_update_file (model, (GFile *)l->data, (GFileInfo *)i->data);
+
+  thaw_updates (model);
+}
+
 /**
  * _gtk_file_system_model_set_filter:
  * @mode: a #GtkFileSystemModel
@@ -2135,46 +2136,47 @@ _gtk_file_system_model_add_and_query_file (GtkFileSystemModel *model,
                            model);
 }
 
-/**
- * _gtk_file_system_model_add_editable:
- * @model: a #GtkFileSystemModel
- * @iter: Location to return the iter corresponding to the editable row
- * 
- * Adds an “empty” row at the beginning of the model.  This does not refer to
- * any file, but is a temporary placeholder for a file name that the user will
- * type when a corresponding cell is made editable.  When your code is done
- * using this temporary row, call _gtk_file_system_model_remove_editable().
- **/
-void
-_gtk_file_system_model_add_editable (GtkFileSystemModel *model, GtkTreeIter *iter)
+static void
+gtk_file_system_model_one_query_done (GObject *     object,
+                                      GAsyncResult *res,
+                                      gpointer      data)
 {
-  g_return_if_fail (GTK_IS_FILE_SYSTEM_MODEL (model));
-  g_return_if_fail (!get_node (model, 0)->visible);
-
-  node_set_visible_and_filtered_out (model, 0, TRUE, FALSE);
-  ITER_INIT_FROM_INDEX (model, iter, 0);
-
-  /* we don't want file system changes to affect the model while
-   * editing is in place
-   */
-  freeze_updates (model);
-}
-
-/**
- * _gtk_file_system_model_remove_editable:
- * @model: a #GtkFileSystemModel
- * 
- * Removes the “empty” row at the beginning of the model that was
- * created with _gtk_file_system_model_add_editable().  You should call
- * this function when your code is finished editing this temporary row.
- **/
-void
-_gtk_file_system_model_remove_editable (GtkFileSystemModel *model)
-{
-  g_return_if_fail (GTK_IS_FILE_SYSTEM_MODEL (model));
-  g_return_if_fail (get_node (model, 0)->visible);
+  GtkFileSystemModel *model = data; /* only a valid pointer if not cancelled */
 
+  gtk_file_system_model_query_done (object, res, data);
   thaw_updates (model);
-
-  node_set_visible_and_filtered_out (model, 0, FALSE, FALSE);
 }
+
+void
+_gtk_file_system_model_add_and_query_files (GtkFileSystemModel *model,
+                                            GList              *list,
+                                            const char         *attributes)
+{
+  GList *l;
+  GFile *file;
+
+  g_return_if_fail (GTK_IS_FILE_SYSTEM_MODEL (model));
+  g_return_if_fail (attributes != NULL);
+
+  for (l = list; l; l = l->next)
+    {
+      file = (GFile *)l->data;
+      freeze_updates (model);
+      g_file_query_info_async (file,
+                               attributes,
+                               G_FILE_QUERY_INFO_NONE,
+                               IO_PRIORITY,
+                               model->cancellable,
+                               gtk_file_system_model_one_query_done,
+                               model);
+    }
+}
+
+GFile *
+_gtk_file_system_model_get_directory (GtkFileSystemModel *model)
+{
+  g_return_val_if_fail (GTK_IS_FILE_SYSTEM_MODEL (model), NULL);
+
+  return model->dir;
+}
+
