@@ -19,15 +19,14 @@
 #define __GTK_IM_MODULE_PRIVATE_H__
 
 #include <gdk/gdk.h>
-#include <gtk/gtkimcontext.h>
-#include <gtk/gtkimcontextinfo.h>
+#include "gtkimcontext.h"
+#include "gtkimmodule.h"
 
 G_BEGIN_DECLS
 
-void           _gtk_im_module_list                   (const GtkIMContextInfo ***contexts,
-						      guint                    *n_contexts);
-GtkIMContext * _gtk_im_module_create                 (const gchar              *context_id);
-const gchar  * _gtk_im_module_get_default_context_id (void);
+void           gtk_im_module_ensure_extension_point  (void);
+GtkIMContext * _gtk_im_module_create                 (const char *context_id);
+const gchar  * _gtk_im_module_get_default_context_id (GdkDisplay *display);
 
 G_END_DECLS
 
