@@ -42,7 +42,7 @@ struct _GtkIMMulticontextPrivate
 {
   GtkIMContext          *slave;
 
-  GdkWindow             *client_window;
+  GtkWidget             *client_widget;
   GdkRectangle           cursor_location;
 
   gchar                 *context_id;
@@ -61,8 +61,8 @@ static void     gtk_im_multicontext_set_slave          (GtkIMMulticontext
 							GtkIMContext            *slave,
 							gboolean                 finalizing);
 
-static void     gtk_im_multicontext_set_client_window  (GtkIMContext            *context,
-							GdkWindow               *window);
+static void     gtk_im_multicontext_set_client_widget  (GtkIMContext            *context,
+							GtkWidget               *widget);
 static void     gtk_im_multicontext_get_preedit_string (GtkIMContext            *context,
 							gchar                  **str,
 							PangoAttrList          **attrs,
@@ -102,8 +102,6 @@ static gboolean gtk_im_multicontext_delete_surrounding_cb   (GtkIMContext      *
 
 static void propagate_purpose (GtkIMMulticontext *context);
 
-static const gchar *global_context_id = NULL;
-
 G_DEFINE_TYPE_WITH_PRIVATE (GtkIMMulticontext, gtk_im_multicontext, GTK_TYPE_IM_CONTEXT)
 
 static void
@@ -114,7 +112,7 @@ gtk_im_multicontext_class_init (GtkIMMulticontextClass *class)
 
   gobject_class->notify = gtk_im_multicontext_notify;
 
-  im_context_class->set_client_window = gtk_im_multicontext_set_client_window;
+  im_context_class->set_client_widget = gtk_im_multicontext_set_client_widget;
   im_context_class->get_preedit_string = gtk_im_multicontext_get_preedit_string;
   im_context_class->filter_keypress = gtk_im_multicontext_filter_keypress;
   im_context_class->focus_in = gtk_im_multicontext_focus_in;
@@ -193,12 +191,6 @@ gtk_im_multicontext_set_slave (GtkIMMulticontext *multicontext,
       g_signal_handlers_disconnect_by_func (priv->slave,
 					    gtk_im_multicontext_commit_cb,
 					    multicontext);
-      g_signal_handlers_disconnect_by_func (priv->slave,
-					    gtk_im_multicontext_retrieve_surrounding_cb,
-					    multicontext);
-      g_signal_handlers_disconnect_by_func (priv->slave,
-					    gtk_im_multicontext_delete_surrounding_cb,
-					    multicontext);
 
       g_object_unref (priv->slave);
       priv->slave = NULL;
@@ -236,8 +228,8 @@ gtk_im_multicontext_set_slave (GtkIMMulticontext *multicontext,
 
       if (!priv->use_preedit)	/* Default is TRUE */
 	gtk_im_context_set_use_preedit (slave, FALSE);
-      if (priv->client_window)
-	gtk_im_context_set_client_window (slave, priv->client_window);
+      if (priv->client_widget)
+	gtk_im_context_set_client_widget (slave, priv->client_widget);
       if (priv->have_cursor_location)
 	gtk_im_context_set_cursor_location (slave, &priv->cursor_location);
       if (priv->focus_in)
@@ -252,14 +244,17 @@ static const gchar *
 get_effective_context_id (GtkIMMulticontext *multicontext)
 {
   GtkIMMulticontextPrivate *priv = multicontext->priv;
+  GdkDisplay *display;
 
   if (priv->context_id_aux)
     return priv->context_id_aux;
 
-  if (!global_context_id)
-    global_context_id = _gtk_im_module_get_default_context_id ();
+  if (priv->client_widget)
+    display = gtk_widget_get_display (priv->client_widget);
+  else
+    display = gdk_display_get_default ();
 
-  return global_context_id;
+  return _gtk_im_module_get_default_context_id (display);
 }
 
 static GtkIMContext *
@@ -267,9 +262,6 @@ gtk_im_multicontext_get_slave (GtkIMMulticontext *multicontext)
 {
   GtkIMMulticontextPrivate *priv = multicontext->priv;
 
-  if (g_strcmp0 (priv->context_id, get_effective_context_id (multicontext)) != 0)
-    gtk_im_multicontext_set_slave (multicontext, NULL, FALSE);
-
   if (!priv->slave)
     {
       GtkIMContext *slave;
@@ -290,48 +282,45 @@ gtk_im_multicontext_get_slave (GtkIMMulticontext *multicontext)
 }
 
 static void
-im_module_setting_changed (GtkSettings       *settings,
+im_module_setting_changed (GtkSettings       *settings, 
                            GParamSpec        *pspec,
                            GtkIMMulticontext *self)
 {
-  global_context_id = NULL;
+  gtk_im_multicontext_set_slave (self, NULL, FALSE);
 }
 
-
 static void
-gtk_im_multicontext_set_client_window (GtkIMContext *context,
-				       GdkWindow    *window)
+gtk_im_multicontext_set_client_widget (GtkIMContext *context,
+				       GtkWidget    *widget)
 {
-  GtkIMMulticontext *multicontext = GTK_IM_MULTICONTEXT (context);
-  GtkIMMulticontextPrivate *priv = multicontext->priv;
+  GtkIMMulticontext *self = GTK_IM_MULTICONTEXT (context);
+  GtkIMMulticontextPrivate *priv = self->priv;
   GtkIMContext *slave;
-  GdkScreen *screen;
   GtkSettings *settings;
-  gboolean connected;
 
-  priv->client_window = window;
-
-  if (window)
+  if (priv->client_widget != NULL)
     {
-      screen = gdk_window_get_screen (window);
-      settings = gtk_settings_get_for_screen (screen);
+      settings = gtk_widget_get_settings (priv->client_widget);
 
-      connected = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (settings),
-                                                      "gtk-im-module-connected"));
-      if (!connected)
-        {
-          g_signal_connect (settings, "notify::gtk-im-module",
-                            G_CALLBACK (im_module_setting_changed), NULL);
-          g_object_set_data (G_OBJECT (settings), "gtk-im-module-connected",
-                             GINT_TO_POINTER (TRUE));
-
-          global_context_id = NULL;
-        }
+      g_signal_handlers_disconnect_by_func (settings,
+                                            im_module_setting_changed,
+                                            self);
     }
 
-  slave = gtk_im_multicontext_get_slave (multicontext);
+  priv->client_widget = widget;
+
+  if (widget)
+    {
+      settings = gtk_widget_get_settings (widget);
+
+      g_signal_connect (settings, "notify::gtk-im-module",
+                        G_CALLBACK (im_module_setting_changed),
+                        self);
+    }
+
+  slave = gtk_im_multicontext_get_slave (self);
   if (slave)
-    gtk_im_context_set_client_window (slave, window);
+    gtk_im_context_set_client_widget (slave, widget);
 }
 
 static void
@@ -360,28 +349,30 @@ gtk_im_multicontext_filter_keypress (GtkIMContext *context,
 {
   GtkIMMulticontext *multicontext = GTK_IM_MULTICONTEXT (context);
   GtkIMContext *slave = gtk_im_multicontext_get_slave (multicontext);
+  guint keyval, state;
 
   if (slave)
     {
       return gtk_im_context_filter_keypress (slave, event);
     }
-  else
+  else if (gdk_event_get_keyval ((GdkEvent *) event, &keyval) &&
+           gdk_event_get_state ((GdkEvent *) event, &state))
     {
       GdkDisplay *display;
       GdkModifierType no_text_input_mask;
 
-      display = gdk_window_get_display (event->window);
+      display = gdk_surface_get_display (gdk_event_get_surface ((GdkEvent *) event));
 
       no_text_input_mask =
-        gdk_keymap_get_modifier_mask (gdk_keymap_get_for_display (display),
+        gdk_keymap_get_modifier_mask (gdk_display_get_keymap (display),
                                       GDK_MODIFIER_INTENT_NO_TEXT_INPUT);
 
-      if (event->type == GDK_KEY_PRESS &&
-          (event->state & no_text_input_mask) == 0)
+      if (gdk_event_get_event_type ((GdkEvent *) event) == GDK_KEY_PRESS &&
+          (state & no_text_input_mask) == 0)
         {
           gunichar ch;
 
-          ch = gdk_keyval_to_unicode (event->keyval);
+          ch = gdk_keyval_to_unicode (keyval);
           if (ch != 0 && !g_unichar_iscntrl (ch))
             {
               gint len;
@@ -555,162 +546,6 @@ gtk_im_multicontext_delete_surrounding_cb (GtkIMContext      *slave,
   return result;
 }
 
-static void
-activate_cb (GtkWidget         *menuitem,
-	     GtkIMMulticontext *context)
-{
-  if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menuitem)))
-    {
-      const gchar *id = g_object_get_data (G_OBJECT (menuitem), "gtk-context-id");
-
-      gtk_im_multicontext_set_context_id (context, id);
-    }
-}
-
-static int
-pathnamecmp (const char *a,
-	     const char *b)
-{
-#ifndef G_OS_WIN32
-  return strcmp (a, b);
-#else
-  /* Ignore case insensitivity, probably not that relevant here. Just
-   * make sure slash and backslash compare equal.
-   */
-  while (*a && *b)
-    if ((G_IS_DIR_SEPARATOR (*a) && G_IS_DIR_SEPARATOR (*b)) ||
-	*a == *b)
-      a++, b++;
-    else
-      return (*a - *b);
-  return (*a - *b);
-#endif
-}
-
-/**
- * gtk_im_multicontext_append_menuitems:
- * @context: a #GtkIMMulticontext
- * @menushell: a #GtkMenuShell
- * 
- * Add menuitems for various available input methods to a menu;
- * the menuitems, when selected, will switch the input method
- * for the context and the global default input method.
- *
- * Deprecated: 3.10: It is better to use the system-wide input
- *     method framework for changing input methods. Modern
- *     desktop shells offer on-screen displays for this that
- *     can triggered with a keyboard shortcut, e.g. Super-Space.
- **/
-void
-gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
-				      GtkMenuShell      *menushell)
-{
-  GtkIMMulticontextPrivate *priv = context->priv;
-  const GtkIMContextInfo **contexts;
-  guint n_contexts, i;
-  GSList *group = NULL;
-  GtkWidget *menuitem, *system_menuitem;
-  const char *system_context_id; 
-
-  system_context_id = _gtk_im_module_get_default_context_id ();
-  system_menuitem = menuitem = gtk_radio_menu_item_new_with_label (group, C_("input method menu", "System"));
-  if (!priv->context_id_aux)
-    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem), TRUE);
-  group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
-  g_object_set_data (G_OBJECT (menuitem), I_("gtk-context-id"), NULL);
-  g_signal_connect (menuitem, "activate", G_CALLBACK (activate_cb), context);
-
-  gtk_widget_show (menuitem);
-  gtk_menu_shell_append (menushell, menuitem);
-
-  _gtk_im_module_list (&contexts, &n_contexts);
-
-  for (i = 0; i < n_contexts; i++)
-    {
-      const gchar *translated_name;
-#ifdef ENABLE_NLS
-      if (contexts[i]->domain && contexts[i]->domain[0])
-	{
-	  if (strcmp (contexts[i]->domain, GETTEXT_PACKAGE) == 0)
-	    {
-	      /* Same translation domain as GTK+ */
-	      if (!(contexts[i]->domain_dirname && contexts[i]->domain_dirname[0]) ||
-		  pathnamecmp (contexts[i]->domain_dirname, _gtk_get_localedir ()) == 0)
-		{
-		  /* Empty or NULL, domain directory, or same as
-		   * GTK+. Input method may have a name in the GTK+
-		   * message catalog.
-		   */
-		  translated_name = g_dpgettext2 (GETTEXT_PACKAGE, "input method menu", contexts[i]->context_name);
-		}
-	      else
-		{
-		  /* Separate domain directory but the same
-		   * translation domain as GTK+. We can't call
-		   * bindtextdomain() as that would make GTK+ forget
-		   * its own messages.
-		   */
-		  g_warning ("Input method %s should not use GTK's translation domain %s",
-			     contexts[i]->context_id, GETTEXT_PACKAGE);
-		  /* Try translating the name in GTK+'s domain */
-		  translated_name = g_dpgettext2 (GETTEXT_PACKAGE, "input method menu", contexts[i]->context_name);
-		}
-	    }
-	  else if (contexts[i]->domain_dirname && contexts[i]->domain_dirname[0])
-	    /* Input method has own translation domain and message catalog */
-	    {
-	      bindtextdomain (contexts[i]->domain,
-			      contexts[i]->domain_dirname);
-#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
-	      bind_textdomain_codeset (contexts[i]->domain, "UTF-8");
-#endif
-	      translated_name = g_dgettext (contexts[i]->domain, contexts[i]->context_name);
-	    }
-	  else
-	    {
-	      /* Different translation domain, but no domain directory */
-	      translated_name = contexts[i]->context_name;
-	    }
-	}
-      else
-	/* Empty or NULL domain. We assume that input method does not
-	 * want a translated name in this case.
-	 */
-	translated_name = contexts[i]->context_name;
-#else
-      translated_name = contexts[i]->context_name;
-#endif
-      menuitem = gtk_radio_menu_item_new_with_label (group, translated_name);
-
-      if ((priv->context_id_aux &&
-           strcmp (contexts[i]->context_id, priv->context_id_aux) == 0))
-        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem), TRUE);
-
-      if (strcmp (contexts[i]->context_id, system_context_id) == 0)
-        {
-          GtkWidget *label;
-          char *text;
-
-          label = gtk_bin_get_child (GTK_BIN (system_menuitem));
-          text = g_strdup_printf (C_("input method menu", "System (%s)"), translated_name);
-          gtk_label_set_text (GTK_LABEL (label), text);
-          g_free (text);
-        }
-
-      group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
-
-      g_object_set_data (G_OBJECT (menuitem), I_("gtk-context-id"),
-			 (char *)contexts[i]->context_id);
-      g_signal_connect (menuitem, "activate",
-			G_CALLBACK (activate_cb), context);
-
-      gtk_widget_show (menuitem);
-      gtk_menu_shell_append (menushell, menuitem);
-    }
-
-  g_free (contexts);
-}
-
 /**
  * gtk_im_multicontext_get_context_id:
  * @context: a #GtkIMMulticontext
@@ -718,15 +553,18 @@ gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
  * Gets the id of the currently active slave of the @context.
  *
  * Returns: the id of the currently active slave
- *
- * Since: 2.16
  */
 const char *
 gtk_im_multicontext_get_context_id (GtkIMMulticontext *context)
 {
+  GtkIMMulticontextPrivate *priv = context->priv;
+
   g_return_val_if_fail (GTK_IS_IM_MULTICONTEXT (context), NULL);
 
-  return context->priv->context_id;
+  if (priv->context_id == NULL)
+    gtk_im_multicontext_get_slave (context);
+
+  return priv->context_id;
 }
 
 /**
@@ -738,8 +576,6 @@ gtk_im_multicontext_get_context_id (GtkIMMulticontext *context)
  *
  * This causes the currently active slave of @context to be
  * replaced by the slave corresponding to the new context id.
- *
- * Since: 2.16
  */
 void
 gtk_im_multicontext_set_context_id (GtkIMMulticontext *context,
