@@ -25,19 +25,20 @@
 
 #include "config.h"
 
-#include <string.h>
-
 #include "gtkmessagedialog.h"
+
 #include "gtkaccessible.h"
-#include "gtkbuildable.h"
-#include "gtklabel.h"
 #include "gtkbox.h"
-#include "gtkbbox.h"
-#include "gtkimage.h"
+#include "gtkbuildable.h"
+#include "gtkdialogprivate.h"
 #include "gtkintl.h"
+#include "gtklabel.h"
 #include "gtkprivate.h"
+#include "gtkstylecontext.h"
 #include "gtktypebuiltins.h"
 
+#include <string.h>
+
 /**
  * SECTION:gtkmessagedialog
  * @Short_description: A convenient message window
@@ -48,10 +49,6 @@
  * convenience widget; you could construct the equivalent of #GtkMessageDialog
  * from #GtkDialog without too much effort, but #GtkMessageDialog saves typing.
  *
- * One difference from #GtkDialog is that #GtkMessageDialog sets the
- * #GtkWindow:skip-taskbar-hint property to %TRUE, so that the dialog is hidden
- * from the taskbar by default.
- *
  * The easiest way to do a modal message dialog is to use gtk_dialog_run(), though
  * you can also pass in the %GTK_DIALOG_MODAL flag, gtk_dialog_run() automatically
  * makes the dialog modal and waits for the user to respond to it. gtk_dialog_run()
@@ -98,9 +95,8 @@
  * the message area as an internal child with the name “message_area”.
  */
 
-struct _GtkMessageDialogPrivate
+typedef struct
 {
-  GtkWidget     *image;
   GtkWidget     *label;
   GtkWidget     *message_area; /* vbox for the primary and secondary labels, and any extra content from the caller */
   GtkWidget     *secondary_label;
@@ -108,9 +104,7 @@ struct _GtkMessageDialogPrivate
   guint          has_primary_markup : 1;
   guint          has_secondary_text : 1;
   guint          message_type       : 3;
-};
-
-static void gtk_message_dialog_style_updated (GtkWidget       *widget);
+} GtkMessageDialogPrivate;
 
 static void gtk_message_dialog_constructed  (GObject          *object);
 static void gtk_message_dialog_set_property (GObject          *object,
@@ -161,23 +155,12 @@ gtk_message_dialog_class_init (GtkMessageDialogClass *class)
   widget_class = GTK_WIDGET_CLASS (class);
   gobject_class = G_OBJECT_CLASS (class);
   
-  widget_class->style_updated = gtk_message_dialog_style_updated;
-
   gtk_widget_class_set_accessible_role (widget_class, ATK_ROLE_ALERT);
 
   gobject_class->constructed = gtk_message_dialog_constructed;
   gobject_class->set_property = gtk_message_dialog_set_property;
   gobject_class->get_property = gtk_message_dialog_get_property;
   
-  gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_int ("message-border",
-                                                             P_("label border"),
-                                                             P_("Width of border around the label in the message dialog"),
-                                                             0,
-                                                             G_MAXINT,
-                                                             12,
-                                                             GTK_PARAM_READABLE));
-
   /**
    * GtkMessageDialog:message-type:
    *
@@ -205,8 +188,6 @@ gtk_message_dialog_class_init (GtkMessageDialogClass *class)
    * 
    * The primary text of the message dialog. If the dialog has 
    * a secondary text, this will appear as the title.
-   *
-   * Since: 2.10
    */
   g_object_class_install_property (gobject_class,
                                    PROP_TEXT,
@@ -221,8 +202,6 @@ gtk_message_dialog_class_init (GtkMessageDialogClass *class)
    * 
    * %TRUE if the primary text of the dialog includes Pango markup. 
    * See pango_parse_markup(). 
-   *
-   * Since: 2.10
    */
   g_object_class_install_property (gobject_class,
 				   PROP_USE_MARKUP,
@@ -236,8 +215,6 @@ gtk_message_dialog_class_init (GtkMessageDialogClass *class)
    * GtkMessageDialog:secondary-text:
    * 
    * The secondary text of the message dialog. 
-   *
-   * Since: 2.10
    */
   g_object_class_install_property (gobject_class,
                                    PROP_SECONDARY_TEXT,
@@ -252,8 +229,6 @@ gtk_message_dialog_class_init (GtkMessageDialogClass *class)
    * 
    * %TRUE if the secondary text of the dialog includes Pango markup. 
    * See pango_parse_markup(). 
-   *
-   * Since: 2.10
    */
   g_object_class_install_property (gobject_class,
 				   PROP_SECONDARY_USE_MARKUP,
@@ -263,36 +238,18 @@ gtk_message_dialog_class_init (GtkMessageDialogClass *class)
 							 FALSE,
 							 GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
-  /**
-   * GtkMessageDialog:image:
-   *
-   * The image for this dialog.
-   *
-   * Since: 2.10
-   * Deprecated: 3.12: Use #GtkDialog to create dialogs with images
-   */
-  g_object_class_install_property (gobject_class,
-                                   PROP_IMAGE,
-                                   g_param_spec_object ("image",
-                                                        P_("Image"),
-                                                        P_("The image"),
-                                                        GTK_TYPE_WIDGET,
-                                                        GTK_PARAM_READWRITE|G_PARAM_DEPRECATED));
-
   /**
    * GtkMessageDialog:message-area:
    *
    * The #GtkBox that corresponds to the message area of this dialog.  See
    * gtk_message_dialog_get_message_area() for a detailed description of this
    * area.
-   *
-   * Since: 2.22
    */
   g_object_class_install_property (gobject_class,
 				   PROP_MESSAGE_AREA,
 				   g_param_spec_object ("message-area",
 							P_("Message area"),
-							P_("GtkBox that holds the dialog's primary and secondary labels"),
+							P_("GtkBox that holds the dialog’s primary and secondary labels"),
 							GTK_TYPE_WIDGET,
 							GTK_PARAM_READABLE));
 
@@ -302,20 +259,17 @@ gtk_message_dialog_class_init (GtkMessageDialogClass *class)
   gtk_widget_class_bind_template_child_private (widget_class, GtkMessageDialog, secondary_label);
   gtk_widget_class_bind_template_child_internal_private (widget_class, GtkMessageDialog, message_area);
 
-  gtk_widget_class_set_css_name (widget_class, "messagedialog");
+  gtk_widget_class_set_css_name (widget_class, I_("messagedialog"));
 }
 
 static void
 gtk_message_dialog_init (GtkMessageDialog *dialog)
 {
-  GtkMessageDialogPrivate *priv;
+  GtkMessageDialogPrivate *priv = gtk_message_dialog_get_instance_private (dialog);
   GtkWidget *action_area;
   GtkSettings *settings;
   gboolean use_caret;
 
-  dialog->priv = gtk_message_dialog_get_instance_private (dialog);
-  priv = dialog->priv;
-
   priv->has_primary_markup = FALSE;
   priv->has_secondary_text = FALSE;
   priv->has_primary_markup = FALSE;
@@ -323,11 +277,9 @@ gtk_message_dialog_init (GtkMessageDialog *dialog)
   priv->message_type = GTK_MESSAGE_OTHER;
 
   gtk_widget_init_template (GTK_WIDGET (dialog));
-  gtk_message_dialog_style_updated (GTK_WIDGET (dialog));
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   action_area = gtk_dialog_get_action_area (GTK_DIALOG (dialog));
-G_GNUC_END_IGNORE_DEPRECATIONS
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (action_area), GTK_BUTTONBOX_EXPAND);
+  gtk_widget_set_halign (action_area, GTK_ALIGN_FILL);
+  gtk_box_set_homogeneous (GTK_BOX (action_area), TRUE);
 
   settings = gtk_widget_get_settings (GTK_WIDGET (dialog));
   g_object_get (settings, "gtk-keynav-use-caret", &use_caret, NULL);
@@ -338,7 +290,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 static void
 setup_primary_label_font (GtkMessageDialog *dialog)
 {
-  GtkMessageDialogPrivate *priv = dialog->priv;
+  GtkMessageDialogPrivate *priv = gtk_message_dialog_get_instance_private (dialog);
 
   if (!priv->has_primary_markup)
     {
@@ -366,55 +318,12 @@ setup_primary_label_font (GtkMessageDialog *dialog)
     }
 }
 
-static void
-update_accessible_name (GtkMessageDialog *dialog)
-{
-  AtkObject *atk_obj = gtk_widget_get_accessible (GTK_WIDGET (dialog));
-  if (!GTK_IS_ACCESSIBLE (atk_obj))
-    return;
-
-  const char *name = gtk_window_get_title (GTK_WINDOW (dialog));
-  if (name && name[0])
-  {
-    atk_object_set_name (atk_obj, name);
-    return;
-  }
-
-  switch (dialog->priv->message_type)
-  {
-  case GTK_MESSAGE_INFO:
-    name = _("Information");
-    break;
-
-  case GTK_MESSAGE_QUESTION:
-    name = _("Question");
-    break;
-
-  case GTK_MESSAGE_WARNING:
-    name = _("Warning");
-    break;
-
-  case GTK_MESSAGE_ERROR:
-    name = _("Error");
-    break;
-
-  case GTK_MESSAGE_OTHER:
-    break;
-
-  default:
-    g_warning ("Unknown GtkMessageType %u", dialog->priv->message_type);
-    break;
-  }
-
-  if (name)
-      atk_object_set_name (atk_obj, name);
-}
-
 static void
 setup_type (GtkMessageDialog *dialog,
 	    GtkMessageType    type)
 {
-  GtkMessageDialogPrivate *priv = dialog->priv;
+  GtkMessageDialogPrivate *priv = gtk_message_dialog_get_instance_private (dialog);
+  const gchar *name = NULL;
   AtkObject *atk_obj;
 
   if (priv->message_type == type)
@@ -422,12 +331,38 @@ setup_type (GtkMessageDialog *dialog,
 
   priv->message_type = type;
 
+  switch (type)
+    {
+    case GTK_MESSAGE_INFO:
+      name = _("Information");
+      break;
+
+    case GTK_MESSAGE_QUESTION:
+      name = _("Question");
+      break;
+
+    case GTK_MESSAGE_WARNING:
+      name = _("Warning");
+      break;
+
+    case GTK_MESSAGE_ERROR:
+      name = _("Error");
+      break;
+
+    case GTK_MESSAGE_OTHER:
+      break;
+
+    default:
+      g_warning ("Unknown GtkMessageType %u", type);
+      break;
+    }
 
   atk_obj = gtk_widget_get_accessible (GTK_WIDGET (dialog));
   if (GTK_IS_ACCESSIBLE (atk_obj))
     {
       atk_object_set_role (atk_obj, ATK_ROLE_ALERT);
-      update_accessible_name (dialog);
+      if (name)
+        atk_object_set_name (atk_obj, name);
     }
 
   g_object_notify (G_OBJECT (dialog), "message-type");
@@ -443,8 +378,6 @@ update_title (GObject    *dialog,
   title = gtk_window_get_title (GTK_WINDOW (dialog));
   gtk_label_set_label (GTK_LABEL (label), title);
   gtk_widget_set_visible (label, title && title[0]);
-
-  update_accessible_name (GTK_MESSAGE_DIALOG (dialog));
 }
 
 static void
@@ -468,11 +401,13 @@ gtk_message_dialog_constructed (GObject *object)
       gtk_widget_show (box);
       gtk_widget_set_size_request (box, -1, 16);
       label = gtk_label_new ("");
-      gtk_widget_set_no_show_all (label, TRUE);
+      gtk_widget_hide (label);
       gtk_widget_set_margin_top (label, 6);
       gtk_widget_set_margin_bottom (label, 6);
+      gtk_widget_set_halign (label, GTK_ALIGN_CENTER);
+      gtk_widget_set_hexpand (label, TRUE);
       gtk_style_context_add_class (gtk_widget_get_style_context (label), "title");
-      gtk_box_set_center_widget (GTK_BOX (box), label);
+      gtk_container_add (GTK_CONTAINER (box), label);
       g_signal_connect_object (dialog, "notify::title", G_CALLBACK (update_title), label, 0);
 
       gtk_window_set_titlebar (GTK_WINDOW (dialog), box);
@@ -486,7 +421,7 @@ gtk_message_dialog_set_property (GObject      *object,
 				 GParamSpec   *pspec)
 {
   GtkMessageDialog *dialog = GTK_MESSAGE_DIALOG (object);
-  GtkMessageDialogPrivate *priv = dialog->priv;
+  GtkMessageDialogPrivate *priv = gtk_message_dialog_get_instance_private (dialog);
 
   switch (prop_id)
     {
@@ -542,11 +477,6 @@ gtk_message_dialog_set_property (GObject      *object,
           g_object_notify_by_pspec (object, pspec);
         }
       break;
-    case PROP_IMAGE:
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      gtk_message_dialog_set_image (dialog, g_value_get_object (value));
-G_GNUC_END_IGNORE_DEPRECATIONS
-      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -561,7 +491,7 @@ gtk_message_dialog_get_property (GObject     *object,
 				 GParamSpec  *pspec)
 {
   GtkMessageDialog *dialog = GTK_MESSAGE_DIALOG (object);
-  GtkMessageDialogPrivate *priv = dialog->priv;
+  GtkMessageDialogPrivate *priv = gtk_message_dialog_get_instance_private (dialog);
 
   switch (prop_id)
     {
@@ -588,9 +518,6 @@ gtk_message_dialog_get_property (GObject     *object,
       else
 	g_value_set_boolean (value, FALSE);
       break;
-    case PROP_IMAGE:
-      g_value_set_object (value, priv->image);
-      break;
     case PROP_MESSAGE_AREA:
       g_value_set_object (value, priv->message_area);
       break;
@@ -640,18 +567,19 @@ gtk_message_dialog_new (GtkWindow     *parent,
 
   if (message_format)
     {
+      GtkMessageDialogPrivate *priv = gtk_message_dialog_get_instance_private ((GtkMessageDialog*)dialog);
       va_start (args, message_format);
       msg = g_strdup_vprintf (message_format, args);
       va_end (args);
 
-      gtk_label_set_text (GTK_LABEL (GTK_MESSAGE_DIALOG (widget)->priv->label), msg);
+      gtk_label_set_text (GTK_LABEL (priv->label), msg);
 
       g_free (msg);
     }
 
   if (parent != NULL)
     gtk_window_set_transient_for (GTK_WINDOW (widget), GTK_WINDOW (parent));
-  
+
   if (flags & GTK_DIALOG_MODAL)
     gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 
@@ -697,8 +625,6 @@ gtk_message_dialog_new (GtkWindow     *parent,
  * ]|
  * 
  * Returns: a new #GtkMessageDialog
- *
- * Since: 2.4
  **/
 GtkWidget*
 gtk_message_dialog_new_with_markup (GtkWindow     *parent,
@@ -730,67 +656,6 @@ gtk_message_dialog_new_with_markup (GtkWindow     *parent,
   return widget;
 }
 
-/**
- * gtk_message_dialog_set_image:
- * @dialog: a #GtkMessageDialog
- * @image: the image
- * 
- * Sets the dialog’s image to @image.
- *
- * Since: 2.10
- * Deprecated: 3.12: Use #GtkDialog to create dialogs with images
- **/
-void
-gtk_message_dialog_set_image (GtkMessageDialog *dialog,
-			      GtkWidget        *image)
-{
-  GtkMessageDialogPrivate *priv;
-  GtkWidget *parent;
-
-  g_return_if_fail (GTK_IS_MESSAGE_DIALOG (dialog));
-  g_return_if_fail (image == NULL || GTK_IS_WIDGET (image));
-
-  priv = dialog->priv;
-  
-  if (priv->image)
-    gtk_widget_destroy (priv->image);
-
-  priv->image = image;
- 
-  if (priv->image)
-    { 
-      gtk_widget_set_halign (priv->image, GTK_ALIGN_CENTER);
-      gtk_widget_set_valign (priv->image, GTK_ALIGN_START);
-      parent = gtk_widget_get_parent (priv->message_area);
-      gtk_container_add (GTK_CONTAINER (parent), priv->image);
-      gtk_box_reorder_child (GTK_BOX (parent), priv->image, 0);
-    }
-
-  priv->message_type = GTK_MESSAGE_OTHER;
-
-  g_object_notify (G_OBJECT (dialog), "image");
-  g_object_notify (G_OBJECT (dialog), "message-type");
-}
-
-/**
- * gtk_message_dialog_get_image:
- * @dialog: a #GtkMessageDialog
- *
- * Gets the dialog’s image.
- *
- * Returns: (transfer none): the dialog’s image
- *
- * Since: 2.14
- * Deprecated: 3.12: Use #GtkDialog for dialogs with images
- **/
-GtkWidget *
-gtk_message_dialog_get_image (GtkMessageDialog *dialog)
-{
-  g_return_val_if_fail (GTK_IS_MESSAGE_DIALOG (dialog), NULL);
-
-  return dialog->priv->image;
-}
-
 /**
  * gtk_message_dialog_set_markup:
  * @message_dialog: a #GtkMessageDialog
@@ -798,19 +663,15 @@ gtk_message_dialog_get_image (GtkMessageDialog *dialog)
  * 
  * Sets the text of the message dialog to be @str, which is marked
  * up with the [Pango text markup language][PangoMarkupFormat].
- *
- * Since: 2.4
  **/
 void
 gtk_message_dialog_set_markup (GtkMessageDialog *message_dialog,
                                const gchar      *str)
 {
-  GtkMessageDialogPrivate *priv;
+  GtkMessageDialogPrivate *priv = gtk_message_dialog_get_instance_private (message_dialog);
 
   g_return_if_fail (GTK_IS_MESSAGE_DIALOG (message_dialog));
 
-  priv = message_dialog->priv;
-
   priv->has_primary_markup = TRUE;
   gtk_label_set_markup (GTK_LABEL (priv->label), str);
 }
@@ -823,22 +684,18 @@ gtk_message_dialog_set_markup (GtkMessageDialog *message_dialog,
  *
  * Sets the secondary text of the message dialog to be @message_format
  * (with printf()-style).
- *
- * Since: 2.6
  */
 void
 gtk_message_dialog_format_secondary_text (GtkMessageDialog *message_dialog,
                                           const gchar      *message_format,
                                           ...)
 {
+  GtkMessageDialogPrivate *priv = gtk_message_dialog_get_instance_private (message_dialog);
   va_list args;
   gchar *msg = NULL;
-  GtkMessageDialogPrivate *priv;
 
   g_return_if_fail (GTK_IS_MESSAGE_DIALOG (message_dialog));
 
-  priv = message_dialog->priv;
-
   if (message_format)
     {
       priv->has_secondary_text = TRUE;
@@ -885,22 +742,18 @@ gtk_message_dialog_format_secondary_text (GtkMessageDialog *message_dialog,
  *                                             "%s", msg);
  * g_free (msg);
  * ]|
- *
- * Since: 2.6
  */
 void
 gtk_message_dialog_format_secondary_markup (GtkMessageDialog *message_dialog,
                                             const gchar      *message_format,
                                             ...)
 {
+  GtkMessageDialogPrivate *priv = gtk_message_dialog_get_instance_private (message_dialog);
   va_list args;
   gchar *msg = NULL;
-  GtkMessageDialogPrivate *priv;
 
   g_return_if_fail (GTK_IS_MESSAGE_DIALOG (message_dialog));
 
-  priv = message_dialog->priv;
-
   if (message_format)
     {
       priv->has_secondary_text = TRUE;
@@ -935,15 +788,15 @@ gtk_message_dialog_format_secondary_markup (GtkMessageDialog *message_dialog,
  *
  * Returns: (transfer none): A #GtkBox corresponding to the
  *     “message area” in the @message_dialog.
- *
- * Since: 2.22
  **/
 GtkWidget *
 gtk_message_dialog_get_message_area (GtkMessageDialog *message_dialog)
 {
+  GtkMessageDialogPrivate *priv = gtk_message_dialog_get_instance_private (message_dialog);
+
   g_return_val_if_fail (GTK_IS_MESSAGE_DIALOG (message_dialog), NULL);
 
-  return message_dialog->priv->message_area;
+  return priv->message_area;
 }
 
 static void
@@ -952,7 +805,6 @@ gtk_message_dialog_add_buttons (GtkMessageDialog* message_dialog,
 {
   GtkDialog* dialog = GTK_DIALOG (message_dialog);
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   switch (buttons)
     {
     case GTK_BUTTONS_NONE:
@@ -974,47 +826,18 @@ G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     case GTK_BUTTONS_YES_NO:
       gtk_dialog_add_button (dialog, _("_No"), GTK_RESPONSE_NO);
       gtk_dialog_add_button (dialog, _("_Yes"), GTK_RESPONSE_YES);
-      gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-					       GTK_RESPONSE_YES,
-					       GTK_RESPONSE_NO,
-					       -1);
       break;
 
     case GTK_BUTTONS_OK_CANCEL:
       gtk_dialog_add_button (dialog, _("_Cancel"), GTK_RESPONSE_CANCEL);
       gtk_dialog_add_button (dialog, _("_OK"), GTK_RESPONSE_OK);
-      gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-					       GTK_RESPONSE_OK,
-					       GTK_RESPONSE_CANCEL,
-					       -1);
       break;
       
     default:
       g_warning ("Unknown GtkButtonsType");
       break;
     } 
-G_GNUC_END_IGNORE_DEPRECATIONS
 
   g_object_notify (G_OBJECT (message_dialog), "buttons");
 }
 
-static void
-gtk_message_dialog_style_updated (GtkWidget *widget)
-{
-  GtkMessageDialog *dialog = GTK_MESSAGE_DIALOG (widget);
-  GtkWidget *parent;
-  gint border_width;
-
-  parent = gtk_widget_get_parent (dialog->priv->message_area);
-
-  if (parent)
-    {
-      gtk_widget_style_get (widget, "message-border",
-                            &border_width, NULL);
-
-      gtk_container_set_border_width (GTK_CONTAINER (parent),
-                                      MAX (0, border_width - 7));
-    }
-
-  GTK_WIDGET_CLASS (gtk_message_dialog_parent_class)->style_updated (widget);
-}
