@@ -28,18 +28,16 @@
 #include "gtkimage.h"
 #include "gtklabel.h"
 #include "gtkbox.h"
-#include "gtkrender.h"
 #include "gtkstylecontext.h"
 #include "gtktypebuiltins.h"
 #include "gtkstack.h"
 #include "gtkpopover.h"
 #include "gtkintl.h"
 #include "gtkcssnodeprivate.h"
-#include "gtkcsscustomgadgetprivate.h"
-#include "gtkbuiltiniconprivate.h"
 #include "gtkcsstypesprivate.h"
 #include "gtkstylecontextprivate.h"
 #include "gtkcontainerprivate.h"
+#include "gtkiconprivate.h"
 
 /**
  * SECTION:gtkmodelbutton
@@ -154,8 +152,7 @@ struct _GtkModelButton
   GtkWidget *box;
   GtkWidget *image;
   GtkWidget *label;
-  GtkCssGadget *gadget;
-  GtkCssGadget *indicator_gadget;
+  GtkWidget *indicator_widget;
   gboolean active;
   gboolean centered;
   gboolean inverted;
@@ -195,8 +192,7 @@ indicator_is_left (GtkWidget *widget)
 }
 
 static void
-gtk_model_button_update_state (GtkModelButton *button,
-                               GtkStateFlags   previous_flags)
+gtk_model_button_update_state (GtkModelButton *button)
 {
   GtkStateFlags state;
   GtkStateFlags indicator_state;
@@ -205,8 +201,6 @@ gtk_model_button_update_state (GtkModelButton *button,
   state = gtk_widget_get_state_flags (GTK_WIDGET (button));
   indicator_state = state;
 
-  gtk_css_gadget_set_state (button->gadget, state);
-
   if (button->role == GTK_BUTTON_ROLE_CHECK)
     {
       if (button->active && !button->menu_name)
@@ -240,51 +234,41 @@ gtk_model_button_update_state (GtkModelButton *button,
         image_type = GTK_CSS_IMAGE_BUILTIN_ARROW_RIGHT;
     }
 
-  gtk_builtin_icon_set_image (GTK_BUILTIN_ICON (button->indicator_gadget), image_type);
+  gtk_icon_set_image (GTK_ICON (button->indicator_widget), image_type);
 
   if (button->iconic)
-    gtk_css_gadget_set_state (button->gadget, indicator_state);
+    gtk_widget_set_state_flags (GTK_WIDGET (button), indicator_state, TRUE);
   else
-    gtk_css_gadget_set_state (button->gadget, state);
+    gtk_widget_set_state_flags (GTK_WIDGET (button), state, TRUE);
 
-  gtk_css_gadget_set_state (button->indicator_gadget, indicator_state);
-
-  if (button->role == GTK_BUTTON_ROLE_CHECK ||
-      button->role == GTK_BUTTON_ROLE_RADIO)
-    {
-      AtkObject *object = _gtk_widget_peek_accessible (GTK_WIDGET (button));
-      gboolean was_checked = (previous_flags & GTK_STATE_FLAG_CHECKED) != 0;
-      gboolean is_checked = (indicator_state & GTK_STATE_FLAG_CHECKED) != 0;
-      if (object && (was_checked != is_checked))
-        atk_object_notify_state_change (object, ATK_STATE_CHECKED, is_checked);
-    }
+  gtk_widget_set_state_flags (button->indicator_widget, indicator_state, TRUE);
 }
 
 static void
 update_node_ordering (GtkModelButton *button)
 {
-  GtkCssNode *widget_node, *indicator_node, *node;
+  GtkStyleContext *indicator_context;
+  GtkWidget *child;
 
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (button));
-  indicator_node = gtk_css_gadget_get_node (button->indicator_gadget);
+  indicator_context = gtk_widget_get_style_context (button->indicator_widget);
 
   if (indicator_is_left (GTK_WIDGET (button)))
     {
-      gtk_css_gadget_add_class (button->indicator_gadget, GTK_STYLE_CLASS_LEFT);
-      gtk_css_gadget_remove_class (button->indicator_gadget, GTK_STYLE_CLASS_RIGHT);
+      gtk_style_context_add_class (indicator_context, GTK_STYLE_CLASS_LEFT);
+      gtk_style_context_remove_class (indicator_context, GTK_STYLE_CLASS_RIGHT);
 
-      node = gtk_css_node_get_first_child (widget_node);
-      if (node != indicator_node)
-        gtk_css_node_insert_before (widget_node, indicator_node, node);
+      child = gtk_widget_get_first_child (GTK_WIDGET (button));
+      if (child != button->indicator_widget)
+        gtk_widget_insert_before (button->indicator_widget, GTK_WIDGET (button), child);
     }
   else
     {
-      gtk_css_gadget_remove_class (button->indicator_gadget, GTK_STYLE_CLASS_LEFT);
-      gtk_css_gadget_add_class (button->indicator_gadget, GTK_STYLE_CLASS_RIGHT);
+      gtk_style_context_add_class (indicator_context, GTK_STYLE_CLASS_RIGHT);
+      gtk_style_context_remove_class (indicator_context, GTK_STYLE_CLASS_LEFT);
 
-      node = gtk_css_node_get_last_child (widget_node);
-      if (node != indicator_node)
-        gtk_css_node_insert_after (widget_node, indicator_node, node);
+      child = gtk_widget_get_first_child (GTK_WIDGET (button));
+      if (child != button->indicator_widget)
+        gtk_widget_insert_after (button->indicator_widget, GTK_WIDGET (button), child);
     }
 }
 
@@ -292,7 +276,7 @@ static void
 gtk_model_button_state_flags_changed (GtkWidget     *widget,
                                       GtkStateFlags  previous_flags)
 {
-  gtk_model_button_update_state (GTK_MODEL_BUTTON (widget), previous_flags);
+  gtk_model_button_update_state (GTK_MODEL_BUTTON (widget));
 
   GTK_WIDGET_CLASS (gtk_model_button_parent_class)->state_flags_changed (widget, previous_flags);
 }
@@ -303,7 +287,7 @@ gtk_model_button_direction_changed (GtkWidget        *widget,
 {
   GtkModelButton *button = GTK_MODEL_BUTTON (widget);
 
-  gtk_model_button_update_state (button, GTK_STATE_FLAG_NORMAL);
+  gtk_model_button_update_state (button);
   update_node_ordering (button);
 
   GTK_WIDGET_CLASS (gtk_model_button_parent_class)->direction_changed (widget, previous_dir);
@@ -316,7 +300,6 @@ update_node_name (GtkModelButton *button)
   AtkRole a11y_role;
   const gchar *indicator_name;
   gboolean indicator_visible;
-  GtkCssNode *indicator_node;
 
   accessible = gtk_widget_get_accessible (GTK_WIDGET (button));
   switch (button->role)
@@ -356,9 +339,8 @@ update_node_name (GtkModelButton *button)
 
   atk_object_set_role (accessible, a11y_role);
 
-  indicator_node = gtk_css_gadget_get_node (button->indicator_gadget);
-  gtk_css_node_set_name (indicator_node, indicator_name);
-  gtk_css_node_set_visible (indicator_node, indicator_visible);
+  gtk_icon_set_css_name (GTK_ICON (button->indicator_widget), indicator_name);
+  gtk_widget_set_visible (button->indicator_widget, indicator_visible);
 }
 
 static void
@@ -372,7 +354,7 @@ gtk_model_button_set_role (GtkModelButton *button,
 
   update_node_name (button);
 
-  gtk_model_button_update_state (button, GTK_STATE_FLAG_NORMAL);
+  gtk_model_button_update_state (button);
   gtk_widget_queue_draw (GTK_WIDGET (button));
   g_object_notify_by_pspec (G_OBJECT (button), properties[PROP_ROLE]);
 }
@@ -394,7 +376,7 @@ static void
 gtk_model_button_set_icon (GtkModelButton *button,
                            GIcon          *icon)
 {
-  gtk_image_set_from_gicon (GTK_IMAGE (button->image), icon, GTK_ICON_SIZE_MENU);
+  gtk_image_set_from_gicon (GTK_IMAGE (button->image), icon);
   update_visibility (button);
   g_object_notify_by_pspec (G_OBJECT (button), properties[PROP_ICON]);
 }
@@ -430,7 +412,7 @@ gtk_model_button_set_active (GtkModelButton *button,
     return;
 
   button->active = active;
-  gtk_model_button_update_state (button, GTK_STATE_FLAG_NORMAL);
+  gtk_model_button_update_state (button);
   gtk_widget_queue_draw (GTK_WIDGET (button));
   g_object_notify_by_pspec (G_OBJECT (button), properties[PROP_ACTIVE]);
 }
@@ -443,7 +425,7 @@ gtk_model_button_set_menu_name (GtkModelButton *button,
   button->menu_name = g_strdup (menu_name);
 
   update_node_name (button);
-  gtk_model_button_update_state (button, GTK_STATE_FLAG_NORMAL);
+  gtk_model_button_update_state (button);
 
   gtk_widget_queue_resize (GTK_WIDGET (button));
   g_object_notify_by_pspec (G_OBJECT (button), properties[PROP_MENU_NAME]);
@@ -458,7 +440,7 @@ gtk_model_button_set_inverted (GtkModelButton *button,
     return;
 
   button->inverted = inverted;
-  gtk_model_button_update_state (button, GTK_STATE_FLAG_NORMAL);
+  gtk_model_button_update_state (button);
   update_node_ordering (button);
   gtk_widget_queue_resize (GTK_WIDGET (button));
   g_object_notify_by_pspec (G_OBJECT (button), properties[PROP_INVERTED]);
@@ -483,7 +465,7 @@ gtk_model_button_set_iconic (GtkModelButton *button,
                              gboolean        iconic)
 {
   GtkCssNode *widget_node;
-  GtkCssNode *indicator_node;
+  GtkStyleContext *context;
 
   iconic = !!iconic;
   if (button->iconic == iconic)
@@ -492,24 +474,24 @@ gtk_model_button_set_iconic (GtkModelButton *button,
   button->iconic = iconic;
 
   widget_node = gtk_widget_get_css_node (GTK_WIDGET (button));
-  indicator_node = gtk_css_gadget_get_node (button->indicator_gadget);
+  context = gtk_widget_get_style_context (GTK_WIDGET (button));
   if (iconic)
     {
       gtk_css_node_set_name (widget_node, I_("button"));
-      gtk_css_gadget_add_class (button->gadget, "model");
-      gtk_css_gadget_add_class (button->gadget, "image-button");
+      gtk_style_context_add_class (context, "model");
+      gtk_style_context_add_class (context, "image-button");
       gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NORMAL);
-      gtk_css_node_set_visible (indicator_node, FALSE);
+      gtk_widget_set_visible (button->indicator_widget, FALSE);
     }
   else
     {
       gtk_css_node_set_name (widget_node, I_("modelbutton"));
-      gtk_css_gadget_remove_class (button->gadget, "model");
-      gtk_css_gadget_remove_class (button->gadget, "image-button");
+      gtk_style_context_remove_class (context, "model");
+      gtk_style_context_remove_class (context, "image-button");
       gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-      gtk_css_node_set_visible (indicator_node,
-                                button->role != GTK_BUTTON_ROLE_NORMAL ||
-                                button->menu_name == NULL);
+      gtk_widget_set_visible (button->indicator_widget,
+                              button->role != GTK_BUTTON_ROLE_NORMAL ||
+                              button->menu_name == NULL);
     }
 
   update_visibility (button);
@@ -532,11 +514,7 @@ gtk_model_button_get_property (GObject    *object,
       break;
 
     case PROP_ICON:
-      {
-        GIcon *icon;
-        gtk_image_get_gicon (GTK_IMAGE (button->image), &icon, NULL);
-        g_value_set_object (value, icon);
-      }
+      g_value_set_object (value, gtk_image_get_gicon (GTK_IMAGE (button->image)));
       break;
 
     case PROP_TEXT:
@@ -671,390 +649,209 @@ needs_indicator (GtkModelButton *button)
 }
 
 static void
-gtk_model_button_get_preferred_width_for_height (GtkWidget *widget,
-                                                 gint       height,
-                                                 gint      *minimum,
-                                                 gint      *natural)
-{
-  GtkCssGadget *gadget;
-
-  if (GTK_MODEL_BUTTON (widget)->iconic)
-    gadget = GTK_BUTTON (widget)->priv->gadget;
-  else
-    gadget = GTK_MODEL_BUTTON (widget)->gadget;
-
-  gtk_css_gadget_get_preferred_size (gadget,
-                                     GTK_ORIENTATION_HORIZONTAL,
-                                     height,
-                                     minimum, natural,
-                                     NULL, NULL);
-}
-
-static void
-gtk_model_button_get_preferred_width (GtkWidget *widget,
-                                      gint      *minimum,
-                                      gint      *natural)
-{
-  GtkCssGadget *gadget;
-
-  if (GTK_MODEL_BUTTON (widget)->iconic)
-    gadget = GTK_BUTTON (widget)->priv->gadget;
-  else
-    gadget = GTK_MODEL_BUTTON (widget)->gadget;
-
-  gtk_css_gadget_get_preferred_size (gadget,
-                                     GTK_ORIENTATION_HORIZONTAL,
-                                     -1,
-                                     minimum, natural,
-                                     NULL, NULL);
-}
-
-static void
-gtk_model_button_get_preferred_height_and_baseline_for_width (GtkWidget *widget,
-                                                              gint       width,
-                                                              gint      *minimum,
-                                                              gint      *natural,
-                                                              gint      *minimum_baseline,
-                                                              gint      *natural_baseline)
-{
-  GtkCssGadget *gadget;
-
-  if (GTK_MODEL_BUTTON (widget)->iconic)
-    gadget = GTK_BUTTON (widget)->priv->gadget;
-  else
-    gadget = GTK_MODEL_BUTTON (widget)->gadget;
-
-  gtk_css_gadget_get_preferred_size (gadget,
-                                     GTK_ORIENTATION_VERTICAL,
-                                     width,
-                                     minimum, natural,
-                                     minimum_baseline, natural_baseline);
-}
-
-static void
-gtk_model_button_get_preferred_height_for_width (GtkWidget *widget,
-                                                 gint       width,
-                                                 gint      *minimum,
-                                                 gint      *natural)
-{
-  GtkCssGadget *gadget;
-
-  if (GTK_MODEL_BUTTON (widget)->iconic)
-    gadget = GTK_BUTTON (widget)->priv->gadget;
-  else
-    gadget = GTK_MODEL_BUTTON (widget)->gadget;
-
-  gtk_css_gadget_get_preferred_size (gadget,
-                                     GTK_ORIENTATION_VERTICAL,
-                                     width,
-                                     minimum, natural,
-                                     NULL, NULL);
-}
-
-static void
-gtk_model_button_get_preferred_height (GtkWidget *widget,
-                                       gint      *minimum,
-                                       gint      *natural)
-{
-  GtkCssGadget *gadget;
-
-  if (GTK_MODEL_BUTTON (widget)->iconic)
-    gadget = GTK_BUTTON (widget)->priv->gadget;
-  else
-    gadget = GTK_MODEL_BUTTON (widget)->gadget;
-
-  gtk_css_gadget_get_preferred_size (gadget,
-                                     GTK_ORIENTATION_VERTICAL,
-                                     -1,
-                                     minimum, natural,
-                                     NULL, NULL);
-}
-
-static void
-gtk_model_button_measure (GtkCssGadget   *gadget,
+gtk_model_button_measure (GtkWidget      *widget,
                           GtkOrientation  orientation,
                           int             for_size,
                           int            *minimum,
                           int            *natural,
                           int            *minimum_baseline,
-                          int            *natural_baseline,
-                          gpointer        data)
+                          int            *natural_baseline)
 {
-  GtkWidget *widget;
-  GtkModelButton *button;
-  GtkWidget *child;
-
-  widget = gtk_css_gadget_get_owner (gadget);
-  button = GTK_MODEL_BUTTON (widget);
-  child = gtk_bin_get_child (GTK_BIN (widget));
-
-  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+  if (GTK_MODEL_BUTTON (widget)->iconic)
     {
-      gint check_min, check_nat;
-
-      gtk_css_gadget_get_preferred_size (button->indicator_gadget,
-                                         GTK_ORIENTATION_HORIZONTAL,
-                                         -1,
-                                         &check_min, &check_nat,
-                                         NULL, NULL);
-
-      if (child && gtk_widget_get_visible (child))
-        {
-          _gtk_widget_get_preferred_size_for_size (child,
-                                                   GTK_ORIENTATION_HORIZONTAL,
-                                                   for_size,
-                                                   minimum, natural,
-                                                   minimum_baseline, natural_baseline);
-        }
-      else
-        {
-          *minimum = 0;
-          *natural = 0;
-        }
-
-      if (button->centered)
-        {
-          *minimum += 2 * check_min;
-          *natural += 2 * check_nat;
-        }
-      else if (needs_indicator (button))
-        {
-          *minimum += check_min;
-          *natural += check_nat;
-        }
+      GTK_WIDGET_CLASS (gtk_model_button_parent_class)->measure (widget, orientation, for_size,
+                                                                 minimum, natural,
+                                                                 minimum_baseline, natural_baseline);
     }
   else
     {
-      gint check_min, check_nat;
+      GtkModelButton *button;
+      GtkWidget *child;
 
-      gtk_css_gadget_get_preferred_size (button->indicator_gadget,
-                                         GTK_ORIENTATION_VERTICAL,
-                                         -1,
-                                         &check_min, &check_nat,
-                                         NULL, NULL);
+      button = GTK_MODEL_BUTTON (widget);
+      child = gtk_bin_get_child (GTK_BIN (widget));
 
-      if (child && gtk_widget_get_visible (child))
+      if (orientation == GTK_ORIENTATION_HORIZONTAL)
         {
-          gint child_min, child_nat;
-          gint child_min_baseline = -1, child_nat_baseline = -1;
+          gint check_min, check_nat;
 
-          if (for_size > -1)
-            {
-              if (button->centered)
-                for_size -= 2 * check_nat;
-              else if (needs_indicator (button))
-                for_size -= check_nat;
-            }
+          gtk_widget_measure (button->indicator_widget,
+                              GTK_ORIENTATION_HORIZONTAL,
+                              -1,
+                              &check_min, &check_nat,
+                               NULL, NULL);
 
-          gtk_widget_get_preferred_height_and_baseline_for_width (child, for_size,
-                                                                  &child_min, &child_nat,
-                                                                  &child_min_baseline, &child_nat_baseline);
-
-          if (button->centered)
+          if (child && gtk_widget_get_visible (child))
             {
-              *minimum = MAX (2 * check_min, child_min);
-              *natural = MAX (2 * check_nat, child_nat);
-            }
-          else if (needs_indicator (button))
-            {
-              *minimum = MAX (check_min, child_min);
-              *natural = MAX (check_nat, child_nat);
-            }
-          else
-            {
-              *minimum = child_min;
-              *natural = child_nat;
-            }
-
-          if (minimum_baseline && child_min_baseline >= 0)
-            *minimum_baseline = child_min_baseline + (*minimum - child_min) / 2;
-          if (natural_baseline && child_nat_baseline >= 0)
-            *natural_baseline = child_nat_baseline + (*natural - child_nat) / 2;
-        }
-      else
-        {
-          if (button->centered)
-            {
-              *minimum = 2 * check_min;
-              *natural = 2 * check_nat;
-            }
-          else if (needs_indicator (button))
-            {
-              *minimum = check_min;
-              *natural = check_nat;
+              gtk_widget_measure (child,
+                                  orientation,
+                                  for_size,
+                                  minimum, natural,
+                                  minimum_baseline, natural_baseline);
             }
           else
             {
               *minimum = 0;
               *natural = 0;
             }
+
+          if (button->centered)
+            {
+              *minimum += 2 * check_min;
+              *natural += 2 * check_nat;
+            }
+          else if (needs_indicator (button))
+            {
+              *minimum += check_min;
+              *natural += check_nat;
+            }
         }
-    }
-}
-
-static void
-gtk_model_button_size_allocate (GtkWidget     *widget,
-                                GtkAllocation *allocation)
-{
-  GtkCssGadget *gadget;
-  GdkRectangle clip;
-
-  if (GTK_MODEL_BUTTON (widget)->iconic)
-    gadget = GTK_BUTTON (widget)->priv->gadget;
-  else
-    gadget = GTK_MODEL_BUTTON (widget)->gadget;
-
-  gtk_widget_set_allocation (widget, allocation);
-  gtk_css_gadget_allocate (gadget,
-                           allocation,
-                           gtk_widget_get_allocated_baseline (widget),
-                           &clip);
-
-  gtk_widget_set_clip (widget, &clip);
-}
-
-static void
-gtk_model_button_allocate (GtkCssGadget        *gadget,
-                           const GtkAllocation *allocation,
-                           int                  baseline,
-                           GtkAllocation       *out_clip,
-                           gpointer             unused)
-{
-  GtkWidget *widget;
-  GtkModelButton *button;
-  PangoContext *pango_context;
-  PangoFontMetrics *metrics;
-  GtkAllocation child_allocation;
-  GtkWidget *child;
-  gint check_min_width, check_nat_width;
-  gint check_min_height, check_nat_height;
-  GdkRectangle check_clip;
-
-  widget = gtk_css_gadget_get_owner (gadget);
-  button = GTK_MODEL_BUTTON (widget);
-  child = gtk_bin_get_child (GTK_BIN (widget));
-
-
-
-  gtk_css_gadget_get_preferred_size (button->indicator_gadget,
-                                     GTK_ORIENTATION_HORIZONTAL,
-                                     -1,
-                                     &check_min_width, &check_nat_width,
-                                     NULL, NULL);
-  gtk_css_gadget_get_preferred_size (button->indicator_gadget,
-                                     GTK_ORIENTATION_VERTICAL,
-                                     -1,
-                                     &check_min_height, &check_nat_height,
-                                     NULL, NULL);
-
-  if (indicator_is_left (widget))
-    child_allocation.x = allocation->x;
-  else
-    child_allocation.x = allocation->x + allocation->width - check_nat_width;
-  child_allocation.y = allocation->y + (allocation->height - check_nat_height) / 2;
-  child_allocation.width = check_nat_width;
-  child_allocation.height = check_nat_height;
-
-  gtk_css_gadget_allocate (button->indicator_gadget,
-                           &child_allocation,
-                           baseline,
-                           &check_clip);
-
-  if (child && gtk_widget_get_visible (child))
-    {
-      GtkBorder border = { 0, };
-
-      if (button->centered)
+      else
         {
-          border.left = check_nat_width;
-          border.right = check_nat_width;
-        }
-      else if (needs_indicator (button))
-        {
-          if (indicator_is_left (widget))
-            border.left += check_nat_width;
+          gint check_min, check_nat;
+
+          gtk_widget_measure (button->indicator_widget,
+                              GTK_ORIENTATION_VERTICAL,
+                              -1,
+                              &check_min, &check_nat,
+                              NULL, NULL);
+
+          if (child && gtk_widget_get_visible (child))
+            {
+              gint child_min, child_nat;
+              gint child_min_baseline = -1, child_nat_baseline = -1;
+
+              if (for_size > -1)
+                {
+                  if (button->centered)
+                    for_size -= 2 * check_nat;
+                  else if (needs_indicator (button))
+                    for_size -= check_nat;
+                }
+
+              gtk_widget_measure (child, GTK_ORIENTATION_VERTICAL,
+                                  for_size,
+                                  &child_min, &child_nat,
+                                  &child_min_baseline, &child_nat_baseline);
+
+              if (button->centered)
+                {
+                  *minimum = MAX (2 * check_min, child_min);
+                  *natural = MAX (2 * check_nat, child_nat);
+                }
+              else if (needs_indicator (button))
+                {
+                  *minimum = MAX (check_min, child_min);
+                  *natural = MAX (check_nat, child_nat);
+                }
+              else
+                {
+                  *minimum = child_min;
+                  *natural = child_nat;
+                }
+
+              if (minimum_baseline && child_min_baseline >= 0)
+                *minimum_baseline = child_min_baseline + (*minimum - child_min) / 2;
+              if (natural_baseline && child_nat_baseline >= 0)
+                *natural_baseline = child_nat_baseline + (*natural - child_nat) / 2;
+            }
           else
-            border.right += check_nat_width;
+            {
+              if (button->centered)
+                {
+                  *minimum = 2 * check_min;
+                  *natural = 2 * check_nat;
+                }
+              else if (needs_indicator (button))
+                {
+                  *minimum = check_min;
+                  *natural = check_nat;
+                }
+              else
+                {
+                  *minimum = 0;
+                  *natural = 0;
+                }
+            }
         }
-
-      child_allocation.x = allocation->x + border.left;
-      child_allocation.y = allocation->y + border.top;
-      child_allocation.width = allocation->width - border.left - border.right;
-      child_allocation.height = allocation->height - border.top - border.bottom;
-
-      baseline = gtk_widget_get_allocated_baseline (widget);
-      if (baseline != -1)
-        baseline -= border.top;
-
-      gtk_widget_size_allocate_with_baseline (child, &child_allocation, baseline);
     }
-
-  pango_context = gtk_widget_get_pango_context (widget);
-  metrics = pango_context_get_metrics (pango_context,
-                                       pango_context_get_font_description (pango_context),
-                                       pango_context_get_language (pango_context));
-  GTK_BUTTON (button)->priv->baseline_align =
-    (double)pango_font_metrics_get_ascent (metrics) /
-    (pango_font_metrics_get_ascent (metrics) + pango_font_metrics_get_descent (metrics));
-  pango_font_metrics_unref (metrics);
-
-
-  if (gtk_widget_get_realized (widget))
-    {
-      GtkAllocation border_allocation;
-      gtk_css_gadget_get_border_allocation (gadget, &border_allocation, NULL);
-
-      gdk_window_move_resize (gtk_button_get_event_window (GTK_BUTTON (widget)),
-                              border_allocation.x,
-                              border_allocation.y,
-                              border_allocation.width,
-                              border_allocation.height);
-    }
-
-  gtk_container_get_children_clip (GTK_CONTAINER (widget), out_clip);
-  gdk_rectangle_union (out_clip, &check_clip, out_clip);
 }
 
-static gint
-gtk_model_button_draw (GtkWidget *widget,
-                       cairo_t   *cr)
+static void
+gtk_model_button_size_allocate (GtkWidget *widget,
+                                int        width,
+                                int        height,
+                                int        baseline)
 {
-  GtkCssGadget *gadget;
-
   if (GTK_MODEL_BUTTON (widget)->iconic)
-    gadget = GTK_BUTTON (widget)->priv->gadget;
+    {
+      GTK_WIDGET_CLASS (gtk_model_button_parent_class)->size_allocate (widget,
+                                                                       width,
+                                                                       height,
+                                                                       baseline);
+    }
   else
-    gadget = GTK_MODEL_BUTTON (widget)->gadget;
+    {
+      GtkModelButton *button;
+      GtkAllocation child_allocation;
+      GtkWidget *child;
+      gint check_min_width, check_nat_width;
+      gint check_min_height, check_nat_height;
 
-  gtk_css_gadget_draw (gadget, cr);
+      button = GTK_MODEL_BUTTON (widget);
+      child = gtk_bin_get_child (GTK_BIN (widget));
 
-  return FALSE;
-}
 
-static gboolean
-gtk_model_button_render (GtkCssGadget *gadget,
-                         cairo_t      *cr,
-                         int           x,
-                         int           y,
-                         int           width,
-                         int           height,
-                         gpointer      data)
-{
-  GtkWidget *widget;
-  GtkModelButton *button;
-  GtkWidget *child;
 
-  widget = gtk_css_gadget_get_owner (gadget);
-  button = GTK_MODEL_BUTTON (widget);
+      gtk_widget_measure (button->indicator_widget,
+                          GTK_ORIENTATION_HORIZONTAL,
+                          -1,
+                          &check_min_width, &check_nat_width,
+                          NULL, NULL);
+      gtk_widget_measure (button->indicator_widget,
+                          GTK_ORIENTATION_VERTICAL,
+                          -1,
+                          &check_min_height, &check_nat_height,
+                          NULL, NULL);
 
-  if (gtk_css_node_get_visible (gtk_css_gadget_get_node (button->indicator_gadget)))
-    gtk_css_gadget_draw (button->indicator_gadget, cr);
+      if (indicator_is_left (widget))
+        child_allocation.x = 0;
+      else
+        child_allocation.x = width - check_nat_width;
+      child_allocation.y = (height - check_nat_height) / 2;
+      child_allocation.width = check_nat_width;
+      child_allocation.height = check_nat_height;
 
-  child = gtk_bin_get_child (GTK_BIN (widget));
-  if (child)
-    gtk_container_propagate_draw (GTK_CONTAINER (widget), child, cr);
+      gtk_widget_size_allocate (button->indicator_widget, &child_allocation, baseline);
 
-  return gtk_widget_has_visible_focus (widget);
+      if (child && gtk_widget_get_visible (child))
+        {
+          GtkBorder border = { 0, };
+
+          if (button->centered)
+            {
+              border.left = check_nat_width;
+              border.right = check_nat_width;
+            }
+          else if (needs_indicator (button))
+            {
+              if (indicator_is_left (widget))
+                border.left += check_nat_width;
+              else
+                border.right += check_nat_width;
+            }
+
+          child_allocation.x = border.left;
+          child_allocation.y = border.top;
+          child_allocation.width = width - border.left - border.right;
+          child_allocation.height = height - border.top - border.bottom;
+
+          if (baseline != -1)
+            baseline -= border.top;
+
+          gtk_widget_size_allocate (child, &child_allocation, baseline);
+        }
+    }
 }
 
 static void
@@ -1095,24 +892,11 @@ gtk_model_button_finalize (GObject *object)
 {
   GtkModelButton *button = GTK_MODEL_BUTTON (object);
 
-  g_clear_object (&button->indicator_gadget);
-  g_clear_object (&button->gadget);
+  gtk_widget_unparent (button->indicator_widget);
 
   G_OBJECT_CLASS (gtk_model_button_parent_class)->finalize (object);
 }
 
-static AtkObject *
-gtk_model_button_get_accessible (GtkWidget *widget)
-{
-  AtkObject *object;
-
-  object = GTK_WIDGET_CLASS (gtk_model_button_parent_class)->get_accessible (widget);
-
-  gtk_model_button_update_state (GTK_MODEL_BUTTON (widget), GTK_STATE_FLAG_NORMAL);
-
-  return object;
-}
-
 static void
 gtk_model_button_class_init (GtkModelButtonClass *class)
 {
@@ -1124,17 +908,11 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
   object_class->get_property = gtk_model_button_get_property;
   object_class->set_property = gtk_model_button_set_property;
 
-  widget_class->get_preferred_width = gtk_model_button_get_preferred_width;
-  widget_class->get_preferred_width_for_height = gtk_model_button_get_preferred_width_for_height;
-  widget_class->get_preferred_height = gtk_model_button_get_preferred_height;
-  widget_class->get_preferred_height_for_width = gtk_model_button_get_preferred_height_for_width;
-  widget_class->get_preferred_height_and_baseline_for_width = gtk_model_button_get_preferred_height_and_baseline_for_width;
+  widget_class->measure = gtk_model_button_measure;
   widget_class->size_allocate = gtk_model_button_size_allocate;
-  widget_class->draw = gtk_model_button_draw;
   widget_class->destroy = gtk_model_button_destroy;
   widget_class->state_flags_changed = gtk_model_button_state_flags_changed;
   widget_class->direction_changed = gtk_model_button_direction_changed;
-  widget_class->get_accessible = gtk_model_button_get_accessible;
 
   button_class->clicked = gtk_model_button_clicked;
 
@@ -1144,8 +922,6 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
    * Specifies whether the button is a plain, check or radio button.
    * When #GtkActionable:action-name is set, the role will be determined
    * from the action and does not have to be set explicitly.
-   *
-   * Since: 3.16
    */
   properties[PROP_ROLE] =
     g_param_spec_enum ("role",
@@ -1160,8 +936,6 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
    *
    * A #GIcon that will be used if iconic appearance for the button is
    * desired.
-   *
-   * Since: 3.16
    */
   properties[PROP_ICON] = 
     g_param_spec_object ("icon",
@@ -1174,8 +948,6 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
    * GtkModelButton:text:
    *
    * The label for the button.
-   *
-   * Since: 3.16
    */
   properties[PROP_TEXT] =
     g_param_spec_string ("text",
@@ -1190,8 +962,6 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
    * If %TRUE, XML tags in the text of the button are interpreted as by
    * pango_parse_markup() to format the enclosed spans of text. If %FALSE, the
    * text will be displayed verbatim.
-   *
-   * Since: 3.24
    */
   properties[PROP_USE_MARKUP] =
     g_param_spec_boolean ("use-markup",
@@ -1205,8 +975,6 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
    *
    * The state of the button. This is reflecting the state of the associated
    * #GAction.
-   *
-   * Since: 3.16
    */
   properties[PROP_ACTIVE] =
     g_param_spec_boolean ("active",
@@ -1220,8 +988,6 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
    *
    * The name of a submenu to open when the button is activated.
    * If this is set, the button should not have an action associated with it.
-   *
-   * Since: 3.16
    */
   properties[PROP_MENU_NAME] =
     g_param_spec_string ("menu-name",
@@ -1236,8 +1002,6 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
    * Whether to show the submenu indicator at the opposite side than normal.
    * This property should be set for model buttons that 'go back' to a parent
    * menu.
-   *
-   * Since: 3.16
    */
   properties[PROP_INVERTED] =
     g_param_spec_boolean ("inverted",
@@ -1251,8 +1015,6 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
    *
    * Whether to render the button contents centered instead of left-aligned.
    * This property should be set for title-like items.
-   *
-   * Since: 3.16
    */
   properties[PROP_CENTERED] =
     g_param_spec_boolean ("centered",
@@ -1267,8 +1029,6 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
    * If this property is set, the button will show an icon if one is set.
    * If no icon is set, the text will be used. This is typically used for
    * horizontal sections of linked buttons.
-   *
-   * Since: 3.16
    */
   properties[PROP_ICONIC] =
     g_param_spec_boolean ("iconic",
@@ -1279,42 +1039,28 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
   g_object_class_install_properties (object_class, LAST_PROPERTY, properties);
 
   gtk_widget_class_set_accessible_role (GTK_WIDGET_CLASS (class), ATK_ROLE_PUSH_BUTTON);
-  gtk_widget_class_set_css_name (GTK_WIDGET_CLASS (class), "modelbutton");
+  gtk_widget_class_set_css_name (GTK_WIDGET_CLASS (class), I_("modelbutton"));
 }
 
 static void
 gtk_model_button_init (GtkModelButton *button)
 {
-  GtkCssNode *widget_node;
-
   button->role = GTK_BUTTON_ROLE_NORMAL;
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
   button->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
   gtk_widget_set_halign (button->box, GTK_ALIGN_FILL);
-  gtk_widget_show (button->box);
   button->image = gtk_image_new ();
-  gtk_widget_set_no_show_all (button->image, TRUE);
+  gtk_widget_hide (button->image);
   button->label = gtk_label_new ("");
-  gtk_widget_set_no_show_all (button->label, TRUE);
+  gtk_widget_hide (button->label);
   gtk_container_add (GTK_CONTAINER (button->box), button->image);
   gtk_container_add (GTK_CONTAINER (button->box), button->label);
   gtk_container_add (GTK_CONTAINER (button), button->box);
 
-  widget_node = gtk_widget_get_css_node (GTK_WIDGET (button));
-  button->gadget = gtk_css_custom_gadget_new_for_node (widget_node,
-                                                       GTK_WIDGET (button),
-                                                       gtk_model_button_measure,
-                                                       gtk_model_button_allocate,
-                                                       gtk_model_button_render,
-                                                       NULL,
-                                                       NULL);
-  button->indicator_gadget = gtk_builtin_icon_new ("check",
-                                                   GTK_WIDGET (button),
-                                                   button->gadget,
-                                                   NULL);
-  gtk_builtin_icon_set_default_size (GTK_BUILTIN_ICON (button->indicator_gadget), 16);
+  button->indicator_widget = gtk_icon_new ("check");
+  gtk_widget_set_parent (button->indicator_widget, GTK_WIDGET (button));
+  gtk_widget_hide (button->indicator_widget);
   update_node_ordering (button);
-  gtk_css_node_set_visible (gtk_css_gadget_get_node (button->indicator_gadget), FALSE);
 }
 
 /**
@@ -1323,8 +1069,6 @@ gtk_model_button_init (GtkModelButton *button)
  * Creates a new GtkModelButton.
  *
  * Returns: the newly created #GtkModelButton widget
- *
- * Since: 3.16
  */
 GtkWidget *
 gtk_model_button_new (void)
