@@ -21,13 +21,13 @@
 #include "gtksidebarrowprivate.h"
 /* For section and place type enums */
 #include "gtkplacessidebarprivate.h"
-#include "gtkplacessidebar.h"
 #include "gtkwidget.h"
 #include "gtkimage.h"
 #include "gtklabel.h"
 #include "gtkstylecontext.h"
 #include "gtkrevealer.h"
 #include "gtkselection.h"
+#include "gtkintl.h"
 #include "gtkspinner.h"
 
 #ifdef HAVE_CLOUDPROVIDERS
@@ -56,7 +56,6 @@ struct _GtkSidebarRow
   GObject *cloud_provider_account;
   gboolean placeholder;
   GtkPlacesSidebar *sidebar;
-  GtkWidget *event_box;
   GtkWidget *revealer;
   GtkWidget *busy_spinner;
 };
@@ -228,9 +227,7 @@ gtk_sidebar_row_set_property (GObject      *object,
         if (object != NULL)
           {
             self->start_icon = G_ICON (g_object_ref (object));
-            gtk_image_set_from_gicon (GTK_IMAGE (self->start_icon_widget),
-                                      self->start_icon,
-                                      GTK_ICON_SIZE_MENU);
+            gtk_image_set_from_gicon (GTK_IMAGE (self->start_icon_widget), self->start_icon);
           }
         else
           {
@@ -246,9 +243,7 @@ gtk_sidebar_row_set_property (GObject      *object,
         if (object != NULL)
           {
             self->end_icon = G_ICON (g_object_ref (object));
-            gtk_image_set_from_gicon (GTK_IMAGE (self->end_icon_widget),
-                                      self->end_icon,
-                                      GTK_ICON_SIZE_MENU);
+            gtk_image_set_from_gicon (GTK_IMAGE (self->end_icon_widget), self->end_icon);
             gtk_widget_show (self->end_icon_widget);
           }
         else
@@ -385,7 +380,7 @@ on_child_revealed (GObject    *self,
 void
 gtk_sidebar_row_reveal (GtkSidebarRow *self)
 {
-  gtk_widget_show_all (GTK_WIDGET (self));
+  gtk_widget_show (GTK_WIDGET (self));
   gtk_revealer_set_reveal_child (GTK_REVEALER (self->revealer), TRUE);
 }
 
@@ -414,8 +409,7 @@ gtk_sidebar_row_set_start_icon (GtkSidebarRow *self,
     {
       g_set_object (&self->start_icon, icon);
       if (self->start_icon != NULL)
-        gtk_image_set_from_gicon (GTK_IMAGE (self->start_icon_widget), self->start_icon,
-                                  GTK_ICON_SIZE_MENU);
+        gtk_image_set_from_gicon (GTK_IMAGE (self->start_icon_widget), self->start_icon);
       else
         gtk_image_clear (GTK_IMAGE (self->start_icon_widget));
 
@@ -433,8 +427,7 @@ gtk_sidebar_row_set_end_icon (GtkSidebarRow *self,
     {
       g_set_object (&self->end_icon, icon);
       if (self->end_icon != NULL)
-        gtk_image_set_from_gicon (GTK_IMAGE (self->end_icon_widget), self->end_icon,
-                                  GTK_ICON_SIZE_MENU);
+        gtk_image_set_from_gicon (GTK_IMAGE (self->end_icon_widget), self->end_icon);
       else
         if (self->end_icon_widget != NULL)
           gtk_image_clear (GTK_IMAGE (self->end_icon_widget));
@@ -621,12 +614,11 @@ gtk_sidebar_row_class_init (GtkSidebarRowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, end_icon_widget);
   gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, label_widget);
   gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, eject_button);
-  gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, event_box);
   gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, revealer);
   gtk_widget_class_bind_template_child (widget_class, GtkSidebarRow, busy_spinner);
 
   gtk_widget_class_bind_template_callback (widget_class, on_child_revealed);
-  gtk_widget_class_set_css_name (widget_class, "row");
+  gtk_widget_class_set_css_name (widget_class, I_("row"));
 }
 
 GtkSidebarRow*
@@ -656,12 +648,6 @@ gtk_sidebar_row_get_eject_button (GtkSidebarRow *self)
   return self->eject_button;
 }
 
-GtkWidget*
-gtk_sidebar_row_get_event_box (GtkSidebarRow *self)
-{
-  return self->event_box;
-}
-
 void
 gtk_sidebar_row_set_busy (GtkSidebarRow *row,
                           gboolean       is_busy)
