@@ -73,17 +73,13 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#define GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
 #include "config.h"
-#include "gtktextattributesprivate.h"
-#include "gtktextdisplay.h"
+#include "gtktextdisplayprivate.h"
+#include "gtktextviewprivate.h"
 #include "gtkwidgetprivate.h"
 #include "gtkstylecontextprivate.h"
 #include "gtkintl.h"
-
-/* DO NOT go putting private headers in here. This file should only
- * use the semi-public headers, as with gtktextview.c.
- */
+#include <gdk/gdktextureprivate.h>
 
 #define GTK_TYPE_TEXT_RENDERER            (_gtk_text_renderer_get_type())
 #define GTK_TEXT_RENDERER(object)         (G_TYPE_CHECK_INSTANCE_CAST ((object), GTK_TYPE_TEXT_RENDERER, GtkTextRenderer))
@@ -166,9 +162,6 @@ get_item_appearance (PangoItem *item)
   return NULL;
 }
 
-extern GtkCssNode *gtk_text_view_get_text_node      (GtkTextView *text_view);
-extern GtkCssNode *gtk_text_view_get_selection_node (GtkTextView *text_view);
-
 static void
 gtk_text_renderer_prepare_run (PangoRenderer  *renderer,
 			       PangoLayoutRun *run)
@@ -187,7 +180,7 @@ gtk_text_renderer_prepare_run (PangoRenderer  *renderer,
   context = gtk_widget_get_style_context (text_renderer->widget);
 
   if (appearance->draw_bg && text_renderer->state == NORMAL)
-    bg_rgba = appearance->rgba[0];
+    bg_rgba = appearance->bg_rgba;
   else
     bg_rgba = NULL;
 
@@ -200,7 +193,7 @@ gtk_text_renderer_prepare_run (PangoRenderer  *renderer,
       selection_node = gtk_text_view_get_selection_node ((GtkTextView *)text_renderer->widget);
       gtk_style_context_save_to_node (context, selection_node);
 
-      gtk_style_context_get (context, gtk_style_context_get_state (context),
+      gtk_style_context_get (context,
                              "color", &fg_rgba,
                              NULL);
 
@@ -208,61 +201,28 @@ gtk_text_renderer_prepare_run (PangoRenderer  *renderer,
     }
   else if (text_renderer->state == CURSOR && gtk_widget_has_focus (text_renderer->widget))
     {
-      gtk_style_context_get (context, gtk_style_context_get_state (context),
+      gtk_style_context_get (context,
                              "background-color", &fg_rgba,
                               NULL);
     }
   else
-    fg_rgba = appearance->rgba[1];
+    fg_rgba = appearance->fg_rgba;
 
   text_renderer_set_rgba (text_renderer, PANGO_RENDER_PART_FOREGROUND, fg_rgba);
 
-  if (GTK_TEXT_APPEARANCE_GET_STRIKETHROUGH_RGBA_SET (appearance))
-    {
-      GdkRGBA rgba;
-
-      GTK_TEXT_APPEARANCE_GET_STRIKETHROUGH_RGBA (appearance, &rgba);
-      text_renderer_set_rgba (text_renderer, PANGO_RENDER_PART_STRIKETHROUGH, &rgba);
-    }
+  if (appearance->strikethrough_rgba)
+    text_renderer_set_rgba (text_renderer, PANGO_RENDER_PART_STRIKETHROUGH, appearance->strikethrough_rgba);
   else
     text_renderer_set_rgba (text_renderer, PANGO_RENDER_PART_STRIKETHROUGH, fg_rgba);
 
-  if (GTK_TEXT_APPEARANCE_GET_UNDERLINE_RGBA_SET (appearance))
-    {
-      GdkRGBA rgba;
-
-      GTK_TEXT_APPEARANCE_GET_UNDERLINE_RGBA (appearance, &rgba);
-      text_renderer_set_rgba (text_renderer, PANGO_RENDER_PART_UNDERLINE, &rgba);
-    }
+  if (appearance->underline_rgba)
+    text_renderer_set_rgba (text_renderer, PANGO_RENDER_PART_UNDERLINE, appearance->underline_rgba);
   else if (appearance->underline == PANGO_UNDERLINE_ERROR)
     {
       if (!text_renderer->error_color)
         {
-	  GdkColor *color = NULL;
-
-          gtk_style_context_get_style (context,
-                                       "error-underline-color", &color,
-                                       NULL);
-
-	  if (color)
-	    {
-	      GdkRGBA rgba;
-
-	      rgba.red = color->red / 65535.;
-	      rgba.green = color->green / 65535.;
-	      rgba.blue = color->blue / 65535.;
-	      rgba.alpha = 1;
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-	      gdk_color_free (color);
-G_GNUC_END_IGNORE_DEPRECATIONS
-
-	      text_renderer->error_color = gdk_rgba_copy (&rgba);
-	    }
-	  else
-	    {
-	      static const GdkRGBA red = { 1, 0, 0, 1 };
-	      text_renderer->error_color = gdk_rgba_copy (&red);
-	    }
+          static const GdkRGBA red = { 1, 0, 0, 1 };
+          text_renderer->error_color = gdk_rgba_copy (&red);
         }
 
       text_renderer_set_rgba (text_renderer, PANGO_RENDER_PART_UNDERLINE, text_renderer->error_color);
@@ -270,7 +230,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
   else
     text_renderer_set_rgba (text_renderer, PANGO_RENDER_PART_UNDERLINE, fg_rgba);
 
-  if (fg_rgba != appearance->rgba[1])
+  if (fg_rgba != appearance->fg_rgba)
     gdk_rgba_free (fg_rgba);
 }
 
@@ -451,19 +411,24 @@ gtk_text_renderer_draw_shape (PangoRenderer   *renderer,
 
       unset_color (text_renderer);
     }
-  else if (GDK_IS_PIXBUF (attr->data))
+  else if (GDK_IS_TEXTURE (attr->data))
     {
       cairo_t *cr = text_renderer->cr;
-      GdkPixbuf *pixbuf = GDK_PIXBUF (attr->data);
-      
+      GdkTexture *texture = GDK_TEXTURE (attr->data);
+      cairo_surface_t *surface;
+
+      surface = gdk_texture_download_surface (texture);
+
       cairo_save (cr);
 
-      gdk_cairo_set_source_pixbuf (cr, pixbuf,
-                                   PANGO_PIXELS (x),
-                                   PANGO_PIXELS (y) -  gdk_pixbuf_get_height (pixbuf));
+      cairo_set_source_surface (cr, surface,
+                                PANGO_PIXELS (x),
+                                PANGO_PIXELS (y) -  gdk_texture_get_height (texture));
       cairo_paint (cr);
 
       cairo_restore (cr);
+
+      cairo_surface_destroy (surface);
     }
   else if (GTK_IS_WIDGET (attr->data))
     {
@@ -475,7 +440,7 @@ gtk_text_renderer_draw_shape (PangoRenderer   *renderer,
 					       g_object_ref (widget));
     }
   else
-    g_assert_not_reached (); /* not a pixbuf or widget */
+    g_assert_not_reached (); /* not a texture or widget */
 }
 
 static void
@@ -520,7 +485,6 @@ text_renderer_begin (GtkTextRenderer *text_renderer,
                      cairo_t         *cr)
 {
   GtkStyleContext *context;
-  GtkStateFlags state;
   GdkRGBA color;
   GtkCssNode *text_node;
 
@@ -532,8 +496,7 @@ text_renderer_begin (GtkTextRenderer *text_renderer,
   text_node = gtk_text_view_get_text_node ((GtkTextView *)widget);
   gtk_style_context_save_to_node (context, text_node);
 
-  state = gtk_style_context_get_state (context);
-  gtk_style_context_get_color (context, state, &color);
+  gtk_style_context_get_color (context, &color);
 
   cairo_save (cr);
 
@@ -542,11 +505,10 @@ text_renderer_begin (GtkTextRenderer *text_renderer,
 
 /* Returns a GSList of (referenced) widgets encountered while drawing.
  */
-static GList *
+static void
 text_renderer_end (GtkTextRenderer *text_renderer)
 {
   GtkStyleContext *context;
-  GList *widgets = text_renderer->widgets;
 
   cairo_restore (text_renderer->cr);
 
@@ -557,15 +519,11 @@ text_renderer_end (GtkTextRenderer *text_renderer)
   text_renderer->widget = NULL;
   text_renderer->cr = NULL;
 
-  text_renderer->widgets = NULL;
-
   if (text_renderer->error_color)
     {
       gdk_rgba_free (text_renderer->error_color);
       text_renderer->error_color = NULL;
     }
-
-  return widgets;
 }
 
 static cairo_region_t *
@@ -611,7 +569,7 @@ render_para (GtkTextRenderer    *text_renderer,
   int byte_offset = 0;
   PangoLayoutIter *iter;
   int screen_width;
-  GdkRGBA selection;
+  GdkRGBA *selection;
   gboolean first = TRUE;
   GtkCssNode *selection_node;
 
@@ -622,11 +580,9 @@ render_para (GtkTextRenderer    *text_renderer,
   selection_node = gtk_text_view_get_selection_node ((GtkTextView*)text_renderer->widget);
   gtk_style_context_save_to_node (context, selection_node);
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  gtk_style_context_get_background_color (context, gtk_style_context_get_state (context), &selection);
-G_GNUC_END_IGNORE_DEPRECATIONS
+  gtk_style_context_get (context, "background-color", &selection, NULL);
 
- gtk_style_context_restore (context);
+  gtk_style_context_restore (context);
 
   do
     {
@@ -671,7 +627,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
           cairo_t *cr = text_renderer->cr;
 
           cairo_save (cr);
-          gdk_cairo_set_source_rgba (cr, &selection);
+          gdk_cairo_set_source_rgba (cr, selection);
           cairo_rectangle (cr, 
                            line_display->left_margin, selection_y,
                            screen_width, selection_height);
@@ -727,7 +683,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
               cairo_clip (cr);
               cairo_region_destroy (clip_region);
 
-              gdk_cairo_set_source_rgba (cr, &selection);
+              gdk_cairo_set_source_rgba (cr, selection);
               cairo_rectangle (cr,
                                PANGO_PIXELS (line_rect.x),
                                selection_y,
@@ -750,7 +706,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
                 {
                   cairo_save (cr);
 
-                  gdk_cairo_set_source_rgba (cr, &selection);
+                  gdk_cairo_set_source_rgba (cr, selection);
                   cairo_rectangle (cr,
                                    line_display->left_margin,
                                    selection_y,
@@ -774,7 +730,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
                   cairo_save (cr);
 
-                  gdk_cairo_set_source_rgba (cr, &selection);
+                  gdk_cairo_set_source_rgba (cr, selection);
                   cairo_rectangle (cr,
                                    PANGO_PIXELS (line_rect.x) + PANGO_PIXELS (line_rect.width),
                                    selection_y,
@@ -815,13 +771,11 @@ G_GNUC_END_IGNORE_DEPRECATIONS
               /* draw text under the cursor if any */
               if (!line_display->cursor_at_line_end)
                 {
-                  GdkRGBA color;
+                  GdkRGBA *color;
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-                  gtk_style_context_get_background_color (context, gtk_style_context_get_state (context), &color);
-G_GNUC_END_IGNORE_DEPRECATIONS
+                  gtk_style_context_get (context, "background-color", &color, NULL);
 
-                  gdk_cairo_set_source_rgba (cr, &color);
+                  gdk_cairo_set_source_rgba (cr, color);
 
 		  text_renderer_set_state (text_renderer, CURSOR);
 
@@ -829,6 +783,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 						   line,
 						   line_rect.x,
 						   baseline);
+                  gdk_rgba_free (color);
                 }
 
               cairo_restore (cr);
@@ -839,6 +794,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
     }
   while (pango_layout_iter_next_line (iter));
 
+  gdk_rgba_free (selection);
   pango_layout_iter_free (iter);
 }
 
@@ -854,10 +810,10 @@ get_text_renderer (void)
 }
 
 void
-gtk_text_layout_draw (GtkTextLayout *layout,
-                      GtkWidget *widget,
-                      cairo_t *cr,
-                      GList **widgets)
+gtk_text_layout_snapshot (GtkTextLayout      *layout,
+                          GtkWidget          *widget,
+                          GtkSnapshot        *snapshot,
+                          const GdkRectangle *clip)
 {
   GtkStyleContext *context;
   gint offset_y;
@@ -866,24 +822,22 @@ gtk_text_layout_draw (GtkTextLayout *layout,
   gboolean have_selection;
   GSList *line_list;
   GSList *tmp_list;
-  GList *tmp_widgets;
-  GdkRectangle clip;
+  cairo_t *cr;
 
   g_return_if_fail (GTK_IS_TEXT_LAYOUT (layout));
   g_return_if_fail (layout->default_style != NULL);
   g_return_if_fail (layout->buffer != NULL);
-  g_return_if_fail (cr != NULL);
-
-  if (!gdk_cairo_get_clip_rectangle (cr, &clip))
-    return;
+  g_return_if_fail (snapshot != NULL);
 
   context = gtk_widget_get_style_context (widget);
 
-  line_list = gtk_text_layout_get_lines (layout, clip.y, clip.y + clip.height, &offset_y);
+  line_list = gtk_text_layout_get_lines (layout, clip->y, clip->y + clip->height, &offset_y);
 
   if (line_list == NULL)
     return; /* nothing on the screen */
 
+  cr = gtk_snapshot_append_cairo (snapshot,
+                                  &GRAPHENE_RECT_INIT(clip->x, clip->y, clip->width, clip->height));
   text_renderer = get_text_renderer ();
   text_renderer_begin (text_renderer, widget, cr);
 
@@ -970,12 +924,9 @@ gtk_text_layout_draw (GtkTextLayout *layout,
     }
 
   gtk_text_layout_wrap_loop_end (layout);
-
-  tmp_widgets = text_renderer_end (text_renderer);
-  if (widgets)
-    *widgets = tmp_widgets;
-  else
-    g_list_free_full (tmp_widgets, g_object_unref);
+  text_renderer_end (text_renderer);
 
   g_slist_free (line_list);
+
+  cairo_destroy (cr);
 }
