@@ -53,7 +53,7 @@
  * @Short_description: A tag that can be applied to text in a GtkTextBuffer
  *
  * You may wish to begin by reading the
- * [text widget conceptual overview](TextWidget.html)
+ * [text widget conceptual overview][TextWidget]
  * which gives an overview of all the objects and
  * data types related to the text widget and how they work together.
  *
@@ -61,7 +61,7 @@
  * before using them with that buffer.
  *
  * gtk_text_buffer_create_tag() is the best way to create tags.
- * See “gtk3-demo” for numerous examples.
+ * See “gtk4-demo” for numerous examples.
  *
  * For each property of #GtkTextTag, there is a “set” property, e.g.
  * “font-set” corresponds to “font”. These “set” properties reflect
@@ -74,21 +74,14 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "gtktextattributesprivate.h"
 #include "gtktexttag.h"
 #include "gtktexttypes.h"
 #include "gtktexttagtable.h"
-#include "gtktexttagtableprivate.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
 #include "gtktypebuiltins.h"
 
-enum {
-  EVENT,
-  LAST_SIGNAL
-};
-
 enum {
   PROP_0,
   /* Construct args */
@@ -97,8 +90,6 @@ enum {
   /* Style args */
   PROP_BACKGROUND,
   PROP_FOREGROUND,
-  PROP_BACKGROUND_GDK,
-  PROP_FOREGROUND_GDK,
   PROP_BACKGROUND_RGBA,
   PROP_FOREGROUND_RGBA,
   PROP_FONT,
@@ -131,7 +122,6 @@ enum {
   PROP_TABS,
   PROP_INVISIBLE,
   PROP_PARAGRAPH_BACKGROUND,
-  PROP_PARAGRAPH_BACKGROUND_GDK,
   PROP_PARAGRAPH_BACKGROUND_RGBA,
   PROP_FALLBACK,
   PROP_LETTER_SPACING,
@@ -185,8 +175,6 @@ static void gtk_text_tag_get_property (GObject         *object,
                                        GValue          *value,
                                        GParamSpec      *pspec);
 
-static guint signals[LAST_SIGNAL] = { 0 };
-
 G_DEFINE_TYPE_WITH_PRIVATE (GtkTextTag, gtk_text_tag, G_TYPE_OBJECT)
 
 static void
@@ -218,27 +206,10 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
                                                         NULL,
                                                         GTK_PARAM_WRITABLE));
 
-  /**
-   * GtkTextTag:background-gdk:
-   *
-   * Background color as a #GdkColor.
-   *
-   * Deprecated: 3.4: Use #GtkTextTag:background-rgba instead.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_BACKGROUND_GDK,
-                                   g_param_spec_boxed ("background-gdk",
-                                                       P_("Background color"),
-                                                       P_("Background color as a GdkColor"),
-                                                       g_type_from_name ("GdkColor"),
-                                                       GTK_PARAM_READWRITE | G_PARAM_DEPRECATED));
-
   /**
    * GtkTextTag:background-rgba:
    *
    * Background color as a #GdkRGBA.
-   *
-   * Since: 3.2
    */
   g_object_class_install_property (object_class,
                                    PROP_BACKGROUND_RGBA,
@@ -264,27 +235,10 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
                                                         NULL,
                                                         GTK_PARAM_WRITABLE));
 
-  /**
-   * GtkTextTag:foreground-gdk:
-   *
-   * Foreground color as a #GdkColor.
-   *
-   * Deprecated: 3.4: Use #GtkTextTag:foreground-rgba instead.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_FOREGROUND_GDK,
-                                   g_param_spec_boxed ("foreground-gdk",
-                                                       P_("Foreground color"),
-                                                       P_("Foreground color as a GdkColor"),
-                                                       g_type_from_name ("GdkColor"),
-                                                       GTK_PARAM_READWRITE | G_PARAM_DEPRECATED));
-
   /**
    * GtkTextTag:foreground-rgba:
    *
    * Foreground color as a #GdkRGBA.
-   *
-   * Since: 3.2
    */
   g_object_class_install_property (object_class,
                                    PROP_FOREGROUND_RGBA,
@@ -323,7 +277,7 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
                                    PROP_FONT,
                                    g_param_spec_string ("font",
                                                         P_("Font"),
-                                                        P_("Font description as a string, e.g. \"Sans Italic 12\""),
+                                                        P_("Font description as a string, e.g. “Sans Italic 12”"),
                                                         NULL,
                                                         GTK_PARAM_READWRITE));
 
@@ -536,8 +490,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
    * If #GtkTextTag:underline is set to %PANGO_UNDERLINE_ERROR, an alternate
    * color may be applied instead of the foreground. Setting this property
    * will always override those defaults.
-   *
-   * Since: 3.16
    */
   g_object_class_install_property (object_class,
                                    PROP_UNDERLINE_RGBA,
@@ -552,8 +504,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
    *
    * This property modifies the color of strikeouts. If not set, strikeouts
    * will use the forground color.
-   *
-   * Since: 3.16
    */
   g_object_class_install_property (object_class,
                                    PROP_STRIKETHROUGH_RGBA,
@@ -589,8 +539,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
    * Note that there may still be problems with the support for invisible 
    * text, in particular when navigating programmatically inside a buffer
    * containing invisible segments. 
-   *
-   * Since: 2.8
    */
   g_object_class_install_property (object_class,
                                    PROP_INVISIBLE,
@@ -604,8 +552,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
    * GtkTextTag:paragraph-background:
    *
    * The paragraph background color as a string.
-   *
-   * Since: 2.8
    */
   g_object_class_install_property (object_class,
                                    PROP_PARAGRAPH_BACKGROUND,
@@ -615,29 +561,10 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
                                                         NULL,
                                                         GTK_PARAM_WRITABLE));
 
-  /**
-   * GtkTextTag:paragraph-background-gdk:
-   *
-   * The paragraph background color as a #GdkColor.
-   *
-   * Since: 2.8
-   *
-   * Deprecated: 3.4: Use #GtkTextTag:paragraph-background-rgba instead.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_PARAGRAPH_BACKGROUND_GDK,
-                                   g_param_spec_boxed ("paragraph-background-gdk",
-                                                       P_("Paragraph background color"),
-                                                       P_("Paragraph background color as a GdkColor"),
-                                                       g_type_from_name ("GdkColor"),
-                                                       GTK_PARAM_READWRITE | G_PARAM_DEPRECATED));
-
   /**
    * GtkTextTag:paragraph-background-rgba:
    *
    * The paragraph background color as a #GdkRGBA.
-   *
-   * Since: 3.2
    */
   g_object_class_install_property (object_class,
                                    PROP_PARAGRAPH_BACKGROUND_RGBA,
@@ -654,8 +581,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
    *
    * When set to %TRUE, other fonts will be substituted
    * where the current font is missing glyphs.
-   *
-   * Since: 3.16
    */
   g_object_class_install_property (object_class,
                                    PROP_FALLBACK,
@@ -669,8 +594,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
    * GtkTextTag:letter-spacing:
    *
    * Extra spacing between graphemes, in Pango units.
-   *
-   * Since: 3.16
    */
   g_object_class_install_property (object_class,
                                    PROP_LETTER_SPACING,
@@ -684,8 +607,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
    * GtkTextTag:font-features:
    *
    * OpenType font features, as a string.
-   *
-   * Since: 3.18
    */
   g_object_class_install_property (object_class,
                                    PROP_FONT_FEATURES,
@@ -703,8 +624,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
    * When set to %TRUE the margins of this tag are added to the margins 
    * of any other non-accumulative margins present. When set to %FALSE 
    * the margins override one another (the default).
-   *
-   * Since: 2.12
    */
   g_object_class_install_property (object_class,
                                    PROP_ACCUMULATIVE_MARGIN,
@@ -810,8 +729,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
    * GtkTextTag:underline-rgba-set:
    *
    * If the #GtkTextTag:underline-rgba property has been set.
-   *
-   * Since: 3.16
    */
   ADD_SET_PROP ("underline-rgba-set", PROP_UNDERLINE_RGBA_SET,
                 P_("Underline RGBA set"),
@@ -821,8 +738,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
    * GtkTextTag:strikethrough-rgba-set:
    *
    * If the #GtkTextTag:strikethrough-rgba property has been set.
-   *
-   * Since: 3.16
    */
   ADD_SET_PROP ("strikethrough-rgba-set", PROP_STRIKETHROUGH_RGBA_SET,
                 P_("Strikethrough RGBA set"),
@@ -855,35 +770,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
   ADD_SET_PROP ("font-features-set", PROP_FONT_FEATURES_SET,
                 P_("Font features set"),
                 P_("Whether this tag affects font features"));
-
-  /**
-   * GtkTextTag::event:
-   * @tag: the #GtkTextTag on which the signal is emitted
-   * @object: the object the event was fired from (typically a #GtkTextView)
-   * @event: the event which triggered the signal
-   * @iter: a #GtkTextIter pointing at the location the event occurred
-   *
-   * The ::event signal is emitted when an event occurs on a region of the
-   * buffer marked with this tag.
-   *
-   * Returns: %TRUE to stop other handlers from being invoked for the
-   * event. %FALSE to propagate the event further.
-   */
-  signals[EVENT] =
-    g_signal_new (I_("event"),
-                  G_OBJECT_CLASS_TYPE (object_class),
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (GtkTextTagClass, event),
-                  _gtk_boolean_handled_accumulator, NULL,
-                  _gtk_marshal_BOOLEAN__OBJECT_BOXED_BOXED,
-                  G_TYPE_BOOLEAN,
-                  3,
-                  G_TYPE_OBJECT,
-                  GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE,
-                  GTK_TYPE_TEXT_ITER);
-  g_signal_set_va_marshaller (signals[EVENT],
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              _gtk_marshal_BOOLEAN__OBJECT_BOXED_BOXEDv);
 }
 
 static void
@@ -932,50 +818,31 @@ gtk_text_tag_finalize (GObject *object)
   G_OBJECT_CLASS (gtk_text_tag_parent_class)->finalize (object);
 }
 
-static void
-copy_rgba_to_gdk_color (GdkRGBA  *src,
-			GdkColor *dest)
-{
-  dest->red   = CLAMP (src->red,   0.0, 1.0) * 65535.0;
-  dest->green = CLAMP (src->green, 0.0, 1.0) * 65535.0;
-  dest->blue  = CLAMP (src->blue,  0.0, 1.0) * 65535.0;
-}
-
-static void
-copy_gdk_color_to_rgba (GdkColor *src,
-			GdkRGBA  *dest)
-{
-  dest->red   = src->red / 65535.;
-  dest->green = src->green / 65535.;
-  dest->blue  = src->blue / 65535.;
-  dest->alpha = 1;
-}
-
 static void
 set_underline_rgba (GtkTextTag    *tag,
                     const GdkRGBA *rgba)
 {
   GtkTextTagPrivate *priv = tag->priv;
 
+  if (priv->values->appearance.underline_rgba)
+    gdk_rgba_free (priv->values->appearance.underline_rgba);
+  priv->values->appearance.underline_rgba = NULL;
+
   if (rgba)
     {
-      GTK_TEXT_APPEARANCE_SET_UNDERLINE_RGBA (&priv->values->appearance, rgba);
+      priv->values->appearance.underline_rgba = gdk_rgba_copy (rgba);
 
-      if (!GTK_TEXT_APPEARANCE_GET_UNDERLINE_RGBA_SET (&priv->values->appearance))
+      if (!priv->underline_rgba_set)
         {
-          GTK_TEXT_APPEARANCE_SET_UNDERLINE_RGBA_SET (&priv->values->appearance, TRUE);
+          priv->underline_rgba_set = TRUE;
           g_object_notify (G_OBJECT (tag), "underline-rgba-set");
         }
     }
   else
     {
-      GdkRGBA black = { 0 };
-
-      GTK_TEXT_APPEARANCE_SET_UNDERLINE_RGBA (&priv->values->appearance, &black);
-
-      if (GTK_TEXT_APPEARANCE_GET_UNDERLINE_RGBA_SET (&priv->values->appearance))
+      if (priv->underline_rgba_set)
         {
-          GTK_TEXT_APPEARANCE_SET_UNDERLINE_RGBA_SET (&priv->values->appearance, FALSE);
+          priv->underline_rgba_set = FALSE;
           g_object_notify (G_OBJECT (tag), "underline-rgba-set");
         }
     }
@@ -987,25 +854,25 @@ set_strikethrough_rgba (GtkTextTag    *tag,
 {
   GtkTextTagPrivate *priv = tag->priv;
 
+  if (priv->values->appearance.strikethrough_rgba)
+    gdk_rgba_free (priv->values->appearance.strikethrough_rgba);
+  priv->values->appearance.strikethrough_rgba = NULL;
+
   if (rgba)
     {
-      GTK_TEXT_APPEARANCE_SET_STRIKETHROUGH_RGBA (&priv->values->appearance, rgba);
+      priv->values->appearance.strikethrough_rgba = gdk_rgba_copy (rgba);
 
-      if (!GTK_TEXT_APPEARANCE_GET_STRIKETHROUGH_RGBA_SET (&priv->values->appearance))
+      if (!priv->strikethrough_rgba_set)
         {
-          GTK_TEXT_APPEARANCE_SET_STRIKETHROUGH_RGBA_SET (&priv->values->appearance, TRUE);
+          priv->strikethrough_rgba_set = TRUE;
           g_object_notify (G_OBJECT (tag), "strikethrough-rgba-set");
         }
     }
   else
     {
-      GdkRGBA black = { 0 };
-
-      GTK_TEXT_APPEARANCE_SET_STRIKETHROUGH_RGBA (&priv->values->appearance, &black);
-
-      if (GTK_TEXT_APPEARANCE_GET_STRIKETHROUGH_RGBA_SET (&priv->values->appearance))
+      if (priv->strikethrough_rgba_set)
         {
-          GTK_TEXT_APPEARANCE_SET_STRIKETHROUGH_RGBA_SET (&priv->values->appearance, FALSE);
+          priv->strikethrough_rgba_set = FALSE;
           g_object_notify (G_OBJECT (tag), "strikethrough-rgba-set");
         }
     }
@@ -1016,10 +883,9 @@ set_bg_rgba (GtkTextTag *tag, GdkRGBA *rgba)
 {
   GtkTextTagPrivate *priv = tag->priv;
 
-  if (priv->values->appearance.rgba[0])
-    gdk_rgba_free (priv->values->appearance.rgba[0]);
-
-  priv->values->appearance.rgba[0] = NULL;
+  if (priv->values->appearance.bg_rgba)
+    gdk_rgba_free (priv->values->appearance.bg_rgba);
+  priv->values->appearance.bg_rgba = NULL;
 
   if (rgba)
     {
@@ -1029,9 +895,7 @@ set_bg_rgba (GtkTextTag *tag, GdkRGBA *rgba)
           g_object_notify (G_OBJECT (tag), "background-set");
         }
 
-      priv->values->appearance.rgba[0] = gdk_rgba_copy (rgba);
-
-      copy_rgba_to_gdk_color (rgba, &priv->values->appearance.bg_color);
+      priv->values->appearance.bg_rgba = gdk_rgba_copy (rgba);
     }
   else
     {
@@ -1048,10 +912,9 @@ set_fg_rgba (GtkTextTag *tag, GdkRGBA *rgba)
 {
   GtkTextTagPrivate *priv = tag->priv;
 
-  if (priv->values->appearance.rgba[1])
-    gdk_rgba_free (priv->values->appearance.rgba[1]);
-
-  priv->values->appearance.rgba[1] = NULL;
+  if (priv->values->appearance.fg_rgba)
+    gdk_rgba_free (priv->values->appearance.fg_rgba);
+  priv->values->appearance.fg_rgba = NULL;
 
   if (rgba)
     {
@@ -1061,9 +924,7 @@ set_fg_rgba (GtkTextTag *tag, GdkRGBA *rgba)
           g_object_notify (G_OBJECT (tag), "foreground-set");
         }
 
-      priv->values->appearance.rgba[1] = gdk_rgba_copy (rgba);
-
-      copy_rgba_to_gdk_color (rgba, &priv->values->appearance.fg_color);
+      priv->values->appearance.fg_rgba = gdk_rgba_copy (rgba);
     }
   else
     {
@@ -1082,19 +943,10 @@ set_pg_bg_rgba (GtkTextTag *tag, GdkRGBA *rgba)
 
   if (priv->values->pg_bg_rgba)
     gdk_rgba_free (priv->values->pg_bg_rgba);
-
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  if (priv->values->pg_bg_color)
-    gdk_color_free (priv->values->pg_bg_color);
-G_GNUC_END_IGNORE_DEPRECATIONS
-
   priv->values->pg_bg_rgba = NULL;
-  priv->values->pg_bg_color = NULL;
 
   if (rgba)
     {
-      GdkColor color = { 0, };
-
       if (!priv->pg_bg_color_set)
         {
           priv->pg_bg_color_set = TRUE;
@@ -1102,11 +954,6 @@ G_GNUC_END_IGNORE_DEPRECATIONS
         }
 
       priv->values->pg_bg_rgba = gdk_rgba_copy (rgba);
-
-      copy_rgba_to_gdk_color (rgba, &color);
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      priv->values->pg_bg_color = gdk_color_copy (&color);
-G_GNUC_END_IGNORE_DEPRECATIONS
     }
   else
     {
@@ -1118,49 +965,6 @@ G_GNUC_END_IGNORE_DEPRECATIONS
     }
 }
 
-
-static void
-set_bg_color (GtkTextTag *tag, GdkColor *color)
-{
-  if (color)
-    {
-      GdkRGBA rgba;
-
-      copy_gdk_color_to_rgba (color, &rgba);
-      set_bg_rgba (tag, &rgba);
-    }
-  else
-    set_bg_rgba (tag, NULL);
-}
-
-static void
-set_fg_color (GtkTextTag *tag, GdkColor *color)
-{
-  if (color)
-    {
-      GdkRGBA rgba;
-
-      copy_gdk_color_to_rgba (color, &rgba);
-      set_fg_rgba (tag, &rgba);
-    }
-  else
-    set_fg_rgba (tag, NULL);
-}
-
-static void
-set_pg_bg_color (GtkTextTag *tag, GdkColor *color)
-{
-  if (color)
-    {
-      GdkRGBA rgba;
-
-      copy_gdk_color_to_rgba (color, &rgba);
-      set_pg_bg_rgba (tag, &rgba);
-    }
-  else
-    set_pg_bg_rgba (tag, NULL);
-}
-
 static PangoFontMask
 get_property_font_set_mask (guint prop_id)
 {
@@ -1178,9 +982,9 @@ get_property_font_set_mask (guint prop_id)
       return PANGO_FONT_MASK_STRETCH;
     case PROP_SIZE_SET:
       return PANGO_FONT_MASK_SIZE;
+    default:
+      return 0;
     }
-
-  return 0;
 }
 
 static PangoFontMask
@@ -1349,8 +1153,6 @@ gtk_text_tag_set_property (GObject      *object,
           set_bg_rgba (text_tag, &rgba);
         else
           g_warning ("Don't know color '%s'", g_value_get_string (value));
-
-        g_object_notify (object, "background-gdk");
       }
       break;
 
@@ -1364,24 +1166,6 @@ gtk_text_tag_set_property (GObject      *object,
           set_fg_rgba (text_tag, &rgba);
         else
           g_warning ("Don't know color '%s'", g_value_get_string (value));
-
-        g_object_notify (object, "foreground-gdk");
-      }
-      break;
-
-    case PROP_BACKGROUND_GDK:
-      {
-        GdkColor *color = g_value_get_boxed (value);
-
-        set_bg_color (text_tag, color);
-      }
-      break;
-
-    case PROP_FOREGROUND_GDK:
-      {
-        GdkColor *color = g_value_get_boxed (value);
-
-        set_fg_color (text_tag, color);
       }
       break;
 
@@ -1476,6 +1260,9 @@ gtk_text_tag_set_property (GObject      *object,
 					     g_value_get_double (value) * PANGO_SCALE);
 	    g_object_notify (object, "size");
 	    break;
+
+          default:
+            break;
 	  }
 
 	size_changed = TRUE;
@@ -1636,16 +1423,6 @@ gtk_text_tag_set_property (GObject      *object,
           set_pg_bg_rgba (text_tag, &rgba);
         else
           g_warning ("Don't know color '%s'", g_value_get_string (value));
-
-        g_object_notify (object, "paragraph-background-gdk");
-      }
-      break;
-
-    case PROP_PARAGRAPH_BACKGROUND_GDK:
-      {
-        GdkColor *color = g_value_get_boxed (value);
-
-        set_pg_bg_color (text_tag, color);
       }
       break;
 
@@ -1763,8 +1540,7 @@ gtk_text_tag_set_property (GObject      *object,
       break;
 
     case PROP_STRIKETHROUGH_RGBA_SET:
-      GTK_TEXT_APPEARANCE_SET_STRIKETHROUGH_RGBA_SET (&priv->values->appearance,
-                                                      g_value_get_boolean (value));
+      priv->strikethrough_rgba_set = g_value_get_boolean (value);
       break;
 
     case PROP_RIGHT_MARGIN_SET:
@@ -1777,8 +1553,7 @@ gtk_text_tag_set_property (GObject      *object,
       break;
 
     case PROP_UNDERLINE_RGBA_SET:
-      GTK_TEXT_APPEARANCE_SET_UNDERLINE_RGBA_SET (&priv->values->appearance,
-                                                  g_value_get_boolean (value));
+      priv->underline_rgba_set = g_value_get_boolean (value);
       break;
 
     case PROP_RISE_SET:
@@ -1849,20 +1624,12 @@ gtk_text_tag_get_property (GObject      *object,
       g_value_set_string (value, priv->name);
       break;
 
-    case PROP_BACKGROUND_GDK:
-      g_value_set_boxed (value, &priv->values->appearance.bg_color);
-      break;
-
     case PROP_BACKGROUND_RGBA:
-      g_value_set_boxed (value, priv->values->appearance.rgba[0]);
-      break;
-
-    case PROP_FOREGROUND_GDK:
-      g_value_set_boxed (value, &priv->values->appearance.fg_color);
+      g_value_set_boxed (value, priv->values->appearance.bg_rgba);
       break;
 
     case PROP_FOREGROUND_RGBA:
-      g_value_set_boxed (value, priv->values->appearance.rgba[1]);
+      g_value_set_boxed (value, priv->values->appearance.fg_rgba);
       break;
 
     case PROP_FONT:
@@ -1882,45 +1649,40 @@ gtk_text_tag_get_property (GObject      *object,
       break;
 
     case PROP_FAMILY:
-    case PROP_STYLE:
-    case PROP_VARIANT:
-    case PROP_WEIGHT:
-    case PROP_STRETCH:
-    case PROP_SIZE:
-    case PROP_SIZE_POINTS:
       gtk_text_tag_ensure_font (tag);
-      switch (prop_id)
-	{
-	case PROP_FAMILY:
-	  g_value_set_string (value, pango_font_description_get_family (priv->values->font));
-	  break;
-	  
-	case PROP_STYLE:
-	  g_value_set_enum (value, pango_font_description_get_style (priv->values->font));
-	  break;
-	  
-	case PROP_VARIANT:
-	  g_value_set_enum (value, pango_font_description_get_variant (priv->values->font));
-	  break;
-	  
-	case PROP_WEIGHT:
-	  g_value_set_int (value, pango_font_description_get_weight (priv->values->font));
-	  break;
-	  
-	case PROP_STRETCH:
-	  g_value_set_enum (value, pango_font_description_get_stretch (priv->values->font));
-	  break;
-	  
-	case PROP_SIZE:
-	  g_value_set_int (value, pango_font_description_get_size (priv->values->font));
-	  break;
-	  
-	case PROP_SIZE_POINTS:
-	  g_value_set_double (value, ((double)pango_font_description_get_size (priv->values->font)) / (double)PANGO_SCALE);
-	  break;
-	}
+      g_value_set_string (value, pango_font_description_get_family (priv->values->font));
       break;
       
+    case PROP_STYLE:
+      gtk_text_tag_ensure_font (tag);
+      g_value_set_enum (value, pango_font_description_get_style (priv->values->font));
+      break;
+      
+    case PROP_VARIANT:
+      gtk_text_tag_ensure_font (tag);
+      g_value_set_enum (value, pango_font_description_get_variant (priv->values->font));
+      break;
+      
+    case PROP_WEIGHT:
+      gtk_text_tag_ensure_font (tag);
+      g_value_set_int (value, pango_font_description_get_weight (priv->values->font));
+      break;
+      
+    case PROP_STRETCH:
+      gtk_text_tag_ensure_font (tag);
+      g_value_set_enum (value, pango_font_description_get_stretch (priv->values->font));
+      break;
+      
+    case PROP_SIZE:
+      gtk_text_tag_ensure_font (tag);
+      g_value_set_int (value, pango_font_description_get_size (priv->values->font));
+      break;
+      
+    case PROP_SIZE_POINTS:
+      gtk_text_tag_ensure_font (tag);
+      g_value_set_double (value, ((double)pango_font_description_get_size (priv->values->font)) / (double)PANGO_SCALE);
+      break;
+  
     case PROP_SCALE:
       g_value_set_double (value, priv->values->font_scale);
       break;
@@ -1966,13 +1728,8 @@ gtk_text_tag_get_property (GObject      *object,
       break;
 
     case PROP_STRIKETHROUGH_RGBA:
-      if (GTK_TEXT_APPEARANCE_GET_STRIKETHROUGH_RGBA_SET (&priv->values->appearance))
-        {
-          GdkRGBA rgba;
-
-          GTK_TEXT_APPEARANCE_GET_STRIKETHROUGH_RGBA (&priv->values->appearance, &rgba);
-          g_value_set_boxed (value, &rgba);
-        }
+      if (priv->strikethrough_rgba_set)
+        g_value_set_boxed (value, priv->values->appearance.strikethrough_rgba);
       break;
 
     case PROP_RIGHT_MARGIN:
@@ -1984,13 +1741,8 @@ gtk_text_tag_get_property (GObject      *object,
       break;
 
     case PROP_UNDERLINE_RGBA:
-      if (GTK_TEXT_APPEARANCE_GET_UNDERLINE_RGBA_SET (&priv->values->appearance))
-        {
-          GdkRGBA rgba;
-
-          GTK_TEXT_APPEARANCE_GET_UNDERLINE_RGBA (&priv->values->appearance, &rgba);
-          g_value_set_boxed (value, &rgba);
-        }
+      if (priv->underline_rgba_set)
+        g_value_set_boxed (value, priv->values->appearance.underline_rgba);
       break;
 
     case PROP_RISE:
@@ -2014,9 +1766,6 @@ gtk_text_tag_get_property (GObject      *object,
       g_value_set_boolean (value, priv->values->invisible);
       break;
       
-    case PROP_PARAGRAPH_BACKGROUND_GDK:
-      g_value_set_boxed (value, priv->values->pg_bg_color);
-      break;
 
     case PROP_PARAGRAPH_BACKGROUND_RGBA:
       g_value_set_boxed (value, priv->values->pg_bg_rgba);
@@ -2101,8 +1850,7 @@ gtk_text_tag_get_property (GObject      *object,
       break;
 
     case PROP_STRIKETHROUGH_RGBA_SET:
-      g_value_set_boolean (value,
-                           GTK_TEXT_APPEARANCE_GET_STRIKETHROUGH_RGBA_SET (&priv->values->appearance));
+      g_value_set_boolean (value, priv->strikethrough_rgba_set);
       break;
 
     case PROP_RIGHT_MARGIN_SET:
@@ -2114,8 +1862,7 @@ gtk_text_tag_get_property (GObject      *object,
       break;
 
     case PROP_UNDERLINE_RGBA_SET:
-      g_value_set_boolean (value,
-                           GTK_TEXT_APPEARANCE_GET_UNDERLINE_RGBA_SET (&priv->values->appearance));
+      g_value_set_boolean (value, priv->underline_rgba_set);
       break;
 
     case PROP_RISE_SET:
@@ -2256,40 +2003,6 @@ gtk_text_tag_set_priority (GtkTextTag *tag,
   priv->priority = priority;
 }
 
-/**
- * gtk_text_tag_event:
- * @tag: a #GtkTextTag
- * @event_object: object that received the event, such as a widget
- * @event: the event
- * @iter: location where the event was received
- * 
- * Emits the “event” signal on the #GtkTextTag.
- * 
- * Returns: result of signal emission (whether the event was handled)
- **/
-gboolean
-gtk_text_tag_event (GtkTextTag        *tag,
-                    GObject           *event_object,
-                    GdkEvent          *event,
-                    const GtkTextIter *iter)
-{
-  gboolean retval = FALSE;
-
-  g_return_val_if_fail (GTK_IS_TEXT_TAG (tag), FALSE);
-  g_return_val_if_fail (G_IS_OBJECT (event_object), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  g_signal_emit (tag,
-                 signals[EVENT],
-                 0,
-                 event_object,
-                 event,
-                 iter,
-                 &retval);
-
-  return retval;
-}
-
 /**
  * gtk_text_tag_changed:
  * @tag: a #GtkTextTag.
@@ -2300,8 +2013,6 @@ gtk_text_tag_event (GtkTextTag        *tag,
  *
  * The signal is already emitted when setting a #GtkTextTag property. This
  * function is useful for a #GtkTextTag subclass.
- *
- * Since: 3.20
  */
 void
 gtk_text_tag_changed (GtkTextTag *tag,
@@ -2318,7 +2029,10 @@ gtk_text_tag_changed (GtkTextTag *tag,
    * added, this would increase significantly the number of signal connections.
    */
   if (priv->table != NULL)
-    _gtk_text_tag_table_tag_changed (priv->table, tag, size_changed);
+    g_signal_emit_by_name (priv->table,
+                           "tag-changed",
+                           tag,
+                           size_changed);
 }
 
 static int
