@@ -28,9 +28,6 @@
 #include "gtkintl.h"
 #include "gtkwindow.h"
 #include "gtkmain.h"
-#include "gtklabel.h"
-#include "gtkimage.h"
-#include "gtkbox.h"
 #include "gtksettings.h"
 #include "gtksizerequest.h"
 #include "gtkstylecontext.h"
@@ -38,12 +35,13 @@
 #include "gtkwindowprivate.h"
 #include "gtkwidgetprivate.h"
 #include "gtkaccessible.h"
+#include "gtkroot.h"
 
 #ifdef GDK_WINDOWING_WAYLAND
 #include "wayland/gdkwayland.h"
 #endif
-#ifdef GDK_WINDOWING_X11
-#include "x11/gdkx.h"
+#ifdef GDK_WINDOWING_MIR
+#include "mir/gdkmir.h"
 #endif
 
 
@@ -104,17 +102,12 @@ struct _GtkTooltip
   GObject parent_instance;
 
   GtkWidget *window;
-  GtkWidget *box;
-  GtkWidget *image;
-  GtkWidget *label;
-  GtkWidget *custom_widget;
 
   GtkWindow *current_window;
-  GtkWidget *keyboard_widget;
 
   GtkWidget *tooltip_widget;
 
-  GdkWindow *last_window;
+  GdkSurface *last_surface;
 
   guint timeout_id;
   guint browse_mode_timeout_id;
@@ -122,7 +115,6 @@ struct _GtkTooltip
   GdkRectangle tip_area;
 
   guint browse_mode_enabled : 1;
-  guint keyboard_mode_enabled : 1;
   guint tip_area_set : 1;
   guint custom_was_reset : 1;
 };
@@ -134,9 +126,6 @@ struct _GtkTooltipClass
 
 #define GTK_TOOLTIP_VISIBLE(tooltip) ((tooltip)->current_window && gtk_widget_get_visible (GTK_WIDGET((tooltip)->current_window)))
 
-
-static void       gtk_tooltip_class_init           (GtkTooltipClass *klass);
-static void       gtk_tooltip_init                 (GtkTooltip      *tooltip);
 static void       gtk_tooltip_dispose              (GObject         *object);
 
 static void       gtk_tooltip_window_hide          (GtkWidget       *widget,
@@ -144,10 +133,14 @@ static void       gtk_tooltip_window_hide          (GtkWidget       *widget,
 static void       gtk_tooltip_display_closed       (GdkDisplay      *display,
 						    gboolean         was_error,
 						    GtkTooltip      *tooltip);
-static void       gtk_tooltip_set_last_window      (GtkTooltip      *tooltip,
-						    GdkWindow       *window);
+static void       gtk_tooltip_set_last_surface      (GtkTooltip      *tooltip,
+						    GdkSurface       *surface);
 
-static void       gtk_tooltip_handle_event_internal (GdkEvent        *event);
+static void       gtk_tooltip_handle_event_internal (GdkEventType   event_type,
+                                                     GdkSurface    *surface,
+                                                     GtkWidget     *target_widget,
+                                                     gdouble       dx,
+                                                     gdouble       dy);
 
 static inline GQuark tooltip_quark (void)
 {
@@ -179,14 +172,12 @@ gtk_tooltip_init (GtkTooltip *tooltip)
   tooltip->browse_mode_timeout_id = 0;
 
   tooltip->browse_mode_enabled = FALSE;
-  tooltip->keyboard_mode_enabled = FALSE;
 
   tooltip->current_window = NULL;
-  tooltip->keyboard_widget = NULL;
 
   tooltip->tooltip_widget = NULL;
 
-  tooltip->last_window = NULL;
+  tooltip->last_surface = NULL;
 
   tooltip->window = gtk_tooltip_window_new ();
   g_signal_connect (tooltip->window, "hide",
@@ -212,7 +203,7 @@ gtk_tooltip_dispose (GObject *object)
     }
 
   gtk_tooltip_set_custom (tooltip, NULL);
-  gtk_tooltip_set_last_window (tooltip, NULL);
+  gtk_tooltip_set_last_surface (tooltip, NULL);
 
   if (tooltip->window)
     {
@@ -239,8 +230,6 @@ gtk_tooltip_dispose (GObject *object)
  * Sets the text of the tooltip to be @markup, which is marked up
  * with the [Pango text markup language][PangoMarkupFormat].
  * If @markup is %NULL, the label will be hidden.
- *
- * Since: 2.12
  */
 void
 gtk_tooltip_set_markup (GtkTooltip  *tooltip,
@@ -258,8 +247,6 @@ gtk_tooltip_set_markup (GtkTooltip  *tooltip,
  *
  * Sets the text of the tooltip to be @text. If @text is %NULL, the label
  * will be hidden. See also gtk_tooltip_set_markup().
- *
- * Since: 2.12
  */
 void
 gtk_tooltip_set_text (GtkTooltip  *tooltip,
@@ -273,95 +260,57 @@ gtk_tooltip_set_text (GtkTooltip  *tooltip,
 /**
  * gtk_tooltip_set_icon:
  * @tooltip: a #GtkTooltip
- * @pixbuf: (allow-none): a #GdkPixbuf, or %NULL
+ * @paintable: (allow-none): a #GdkPaintable, or %NULL
  *
  * Sets the icon of the tooltip (which is in front of the text) to be
- * @pixbuf.  If @pixbuf is %NULL, the image will be hidden.
- *
- * Since: 2.12
+ * @paintable.  If @paintable is %NULL, the image will be hidden.
  */
 void
-gtk_tooltip_set_icon (GtkTooltip *tooltip,
-		      GdkPixbuf  *pixbuf)
+gtk_tooltip_set_icon (GtkTooltip   *tooltip,
+		      GdkPaintable *paintable)
 {
   g_return_if_fail (GTK_IS_TOOLTIP (tooltip));
-  g_return_if_fail (pixbuf == NULL || GDK_IS_PIXBUF (pixbuf));
+  g_return_if_fail (paintable == NULL || GDK_IS_PAINTABLE (paintable));
 
-  gtk_tooltip_window_set_image_icon (GTK_TOOLTIP_WINDOW (tooltip->window), pixbuf);
-}
-
-/**
- * gtk_tooltip_set_icon_from_stock:
- * @tooltip: a #GtkTooltip
- * @stock_id: (allow-none): a stock id, or %NULL
- * @size: (type int): a stock icon size (#GtkIconSize)
- *
- * Sets the icon of the tooltip (which is in front of the text) to be
- * the stock item indicated by @stock_id with the size indicated
- * by @size.  If @stock_id is %NULL, the image will be hidden.
- *
- * Since: 2.12
- *
- * Deprecated: 3.10: Use gtk_tooltip_set_icon_from_icon_name() instead.
- */
-void
-gtk_tooltip_set_icon_from_stock (GtkTooltip  *tooltip,
-				 const gchar *stock_id,
-				 GtkIconSize  size)
-{
-  g_return_if_fail (GTK_IS_TOOLTIP (tooltip));
-
-  gtk_tooltip_window_set_image_icon_from_stock (GTK_TOOLTIP_WINDOW (tooltip->window),
-                                                stock_id,
-                                                size);
+  gtk_tooltip_window_set_image_icon (GTK_TOOLTIP_WINDOW (tooltip->window), paintable);
 }
 
 /**
  * gtk_tooltip_set_icon_from_icon_name:
  * @tooltip: a #GtkTooltip
  * @icon_name: (allow-none): an icon name, or %NULL
- * @size: (type int): a stock icon size (#GtkIconSize)
  *
  * Sets the icon of the tooltip (which is in front of the text) to be
  * the icon indicated by @icon_name with the size indicated
  * by @size.  If @icon_name is %NULL, the image will be hidden.
- *
- * Since: 2.14
  */
 void
 gtk_tooltip_set_icon_from_icon_name (GtkTooltip  *tooltip,
-				     const gchar *icon_name,
-				     GtkIconSize  size)
+				     const gchar *icon_name)
 {
   g_return_if_fail (GTK_IS_TOOLTIP (tooltip));
 
   gtk_tooltip_window_set_image_icon_from_name (GTK_TOOLTIP_WINDOW (tooltip->window),
-                                               icon_name,
-                                               size);
+                                               icon_name);
 }
 
 /**
  * gtk_tooltip_set_icon_from_gicon:
  * @tooltip: a #GtkTooltip
  * @gicon: (allow-none): a #GIcon representing the icon, or %NULL
- * @size: (type int): a stock icon size (#GtkIconSize)
  *
  * Sets the icon of the tooltip (which is in front of the text)
  * to be the icon indicated by @gicon with the size indicated
  * by @size. If @gicon is %NULL, the image will be hidden.
- *
- * Since: 2.20
  */
 void
 gtk_tooltip_set_icon_from_gicon (GtkTooltip  *tooltip,
-				 GIcon       *gicon,
-				 GtkIconSize  size)
+				 GIcon       *gicon)
 {
   g_return_if_fail (GTK_IS_TOOLTIP (tooltip));
 
   gtk_tooltip_window_set_image_icon_from_gicon (GTK_TOOLTIP_WINDOW (tooltip->window),
-                                                gicon,
-                                                size);
+                                                gicon);
 }
 
 /**
@@ -375,9 +324,6 @@ gtk_tooltip_set_icon_from_gicon (GtkTooltip  *tooltip,
  * By default a box with a #GtkImage and #GtkLabel is embedded in 
  * the tooltip, which can be configured using gtk_tooltip_set_markup() 
  * and gtk_tooltip_set_icon().
-
- *
- * Since: 2.12
  */
 void
 gtk_tooltip_set_custom (GtkTooltip *tooltip,
@@ -407,8 +353,6 @@ gtk_tooltip_set_custom (GtkTooltip *tooltip,
  * For setting tooltips on #GtkTreeView, please refer to the convenience
  * functions for this: gtk_tree_view_set_tooltip_row() and
  * gtk_tree_view_set_tooltip_cell().
- *
- * Since: 2.12
  */
 void
 gtk_tooltip_set_tip_area (GtkTooltip         *tooltip,
@@ -433,49 +377,35 @@ gtk_tooltip_set_tip_area (GtkTooltip         *tooltip,
  * visible tooltip, or to show/hide the current tooltip.  This function is
  * useful to call when, for example, the state of the widget changed by a
  * key press.
- *
- * Since: 2.12
  */
 void
-gtk_tooltip_trigger_tooltip_query (GdkDisplay *display)
+gtk_tooltip_trigger_tooltip_query (GtkWidget *widget)
 {
-  gint x, y;
-  GdkWindow *window;
-  GdkEvent event;
+  GdkDisplay *display;
+  double x, y;
+  GdkSurface *surface;
   GdkDevice *device;
+  GtkWidget *toplevel;
+  int dx, dy;
+
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+
+  display = gtk_widget_get_display (widget);
 
   /* Trigger logic as if the mouse moved */
   device = gdk_seat_get_pointer (gdk_display_get_default_seat (display));
-  window = gdk_device_get_window_at_position (device, &x, &y);
-  if (!window)
+  surface = gdk_device_get_surface_at_position (device, &x, &y);
+  if (!surface)
     return;
 
-  event.type = GDK_MOTION_NOTIFY;
-  event.motion.window = window;
-  event.motion.x = x;
-  event.motion.y = y;
-  event.motion.is_hint = FALSE;
+  toplevel = gtk_widget_get_toplevel (widget);
 
-  gdk_window_get_root_coords (window, x, y, &x, &y);
-  event.motion.x_root = x;
-  event.motion.y_root = y;
+  if (gtk_widget_get_surface (toplevel) != surface)
+    return;
 
-  gtk_tooltip_handle_event_internal (&event);
-}
+  gtk_widget_translate_coordinates (toplevel, widget, round (x), round (y), &dx, &dy);
 
-/* private functions */
-
-static void
-gtk_tooltip_reset (GtkTooltip *tooltip)
-{
-  gtk_tooltip_set_markup (tooltip, NULL);
-  gtk_tooltip_set_icon (tooltip, NULL);
-  gtk_tooltip_set_tip_area (tooltip, NULL);
-
-  /* See if the custom widget is again set from the query-tooltip
-   * callback.
-   */
-  tooltip->custom_was_reset = FALSE;
+  gtk_tooltip_handle_event_internal (GDK_MOTION_NOTIFY, surface, widget, dx, dy);
 }
 
 static void
@@ -487,265 +417,29 @@ gtk_tooltip_window_hide (GtkWidget *widget,
   gtk_tooltip_set_custom (tooltip, NULL);
 }
 
-/* event handling, etc */
-
-struct ChildLocation
-{
-  GtkWidget *child;
-  GtkWidget *container;
-
-  gint x;
-  gint y;
-};
-
-static void
-prepend_and_ref_widget (GtkWidget *widget,
-                        gpointer   data)
-{
-  GSList **slist_p = data;
-
-  *slist_p = g_slist_prepend (*slist_p, g_object_ref (widget));
-}
-
-static void
-child_location_foreach (GtkWidget *child,
-			gpointer   data)
-{
-  GtkAllocation child_allocation;
-  gint x, y;
-  struct ChildLocation *child_loc = data;
-
-  /* Ignore invisible widgets */
-  if (!gtk_widget_is_drawable (child))
-    return;
-
-  gtk_widget_get_allocation (child, &child_allocation);
-
-  x = 0;
-  y = 0;
-
-  /* (child_loc->x, child_loc->y) are relative to
-   * child_loc->container's allocation.
-   */
-
-  if (!child_loc->child &&
-      gtk_widget_translate_coordinates (child_loc->container, child,
-					child_loc->x, child_loc->y,
-					&x, &y))
-    {
-      /* (x, y) relative to child's allocation. */
-      if (x >= 0 && x < child_allocation.width
-	  && y >= 0 && y < child_allocation.height)
-        {
-	  if (GTK_IS_CONTAINER (child))
-	    {
-	      struct ChildLocation tmp = { NULL, NULL, 0, 0 };
-              GSList *children = NULL, *tmp_list;
-
-	      /* Take (x, y) relative the child's allocation and
-	       * recurse.
-	       */
-	      tmp.x = x;
-	      tmp.y = y;
-	      tmp.container = child;
-
-	      gtk_container_forall (GTK_CONTAINER (child),
-				    prepend_and_ref_widget, &children);
-
-              for (tmp_list = children; tmp_list; tmp_list = tmp_list->next)
-                {
-                  child_location_foreach (tmp_list->data, &tmp);
-                  g_object_unref (tmp_list->data);
-                }
-
-	      if (tmp.child)
-		child_loc->child = tmp.child;
-	      else
-		child_loc->child = child;
-
-              g_slist_free (children);
-	    }
-	  else
-	    child_loc->child = child;
-	}
-    }
-}
-
-/* Translates coordinates from dest_widget->window relative (src_x, src_y),
- * to allocation relative (dest_x, dest_y) of dest_widget.
- */
-static void
-window_to_alloc (GtkWidget *dest_widget,
-		 gint       src_x,
-		 gint       src_y,
-		 gint      *dest_x,
-		 gint      *dest_y)
-{
-  GtkAllocation allocation;
-
-  gtk_widget_get_allocation (dest_widget, &allocation);
-
-  /* Translate from window relative to allocation relative */
-  if (gtk_widget_get_has_window (dest_widget) &&
-      gtk_widget_get_parent (dest_widget))
-    {
-      gint wx, wy;
-      gdk_window_get_position (gtk_widget_get_window (dest_widget),
-                               &wx, &wy);
-
-      /* Offset coordinates if widget->window is smaller than
-       * widget->allocation.
-       */
-      src_x += wx - allocation.x;
-      src_y += wy - allocation.y;
-    }
-  else
-    {
-      src_x -= allocation.x;
-      src_y -= allocation.y;
-    }
-
-  if (dest_x)
-    *dest_x = src_x;
-  if (dest_y)
-    *dest_y = src_y;
-}
-
-/* Translates coordinates from window relative (x, y) to
- * allocation relative (x, y) of the returned widget.
- */
 GtkWidget *
-_gtk_widget_find_at_coords (GdkWindow *window,
-                            gint       window_x,
-                            gint       window_y,
+_gtk_widget_find_at_coords (GdkSurface *surface,
+                            gint       surface_x,
+                            gint       surface_y,
                             gint      *widget_x,
                             gint      *widget_y)
 {
   GtkWidget *event_widget;
-  struct ChildLocation child_loc = { NULL, NULL, 0, 0 };
+  GtkWidget *picked_widget;
 
-  g_return_val_if_fail (GDK_IS_WINDOW (window), NULL);
+  g_return_val_if_fail (GDK_IS_SURFACE (surface), NULL);
 
-  gdk_window_get_user_data (window, (void **)&event_widget);
+  event_widget = gtk_root_get_for_surface (surface);
 
   if (!event_widget)
     return NULL;
 
-  /* Coordinates are relative to event window */
-  child_loc.x = window_x;
-  child_loc.y = window_y;
+  picked_widget = gtk_widget_pick (event_widget, surface_x, surface_y, GTK_PICK_INSENSITIVE);
 
-  /* We go down the window hierarchy to the widget->window,
-   * coordinates stay relative to the current window.
-   * We end up with window == widget->window, coordinates relative to that.
-   */
-  while (window && window != gtk_widget_get_window (event_widget))
-    {
-      gdouble px, py;
+  if (picked_widget != NULL)
+    gtk_widget_translate_coordinates (event_widget, picked_widget, surface_x, surface_y, widget_x, widget_y);
 
-      gdk_window_coords_to_parent (window,
-                                   child_loc.x, child_loc.y,
-                                   &px, &py);
-      child_loc.x = px;
-      child_loc.y = py;
-
-      window = gdk_window_get_effective_parent (window);
-    }
-
-  /* Failing to find widget->window can happen for e.g. a detached handle box;
-   * chaining ::query-tooltip up to its parent probably makes little sense,
-   * and users better implement tooltips on handle_box->child.
-   * so we simply ignore the event for tooltips here.
-   */
-  if (!window)
-    return NULL;
-
-  /* Convert the window relative coordinates to allocation
-   * relative coordinates.
-   */
-  window_to_alloc (event_widget,
-		   child_loc.x, child_loc.y,
-		   &child_loc.x, &child_loc.y);
-
-  if (GTK_IS_CONTAINER (event_widget))
-    {
-      GtkWidget *container = event_widget;
-
-      child_loc.container = event_widget;
-      child_loc.child = NULL;
-
-      gtk_container_forall (GTK_CONTAINER (event_widget),
-			    child_location_foreach, &child_loc);
-
-      /* Here we have a widget, with coordinates relative to
-       * child_loc.container's allocation.
-       */
-
-      if (child_loc.child)
-	event_widget = child_loc.child;
-      else if (child_loc.container)
-	event_widget = child_loc.container;
-
-      /* Translate to event_widget's allocation */
-      gtk_widget_translate_coordinates (container, event_widget,
-					child_loc.x, child_loc.y,
-					&child_loc.x, &child_loc.y);
-    }
-
-  /* We return (x, y) relative to the allocation of event_widget. */
-  if (widget_x)
-    *widget_x = child_loc.x;
-  if (widget_y)
-    *widget_y = child_loc.y;
-
-  return event_widget;
-}
-
-/* Ignores (x, y) on input, translates event coordinates to
- * allocation relative (x, y) of the returned widget.
- */
-static GtkWidget *
-find_topmost_widget_coords_from_event (GdkEvent *event,
-				       gint     *x,
-				       gint     *y)
-{
-  GtkAllocation allocation;
-  gint tx, ty;
-  gdouble dx, dy;
-  GtkWidget *tmp;
-
-  gdk_event_get_coords (event, &dx, &dy);
-
-  /* Returns coordinates relative to tmp's allocation. */
-  tmp = _gtk_widget_find_at_coords (event->any.window, dx, dy, &tx, &ty);
-
-  if (!tmp)
-    return NULL;
-
-  /* Make sure the pointer can actually be on the widget returned. */
-  gtk_widget_get_allocation (tmp, &allocation);
-  allocation.x = 0;
-  allocation.y = 0;
-  if (GTK_IS_WINDOW (tmp))
-    {
-      GtkBorder border;
-      _gtk_window_get_shadow_width (GTK_WINDOW (tmp), &border);
-      allocation.x = border.left;
-      allocation.y = border.top;
-      allocation.width -= border.left + border.right;
-      allocation.height -= border.top + border.bottom;
-    }
-
-  if (tx < allocation.x || tx >= allocation.width ||
-      ty < allocation.y || ty >= allocation.height)
-    return NULL;
-
-  if (x)
-    *x = tx;
-  if (y)
-    *y = ty;
-
-  return tmp;
+  return picked_widget;
 }
 
 static gint
@@ -787,26 +481,26 @@ gtk_tooltip_display_closed (GdkDisplay *display,
 }
 
 static void
-gtk_tooltip_set_last_window (GtkTooltip *tooltip,
-			     GdkWindow  *window)
+gtk_tooltip_set_last_surface (GtkTooltip *tooltip,
+                              GdkSurface  *surface)
 {
   GtkWidget *window_widget = NULL;
 
-  if (tooltip->last_window == window)
+  if (tooltip->last_surface == surface)
     return;
 
-  if (tooltip->last_window)
-    g_object_remove_weak_pointer (G_OBJECT (tooltip->last_window),
-				  (gpointer *) &tooltip->last_window);
+  if (tooltip->last_surface)
+    g_object_remove_weak_pointer (G_OBJECT (tooltip->last_surface),
+				  (gpointer *) &tooltip->last_surface);
 
-  tooltip->last_window = window;
+  tooltip->last_surface = surface;
 
-  if (tooltip->last_window)
-    g_object_add_weak_pointer (G_OBJECT (tooltip->last_window),
-			       (gpointer *) &tooltip->last_window);
+  if (tooltip->last_surface)
+    g_object_add_weak_pointer (G_OBJECT (tooltip->last_surface),
+			       (gpointer *) &tooltip->last_surface);
 
-  if (window)
-    gdk_window_get_user_data (window, (gpointer *) &window_widget);
+  if (surface)
+    window_widget = gtk_root_get_for_surface (surface);
 
   if (window_widget)
     window_widget = gtk_widget_get_toplevel (window_widget);
@@ -830,14 +524,22 @@ gtk_tooltip_run_requery (GtkWidget  **widget,
   gboolean has_tooltip = FALSE;
   gboolean return_value = FALSE;
 
-  gtk_tooltip_reset (tooltip);
+  /* Reset tooltip */
+  gtk_tooltip_set_markup (tooltip, NULL);
+  gtk_tooltip_set_icon (tooltip, NULL);
+  gtk_tooltip_set_tip_area (tooltip, NULL);
+
+  /* See if the custom widget is again set from the query-tooltip
+   * callback.
+   */
+  tooltip->custom_was_reset = FALSE;
 
   do
     {
       has_tooltip = gtk_widget_get_has_tooltip (*widget);
 
       if (has_tooltip)
-        return_value = gtk_widget_query_tooltip (*widget, *x, *y, tooltip->keyboard_mode_enabled, tooltip);
+        return_value = gtk_widget_query_tooltip (*widget, *x, *y, FALSE, tooltip);
 
       if (!return_value)
         {
@@ -868,49 +570,41 @@ gtk_tooltip_position (GtkTooltip *tooltip,
 		      GtkWidget  *new_tooltip_widget,
                       GdkDevice  *device)
 {
-  GdkScreen *screen;
   GtkSettings *settings;
+  graphene_rect_t anchor_bounds;
   GdkRectangle anchor_rect;
-  GdkWindow *window;
-  GdkWindow *widget_window;
-  GdkWindow *effective_toplevel;
+  GdkSurface *surface;
+  GdkSurface *effective_toplevel;
   GtkWidget *toplevel;
   int rect_anchor_dx = 0;
   int cursor_size;
   int anchor_rect_padding;
 
   gtk_widget_realize (GTK_WIDGET (tooltip->current_window));
-  gtk_window_move_resize (tooltip->current_window);
-  window = _gtk_widget_get_window (GTK_WIDGET (tooltip->current_window));
+  surface = _gtk_widget_get_surface (GTK_WIDGET (tooltip->current_window));
 
   tooltip->tooltip_widget = new_tooltip_widget;
 
   toplevel = _gtk_widget_get_toplevel (new_tooltip_widget);
-  gtk_widget_translate_coordinates (new_tooltip_widget, toplevel,
-                                    0, 0,
-                                    &anchor_rect.x, &anchor_rect.y);
+  if (gtk_widget_compute_bounds (new_tooltip_widget, toplevel, &anchor_bounds))
+    {
+      anchor_rect = (GdkRectangle) {
+        floorf (anchor_bounds.origin.x), floorf (anchor_bounds.origin.y),
+        ceilf (anchor_bounds.size.width), ceilf (anchor_bounds.size.height)
+      };
+    }
+  else
+    {
+      anchor_rect = (GdkRectangle) { 0, 0, 0, 0 };
+    }
 
-  anchor_rect.width = gtk_widget_get_allocated_width (new_tooltip_widget);
-  anchor_rect.height = gtk_widget_get_allocated_height (new_tooltip_widget);
-
-  screen = gdk_window_get_screen (window);
-  settings = gtk_settings_get_for_screen (screen);
+  settings = gtk_settings_get_for_display (display);
   g_object_get (settings,
                 "gtk-cursor-theme-size", &cursor_size,
                 NULL);
 
   if (cursor_size == 0)
-    cursor_size = gdk_display_get_default_cursor_size (display);
-
-#ifdef GDK_WINDOWING_X11
-  if (GDK_IS_X11_SCREEN (screen))
-    {
-      /* Cursor size on X11 comes directly from XSettings which
-       * report physical sizes, unlike on other backends. So in
-       * that case we have to scale the retrieved cursor_size */
-       cursor_size /= gtk_widget_get_scale_factor (new_tooltip_widget);
-    }
-#endif
+    cursor_size = 16;
 
   if (device)
     anchor_rect_padding = MAX (4, cursor_size - 32);
@@ -927,6 +621,7 @@ gtk_tooltip_position (GtkTooltip *tooltip,
       const int max_x_distance = 32;
       /* Max 48x48 icon + default padding */
       const int max_anchor_rect_height = 48 + 8;
+      double px, py;
       int pointer_x, pointer_y;
 
       /*
@@ -941,11 +636,10 @@ gtk_tooltip_position (GtkTooltip *tooltip,
        * If the anchor rectangle isn't to tall, make sure the tooltip isn't too
        * far away from the pointer position.
        */
-      widget_window = _gtk_widget_get_window (new_tooltip_widget);
-      effective_toplevel = gdk_window_get_effective_toplevel (widget_window);
-      gdk_window_get_device_position (effective_toplevel,
-                                      device,
-                                      &pointer_x, &pointer_y, NULL);
+      effective_toplevel = _gtk_widget_get_surface (toplevel);
+      gdk_surface_get_device_position (effective_toplevel, device, &px, &py, NULL);
+      pointer_x = round (px);
+      pointer_y = round (py);
 
       if (anchor_rect.height > max_anchor_rect_height)
         {
@@ -972,51 +666,42 @@ gtk_tooltip_position (GtkTooltip *tooltip,
   gtk_window_set_transient_for (GTK_WINDOW (tooltip->current_window),
                                 GTK_WINDOW (toplevel));
 
-  gdk_window_move_to_rect (window,
-                           &anchor_rect,
-                           GDK_GRAVITY_SOUTH,
-                           GDK_GRAVITY_NORTH,
-                           GDK_ANCHOR_FLIP_Y | GDK_ANCHOR_SLIDE_X,
-                           rect_anchor_dx, 0);
+  gdk_surface_move_to_rect (surface,
+                            &anchor_rect,
+                            GDK_GRAVITY_SOUTH,
+                            GDK_GRAVITY_NORTH,
+                            GDK_ANCHOR_FLIP_Y | GDK_ANCHOR_SLIDE_X,
+                            rect_anchor_dx, 0);
   gtk_widget_show (GTK_WIDGET (tooltip->current_window));
 }
 
 static void
 gtk_tooltip_show_tooltip (GdkDisplay *display)
 {
+  double px, py;
   gint x, y;
-  GdkScreen *screen;
-  GdkDevice *device;
-  GdkWindow *window;
+  GdkSurface *surface;
   GtkWidget *tooltip_widget;
+  GdkDevice *device;
   GtkTooltip *tooltip;
   gboolean return_value = FALSE;
 
   tooltip = g_object_get_qdata (G_OBJECT (display), quark_current_tooltip);
 
-  if (tooltip->keyboard_mode_enabled)
-    {
-      x = y = -1;
-      tooltip_widget = tooltip->keyboard_widget;
-      device = NULL;
-    }
-  else
-    {
-      gint tx, ty;
+  {
+    surface = tooltip->last_surface;
 
-      window = tooltip->last_window;
+    if (!GDK_IS_SURFACE (surface))
+      return;
 
-      if (!GDK_IS_WINDOW (window))
-        return;
+    device = gdk_seat_get_pointer (gdk_display_get_default_seat (display));
 
-      device = gdk_seat_get_pointer (gdk_display_get_default_seat (display));
+    gdk_surface_get_device_position (surface, device, &px, &py, NULL);
+    x = round (px);
+    y = round (py);
 
-      gdk_window_get_device_position (window, device, &x, &y, NULL);
-
-      gdk_window_get_root_coords (window, x, y, &tx, &ty);
-
-      tooltip_widget = _gtk_widget_find_at_coords (window, x, y, &x, &y);
-    }
+    tooltip_widget = _gtk_widget_find_at_coords (surface, x, y, &x, &y);
+  }
 
   if (!tooltip_widget)
     return;
@@ -1033,16 +718,14 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
         tooltip->current_window = GTK_WINDOW (GTK_TOOLTIP (tooltip)->window);
     }
 
-  screen = gtk_widget_get_screen (tooltip_widget);
-
   /* FIXME: should use tooltip->current_window iso tooltip->window */
-  if (screen != gtk_widget_get_screen (tooltip->window))
+  if (display != gtk_widget_get_display (tooltip->window))
     {
       g_signal_handlers_disconnect_by_func (display,
                                             gtk_tooltip_display_closed,
                                             tooltip);
 
-      gtk_window_set_screen (GTK_WINDOW (tooltip->window), screen);
+      gtk_window_set_display (GTK_WINDOW (tooltip->window), display);
 
       g_signal_connect (display, "closed",
                         G_CALLBACK (gtk_tooltip_display_closed), tooltip);
@@ -1064,6 +747,8 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
 static void
 gtk_tooltip_hide_tooltip (GtkTooltip *tooltip)
 {
+  guint timeout = BROWSE_DISABLE_TIMEOUT;
+
   if (!tooltip)
     return;
 
@@ -1078,30 +763,17 @@ gtk_tooltip_hide_tooltip (GtkTooltip *tooltip)
 
   tooltip->tooltip_widget = NULL;
 
-  if (!tooltip->keyboard_mode_enabled)
+  /* The tooltip is gone, after (by default, should be configurable) 500ms
+   * we want to turn off browse mode
+   */
+  if (!tooltip->browse_mode_timeout_id)
     {
-      guint timeout = BROWSE_DISABLE_TIMEOUT;
-
-      /* The tooltip is gone, after (by default, should be configurable) 500ms
-       * we want to turn off browse mode
-       */
-      if (!tooltip->browse_mode_timeout_id)
-        {
-	  tooltip->browse_mode_timeout_id =
-	    gdk_threads_add_timeout_full (0, timeout,
-					  tooltip_browse_mode_expired,
-					  g_object_ref (tooltip),
-					  g_object_unref);
-	  g_source_set_name_by_id (tooltip->browse_mode_timeout_id, "[gtk+] tooltip_browse_mode_expired");
-	}
-    }
-  else
-    {
-      if (tooltip->browse_mode_timeout_id)
-        {
-	  g_source_remove (tooltip->browse_mode_timeout_id);
-	  tooltip->browse_mode_timeout_id = 0;
-	}
+      tooltip->browse_mode_timeout_id =
+        g_timeout_add_full (0, timeout,
+                            tooltip_browse_mode_expired,
+                            g_object_ref (tooltip),
+                            g_object_unref);
+      g_source_set_name_by_id (tooltip->browse_mode_timeout_id, "[gtk] tooltip_browse_mode_expired");
     }
 
   if (tooltip->current_window)
@@ -1152,127 +824,11 @@ gtk_tooltip_start_delay (GdkDisplay *display)
   else
     timeout = HOVER_TIMEOUT;
 
-  tooltip->timeout_id = gdk_threads_add_timeout_full (0, timeout,
-						      tooltip_popup_timeout,
-						      g_object_ref (display),
-						      g_object_unref);
-  g_source_set_name_by_id (tooltip->timeout_id, "[gtk+] tooltip_popup_timeout");
-}
-
-void
-_gtk_tooltip_focus_in (GtkWidget *widget)
-{
-  gint x, y;
-  gboolean return_value = FALSE;
-  GdkDisplay *display;
-  GtkTooltip *tooltip;
-  GdkDevice *device;
-
-  /* Get current tooltip for this display */
-  display = gtk_widget_get_display (widget);
-  tooltip = g_object_get_qdata (G_OBJECT (display), quark_current_tooltip);
-
-  /* Check if keyboard mode is enabled at this moment */
-  if (!tooltip || !tooltip->keyboard_mode_enabled)
-    return;
-
-  device = gtk_get_current_event_device ();
-
-  if (device && gdk_device_get_source (device) == GDK_SOURCE_KEYBOARD)
-    device = gdk_device_get_associated_device (device);
-
-  /* This function should be called by either a focus in event,
-   * or a key binding. In either case there should be a device.
-   */
-  if (!device)
-    return;
-
-  if (tooltip->keyboard_widget)
-    g_object_unref (tooltip->keyboard_widget);
-
-  tooltip->keyboard_widget = g_object_ref (widget);
-
-  gdk_window_get_device_position (gtk_widget_get_window (widget),
-                                  device, &x, &y, NULL);
-
-  return_value = gtk_tooltip_run_requery (&widget, tooltip, &x, &y);
-  if (!return_value)
-    {
-      gtk_tooltip_hide_tooltip (tooltip);
-      return;
-    }
-
-  if (!tooltip->current_window)
-    {
-      if (gtk_widget_get_tooltip_window (widget))
-	tooltip->current_window = gtk_widget_get_tooltip_window (widget);
-      else
-	tooltip->current_window = GTK_WINDOW (GTK_TOOLTIP (tooltip)->window);
-    }
-
-  gtk_tooltip_show_tooltip (display);
-}
-
-void
-_gtk_tooltip_focus_out (GtkWidget *widget)
-{
-  GdkDisplay *display;
-  GtkTooltip *tooltip;
-
-  /* Get current tooltip for this display */
-  display = gtk_widget_get_display (widget);
-  tooltip = g_object_get_qdata (G_OBJECT (display), quark_current_tooltip);
-
-  if (!tooltip || !tooltip->keyboard_mode_enabled)
-    return;
-
-  if (tooltip->keyboard_widget)
-    {
-      g_object_unref (tooltip->keyboard_widget);
-      tooltip->keyboard_widget = NULL;
-    }
-
-  gtk_tooltip_hide_tooltip (tooltip);
-}
-
-void
-_gtk_tooltip_toggle_keyboard_mode (GtkWidget *widget)
-{
-  GdkDisplay *display;
-  GtkTooltip *tooltip;
-
-  display = gtk_widget_get_display (widget);
-  tooltip = g_object_get_qdata (G_OBJECT (display), quark_current_tooltip);
-
-  if (!tooltip)
-    {
-      tooltip = g_object_new (GTK_TYPE_TOOLTIP, NULL);
-      g_object_set_qdata_full (G_OBJECT (display),
-			       quark_current_tooltip,
-			       tooltip,
-                               g_object_unref);
-      g_signal_connect (display, "closed",
-			G_CALLBACK (gtk_tooltip_display_closed),
-			tooltip);
-    }
-
-  tooltip->keyboard_mode_enabled ^= 1;
-
-  if (tooltip->keyboard_mode_enabled)
-    {
-      tooltip->keyboard_widget = g_object_ref (widget);
-      _gtk_tooltip_focus_in (widget);
-    }
-  else
-    {
-      if (tooltip->keyboard_widget)
-        {
-	  g_object_unref (tooltip->keyboard_widget);
-	  tooltip->keyboard_widget = NULL;
-	}
-
-      gtk_tooltip_hide_tooltip (tooltip);
-    }
+  tooltip->timeout_id = g_timeout_add_full (0, timeout,
+                                            tooltip_popup_timeout,
+                                            g_object_ref (display),
+                                            g_object_unref);
+  g_source_set_name_by_id (tooltip->timeout_id, "[gtk] tooltip_popup_timeout");
 }
 
 void
@@ -1291,27 +847,38 @@ _gtk_tooltip_hide (GtkWidget *widget)
     gtk_tooltip_hide_tooltip (tooltip);
 }
 
-void
-_gtk_tooltip_hide_in_display (GdkDisplay *display)
-{
-  GtkTooltip *tooltip;
-
-  if (!display)
-    return;
-
-  tooltip = g_object_get_qdata (G_OBJECT (display), quark_current_tooltip);
-
-  if (!tooltip || !GTK_TOOLTIP_VISIBLE (tooltip))
-    return;
-
-  gtk_tooltip_hide_tooltip (tooltip);
-}
-
 static gboolean
 tooltips_enabled (GdkEvent *event)
 {
   GdkDevice *source_device;
   GdkInputSource source;
+  GdkModifierType event_state = 0;
+
+  switch ((guint)gdk_event_get_event_type (event))
+    {
+    case GDK_ENTER_NOTIFY:
+    case GDK_LEAVE_NOTIFY:
+    case GDK_BUTTON_PRESS:
+    case GDK_KEY_PRESS:
+    case GDK_DRAG_ENTER:
+    case GDK_GRAB_BROKEN:
+    case GDK_MOTION_NOTIFY:
+    case GDK_TOUCH_UPDATE:
+    case GDK_SCROLL:
+      break; /* OK */
+
+    default:
+      return FALSE;
+    }
+
+  gdk_event_get_state (event, &event_state);
+  if ((event_state &
+       (GDK_BUTTON1_MASK |
+        GDK_BUTTON2_MASK |
+        GDK_BUTTON3_MASK |
+        GDK_BUTTON4_MASK |
+        GDK_BUTTON5_MASK)) != 0)
+    return FALSE;
 
   source_device = gdk_event_get_source_device (event);
 
@@ -1329,55 +896,42 @@ tooltips_enabled (GdkEvent *event)
 void
 _gtk_tooltip_handle_event (GdkEvent *event)
 {
+  GdkEventType event_type;
+  GtkWidget *target;
+  GdkSurface *surface;
+  gdouble dx, dy;
+
   if (!tooltips_enabled (event))
     return;
 
-  gtk_tooltip_handle_event_internal (event);
+  event_type = gdk_event_get_event_type (event);
+  surface = gdk_event_get_surface (event);
+  gdk_event_get_coords (event, &dx, &dy);
+  target = gtk_get_event_target (event);
+
+  gtk_tooltip_handle_event_internal (event_type, surface, target, dx, dy);
 }
 
+/* dx/dy must be in @target_widget's coordinates */
 static void
-gtk_tooltip_handle_event_internal (GdkEvent *event)
+gtk_tooltip_handle_event_internal (GdkEventType   event_type,
+                                   GdkSurface    *surface,
+                                   GtkWidget     *target_widget,
+                                   gdouble       dx,
+                                   gdouble       dy)
 {
-  gint x, y;
-  GtkWidget *has_tooltip_widget;
+  int x = dx, y = dy;
   GdkDisplay *display;
   GtkTooltip *current_tooltip;
 
-  /* Returns coordinates relative to has_tooltip_widget's allocation. */
-  has_tooltip_widget = find_topmost_widget_coords_from_event (event, &x, &y);
-  display = gdk_window_get_display (event->any.window);
+  display = gdk_surface_get_display (surface);
   current_tooltip = g_object_get_qdata (G_OBJECT (display), quark_current_tooltip);
 
   if (current_tooltip)
-    {
-      gtk_tooltip_set_last_window (current_tooltip, event->any.window);
-    }
-
-  if (current_tooltip && current_tooltip->keyboard_mode_enabled)
-    {
-      gboolean return_value;
-
-      has_tooltip_widget = current_tooltip->keyboard_widget;
-      if (!has_tooltip_widget)
-	return;
-
-      return_value = gtk_tooltip_run_requery (&has_tooltip_widget,
-					      current_tooltip,
-					      &x, &y);
-
-      if (!return_value)
-	gtk_tooltip_hide_tooltip (current_tooltip);
-      else
-	gtk_tooltip_start_delay (display);
-
-      return;
-    }
-
-  /* Always poll for a next motion event */
-  gdk_event_request_motions (&event->motion);
+    gtk_tooltip_set_last_surface (current_tooltip, surface);
 
   /* Hide the tooltip when there's no new tooltip widget */
-  if (!has_tooltip_widget)
+  if (!target_widget)
     {
       if (current_tooltip)
 	gtk_tooltip_hide_tooltip (current_tooltip);
@@ -1385,11 +939,9 @@ gtk_tooltip_handle_event_internal (GdkEvent *event)
       return;
     }
 
-  switch (event->type)
+  switch ((guint) event_type)
     {
       case GDK_BUTTON_PRESS:
-      case GDK_2BUTTON_PRESS:
-      case GDK_3BUTTON_PRESS:
       case GDK_KEY_PRESS:
       case GDK_DRAG_ENTER:
       case GDK_GRAB_BROKEN:
@@ -1403,29 +955,26 @@ gtk_tooltip_handle_event_internal (GdkEvent *event)
 	if (current_tooltip)
 	  {
 	    gboolean tip_area_set;
-	    GdkRectangle tip_area;
+            GdkRectangle tip_area;
 	    gboolean hide_tooltip;
 
 	    tip_area_set = current_tooltip->tip_area_set;
 	    tip_area = current_tooltip->tip_area;
 
-	    gtk_tooltip_run_requery (&has_tooltip_widget,
+	    gtk_tooltip_run_requery (&target_widget,
                                      current_tooltip,
                                      &x, &y);
 
 	    /* Leave notify should override the query function */
-	    hide_tooltip = (event->type == GDK_LEAVE_NOTIFY);
+	    hide_tooltip = (event_type == GDK_LEAVE_NOTIFY);
 
 	    /* Is the pointer above another widget now? */
 	    if (GTK_TOOLTIP_VISIBLE (current_tooltip))
-	      hide_tooltip |= has_tooltip_widget != current_tooltip->tooltip_widget;
+	      hide_tooltip |= target_widget != current_tooltip->tooltip_widget;
 
 	    /* Did the pointer move out of the previous "context area"? */
 	    if (tip_area_set)
-	      hide_tooltip |= (x <= tip_area.x
-			       || x >= tip_area.x + tip_area.width
-			       || y <= tip_area.y
-			       || y >= tip_area.y + tip_area.height);
+	      hide_tooltip |= !gdk_rectangle_contains_point (&tip_area, x, y);
 
 	    if (hide_tooltip)
 	      gtk_tooltip_hide_tooltip (current_tooltip);
@@ -1444,7 +993,7 @@ gtk_tooltip_handle_event_internal (GdkEvent *event)
 			      G_CALLBACK (gtk_tooltip_display_closed),
 			      current_tooltip);
 
-	    gtk_tooltip_set_last_window (current_tooltip, event->any.window);
+	    gtk_tooltip_set_last_surface (current_tooltip, surface);
 
 	    gtk_tooltip_start_delay (display);
 	  }
