@@ -31,16 +31,13 @@
 
 G_BEGIN_DECLS
 
-void _gtk_tooltip_focus_in               (GtkWidget          *widget);
-void _gtk_tooltip_focus_out              (GtkWidget          *widget);
-void _gtk_tooltip_toggle_keyboard_mode   (GtkWidget          *widget);
 void _gtk_tooltip_handle_event           (GdkEvent           *event);
 void _gtk_tooltip_hide                   (GtkWidget          *widget);
-void _gtk_tooltip_hide_in_display        (GdkDisplay         *display);
+void gtk_tooltip_trigger_tooltip_query   (GtkWidget          *widget);
 
-GtkWidget * _gtk_widget_find_at_coords   (GdkWindow          *window,
-                                          gint                window_x,
-                                          gint                window_y,
+GtkWidget * _gtk_widget_find_at_coords   (GdkSurface         *surface,
+                                          gint                surface_x,
+                                          gint                surface_y,
                                           gint               *widget_x,
                                           gint               *widget_y);
 
