@@ -44,13 +44,6 @@
 #include "gtkmenushellprivate.h"
 #include "gtkprivate.h"
 
-#undef GDK_DEPRECATED
-#undef GDK_DEPRECATED_FOR
-#define GDK_DEPRECATED
-#define GDK_DEPRECATED_FOR(f)
-
-#include "deprecated/gtktearoffmenuitem.h"
-
 /* GObjectClass */
 static void      gtk_tree_menu_constructed                    (GObject            *object);
 static void      gtk_tree_menu_dispose                        (GObject            *object);
@@ -65,20 +58,13 @@ static void      gtk_tree_menu_get_property                   (GObject
                                                                GParamSpec         *pspec);
 
 /* GtkWidgetClass */
-static void      gtk_tree_menu_get_preferred_width            (GtkWidget           *widget,
-                                                               gint                *minimum_size,
-                                                               gint                *natural_size);
-static void      gtk_tree_menu_get_preferred_height           (GtkWidget           *widget,
-                                                               gint                *minimum_size,
-                                                               gint                *natural_size);
-static void      gtk_tree_menu_get_preferred_width_for_height (GtkWidget           *widget,
-                                                               gint                 for_height,
-                                                               gint                *minimum_size,
-                                                               gint                *natural_size);
-static void      gtk_tree_menu_get_preferred_height_for_width (GtkWidget           *widget,
-                                                               gint                 for_width,
-                                                               gint                *minimum_size,
-                                                               gint                *natural_size);
+static void gtk_tree_menu_measure (GtkWidget      *widget,
+                                   GtkOrientation  orientation,
+                                   int             for_size,
+                                   int            *minimum,
+                                   int            *natural,
+                                   int            *minimum_baseline,
+                                   int            *natural_baseline);
 
 /* GtkCellLayoutIface */
 static void      gtk_tree_menu_cell_layout_init               (GtkCellLayoutIface  *iface);
@@ -87,15 +73,6 @@ static GtkCellArea *gtk_tree_menu_cell_layout_get_area        (GtkCellLayout
 
 /* TreeModel/DrawingArea callbacks and building menus/submenus */
 static inline void rebuild_menu                               (GtkTreeMenu          *menu);
-static gboolean   menu_occupied                               (GtkTreeMenu          *menu,
-                                                               guint                 left_attach,
-                                                               guint                 right_attach,
-                                                               guint                 top_attach,
-                                                               guint                 bottom_attach);
-static void       relayout_item                               (GtkTreeMenu          *menu,
-                                                               GtkWidget            *item,
-                                                               GtkTreeIter          *iter,
-                                                               GtkWidget            *prev);
 static void       gtk_tree_menu_populate                      (GtkTreeMenu          *menu);
 static GtkWidget *gtk_tree_menu_create_item                   (GtkTreeMenu          *menu,
                                                                GtkTreeIter          *iter,
@@ -163,14 +140,8 @@ struct _GtkTreeMenuPrivate
   gulong               row_reordered_id;
   gulong               row_changed_id;
 
-  /* Grid menu mode */
-  gint                 wrap_width;
-  gint                 row_span_col;
-  gint                 col_span_col;
-
   /* Flags */
   guint32              menu_with_header : 1;
-  guint32              tearoff     : 1;
 
   /* Row separators */
   GtkTreeViewRowSeparatorFunc row_separator_func;
@@ -182,11 +153,7 @@ enum {
   PROP_0,
   PROP_MODEL,
   PROP_ROOT,
-  PROP_CELL_AREA,
-  PROP_TEAROFF,
-  PROP_WRAP_WIDTH,
-  PROP_ROW_SPAN_COL,
-  PROP_COL_SPAN_COL
+  PROP_CELL_AREA
 };
 
 enum {
@@ -206,8 +173,6 @@ static void
 _gtk_tree_menu_init (GtkTreeMenu *menu)
 {
   menu->priv = _gtk_tree_menu_get_instance_private (menu);
-  menu->priv->row_span_col = -1;
-  menu->priv->col_span_col = -1;
 
   gtk_menu_set_reserve_toggle_size (GTK_MENU (menu), FALSE);
 }
@@ -226,10 +191,7 @@ _gtk_tree_menu_class_init (GtkTreeMenuClass *class)
   object_class->set_property = gtk_tree_menu_set_property;
   object_class->get_property = gtk_tree_menu_get_property;
 
-  widget_class->get_preferred_width  = gtk_tree_menu_get_preferred_width;
-  widget_class->get_preferred_height = gtk_tree_menu_get_preferred_height;
-  widget_class->get_preferred_width_for_height  = gtk_tree_menu_get_preferred_width_for_height;
-  widget_class->get_preferred_height_for_width = gtk_tree_menu_get_preferred_height_for_width;
+  widget_class->measure = gtk_tree_menu_measure;
 
   /*
    * GtkTreeMenu::menu-activate:
@@ -240,8 +202,6 @@ _gtk_tree_menu_class_init (GtkTreeMenuClass *class)
    * This signal is emitted to notify that a menu item in the #GtkTreeMenu
    * was activated and provides the path string from the #GtkTreeModel
    * to specify which row was selected.
-   *
-   * Since: 3.0
    */
   tree_menu_signals[SIGNAL_MENU_ACTIVATE] =
     g_signal_new (I_("menu-activate"),
@@ -256,8 +216,6 @@ _gtk_tree_menu_class_init (GtkTreeMenuClass *class)
    * GtkTreeMenu:model:
    *
    * The #GtkTreeModel from which the menu is constructed.
-   *
-   * Since: 3.0
    */
   g_object_class_install_property (object_class,
                                    PROP_MODEL,
@@ -279,8 +237,6 @@ _gtk_tree_menu_class_init (GtkTreeMenuClass *class)
    * If you dont provide a root for the #GtkTreeMenu then the whole
    * model will be added to the menu. Specifying a root allows you
    * to build a menu for a given #GtkTreePath and its children.
-   * 
-   * Since: 3.0
    */
   g_object_class_install_property (object_class,
                                    PROP_ROOT,
@@ -298,8 +254,6 @@ _gtk_tree_menu_class_init (GtkTreeMenuClass *class)
    *
    * You can provide a different cell area at object construction
    * time, otherwise the #GtkTreeMenu will use a #GtkCellAreaBox.
-   *
-   * Since: 3.0
    */
   g_object_class_install_property (object_class,
                                    PROP_CELL_AREA,
@@ -308,80 +262,6 @@ _gtk_tree_menu_class_init (GtkTreeMenuClass *class)
                                                         P_("The GtkCellArea used to layout cells"),
                                                         GTK_TYPE_CELL_AREA,
                                                         GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
-  /*
-   * GtkTreeMenu:tearoff:
-   *
-   * Specifies whether this menu comes with a leading tearoff menu item
-   *
-   * Since: 3.0
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_TEAROFF,
-                                   g_param_spec_boolean ("tearoff",
-                                                         P_("Tearoff"),
-                                                         P_("Whether the menu has a tearoff item"),
-                                                         FALSE,
-                                                         GTK_PARAM_READWRITE));
-
-  /*
-   * GtkTreeMenu:wrap-width:
-   *
-   * If wrap-width is set to a positive value, items in the popup will be laid
-   * out along multiple columns, starting a new row on reaching the wrap width.
-   *
-   * Since: 3.0
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_WRAP_WIDTH,
-                                   g_param_spec_int ("wrap-width",
-                                                     P_("Wrap Width"),
-                                                     P_("Wrap width for laying out items in a grid"),
-                                                     0,
-                                                     G_MAXINT,
-                                                     0,
-                                                     GTK_PARAM_READWRITE));
-
-  /*
-   * GtkTreeMenu:row-span-column:
-   *
-   * If this is set to a non-negative value, it must be the index of a column
-   * of type %G_TYPE_INT in the model. The value in that column for each item
-   * will determine how many rows that item will span in the popup. Therefore,
-   * values in this column must be greater than zero.
-   *
-   * Since: 3.0
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_ROW_SPAN_COL,
-                                   g_param_spec_int ("row-span-column",
-                                                     P_("Row span column"),
-                                                     P_("TreeModel column containing the row span values"),
-                                                     -1,
-                                                     G_MAXINT,
-                                                     -1,
-                                                     GTK_PARAM_READWRITE));
-
-  /*
-   * GtkTreeMenu:column-span-column:
-   *
-   * If this is set to a non-negative value, it must be the index of a column
-   * of type %G_TYPE_INT in the model. The value in that column for each item
-   * will determine how many columns that item will span in the popup.
-   * Therefore, values in this column must be greater than zero, and the sum of
-   * an item’s column position + span should not exceed #GtkTreeMenu:wrap-width.
-   *
-   * Since: 3.0
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_COL_SPAN_COL,
-                                   g_param_spec_int ("column-span-column",
-                                                     P_("Column span column"),
-                                                     P_("TreeModel column containing the column span values"),
-                                                     -1,
-                                                     G_MAXINT,
-                                                     -1,
-                                                     GTK_PARAM_READWRITE));
 }
 
 /****************************************************************
@@ -474,22 +354,6 @@ gtk_tree_menu_set_property (GObject            *object,
       gtk_tree_menu_set_area (menu, (GtkCellArea *)g_value_get_object (value));
       break;
 
-    case PROP_TEAROFF:
-      _gtk_tree_menu_set_tearoff (menu, g_value_get_boolean (value));
-      break;
-
-    case PROP_WRAP_WIDTH:
-      _gtk_tree_menu_set_wrap_width (menu, g_value_get_int (value));
-      break;
-
-     case PROP_ROW_SPAN_COL:
-      _gtk_tree_menu_set_row_span_column (menu, g_value_get_int (value));
-      break;
-
-     case PROP_COL_SPAN_COL:
-      _gtk_tree_menu_set_column_span_column (menu, g_value_get_int (value));
-      break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -519,10 +383,6 @@ gtk_tree_menu_get_property (GObject            *object,
       g_value_set_object (value, priv->area);
       break;
 
-    case PROP_TEAROFF:
-      g_value_set_boolean (value, priv->tearoff);
-      break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -566,12 +426,17 @@ sync_reserve_submenu_size (GtkTreeMenu *menu)
   g_list_free (children);
 }
 
+
 static void
-gtk_tree_menu_get_preferred_width (GtkWidget           *widget,
-                                   gint                *minimum_size,
-                                   gint                *natural_size)
+gtk_tree_menu_measure (GtkWidget      *widget,
+                       GtkOrientation  orientation,
+                       int             for_size,
+                       int            *minimum,
+                       int            *natural,
+                       int            *minimum_baseline,
+                       int            *natural_baseline)
 {
-  GtkTreeMenu        *menu = GTK_TREE_MENU (widget);
+  GtkTreeMenu *menu = GTK_TREE_MENU (widget);
   GtkTreeMenuPrivate *priv = menu->priv;
 
   /* We leave the requesting work up to the cellviews which operate in the same
@@ -585,70 +450,15 @@ gtk_tree_menu_get_preferred_width (GtkWidget           *widget,
 
   sync_reserve_submenu_size (menu);
 
-  GTK_WIDGET_CLASS (_gtk_tree_menu_parent_class)->get_preferred_width (widget, minimum_size, natural_size);
+  GTK_WIDGET_CLASS (_gtk_tree_menu_parent_class)->measure (widget,
+                                                           orientation,
+                                                           for_size,
+                                                           minimum, natural,
+                                                           minimum_baseline, natural_baseline);
 
   g_signal_handler_unblock (priv->context, priv->size_changed_id);
 }
 
-static void
-gtk_tree_menu_get_preferred_height (GtkWidget           *widget,
-                                    gint                *minimum_size,
-                                    gint                *natural_size)
-{
-  GtkTreeMenu        *menu = GTK_TREE_MENU (widget);
-  GtkTreeMenuPrivate *priv = menu->priv;
-
-  g_signal_handler_block (priv->context, priv->size_changed_id);
-
-  sync_reserve_submenu_size (menu);
-
-  GTK_WIDGET_CLASS (_gtk_tree_menu_parent_class)->get_preferred_height (widget, minimum_size, natural_size);
-
-  g_signal_handler_unblock (priv->context, priv->size_changed_id);
-}
-
-static void
-gtk_tree_menu_get_preferred_width_for_height (GtkWidget           *widget,
-                                              gint                 for_height,
-                                              gint                *minimum_size,
-                                              gint                *natural_size)
-{
-  GtkTreeMenu        *menu = GTK_TREE_MENU (widget);
-  GtkTreeMenuPrivate *priv = menu->priv;
-
-  /* We leave the requesting work up to the cellviews which operate in the same
-   * context, reserving space for the submenu indicator if any of the items have
-   * submenus ensures that every cellview will receive the same allocated width.
-   *
-   * Since GtkMenu does hieght-for-width correctly, we know that the width of
-   * every cell will be requested before the height-for-widths are requested.
-   */
-  g_signal_handler_block (priv->context, priv->size_changed_id);
-
-  sync_reserve_submenu_size (menu);
-
-  GTK_WIDGET_CLASS (_gtk_tree_menu_parent_class)->get_preferred_width_for_height (widget, for_height, minimum_size, natural_size);
-
-  g_signal_handler_unblock (priv->context, priv->size_changed_id);
-}
-
-static void
-gtk_tree_menu_get_preferred_height_for_width (GtkWidget           *widget,
-                                              gint                 for_width,
-                                              gint                *minimum_size,
-                                              gint                *natural_size)
-{
-  GtkTreeMenu        *menu = GTK_TREE_MENU (widget);
-  GtkTreeMenuPrivate *priv = menu->priv;
-
-  g_signal_handler_block (priv->context, priv->size_changed_id);
-
-  sync_reserve_submenu_size (menu);
-
-  GTK_WIDGET_CLASS (_gtk_tree_menu_parent_class)->get_preferred_height_for_width (widget, for_width, minimum_size, natural_size);
-
-  g_signal_handler_unblock (priv->context, priv->size_changed_id);
-}
 
 /****************************************************************
  *                      GtkCellLayoutIface                      *
@@ -703,7 +513,7 @@ gtk_tree_menu_get_path_item (GtkTreeMenu          *menu,
                 item = child;
             }
         }
-      else if (!GTK_IS_TEAROFF_MENU_ITEM (child))
+      else
         {
           GtkWidget *view = gtk_bin_get_child (GTK_BIN (child));
 
@@ -842,7 +652,7 @@ find_empty_submenu (GtkTreeMenu  *menu)
 
       /* Separators dont get submenus, if it already has a submenu then let
        * the submenu handle inserted rows */
-      if (!GTK_IS_SEPARATOR_MENU_ITEM (child) && !GTK_IS_TEAROFF_MENU_ITEM (child))
+      if (!GTK_IS_SEPARATOR_MENU_ITEM (child))
         {
           GtkWidget *view = gtk_bin_get_child (GTK_BIN (child));
 
@@ -879,32 +689,21 @@ row_inserted_cb (GtkTreeModel     *model,
   /* If the iter should be in this menu then go ahead and insert it */
   if (gtk_tree_menu_path_in_menu (menu, path, NULL))
     {
-      if (priv->wrap_width > 0)
-        rebuild_menu (menu);
-      else
-        {
-          /* Get the index of the path for this depth */
-          indices = gtk_tree_path_get_indices (path);
-          depth   = gtk_tree_path_get_depth (path);
-          index   = indices[depth -1];
+      /* Get the index of the path for this depth */
+      indices = gtk_tree_path_get_indices (path);
+      depth   = gtk_tree_path_get_depth (path);
+      index   = indices[depth -1];
 
-          /* Menus with a header include a menuitem for its root node
-           * and a separator menu item */
-          if (priv->menu_with_header)
-            index += 2;
+      /* Menus with a header include a menuitem for its root node
+       * and a separator menu item */
+      if (priv->menu_with_header)
+        index += 2;
 
-          /* Index after the tearoff item for the root menu if
-           * there is a tearoff item
-           */
-          if (priv->root == NULL && priv->tearoff)
-            index += 1;
+      item = gtk_tree_menu_create_item (menu, iter, FALSE);
+      gtk_menu_shell_insert (GTK_MENU_SHELL (menu), item, index);
 
-          item = gtk_tree_menu_create_item (menu, iter, FALSE);
-          gtk_menu_shell_insert (GTK_MENU_SHELL (menu), item, index);
-
-          /* Resize everything */
-          gtk_cell_area_context_reset (menu->priv->context);
-        }
+      /* Resize everything */
+      gtk_cell_area_context_reset (menu->priv->context);
     }
   else
     {
@@ -926,7 +725,6 @@ row_deleted_cb (GtkTreeModel     *model,
                 GtkTreePath      *path,
                 GtkTreeMenu      *menu)
 {
-  GtkTreeMenuPrivate *priv = menu->priv;
   GtkWidget          *item;
 
   /* If it's the header item we leave it to the parent menu
@@ -936,16 +734,11 @@ row_deleted_cb (GtkTreeModel     *model,
 
   if (item)
     {
-      if (priv->wrap_width > 0)
-        rebuild_menu (menu);
-      else
-        {
-          /* Get rid of the deleted item */
-          gtk_widget_destroy (item);
+      /* Get rid of the deleted item */
+      gtk_widget_destroy (item);
 
-          /* Resize everything */
-          gtk_cell_area_context_reset (menu->priv->context);
-        }
+      /* Resize everything */
+      gtk_cell_area_context_reset (menu->priv->context);
     }
   else
     {
@@ -1032,27 +825,16 @@ row_changed_cb (GtkTreeModel         *model,
 
   if (item)
     {
-      if (priv->wrap_width > 0)
-        /* Ugly, we need to rebuild the menu here if
-         * the row-span/row-column values change
-         */
-        rebuild_menu (menu);
-      else
+      if (priv->row_separator_func)
+        is_separator = priv->row_separator_func (model, iter, priv->row_separator_data);
+
+      if (is_separator != GTK_IS_SEPARATOR_MENU_ITEM (item))
         {
-          if (priv->row_separator_func)
-            is_separator =
-              priv->row_separator_func (model, iter,
-                                        priv->row_separator_data);
+          gint position = menu_item_position (menu, item);
 
-
-          if (is_separator != GTK_IS_SEPARATOR_MENU_ITEM (item))
-            {
-              gint position = menu_item_position (menu, item);
-
-              gtk_widget_destroy (item);
-              item = gtk_tree_menu_create_item (menu, iter, FALSE);
-              gtk_menu_shell_insert (GTK_MENU_SHELL (menu), item, position);
-            }
+          gtk_widget_destroy (item);
+          item = gtk_tree_menu_create_item (menu, iter, FALSE);
+          gtk_menu_shell_insert (GTK_MENU_SHELL (menu), item, position);
         }
     }
 }
@@ -1165,93 +947,6 @@ gtk_tree_menu_set_area (GtkTreeMenu *menu,
     }
 }
 
-static gboolean
-menu_occupied (GtkTreeMenu *menu,
-               guint        left_attach,
-               guint        right_attach,
-               guint        top_attach,
-               guint        bottom_attach)
-{
-  GList *i;
-
-  for (i = GTK_MENU_SHELL (menu)->priv->children; i; i = i->next)
-    {
-      guint l, r, b, t;
-
-      gtk_container_child_get (GTK_CONTAINER (menu),
-                               i->data,
-                               "left-attach", &l,
-                               "right-attach", &r,
-                               "bottom-attach", &b,
-                               "top-attach", &t,
-                               NULL);
-
-      /* look if this item intersects with the given coordinates */
-      if (right_attach > l && left_attach < r && bottom_attach > t && top_attach < b)
-        return TRUE;
-    }
-
-  return FALSE;
-}
-
-static void
-relayout_item (GtkTreeMenu *menu,
-               GtkWidget   *item,
-               GtkTreeIter *iter,
-               GtkWidget   *prev)
-{
-  GtkTreeMenuPrivate *priv = menu->priv;
-  gint                current_col = 0, current_row = 0;
-  gint                rows = 1, cols = 1;
-
-  if (priv->col_span_col == -1 &&
-      priv->row_span_col == -1 &&
-      prev)
-    {
-      gtk_container_child_get (GTK_CONTAINER (menu), prev,
-                               "right-attach", &current_col,
-                               "top-attach", &current_row,
-                               NULL);
-      if (current_col + cols > priv->wrap_width)
-        {
-          current_col = 0;
-          current_row++;
-        }
-    }
-  else
-    {
-      if (priv->col_span_col != -1)
-        gtk_tree_model_get (priv->model, iter,
-                            priv->col_span_col, &cols,
-                            -1);
-      if (priv->row_span_col != -1)
-        gtk_tree_model_get (priv->model, iter,
-                            priv->row_span_col, &rows,
-                            -1);
-
-      while (1)
-        {
-          if (current_col + cols > priv->wrap_width)
-            {
-              current_col = 0;
-              current_row++;
-            }
-
-          if (!menu_occupied (menu,
-                              current_col, current_col + cols,
-                              current_row, current_row + rows))
-            break;
-
-          current_col++;
-        }
-    }
-
-  /* set attach props */
-  gtk_menu_attach (GTK_MENU (menu), item,
-                   current_col, current_col + cols,
-                   current_row, current_row + rows);
-}
-
 static void
 gtk_tree_menu_create_submenu (GtkTreeMenu *menu,
                               GtkWidget   *item,
@@ -1271,10 +966,6 @@ gtk_tree_menu_create_submenu (GtkTreeMenu *menu,
                                          priv->row_separator_data,
                                          priv->row_separator_destroy);
 
-  _gtk_tree_menu_set_wrap_width (GTK_TREE_MENU (submenu), priv->wrap_width);
-  _gtk_tree_menu_set_row_span_column (GTK_TREE_MENU (submenu), priv->row_span_col);
-  _gtk_tree_menu_set_column_span_column (GTK_TREE_MENU (submenu), priv->col_span_col);
-
   gtk_tree_menu_set_model_internal (GTK_TREE_MENU (submenu), priv->model);
   _gtk_tree_menu_set_root (GTK_TREE_MENU (submenu), path);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
@@ -1303,7 +994,6 @@ gtk_tree_menu_create_item (GtkTreeMenu *menu,
   if (is_separator)
     {
       item = gtk_separator_menu_item_new ();
-      gtk_widget_show (item);
 
       g_object_set_qdata_full (G_OBJECT (item),
                                tree_menu_path_quark,
@@ -1314,13 +1004,10 @@ gtk_tree_menu_create_item (GtkTreeMenu *menu,
     {
       view = gtk_cell_view_new_with_context (priv->area, priv->context);
       item = gtk_menu_item_new ();
-      gtk_widget_show (view);
-      gtk_widget_show (item);
 
       gtk_cell_view_set_model (GTK_CELL_VIEW (view), priv->model);
       gtk_cell_view_set_displayed_row (GTK_CELL_VIEW (view), path);
 
-      gtk_widget_show (view);
       gtk_container_add (GTK_CONTAINER (item), view);
 
       g_signal_connect (item, "activate", G_CALLBACK (item_activated_cb), menu);
@@ -1359,7 +1046,7 @@ gtk_tree_menu_populate (GtkTreeMenu *menu)
   GtkTreeIter         parent;
   GtkTreeIter         iter;
   gboolean            valid = FALSE;
-  GtkWidget          *menu_item, *prev = NULL;
+  GtkWidget          *menu_item;
 
   if (!priv->model)
     return;
@@ -1376,20 +1063,6 @@ gtk_tree_menu_populate (GtkTreeMenu *menu)
     }
   else
     {
-      /* Tearoff menu items only go in the root menu */
-      if (priv->tearoff)
-        {
-          menu_item = gtk_tearoff_menu_item_new ();
-          gtk_widget_show (menu_item);
-
-          if (priv->wrap_width > 0)
-            gtk_menu_attach (GTK_MENU (menu), menu_item, 0, priv->wrap_width, 0, 1);
-          else
-            gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-
-          prev = menu_item;
-        }
-
       valid = gtk_tree_model_iter_children (priv->model, &iter, NULL);
     }
 
@@ -1401,10 +1074,6 @@ gtk_tree_menu_populate (GtkTreeMenu *menu)
 
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 
-      if (priv->wrap_width > 0)
-        relayout_item (menu, menu_item, &iter, prev);
-
-      prev  = menu_item;
       valid = gtk_tree_model_iter_next (priv->model, &iter);
     }
 }
@@ -1500,8 +1169,6 @@ gtk_tree_menu_set_model_internal (GtkTreeMenu  *menu,
  * Creates a new #GtkTreeMenu.
  *
  * Returns: A newly created #GtkTreeMenu with no model or root.
- *
- * Since: 3.0
  */
 GtkWidget *
 _gtk_tree_menu_new (void)
@@ -1516,8 +1183,6 @@ _gtk_tree_menu_new (void)
  * Creates a new #GtkTreeMenu using @area to render its cells.
  *
  * Returns: A newly created #GtkTreeMenu with no model or root.
- *
- * Since: 3.0
  */
 GtkWidget *
 _gtk_tree_menu_new_with_area (GtkCellArea    *area)
@@ -1536,8 +1201,6 @@ _gtk_tree_menu_new_with_area (GtkCellArea    *area)
  * Creates a new #GtkTreeMenu hierarchy from the provided @model and @root using @area to render its cells.
  *
  * Returns: A newly created #GtkTreeMenu.
- *
- * Since: 3.0
  */
 GtkWidget *
 _gtk_tree_menu_new_full (GtkCellArea         *area,
@@ -1557,8 +1220,6 @@ _gtk_tree_menu_new_full (GtkCellArea         *area,
  * @model: (allow-none): the #GtkTreeModel to build the menu hierarchy from, or %NULL.
  *
  * Sets @model to be used to build the menu heirarhcy.
- *
- * Since: 3.0
  */
 void
 _gtk_tree_menu_set_model (GtkTreeMenu  *menu,
@@ -1580,8 +1241,6 @@ _gtk_tree_menu_set_model (GtkTreeMenu  *menu,
  *
  * Returns: (transfer none): the #GtkTreeModel which is used
  * for @menu’s hierarchy.
- *
- * Since: 3.0
  */
 GtkTreeModel *
 _gtk_tree_menu_get_model (GtkTreeMenu *menu)
@@ -1602,8 +1261,6 @@ _gtk_tree_menu_get_model (GtkTreeMenu *menu)
  *
  * Sets the root of a @menu’s hierarchy to be @path. @menu must already
  * have a model set and @path must point to a valid path inside the model.
- *
- * Since: 3.0
  */
 void
 _gtk_tree_menu_set_root (GtkTreeMenu *menu,
@@ -1636,8 +1293,6 @@ _gtk_tree_menu_set_root (GtkTreeMenu *menu,
  *
  * Returns: (transfer full) (allow-none): A newly created #GtkTreePath
  * pointing to the root of @menu which must be freed with gtk_tree_path_free().
- *
- * Since: 3.0
  */
 GtkTreePath *
 _gtk_tree_menu_get_root (GtkTreeMenu *menu)
@@ -1654,223 +1309,6 @@ _gtk_tree_menu_get_root (GtkTreeMenu *menu)
   return NULL;
 }
 
-/*
- * _gtk_tree_menu_get_tearoff:
- * @menu: a #GtkTreeMenu
- *
- * Gets whether this menu is build with a leading tearoff menu item.
- *
- * Returns: %TRUE if the menu has a tearoff item.
- *
- * Since: 3.0
- */
-gboolean
-_gtk_tree_menu_get_tearoff (GtkTreeMenu *menu)
-{
-  GtkTreeMenuPrivate *priv;
-
-  g_return_val_if_fail (GTK_IS_TREE_MENU (menu), FALSE);
-
-  priv = menu->priv;
-
-  return priv->tearoff;
-}
-
-/*
- * _gtk_tree_menu_set_tearoff:
- * @menu: a #GtkTreeMenu
- * @tearoff: whether the menu should have a leading tearoff menu item.
- *
- * Sets whether this menu has a leading tearoff menu item.
- *
- * Since: 3.0
- */
-void
-_gtk_tree_menu_set_tearoff (GtkTreeMenu *menu,
-                            gboolean     tearoff)
-{
-  GtkTreeMenuPrivate *priv;
-
-  g_return_if_fail (GTK_IS_TREE_MENU (menu));
-
-  priv = menu->priv;
-
-  if (priv->tearoff != tearoff)
-    {
-      priv->tearoff = tearoff;
-
-      rebuild_menu (menu);
-
-      g_object_notify (G_OBJECT (menu), "tearoff");
-    }
-}
-
-/*
- * _gtk_tree_menu_get_wrap_width:
- * @menu: a #GtkTreeMenu
- *
- * Gets the wrap width which is used to determine the number of columns
- * for @menu. If the wrap width is larger than 1, @menu is in table mode.
- *
- * Returns: the wrap width.
- *
- * Since: 3.0
- */
-gint
-_gtk_tree_menu_get_wrap_width (GtkTreeMenu *menu)
-{
-  GtkTreeMenuPrivate *priv;
-
-  g_return_val_if_fail (GTK_IS_TREE_MENU (menu), FALSE);
-
-  priv = menu->priv;
-
-  return priv->wrap_width;
-}
-
-/*
- * _gtk_tree_menu_set_wrap_width:
- * @menu: a #GtkTreeMenu
- * @width: the wrap width
- *
- * Sets the wrap width which is used to determine the number of columns
- * for @menu. If the wrap width is larger than 1, @menu is in table mode.
- *
- * Since: 3.0
- */
-void
-_gtk_tree_menu_set_wrap_width (GtkTreeMenu *menu,
-                               gint         width)
-{
-  GtkTreeMenuPrivate *priv;
-
-  g_return_if_fail (GTK_IS_TREE_MENU (menu));
-  g_return_if_fail (width >= 0);
-
-  priv = menu->priv;
-
-  if (priv->wrap_width != width)
-    {
-      priv->wrap_width = width;
-
-      rebuild_menu (menu);
-
-      g_object_notify (G_OBJECT (menu), "wrap-width");
-    }
-}
-
-/*
- * _gtk_tree_menu_get_row_span_column:
- * @menu: a #GtkTreeMenu
- *
- * Gets the column with row span information for @menu.
- * The row span column contains integers which indicate how many rows
- * a menu item should span.
- *
- * Returns: the column in @menu’s model containing row span information, or -1.
- *
- * Since: 3.0
- */
-gint
-_gtk_tree_menu_get_row_span_column (GtkTreeMenu *menu)
-{
-  GtkTreeMenuPrivate *priv;
-
-  g_return_val_if_fail (GTK_IS_TREE_MENU (menu), FALSE);
-
-  priv = menu->priv;
-
-  return priv->row_span_col;
-}
-
-/*
- * _gtk_tree_menu_set_row_span_column:
- * @menu: a #GtkTreeMenu
- * @row_span: the column in the model to fetch the row span for a given menu item.
- *
- * Sets the column with row span information for @menu to be @row_span.
- * The row span column contains integers which indicate how many rows
- * a menu item should span.
- *
- * Since: 3.0
- */
-void
-_gtk_tree_menu_set_row_span_column (GtkTreeMenu *menu,
-                                    gint         row_span)
-{
-  GtkTreeMenuPrivate *priv;
-
-  g_return_if_fail (GTK_IS_TREE_MENU (menu));
-
-  priv = menu->priv;
-
-  if (priv->row_span_col != row_span)
-    {
-      priv->row_span_col = row_span;
-
-      if (priv->wrap_width > 0)
-        rebuild_menu (menu);
-
-      g_object_notify (G_OBJECT (menu), "row-span-column");
-    }
-}
-
-/*
- * _gtk_tree_menu_get_column_span_column:
- * @menu: a #GtkTreeMenu
- *
- * Gets the column with column span information for @menu.
- * The column span column contains integers which indicate how many columns
- * a menu item should span.
- *
- * Returns: the column in @menu’s model containing column span information, or -1.
- *
- * Since: 3.0
- */
-gint
-_gtk_tree_menu_get_column_span_column (GtkTreeMenu *menu)
-{
-  GtkTreeMenuPrivate *priv;
-
-  g_return_val_if_fail (GTK_IS_TREE_MENU (menu), FALSE);
-
-  priv = menu->priv;
-
-  return priv->col_span_col;
-}
-
-/*
- * _gtk_tree_menu_set_column_span_column:
- * @menu: a #GtkTreeMenu
- * @column_span: the column in the model to fetch the column span for a given menu item.
- *
- * Sets the column with column span information for @menu to be @column_span.
- * The column span column contains integers which indicate how many columns
- * a menu item should span.
- *
- * Since: 3.0
- */
-void
-_gtk_tree_menu_set_column_span_column (GtkTreeMenu *menu,
-                                       gint         column_span)
-{
-  GtkTreeMenuPrivate *priv;
-
-  g_return_if_fail (GTK_IS_TREE_MENU (menu));
-
-  priv = menu->priv;
-
-  if (priv->col_span_col != column_span)
-    {
-      priv->col_span_col = column_span;
-
-      if (priv->wrap_width > 0)
-        rebuild_menu (menu);
-
-      g_object_notify (G_OBJECT (menu), "column-span-column");
-    }
-}
-
 /*
  * _gtk_tree_menu_get_row_separator_func:
  * @menu: a #GtkTreeMenu
@@ -1878,8 +1316,6 @@ _gtk_tree_menu_set_column_span_column (GtkTreeMenu *menu,
  * Gets the current #GtkTreeViewRowSeparatorFunc separator function.
  *
  * Returns: the current row separator function.
- *
- * Since: 3.0
  */
 GtkTreeViewRowSeparatorFunc
 _gtk_tree_menu_get_row_separator_func (GtkTreeMenu *menu)
@@ -1903,8 +1339,6 @@ _gtk_tree_menu_get_row_separator_func (GtkTreeMenu *menu)
  * Sets the row separator function, which is used to determine
  * whether a row should be drawn as a separator. If the row separator
  * function is %NULL, no separators are drawn. This is the default value.
- *
- * Since: 3.0
  */
 void
 _gtk_tree_menu_set_row_separator_func (GtkTreeMenu          *menu,
