@@ -19,6 +19,8 @@
 #include <glib/gi18n-lib.h>
 
 #include "size-groups.h"
+
+#include "highlightoverlay.h"
 #include "window.h"
 
 #include "gtkcomboboxtext.h"
@@ -26,12 +28,15 @@
 #include "gtklabel.h"
 #include "gtklistbox.h"
 #include "gtksizegroup.h"
+#include "gtkstylecontext.h"
 #include "gtkswitch.h"
 #include "gtkwidgetprivate.h"
+#include "gtkstack.h"
 
 
 typedef struct {
   GtkListBoxRow parent;
+  GtkInspectorOverlay *highlight;
   GtkWidget *widget;
 } SizeGroupRow;
 
@@ -139,10 +144,18 @@ size_group_state_flags_changed (GtkWidget     *widget,
   state = gtk_widget_get_state_flags (widget);
   if ((state & GTK_STATE_FLAG_PRELIGHT) != (old_state & GTK_STATE_FLAG_PRELIGHT))
     {
+      GtkInspectorWindow *iw = GTK_INSPECTOR_WINDOW (gtk_widget_get_toplevel (widget));
+
       if (state & GTK_STATE_FLAG_PRELIGHT)
-        gtk_inspector_start_highlight (row->widget);
+        {
+          row->highlight = gtk_highlight_overlay_new (row->widget);
+          gtk_inspector_window_add_overlay (iw, row->highlight);
+        }
       else
-        gtk_inspector_stop_highlight (row->widget);
+        {
+          gtk_inspector_window_remove_overlay (iw, row->highlight);
+          g_clear_object (&row->highlight);
+        }
     }
 }
 
@@ -198,7 +211,6 @@ add_widget (GtkInspectorSizeGroups *sl,
   g_object_set (label, "margin", 10, NULL);
   gtk_widget_set_halign (label, GTK_ALIGN_START);
   gtk_widget_set_valign (label, GTK_ALIGN_BASELINE);
-  gtk_widget_show (label);
   gtk_container_add (GTK_CONTAINER (row), label);
   gtk_container_add (GTK_CONTAINER (listbox), row);
 }
@@ -208,7 +220,7 @@ add_size_group (GtkInspectorSizeGroups *sl,
                 GtkSizeGroup           *group)
 {
   GtkWidget *frame, *box, *box2;
-  GtkWidget *label, *sw, *combo;
+  GtkWidget *label, *combo;
   GSList *widgets, *l;
   GtkWidget *listbox;
 
@@ -221,29 +233,11 @@ add_size_group (GtkInspectorSizeGroups *sl,
   box2 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
   gtk_container_add (GTK_CONTAINER (box), box2);
 
-  label = gtk_label_new (_("Ignore hidden"));
-  g_object_set (label, "margin", 10, NULL);
-  gtk_widget_set_halign (label, GTK_ALIGN_START);
-  gtk_widget_set_valign (label, GTK_ALIGN_BASELINE);
-  gtk_box_pack_start (GTK_BOX (box2), label, TRUE, TRUE, 0);
-
-  sw = gtk_switch_new ();
-  g_object_set (sw, "margin", 10, NULL);
-  gtk_widget_set_halign (sw, GTK_ALIGN_END);
-  gtk_widget_set_valign (sw, GTK_ALIGN_BASELINE);
-  g_object_bind_property (group, "ignore-hidden",
-                          sw, "active",
-                          G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
-  gtk_box_pack_start (GTK_BOX (box2), sw, FALSE, FALSE, 0);
-
-  box2 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
-  gtk_container_add (GTK_CONTAINER (box), box2);
-
   label = gtk_label_new (_("Mode"));
   g_object_set (label, "margin", 10, NULL);
   gtk_widget_set_halign (label, GTK_ALIGN_START);
   gtk_widget_set_valign (label, GTK_ALIGN_BASELINE);
-  gtk_box_pack_start (GTK_BOX (box2), label, TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (box2), label);
 
   combo = gtk_combo_box_text_new ();
   g_object_set (combo, "margin", 10, NULL);
@@ -256,7 +250,7 @@ add_size_group (GtkInspectorSizeGroups *sl,
   g_object_bind_property (group, "mode",
                           combo, "active",
                           G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
-  gtk_box_pack_start (GTK_BOX (box2), combo, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (box2), combo);
 
   listbox = gtk_list_box_new ();
   gtk_container_add (GTK_CONTAINER (box), listbox);
@@ -265,8 +259,6 @@ add_size_group (GtkInspectorSizeGroups *sl,
   widgets = gtk_size_group_get_widgets (group);
   for (l = widgets; l; l = l->next)
     add_widget (sl, GTK_LIST_BOX (listbox), GTK_WIDGET (l->data));
-
-  gtk_widget_show_all (frame);
 }
 
 void
@@ -274,18 +266,22 @@ gtk_inspector_size_groups_set_object (GtkInspectorSizeGroups *sl,
                                       GObject                *object)
 {
   GSList *groups, *l;
+  GtkWidget *stack;
+  GtkStackPage *page;
+
+  stack = gtk_widget_get_parent (GTK_WIDGET (sl));
+  page = gtk_stack_get_page (GTK_STACK (stack), GTK_WIDGET (sl));
+
+  g_object_set (page, "visible", FALSE, NULL);
 
   clear_view (sl);
 
   if (!GTK_IS_WIDGET (object))
-    {
-      gtk_widget_hide (GTK_WIDGET (sl));
-      return;
-    }
+    return;
 
   groups = _gtk_widget_get_sizegroups (GTK_WIDGET (object));
   if (groups)
-    gtk_widget_show (GTK_WIDGET (sl));
+    g_object_set (page, "visible", TRUE, NULL);
   for (l = groups; l; l = l->next)
     {
       GtkSizeGroup *group = l->data;
