@@ -71,40 +71,40 @@ main (int    argc,
       char **argv)
 {
   GtkWidget *window, *label, *entry, *button, *grid, *notebook;
-  GtkWidget *vbox, *hbox, *grid_hbox, *spin, *spin2, *toggle, *combo, *image, *ebox;
+  GtkWidget *vbox, *hbox, *grid_hbox, *spin, *spin2, *toggle, *combo, *image;
   GtkAdjustment *adjustment;
   int i, j;
   GtkCssProvider *provider;
 
-  gtk_init (&argc, &argv);
+  gtk_init ();
 
   provider = gtk_css_provider_new ();
   gtk_css_provider_load_from_data (provider,
     ".small-font { font-size: 5px; }"
     ".medium-font { font-size: 10px; }"
-    ".large-font { font-size: 15px; }", -1, NULL);
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+    ".large-font { font-size: 15px; }", -1);
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
   g_object_unref (provider);
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  g_signal_connect (G_OBJECT (window), "delete-event", G_CALLBACK (gtk_main_quit), NULL);
+  g_signal_connect (G_OBJECT (window), "destroy", G_CALLBACK (gtk_main_quit), NULL);
 
   notebook = gtk_notebook_new ();
   gtk_container_add (GTK_CONTAINER (window), notebook);
 
-  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
 			    vbox, gtk_label_new ("hboxes"));
 
   for (j = 0; j < 2; j++)
     {
+      hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
+      gtk_container_add (GTK_CONTAINER (vbox), hbox);
+
       char *aligns_names[] = { "FILL", "BASELINE" };
       GtkAlign aligns[] = { GTK_ALIGN_FILL, GTK_ALIGN_BASELINE};
 
-      hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
-      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5);
-
       label = gtk_label_new (aligns_names[j]);
       gtk_container_add (GTK_CONTAINER (hbox), label);
 
@@ -120,7 +120,7 @@ main (int    argc,
 
       for (i = 0; i < 3; i++) {
 	entry = gtk_entry_new ();
-	gtk_entry_set_text (GTK_ENTRY (entry), "│XYyj,Ö...");
+	gtk_editable_set_text (GTK_EDITABLE (entry), "│XYyj,Ö...");
 
         set_font_size (entry, i);
 
@@ -133,10 +133,14 @@ main (int    argc,
       gtk_orientable_set_orientation (GTK_ORIENTABLE (spin), GTK_ORIENTATION_VERTICAL);
       gtk_widget_set_valign (spin, aligns[j]);
       gtk_container_add (GTK_CONTAINER (hbox), spin);
+
+      spin = gtk_spin_button_new (NULL, 0, 1);
+      gtk_widget_set_valign (spin, aligns[j]);
+      gtk_container_add (GTK_CONTAINER (hbox), spin);
     }
 
   grid_hbox = hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);
+  gtk_container_add (GTK_CONTAINER (vbox), hbox);
 
   combo = gtk_combo_box_text_new ();
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), baseline_pos_str[0]);
@@ -148,7 +152,7 @@ main (int    argc,
   for (j = 0; j < 2; j++)
     {
       hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
-      gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);
+      gtk_container_add (GTK_CONTAINER (vbox), hbox);
 
       g_signal_connect (G_OBJECT (combo), "changed",
 			G_CALLBACK (baseline_position_changed), hbox);
@@ -173,11 +177,12 @@ main (int    argc,
 
       for (i = 0; i < 3; i++)
 	{
-	  button = gtk_button_new_with_label ("│Xyj,Ö");
+          GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL,  6);
+          button = gtk_button_new ();
 
-	  gtk_button_set_image (GTK_BUTTON (button),
-				gtk_image_new_from_icon_name ("face-sad", GTK_ICON_SIZE_BUTTON));
-	  gtk_button_set_always_show_image (GTK_BUTTON (button), TRUE);
+          gtk_container_add (GTK_CONTAINER (box), gtk_label_new ("│Xyj,Ö"));
+          gtk_container_add (GTK_CONTAINER (box), gtk_image_new_from_icon_name ("face-sad"));
+          gtk_container_add (GTK_CONTAINER (button), box);
 
           set_font_size (button, i);
 
@@ -187,16 +192,11 @@ main (int    argc,
 	  gtk_container_add (GTK_CONTAINER (hbox), button);
 	}
 
-      ebox = gtk_event_box_new ();
-      if (j == 0)
-	gtk_widget_set_valign (ebox, GTK_ALIGN_BASELINE);
-      gtk_container_add (GTK_CONTAINER (hbox), ebox);
-
-      image = gtk_image_new_from_icon_name ("face-sad", GTK_ICON_SIZE_BUTTON);
+      image = gtk_image_new_from_icon_name ("face-sad");
       gtk_image_set_pixel_size (GTK_IMAGE (image), 34);
       if (j == 0)
 	gtk_widget_set_valign (image, GTK_ALIGN_BASELINE);
-      gtk_container_add (GTK_CONTAINER (ebox), image);
+      gtk_container_add (GTK_CONTAINER (hbox), image);
 
       button = gtk_toggle_button_new_with_label ("│Xyj,Ö");
       if (j == 0)
@@ -204,7 +204,6 @@ main (int    argc,
       gtk_container_add (GTK_CONTAINER (hbox), button);
 
       button = gtk_toggle_button_new_with_label ("│Xyj,Ö");
-      gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), TRUE);
       if (j == 0)
 	gtk_widget_set_valign (button, GTK_ALIGN_BASELINE);
       gtk_container_add (GTK_CONTAINER (hbox), button);
@@ -226,7 +225,7 @@ main (int    argc,
 			    vbox, gtk_label_new ("grid"));
 
   grid_hbox = hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);
+  gtk_container_add (GTK_CONTAINER (vbox), hbox);
 
   label = gtk_label_new ("Align me:");
   gtk_widget_set_valign (label, GTK_ALIGN_BASELINE);
@@ -270,11 +269,12 @@ main (int    argc,
 
       for (i = 0; i < 3; i++)
 	{
-	  button = gtk_button_new_with_label ("│Xyj,Ö");
+          GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL,  6);
+          button = gtk_button_new ();
 
-	  gtk_button_set_image (GTK_BUTTON (button),
-				gtk_image_new_from_icon_name ("face-sad", GTK_ICON_SIZE_BUTTON));
-	  gtk_button_set_always_show_image (GTK_BUTTON (button), TRUE);
+          gtk_container_add (GTK_CONTAINER (box), gtk_label_new ("│Xyj,Ö"));
+          gtk_container_add (GTK_CONTAINER (box), gtk_image_new_from_icon_name ("face-sad"));
+          gtk_container_add (GTK_CONTAINER (button), box);
 
           set_font_size (button, i);
 
@@ -292,7 +292,7 @@ main (int    argc,
   gtk_container_add (GTK_CONTAINER (hbox), grid);
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5);
+  gtk_container_add (GTK_CONTAINER (vbox), hbox);
 
   adjustment = gtk_adjustment_new (0.0, -1.0, 5.0, 1.0, 1.0, 0.0);
   spin = gtk_spin_button_new (adjustment, 1.0, 0);
@@ -317,7 +317,7 @@ main (int    argc,
 			    vbox, gtk_label_new ("button box"));
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5);
+  gtk_container_add (GTK_CONTAINER (vbox), hbox);
 
   adjustment = gtk_adjustment_new (34.0, 1.0, 64.0, 1.0, 1.0, 0.0);
   spin = gtk_spin_button_new (adjustment, 1.0, 0);
@@ -329,8 +329,8 @@ main (int    argc,
 
   for (j = 0; j < 3; j++)
     {
-      hbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
-      gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);
+      hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+      gtk_container_add (GTK_CONTAINER (vbox), hbox);
 
       gtk_box_set_baseline_position (GTK_BOX (hbox), j);
 
@@ -338,7 +338,7 @@ main (int    argc,
       gtk_container_add (GTK_CONTAINER (hbox), label);
       gtk_widget_set_vexpand (label, TRUE);
 
-      image = gtk_image_new_from_icon_name ("face-sad", GTK_ICON_SIZE_BUTTON);
+      image = gtk_image_new_from_icon_name ("face-sad");
       gtk_image_set_pixel_size (GTK_IMAGE (image), 34);
       gtk_container_add (GTK_CONTAINER (hbox), image);
 
@@ -358,14 +358,17 @@ main (int    argc,
 
       for (i = 0; i < 3; i++)
 	{
-	  button = gtk_button_new_with_label ("│Xyj,Ö");
+          GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL,  6);
+          button = gtk_button_new ();
+
+          gtk_container_add (GTK_CONTAINER (box), gtk_label_new ("│Xyj,Ö"));
+          image = gtk_image_new_from_icon_name ("face-sad");
+          gtk_image_set_pixel_size (GTK_IMAGE (image), 16);
+          gtk_container_add (GTK_CONTAINER (box), image);
+          gtk_container_add (GTK_CONTAINER (button), box);
 
-	  image = gtk_image_new_from_icon_name ("face-sad", GTK_ICON_SIZE_BUTTON);
-	  gtk_image_set_pixel_size (GTK_IMAGE (image), 16);
-	  gtk_button_set_image (GTK_BUTTON (button), image);
 	  if (i == 0)
 	    g_signal_connect (spin2, "value-changed", (GCallback)image_size_value_changed, image);
-	  gtk_button_set_always_show_image (GTK_BUTTON (button), TRUE);
 
           set_font_size (button, i);
 
@@ -375,7 +378,7 @@ main (int    argc,
 	}
     }
 
-  gtk_widget_show_all (window);
+  gtk_widget_show (window);
 
   gtk_main ();
 
