@@ -254,12 +254,11 @@ main (int argc, char **argv)
   GtkWidget *label;
   GtkWidget *tree;
 
-  gtk_init (&argc, &argv);
+  gtk_init ();
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   g_signal_connect (window, "destroy",
 		    G_CALLBACK (gtk_main_quit), NULL);
-  gtk_container_set_border_width (GTK_CONTAINER (window), 12);
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_container_add (GTK_CONTAINER (window), vbox);
@@ -267,20 +266,22 @@ main (int argc, char **argv)
   /* LTR */
 
   label = gtk_label_new ("Left to right");
-  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (vbox), label);
 
   tree = create_tree (FALSE);
-  gtk_box_pack_start (GTK_BOX (vbox), tree, TRUE, TRUE, 0);
+  gtk_widget_set_vexpand (tree, TRUE);
+  gtk_container_add (GTK_CONTAINER (vbox), tree);
 
   /* RTL */
 
   label = gtk_label_new ("Right to left");
-  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (vbox), label);
 
   tree = create_tree (TRUE);
-  gtk_box_pack_start (GTK_BOX (vbox), tree, TRUE, TRUE, 0);
+  gtk_widget_set_vexpand (tree, TRUE);
+  gtk_container_add (GTK_CONTAINER (vbox), tree);
 
-  gtk_widget_show_all (window);
+  gtk_widget_show (window);
   gtk_main ();
 
   return 0;
