@@ -8,7 +8,7 @@ main (int argc, char *argv[])
   GtkWidget *win;
   GtkWidget *view;
 
-  gtk_init (&argc, &argv);
+  gtk_init ();
 
   win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_default_size (GTK_WINDOW (win), 400, 600);
@@ -16,9 +16,9 @@ main (int argc, char *argv[])
   view = gtk_places_view_new ();
 
   gtk_container_add (GTK_CONTAINER (win), view);
-  gtk_widget_show_all (win);
+  gtk_widget_show (win);
 
-  g_signal_connect (win, "delete-event", G_CALLBACK (gtk_main_quit), win);
+  g_signal_connect (win, "destroy", G_CALLBACK (gtk_main_quit), win);
 
   gtk_main ();
 
