@@ -7,7 +7,7 @@ set_insensitive (GtkButton *b, GtkWidget *w)
 }
 
 static void
-state_changed (GtkWidget *widget)
+state_flags_changed (GtkWidget *widget)
 {
   GtkStateFlags flags;
   const gchar *sep;
@@ -58,28 +58,28 @@ int main (int argc, char *argv[])
   GtkWidget *bu;
   GtkWidget *w, *c;
 
-  gtk_init (&argc, &argv);
+  gtk_init ();
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
   gtk_container_add (GTK_CONTAINER (window), box);
 
   w = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 15);
-  gtk_box_pack_start (GTK_BOX (box), w, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (w), gtk_entry_new (), TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (box), w);
+  gtk_container_add (GTK_CONTAINER (w), gtk_entry_new ());
   bu = gtk_button_new_with_label ("Bu");
-  gtk_box_pack_start (GTK_BOX (w), bu, TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (w), bu);
   c = gtk_switch_new ();
   gtk_switch_set_active (GTK_SWITCH (c), TRUE);
   gtk_widget_set_halign (c, GTK_ALIGN_CENTER);
   gtk_widget_set_valign (c, GTK_ALIGN_CENTER);
-  gtk_box_pack_start (GTK_BOX (box), c, TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (box), c);
   g_signal_connect (bu, "clicked", G_CALLBACK (set_insensitive), w);
-  g_signal_connect (bu, "state-changed", G_CALLBACK (state_changed), NULL);
+  g_signal_connect (bu, "state-flags-changed", G_CALLBACK (state_flags_changed), NULL);
 
   g_object_bind_property (c, "active", w, "sensitive", G_BINDING_BIDIRECTIONAL);
 
-  gtk_widget_show_all (window);
+  gtk_widget_show (window);
 
   gtk_main ();
 
