@@ -703,8 +703,8 @@ selection_changed (GtkTreeSelection *selection, GtkWidget *button)
     gtk_widget_set_sensitive (button, FALSE);
 }
 
-static GtkTargetEntry row_targets[] = {
-  { "GTK_TREE_MODEL_ROW", GTK_TARGET_SAME_APP, 0}
+static const char *row_targets[] = {
+  "GTK_TREE_MODEL_ROW"
 };
 
 int
@@ -718,9 +718,10 @@ main (int argc, char *argv[])
   GtkCellRenderer *cell;
   GtkWidget *swindow;
   GtkTreeModel *sample_model;
+  GdkContentFormats *targets;
   gint i;
 
-  gtk_init (&argc, &argv);
+  gtk_init ();
 
   /* First initialize all the models for signal purposes */
   left_tree_model = (GtkTreeModel *) gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_POINTER);
@@ -749,7 +750,7 @@ main (int argc, char *argv[])
   swindow = gtk_scrolled_window_new (NULL, NULL);
   gtk_container_add (GTK_CONTAINER (window), swindow);
   gtk_container_add (GTK_CONTAINER (swindow), sample_tree_view_top);
-  gtk_widget_show_all (window);
+  gtk_widget_show (window);
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), NULL); 
@@ -758,18 +759,17 @@ main (int argc, char *argv[])
   swindow = gtk_scrolled_window_new (NULL, NULL);
   gtk_container_add (GTK_CONTAINER (window), swindow);
   gtk_container_add (GTK_CONTAINER (swindow), sample_tree_view_bottom);
-  gtk_widget_show_all (window);
+  gtk_widget_show (window);
 
   /* Set up the main window */
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), NULL); 
+  g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), NULL);
   gtk_window_set_default_size (GTK_WINDOW (window), 500, 300);
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 8);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
   gtk_container_add (GTK_CONTAINER (window), vbox);
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (vbox), hbox);
 
   /* Left Pane */
   cell = gtk_cell_renderer_text_new ();
@@ -786,52 +786,50 @@ main (int argc, char *argv[])
   gtk_tree_view_append_column (GTK_TREE_VIEW (left_tree_view), column);
 
   gtk_tree_view_column_set_cell_data_func (column, cell, get_visible, NULL, NULL);
-  gtk_box_pack_start (GTK_BOX (hbox), swindow, TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (hbox), swindow);
 
   /* Middle Pane */
   vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 8);
-  gtk_box_pack_start (GTK_BOX (hbox), vbox2, FALSE, FALSE, 0);
-  
-  bbox = gtk_button_box_new (GTK_ORIENTATION_VERTICAL);
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_SPREAD);
-  gtk_box_pack_start (GTK_BOX (vbox2), bbox, TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (hbox), vbox2);
+
+  bbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  gtk_container_add (GTK_CONTAINER (vbox2), bbox);
 
   button = gtk_button_new_with_mnemonic ("<< (_Q)");
   gtk_widget_set_sensitive (button, FALSE);
   g_signal_connect (button, "clicked", G_CALLBACK (add_left_clicked), top_right_tree_view);
   g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (top_right_tree_view)),
                     "changed", G_CALLBACK (selection_changed), button);
-  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (bbox), button);
 
   button = gtk_button_new_with_mnemonic (">> (_W)");
   gtk_widget_set_sensitive (button, FALSE);
   g_signal_connect (button, "clicked", G_CALLBACK (add_right_clicked), top_right_tree_view);
   g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (left_tree_view)),
                     "changed", G_CALLBACK (selection_changed), button);
-  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (bbox), button);
 
-  bbox = gtk_button_box_new (GTK_ORIENTATION_VERTICAL);
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_SPREAD);
-  gtk_box_pack_start (GTK_BOX (vbox2), bbox, TRUE, TRUE, 0);
+  bbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  gtk_container_add (GTK_CONTAINER (vbox2), bbox);
 
   button = gtk_button_new_with_mnemonic ("<< (_E)");
   gtk_widget_set_sensitive (button, FALSE);
   g_signal_connect (button, "clicked", G_CALLBACK (add_left_clicked), bottom_right_tree_view);
   g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (bottom_right_tree_view)),
                     "changed", G_CALLBACK (selection_changed), button);
-  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (bbox), button);
 
   button = gtk_button_new_with_mnemonic (">> (_R)");
   gtk_widget_set_sensitive (button, FALSE);
   g_signal_connect (button, "clicked", G_CALLBACK (add_right_clicked), bottom_right_tree_view);
   g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (left_tree_view)),
                     "changed", G_CALLBACK (selection_changed), button);
-  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (bbox), button);
+
 
-  
   /* Right Pane */
   vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 8);
-  gtk_box_pack_start (GTK_BOX (hbox), vbox2, TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (hbox), vbox2);
 
   swindow = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -846,7 +844,7 @@ main (int argc, char *argv[])
   gtk_tree_view_append_column (GTK_TREE_VIEW (top_right_tree_view), column);
 
   gtk_container_add (GTK_CONTAINER (swindow), top_right_tree_view);
-  gtk_box_pack_start (GTK_BOX (vbox2), swindow, TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (vbox2), swindow);
 
   swindow = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -860,51 +858,45 @@ main (int argc, char *argv[])
   gtk_tree_view_column_set_cell_data_func (column, cell, get_visible, NULL, NULL);
   gtk_tree_view_append_column (GTK_TREE_VIEW (bottom_right_tree_view), column);
   gtk_container_add (GTK_CONTAINER (swindow), bottom_right_tree_view);
-  gtk_box_pack_start (GTK_BOX (vbox2), swindow, TRUE, TRUE, 0);
+  gtk_container_add (GTK_CONTAINER (vbox2), swindow);
+
 
-  
   /* Drag and Drop */
+  targets = gdk_content_formats_new (row_targets, G_N_ELEMENTS (row_targets));
   gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW (left_tree_view),
 					  GDK_BUTTON1_MASK,
-					  row_targets,
-					  G_N_ELEMENTS (row_targets),
+                                          targets,
 					  GDK_ACTION_MOVE);
   gtk_tree_view_enable_model_drag_dest (GTK_TREE_VIEW (left_tree_view),
-					row_targets,
-					G_N_ELEMENTS (row_targets),
+					targets,
 					GDK_ACTION_MOVE);
 
   gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW (top_right_tree_view),
 					  GDK_BUTTON1_MASK,
-					  row_targets,
-					  G_N_ELEMENTS (row_targets),
+					  targets,
 					  GDK_ACTION_MOVE);
   gtk_tree_view_enable_model_drag_dest (GTK_TREE_VIEW (top_right_tree_view),
-					row_targets,
-					G_N_ELEMENTS (row_targets),
+					targets,
 					GDK_ACTION_MOVE);
 
   gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW (bottom_right_tree_view),
 					  GDK_BUTTON1_MASK,
-					  row_targets,
-					  G_N_ELEMENTS (row_targets),
+					  targets,
 					  GDK_ACTION_MOVE);
   gtk_tree_view_enable_model_drag_dest (GTK_TREE_VIEW (bottom_right_tree_view),
-					row_targets,
-					G_N_ELEMENTS (row_targets),
+					targets,
 					GDK_ACTION_MOVE);
+  gdk_content_formats_unref (targets);
 
-
-  gtk_box_pack_start (GTK_BOX (vbox), gtk_separator_new (GTK_ORIENTATION_HORIZONTAL),
-                      FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (vbox), gtk_separator_new (GTK_ORIENTATION_HORIZONTAL));
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (vbox), hbox);
   button = gtk_button_new_with_mnemonic ("_Add new Column");
   g_signal_connect (button, "clicked", G_CALLBACK (add_clicked), left_tree_model);
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (hbox), button);
 
-  gtk_widget_show_all (window);
+  gtk_widget_show (window);
   gtk_main ();
 
   return 0;
