@@ -11,8 +11,6 @@ static void
 test_parse_selectors (void)
 {
   GtkCssProvider *provider;
-  GError *error;
-  gboolean res;
   gint i;
   const gchar *valid[] = {
     "* {}",
@@ -24,11 +22,10 @@ test_parse_selectors (void)
     "#id {}",
     "tab:first-child {}",
     "tab:last-child {}",
-    "tab:nth-child(first) {}",
-    "tab:nth-child(last) {}",
+    "tab:first-child {}",
+    "tab:last-child {}",
     "tab:nth-child(even) {}",
     "tab:nth-child(odd) {}",
-    "tab:sorted {}",
     ".some-class {}",
     ".some-class.another-class {}",
     ".some-class .another-class {}",
@@ -50,23 +47,18 @@ test_parse_selectors (void)
     "E1.e1_2 #T3_4 {}",
     "E:first-child {}",
     "E:last-child {}",
-    "E:nth-child(first) {}",
-    "E:nth-child(last) {}",
+    "E:first-child {}",
+    "E:last-child {}",
     "E:nth-child(even) {}",
     "E:nth-child(odd) {}",
     "E:focus tab {}",
      NULL
   };
 
-  error = NULL;
   for (i = 0; valid[i]; i++)
     {
       provider = gtk_css_provider_new ();
-      res = gtk_css_provider_load_from_data (provider, valid[i], -1, &error);
-      if (error)
-        g_print ("parsing '%s': got unexpected error: %s\n", valid[i], error->message);
-      g_assert_no_error (error);
-      g_assert (res);
+      gtk_css_provider_load_from_data (provider, valid[i], -1);
 
       g_object_unref (provider);
    }
@@ -78,7 +70,6 @@ test_path (void)
   GtkWidgetPath *path;
   GtkWidgetPath *path2;
   gint pos;
-  GtkRegionFlags flags;
 
   path = gtk_widget_path_new ();
   g_assert_cmpint (gtk_widget_path_length (path), ==, 0);
@@ -121,25 +112,6 @@ test_path (void)
   gtk_widget_path_iter_clear_classes (path, 1);
   g_assert (!gtk_widget_path_iter_has_class (path, 1, "class1"));
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  gtk_widget_path_iter_add_region (path, 1, "tab", 0);
-  gtk_widget_path_iter_add_region (path, 1, "title", GTK_REGION_EVEN | GTK_REGION_FIRST);
-
-  g_assert (gtk_widget_path_iter_has_region (path, 1, "tab", &flags) &&
-            flags == 0);
-  g_assert (gtk_widget_path_iter_has_region (path, 1, "title", &flags) &&
-            flags == (GTK_REGION_EVEN | GTK_REGION_FIRST));
-  g_assert (!gtk_widget_path_iter_has_region (path, 1, "extension", NULL));
-
-  path2 = gtk_widget_path_copy (path);
-  g_assert (gtk_widget_path_iter_has_region (path2, 1, "tab", &flags) &&
-            flags == 0);
-  g_assert (gtk_widget_path_iter_has_region (path2, 1, "title", &flags) &&
-            flags == (GTK_REGION_EVEN | GTK_REGION_FIRST));
-  g_assert (!gtk_widget_path_iter_has_region (path2, 1, "extension", NULL));
-G_GNUC_END_IGNORE_DEPRECATIONS
-
-  gtk_widget_path_free (path2);
 
   gtk_widget_path_free (path);
 }
@@ -150,12 +122,10 @@ test_match (void)
   GtkStyleContext *context;
   GtkWidgetPath *path;
   GtkCssProvider *provider;
-  GError *error;
   const gchar *data;
   GdkRGBA color;
   GdkRGBA expected;
 
-  error = NULL;
   provider = gtk_css_provider_new ();
 
   gdk_rgba_parse (&expected, "#fff");
@@ -178,89 +148,78 @@ test_match (void)
                                   GTK_STYLE_PROVIDER (provider),
                                   GTK_STYLE_PROVIDER_PRIORITY_USER);
 
-  data = "* { color: #fff }";
-  gtk_css_provider_load_from_data (provider, data, -1, &error);
-  g_assert_no_error (error);
-  gtk_style_context_get_color (context, gtk_style_context_get_state (context), &color);
+  data = "* { color: #fff; }";
+  gtk_css_provider_load_from_data (provider, data, -1);
+  gtk_style_context_get_color (context, &color);
   g_assert (gdk_rgba_equal (&color, &expected));
 
-  data = "* { color: #f00 }\n"
-         "button { color: #fff }";
-  gtk_css_provider_load_from_data (provider, data, -1, &error);
-  g_assert_no_error (error);
-  gtk_style_context_get_color (context, gtk_style_context_get_state (context), &color);
+  data = "* { color: #f00; }\n"
+         "button { color: #fff; }";
+  gtk_css_provider_load_from_data (provider, data, -1);
+  gtk_style_context_get_color (context, &color);
   g_assert (gdk_rgba_equal (&color, &expected));
 
-  data = "* { color: #f00 }\n"
-         "button { color: #fff }\n"
-         "window > button { color: #000 }";
-  gtk_css_provider_load_from_data (provider, data, -1, &error);
-  g_assert_no_error (error);
-  gtk_style_context_get_color (context, gtk_style_context_get_state (context), &color);
+  data = "* { color: #f00; }\n"
+         "button { color: #fff; }\n"
+         "window > button { color: #000; }";
+  gtk_css_provider_load_from_data (provider, data, -1);
+  gtk_style_context_get_color (context, &color);
   g_assert (gdk_rgba_equal (&color, &expected));
 
-  data = "* { color: #f00 }\n"
-         ".button { color: #fff }";
-  gtk_css_provider_load_from_data (provider, data, -1, &error);
-  g_assert_no_error (error);
-  gtk_style_context_get_color (context, gtk_style_context_get_state (context), &color);
+  data = "* { color: #f00; }\n"
+         ".button { color: #fff; }";
+  gtk_css_provider_load_from_data (provider, data, -1);
+  gtk_style_context_get_color (context, &color);
   g_assert (gdk_rgba_equal (&color, &expected));
 
-  data = "* { color: #f00 }\n"
-         "button { color: #000 }\n"
-         ".button { color: #fff }";
-  gtk_css_provider_load_from_data (provider, data, -1, &error);
-  g_assert_no_error (error);
-  gtk_style_context_get_color (context, gtk_style_context_get_state (context), &color);
+  data = "* { color: #f00; }\n"
+         "button { color: #000; }\n"
+         ".button { color: #fff; }";
+  gtk_css_provider_load_from_data (provider, data, -1);
+  gtk_style_context_get_color (context, &color);
   g_assert (gdk_rgba_equal (&color, &expected));
 
-  data = "* { color: #f00 }\n"
-         "button { color: #000 }\n"
-         "window button { color: #fff }";
-  gtk_css_provider_load_from_data (provider, data, -1, &error);
-  g_assert_no_error (error);
-  gtk_style_context_get_color (context, gtk_style_context_get_state (context), &color);
+  data = "* { color: #f00; }\n"
+         "button { color: #000; }\n"
+         "window button { color: #fff; }";
+  gtk_css_provider_load_from_data (provider, data, -1);
+  gtk_style_context_get_color (context, &color);
   g_assert (gdk_rgba_equal (&color, &expected));
 
-  data = "* { color: #f00 }\n"
-         ".button { color: #000 }\n"
-         "window .button { color: #fff }";
-  gtk_css_provider_load_from_data (provider, data, -1, &error);
-  g_assert_no_error (error);
-  gtk_style_context_get_color (context, gtk_style_context_get_state (context), &color);
+  data = "* { color: #f00; }\n"
+         ".button { color: #000; }\n"
+         "window .button { color: #fff; }";
+  gtk_css_provider_load_from_data (provider, data, -1);
+  gtk_style_context_get_color (context, &color);
   g_assert (gdk_rgba_equal (&color, &expected));
 
-  data = "* { color: #f00 }\n"
-         "* .button { color: #000 }\n"
-         "#mywindow .button { color: #fff }";
-  gtk_css_provider_load_from_data (provider, data, -1, &error);
-  g_assert_no_error (error);
-  gtk_style_context_get_color (context, gtk_style_context_get_state (context), &color);
+  data = "* { color: #f00; }\n"
+         "* .button { color: #000; }\n"
+         "#mywindow .button { color: #fff; }";
+  gtk_css_provider_load_from_data (provider, data, -1);
+  gtk_style_context_get_color (context, &color);
   g_assert (gdk_rgba_equal (&color, &expected));
 
-  data = "* { color: #f00 }\n"
-         "window .button { color: #000 }\n"
-         "window#mywindow .button { color: #fff }";
-  gtk_css_provider_load_from_data (provider, data, -1, &error);
-  g_assert_no_error (error);
-  gtk_style_context_get_color (context, gtk_style_context_get_state (context), &color);
+  data = "* { color: #f00; }\n"
+         "window .button { color: #000; }\n"
+         "window#mywindow .button { color: #fff; }";
+  gtk_css_provider_load_from_data (provider, data, -1);
+  gtk_style_context_get_color (context, &color);
   g_assert (gdk_rgba_equal (&color, &expected));
 
-  data = "* { color: #f00 }\n"
-         "window .button { color: #000 }\n"
-         "window button.button { color: #fff }";
-  gtk_css_provider_load_from_data (provider, data, -1, &error);
-  g_assert_no_error (error);
-  gtk_style_context_get_color (context, gtk_style_context_get_state (context), &color);
+  data = "* { color: #f00; }\n"
+         "window .button { color: #000; }\n"
+         "window button.button { color: #fff; }";
+  gtk_css_provider_load_from_data (provider, data, -1);
+  gtk_style_context_get_color (context, &color);
   g_assert (gdk_rgba_equal (&color, &expected));
 
-  data = "* { color: #f00 }\n"
-         "window:backdrop .button { color: #000 }\n"
-         "window .button { color: #111 }\n"
-         "window:active .button { color: #fff }";
-  gtk_css_provider_load_from_data (provider, data, -1, &error);
-  g_assert_no_error (error);
-  gtk_style_context_get_color (context, gtk_style_context_get_state (context), &color);
+  data = "* { color: #f00; }\n"
+         "window:backdrop .button { color: #000; }\n"
+         "window .button { color: #111; }\n"
+         "window:active .button { color: #fff; }";
+  gtk_css_provider_load_from_data (provider, data, -1);
+  gtk_style_context_get_color (context, &color);
   g_assert (gdk_rgba_equal (&color, &expected));
 
   g_object_unref (provider);
@@ -281,7 +240,7 @@ test_basic_properties (void)
   gtk_style_context_set_path (context, path);
   gtk_widget_path_free (path);
 
-  gtk_style_context_get (context, gtk_style_context_get_state (context),
+  gtk_style_context_get (context,
                          "color", &color,
                          "background-color", &bg_color,
                          "font", &font,
@@ -297,24 +256,6 @@ test_basic_properties (void)
   g_object_unref (context);
 }
 
-void
-test_invalidate_saved (void)
-{
-  GtkWidget *window;
-  GtkStyleContext *context;
-
-  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-
-  context = gtk_widget_get_style_context (window);
-  gtk_style_context_save (context);
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  gtk_style_context_invalidate (context);
-G_GNUC_END_IGNORE_DEPRECATIONS
-  gtk_style_context_restore (context);
-
-  gtk_widget_destroy (window);
-}
-
 void
 test_widget_path_parent (void)
 {
@@ -373,20 +314,15 @@ static void
 test_style_priorities_setup (PrioritiesFixture *f,
                              gconstpointer      unused)
 {
-  GError *error = NULL;
-  GtkWidgetPath *path;
   f->blue_provider = gtk_css_provider_new ();
   f->red_provider = gtk_css_provider_new ();
   f->green_provider = gtk_css_provider_new ();
   f->context = gtk_style_context_new ();
-  path = gtk_widget_path_new ();
+  GtkWidgetPath *path = gtk_widget_path_new ();
 
-  gtk_css_provider_load_from_data (f->blue_provider, "* { color: blue; }", -1, &error);
-  g_assert_no_error (error);
-  gtk_css_provider_load_from_data (f->red_provider, "* { color: red; }", -1, &error);
-  g_assert_no_error (error);
-  gtk_css_provider_load_from_data (f->green_provider, "* { color: green; }", -1, &error);
-  g_assert_no_error (error);
+  gtk_css_provider_load_from_data (f->blue_provider, "* { color: blue; }", -1);
+  gtk_css_provider_load_from_data (f->red_provider, "* { color: red; }", -1);
+  gtk_css_provider_load_from_data (f->green_provider, "* { color: green; }", -1);
 
   gtk_widget_path_append_type (path, GTK_TYPE_WINDOW);
   gtk_style_context_set_path (f->context, path);
@@ -410,34 +346,32 @@ test_style_priorities_equal (PrioritiesFixture *f,
 {
   GdkRGBA color, ref_color;
 
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (f->blue_provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_USER);
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (f->blue_provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_USER);
   gtk_style_context_add_provider (f->context, GTK_STYLE_PROVIDER (f->red_provider),
                                   GTK_STYLE_PROVIDER_PRIORITY_USER);
 
-  /* When style providers are added to the screen as well as the style context
+  /* When style providers are added to the display as well as the style context
   the one specific to the style context should take priority */
   gdk_rgba_parse (&ref_color, "red");
-  gtk_style_context_get_color (f->context, gtk_style_context_get_state (f->context),
-                               &color);
+  gtk_style_context_get_color (f->context, &color);
 
   g_assert_true (gdk_rgba_equal (&ref_color, &color));
 }
 
 static void
-test_style_priorities_screen_only (PrioritiesFixture *f,
-                                   gconstpointer      unused)
+test_style_priorities_display_only (PrioritiesFixture *f,
+                                    gconstpointer      unused)
 {
   GdkRGBA color, ref_color;
 
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (f->blue_provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_USER);
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (f->blue_provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_USER);
 
   gdk_rgba_parse (&ref_color, "blue");
-  gtk_style_context_get_color (f->context, gtk_style_context_get_state (f->context),
-                               &color);
+  gtk_style_context_get_color (f->context, &color);
 
   g_assert_true (gdk_rgba_equal (&ref_color, &color));
 }
@@ -452,27 +386,25 @@ test_style_priorities_context_only (PrioritiesFixture *f,
                                   GTK_STYLE_PROVIDER_PRIORITY_USER);
 
   gdk_rgba_parse (&ref_color, "red");
-  gtk_style_context_get_color (f->context, gtk_style_context_get_state (f->context),
-                               &color);
+  gtk_style_context_get_color (f->context, &color);
 
   g_assert_true (gdk_rgba_equal (&ref_color, &color));
 }
 
 static void
-test_style_priorities_screen_higher (PrioritiesFixture *f,
-                                     gconstpointer      unused)
+test_style_priorities_display_higher (PrioritiesFixture *f,
+                                      gconstpointer      unused)
 {
   GdkRGBA color, ref_color;
 
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (f->blue_provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_USER + 1);
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (f->blue_provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_USER + 1);
   gtk_style_context_add_provider (f->context, GTK_STYLE_PROVIDER (f->red_provider),
                                   GTK_STYLE_PROVIDER_PRIORITY_USER);
 
   gdk_rgba_parse (&ref_color, "blue");
-  gtk_style_context_get_color (f->context, gtk_style_context_get_state (f->context),
-                               &color);
+  gtk_style_context_get_color (f->context, &color);
 
   g_assert_true (gdk_rgba_equal (&ref_color, &color));
 }
@@ -483,35 +415,33 @@ test_style_priorities_context_higher (PrioritiesFixture *f,
 {
   GdkRGBA color, ref_color;
 
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (f->blue_provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_USER);
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (f->blue_provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_USER);
   gtk_style_context_add_provider (f->context, GTK_STYLE_PROVIDER (f->red_provider),
                                   GTK_STYLE_PROVIDER_PRIORITY_USER + 1);
 
   gdk_rgba_parse (&ref_color, "red");
-  gtk_style_context_get_color (f->context, gtk_style_context_get_state (f->context),
-                               &color);
+  gtk_style_context_get_color (f->context, &color);
 
   g_assert_true (gdk_rgba_equal (&ref_color, &color));
 }
 
 static void
-test_style_priorities_two_screen (PrioritiesFixture *f,
-                                  gconstpointer      unused)
+test_style_priorities_two_display (PrioritiesFixture *f,
+                                   gconstpointer      unused)
 {
   GdkRGBA color, ref_color;
 
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (f->blue_provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_USER);
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (f->red_provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_USER + 1);
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (f->blue_provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_USER);
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (f->red_provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_USER + 1);
 
   gdk_rgba_parse (&ref_color, "red");
-  gtk_style_context_get_color (f->context, gtk_style_context_get_state (f->context),
-                               &color);
+  gtk_style_context_get_color (f->context, &color);
 
   g_assert_true (gdk_rgba_equal (&ref_color, &color));
 }
@@ -528,30 +458,28 @@ test_style_priorities_two_context (PrioritiesFixture *f,
                                   GTK_STYLE_PROVIDER_PRIORITY_USER + 1);
 
   gdk_rgba_parse (&ref_color, "red");
-  gtk_style_context_get_color (f->context, gtk_style_context_get_state (f->context),
-                               &color);
+  gtk_style_context_get_color (f->context, &color);
 
   g_assert_true (gdk_rgba_equal (&ref_color, &color));
 }
 
 static void
-test_style_priorities_three_screen_higher (PrioritiesFixture *f,
-                                           gconstpointer      unused)
+test_style_priorities_three_display_higher (PrioritiesFixture *f,
+                                            gconstpointer      unused)
 {
   GdkRGBA color, ref_color;
 
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (f->blue_provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_USER);
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (f->green_provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_USER + 1);
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (f->blue_provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_USER);
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (f->green_provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_USER + 1);
   gtk_style_context_add_provider (f->context, GTK_STYLE_PROVIDER (f->red_provider),
                                   GTK_STYLE_PROVIDER_PRIORITY_USER);
 
   gdk_rgba_parse (&ref_color, "green");
-  gtk_style_context_get_color (f->context, gtk_style_context_get_state (f->context),
-                               &color);
+  gtk_style_context_get_color (f->context, &color);
 
   g_assert_true (gdk_rgba_equal (&ref_color, &color));
 }
@@ -562,17 +490,16 @@ test_style_priorities_three_context_higher (PrioritiesFixture *f,
 {
   GdkRGBA color, ref_color;
 
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (f->blue_provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_USER);
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (f->blue_provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_USER);
   gtk_style_context_add_provider (f->context, GTK_STYLE_PROVIDER (f->red_provider),
                                   GTK_STYLE_PROVIDER_PRIORITY_USER);
   gtk_style_context_add_provider (f->context, GTK_STYLE_PROVIDER (f->green_provider),
                                   GTK_STYLE_PROVIDER_PRIORITY_USER + 1);
 
   gdk_rgba_parse (&ref_color, "green");
-  gtk_style_context_get_color (f->context, gtk_style_context_get_state (f->context),
-                               &color);
+  gtk_style_context_get_color (f->context, &color);
 
   g_assert_true (gdk_rgba_equal (&ref_color, &color));
 }
@@ -580,14 +507,13 @@ test_style_priorities_three_context_higher (PrioritiesFixture *f,
 int
 main (int argc, char *argv[])
 {
-  gtk_init (NULL, NULL);
+  gtk_init ();
   g_test_init (&argc, &argv, NULL);
 
   g_test_add_func ("/style/parse/selectors", test_parse_selectors);
   g_test_add_func ("/style/path", test_path);
   g_test_add_func ("/style/match", test_match);
   g_test_add_func ("/style/basic", test_basic_properties);
-  g_test_add_func ("/style/invalidate-saved", test_invalidate_saved);
   g_test_add_func ("/style/widget-path-parent", test_widget_path_parent);
   g_test_add_func ("/style/classes", test_style_classes);
 
@@ -596,16 +522,15 @@ main (int argc, char *argv[])
               (func), test_style_priorities_teardown)
 
   ADD_PRIORITIES_TEST ("equal", test_style_priorities_equal);
-  ADD_PRIORITIES_TEST ("screen-only", test_style_priorities_screen_only);
+  ADD_PRIORITIES_TEST ("display-only", test_style_priorities_display_only);
   ADD_PRIORITIES_TEST ("context-only", test_style_priorities_context_only);
-  ADD_PRIORITIES_TEST ("screen-higher", test_style_priorities_screen_higher);
+  ADD_PRIORITIES_TEST ("display-higher", test_style_priorities_display_higher);
   ADD_PRIORITIES_TEST ("context-higher", test_style_priorities_context_higher);
-  ADD_PRIORITIES_TEST ("two-screen", test_style_priorities_two_screen);
+  ADD_PRIORITIES_TEST ("two-display", test_style_priorities_two_display);
   ADD_PRIORITIES_TEST ("two-context", test_style_priorities_two_context);
-  ADD_PRIORITIES_TEST ("three-screen-higher", test_style_priorities_three_screen_higher);
+  ADD_PRIORITIES_TEST ("three-display-higher", test_style_priorities_three_display_higher);
   ADD_PRIORITIES_TEST ("three-context-higher", test_style_priorities_three_context_higher);
 
 #undef ADD_PRIORITIES_TEST
-
   return g_test_run ();
 }
